function hf = modImage(Ismall_in,Ismall_orig,hforig,varargin)
% this function opens a new window with selected image Ismall and allows user
% to modify this particular portion of the image.  The modified image is
% returned as Inew.  If the user decides not to apply the modified image to
% the original, Inew is returned as empty matrix.

%global Ismall Iundo2 himage Inew
global Ismall Inew himage Ismall0

Inew = [];  % default output image
Ismall = Ismall_in;
Ismall0 = Ismall_orig;

lorig = true;  % lorig is true if the original image exists
if isempty(Ismall_orig) || any(size(Ismall_in)~=size(Ismall_orig)) 
    lrogi = false;  % original image does not exist
end  % if isempty(Ismall_orig) || any (size(Ismall_in)~=size(Ismall_orig))

nargin;
if (nargin==3)  % assume rectangular region
    lrect = true;
    indx_mask = logical(ones(size(Ismall)));
elseif (nargin==4)  % assume input being mask 
    lrect = false;
    indx_mask = varargin{1};
else
    disp('!!Error(modImage): nargin needs to be 0 or 1');
end  % if (nargin==0)...
% figure;
% imshow(Ismall);

% set up the figure window
hf = figure;
pos = get(0,'ScreenSize');  % get screen size
pos(1) = pos(3)*0.25;
pos(2) = pos(4)*0.25;
pos(3:4) = 0.5*pos(3:4);
set(hf,'OuterPosition',pos);
set(hf,'Name','Regional Image Manipulation','NumberTitle','off',...
    'menu','none','toolbar','figure');

temp= findall(findall(hf,'Type','uitoolbar'));
delete(temp([2:9,13:numel(temp)]));  % remove some standard icons

htool.pan = findobj(temp,'tag','Exploration.Pan');
htool.zoomOut = findobj(temp,'tag','Exploration.ZoomOut');
htool.zoomIn = findobj(temp,'tag','Exploration.ZoomIn');

htool.origImage=uitoggletool(temp(1),'Tag','Original Image','Cdata',imread('icons/origImage.png'),...
    'Separator','on','TooltipString','Switch between original and processed images',...
    'HandleVisibility','on', 'State','off','ClickedCallback',@viewOrigImage);
htool.recoverRegion = uipushtool(temp(1),'Tag','Recover Region','Cdata',imread('icons/retrieveR.png','BackgroundColor',[1 1 1]),...
    'Separator','on','TooltipString','Draw a region to recover from the original image',...
    'HandleVisibility','on','ClickedCallback',@recoverOrig,'enable','on');
if ~lorig  % if original image does not exist
    set(htool.origImage,'enable','off');
    set(htool.recoverRegion,'enable','off');
end  % if ~lorig

htool.contrast = uipushtool(temp(1),'Tag','contrast','Cdata',imread('icons/contrast-icon.png','BackgroundColor',[1 1 1]),...
    'Separator','on','TooltipString','Adjust contrast',...
    'HandleVisibility','on','ClickedCallback',@adjust_contrast);
% if lrect  % images with full rectangular information
htool.rm_bg = uipushtool(temp(1),'Tag','Remove background','Cdata',imread('icons/Removebackground.png','BackgroundColor',[1 1 1]),...
        'Separator','on','TooltipString','Click remove large background stains-colorization',...
        'HandleVisibility','on','ClickedCallback',@remove_background_withGAUSSFILT);
% else  % there are regions that should not be analyzed
%     uipushtool(temp(1),'Tag','Remove background','Cdata',imread('Removebackground.png','BackgroundColor',[1 1 1]),...
%         'Separator','on','TooltipString','Click remove large background stains-colorization',...
%         'HandleVisibility','on','Enable','off','ClickedCallback',@remove_background_withGAUSSFILT);
% end  % if (numel(find(isanan(Ismall))==0))...
tempim=imnoise(zeros([16,16,3],'uint8'),'salt & pepper',0.2);
for ii=2:16, tempim(ii-1:ii,ii-1:ii,1)=255; tempim(ii-1:ii,ii-1:ii,2:3)=0 ;tempim([ii-1:ii],17-[ii-1:ii],1)=255; tempim([ii-1:ii],17-[ii-1:ii],2:3)=0; end
htool.rm_saltPepper = uipushtool(temp(1),'Cdata',tempim,...
    'Separator','off','TooltipString','Remove salt & pepper noise',...
    'HandleVisibility','on','Enable','on','ClickedCallback',@remove_SaltandPepper);
hrm_bg = uitoggletool(temp(1),'Tag','Remove Region','Cdata',imread('icons/removeS.png','BackgroundColor',[1 1 1]),...
    'Separator','on','TooltipString','Draw a region to remove',...
    'HandleVisibility','on','ClickedCallback',@remove_region);
htool.rm_region = hrm_bg;
hundo = uipushtool(temp(1),'Tag','undo','Cdata',imread('icons/undo.jpeg'),...
    'Separator','on','TooltipString','Undo last action',...
    'HandleVisibility','on','Enable','off','ClickedCallback',@Undo_remove);
htool.undo = hundo;
htool.whole = uipushtool(temp(1),'Tag','Show whole seismogram','Cdata',imread('icons/whole_seismogram.png','BackgroundColor',[1 1 1]),...
    'Separator','on','TooltipString','Show whole seismogram',...
    'HandleVisibility','on','ClickedCallback',@zoom2seismogram);

temp= findall(findall(hf,'Type','uitoolbar'));
% reorder tools 
htmp = [htool.whole; htool.pan; htool.zoomOut; htool.zoomIn; 
    htool.undo; 
    htool.rm_region;
    htool.rm_bg; htool.rm_saltPepper; 
    htool.recoverRegion; htool.origImage; htool.contrast;
    ];
set(temp(1),'Children',htmp);




hax = axes;  % where the image is going to be displayed
pos = [0.13 0.2 0.775 0.7];  % axis object position
set(hax,'position',pos,'parent',hf);
Itmp = Ismall;
Itmp(~indx_mask) = nan;
himage = image(Itmp,'parent',hax);
set(himage,'tag','himage');
colormap(hax,gray(256));
setappdata(hundo,'indx_mask',indx_mask);


% pushbuttons
pos = [0.295 0.05 0.2 0.1];
hpbApply = uicontrol('style','pushbutton','parent',hf,'units','normalized',...
    'position',pos,'string','Apply','callback',{@hpbCallback,hforig});
pos = [0.505 0.05 0.2 0.1];
hpbCancel = uicontrol('style','pushbutton','parent',hf,'units','normalized',...
    'position',pos,'string','Cancel','callback',{@hpbCallback,hforig});



end

function hpbCallback(hObj,eventdata,hforig)
% this function is called when one of the two push buttons is pressed

global Ismall Inew

hf = gcbf;
cstr = get(hObj,'string');

if (strcmpi(cstr,'Apply'))  % apply button has been pressed
    % update Inew to be Ismall
%      disp('updating');
    Inew = Ismall;
%     figure;
%     imshow(Inew)
    setappdata(hforig,'Inew',Inew);
end  % if (strcmpi(cstr,'Apply'))

% close the figure window
close(hf);

end  

function remove_region(hObject, eventdata)

%global Ismall Iundo2
global Ismall

hf = gcbf;
htool = findall(hf,'type','uitoolbar');
hundo = findobj(htool,'tag','undo');
hax = findobj(hf,'type','axes');
hrm_bg = findobj(htool,'tag','Remove Region');

try  % in order to avoid error message as the figure is closed
    % without this try statement, the closure of figure results in
    % "Error while evaluating DestroyedObject ClickedCallback"
    % message
    while strcmpi(get(hrm_bg,'state'),'on')  % while this button is activated
        Iundo2=Ismall; % save an undo copy
        h=imfreehand(hax);
        try; h.Deletable = false; end;
        try
            Ismall(h.createMask)=0;
            delete(h);
            set(hundo,'Enable','on');
            setappdata(hundo,'Iundo2',Iundo2);
            update_plot_image;
        catch
            delete(h);
            %return
        end
    end  % while strcmpi(get(hrm_bg,'state'),'on')
    
    set(hrm_bg,'state','off');
end
end


function adjust_contrast(hObject, eventdata)
%global Ismall Iundo2 himage
global Ismall
Iundo2=Ismall;
hf = gcbf;
htool = findall(hf,'type','uitoolbar');
hundo = findobj(htool,'tag','undo');
indx_mask = getappdata(hundo,'indx_mask');
setappdata(hundo,'Iundo2',Iundo2);
himage = findobj(hf,'tag','himage');

%hundo = findall(hf,'tag','undo');
hFig = figure('Color','w','Toolbar','none','NumberTitle','off',...
   'Menubar','none','Visible','off','closerequestfcn','','Name','Small Image');
Itmp = Ismall;
Itmp(~indx_mask) = nan;
hIm = imshow(Itmp);
hSP = imscrollpanel(hFig,hIm);
set(hSP,'Units','normalized',...
    'Position',[0.0 0.05 1 .95])
hMagBox = immagbox(hFig,hIm);
pos = get(hMagBox,'Position');
set(hMagBox,'Position',[0 0 pos(3) pos(4)])
%imoverview(hIm0);
hImCon=imcontrast(hIm);
set(hFig,'Visible','on')
waitfor(hImCon);
Itmp=get(hIm,'Cdata');
rtmp = Itmp(find(indx_mask));
imin = min(rtmp);
imax = max(rtmp);
rtmp0 = Ismall(find(indx_mask));
idx = find(rtmp==imin);  % minimum value
imin0 = max(rtmp0(idx));
idx = find(rtmp==imax);
imax0 = min(rtmp0(idx));
Ismall = imadjust(Ismall,double([imin0 imax0])/255, double([imin imax])/255);
update_plot_image;
delete(hFig);
% enable undo
set(hundo,'Enable','on');
end

function update_plot_image(hObject, eventdata)
global Ismall

hf = findobj(0,'Name','Regional Image Manipulation');
hax = findobj(hf,'type','axes');
himage = findobj(hf,'tag','himage');
% hax = get(himage,'parent');
% hf = get(hax,'parent');  % image view window

htool = findall(hf,'type','uitoolbar');
hundo = findobj(htool,'tag','undo');
indx_mask = getappdata(hundo,'indx_mask');

xyl=axis(hax);
Itmp = Ismall;
Itmp(~indx_mask) = nan;
set(himage,'Cdata',Itmp)
axis(hax,xyl);
drawnow;
end

function Undo_remove(hObject, eventdata)
%global Iundo2 Ismall
global Ismall
hundo = hObject;
Iundo2 = getappdata(hundo,'Iundo2');
Ismall=Iundo2;
clear Iundo2;
rmappdata(hundo,'Iundo2');
set(hundo,'Enable','off');
update_plot_image;
% htool = findall(gcf,'type','uitoolbar');
% hrm_bg = findobj(htool,'tag','Remove Region');
% set(hrm_bg,'state','off');
end

function remove_SaltandPepper(hObject,eventdata)
%global Ismall Iundo2
global Ismall

hf = gcbf;
htool = findall(hf,'type','uitoolbar');
hundo = findobj(htool,'tag','undo');
indx_mask = getappdata(hundo,'indx_mask');

Fig_remove_SaltandPepper = figure('Units','Normalized','Position',[0.1 0.1 0.5 0.7],...
    'color','w','WindowStyle','Normal','NumberTitle','Off','Name','Remove bacground long wavelength noise of the image with Gaussian filtering');

hbg=uibuttongroup('Visible','on','BackgroundColor',[1 1 1],'Position',[0.01 0.01 0.25 0.04],'Title','Filter type');
uicontrol(hbg,'Style','radiobutton','units','normalized',...
    'String', '2-D Median','BackgroundColor',[1 1 1],...
    'Position', [0.05 0.01 0.4 1]);
uicontrol(hbg,'Style','radiobutton','units','normalized',...
    'String', 'Wiener','BackgroundColor',[1 1 1],...
    'Position', [0.55 0.01 0.4 1]);

uicontrol('Style', 'Text','Parent',Fig_remove_SaltandPepper,'units','normalized',...
    'String', 'm x n neighborhood:','BackgroundColor',[1 1 1],...
    'Position', [0.30 0.015 0.08 0.02],'HorizontalAlignment','right');
Mvalue=uicontrol('Style', 'Edit','Parent',Fig_remove_SaltandPepper,'units','normalized','tag','Vsigma',...
    'String', '4','Tooltip','Enter a positive scalar','BackgroundColor',[1 1 1],...
    'Position', [0.39 0.02 0.06 0.02]);
uicontrol('Style', 'Text','Parent',Fig_remove_SaltandPepper,'units','normalized',...
    'String', 'x','BackgroundColor',[1 1 1],...
    'Position', [0.455 0.015 0.01 0.02]);
Nvalue=uicontrol('Style', 'Edit','Parent',Fig_remove_SaltandPepper,'units','normalized','tag','Vsigma',...
    'String', '4','Tooltip','Enter a positive scalar','BackgroundColor',[1 1 1],...
    'Position', [0.468 0.02 0.06 0.02]);

uicontrol('Style', 'pushbutton','Parent',Fig_remove_SaltandPepper,'units','normalized',...
    'String', 'Preview', 'Position',[0.68 0.01 0.09 0.04],'Callback',@PreviewChanges);

uicontrol('Style', 'pushbutton','Parent',Fig_remove_SaltandPepper,'units','normalized',...
    'String', 'Apply','Position', [0.79 0.01 0.09 0.04],'Callback',@ApplyChanges);

uicontrol('Style', 'pushbutton','Parent',Fig_remove_SaltandPepper,'units','normalized',...
    'String', 'Cancel','Position', [0.90 0.01 0.09 0.04],'Callback',@(hObject,eventdata)(close(Fig_remove_SaltandPepper)));


%%
ax1=subplot(2,1,1);
Itmp = Ismall;
Itmp(~indx_mask) = nan;
imshow(Itmp)
ax2=subplot(2,1,2);
Itemp=Itmp;
HItemp=imshow(Itemp);
ax2.Visible='off';
linkaxes([ax1,ax2])
waitfor(Fig_remove_SaltandPepper)
clear Itemp;


    function PreviewChanges(hObject_l,eventdata_l)
        mn=[str2double(get(Mvalue,'String')) str2double(get(Nvalue,'String'))];
        if strcmp(get(hbg,'SelectedObject'),'Wiener')
            Itemp=wiener2(Ismall,mn);
        else
            Itemp=medfilt2(Ismall,mn);
        end
        Itmp = Itemp;
        Itmp(~indx_mask) = nan;
        set(HItemp,'CData',Itmp);
        drawnow;
%         disp('ok')
    end

    function ApplyChanges(hObject_l,eventdata_l)
        PreviewChanges;
        Iundo2=Ismall;
        setappdata(hundo,'Iundo2',Iundo2);
        Ismall=Itemp;
        update_plot_image;
        delete(Fig_remove_SaltandPepper);
        set(hundo,'Enable','on');
        
    end

end

function remove_background_withGAUSSFILT(hObject,eventdata)
%global Ismall Iundo2
global Ismall

hf = gcbf;
htool = findall(hf,'type','uitoolbar');
hundo = findobj(htool,'tag','undo');
indx_mask = getappdata(hundo,'indx_mask');

Fig_remove_background = figure('Units','Normalized','Position',[0.1 0.1 0.5 0.7],...
    'color','w','WindowStyle','Normal','NumberTitle','Off','Name','Remove bacground long wavelength noise of the image with Gaussian filtering');

uicontrol('Style', 'Text','Parent',Fig_remove_background,'units','normalized',...
    'String', 'Gaussian distribution standard deviation,','BackgroundColor',[1 1 1],...
    'Position', [0.001 0.02 0.31 0.03]);

uicontrol('Style', 'Text','Parent',Fig_remove_background,'units','normalized',...
    'String', 'Vertical:','BackgroundColor',[1 1 1],...
    'Position', [0.32 0.02 0.08 0.03]);


Vsigma=uicontrol('Style', 'Edit','Parent',Fig_remove_background,'units','normalized','tag','Vsigma',...
    'String', '','Tooltip','Enter a positive scalar','BackgroundColor',[1 1 1],...
    'Position', [0.4 0.02 0.04 0.03]);
uicontrol('Style', 'Text','Parent',Fig_remove_background,'units','normalized',...
    'String', 'Horizontal:','BackgroundColor',[1 1 1],...
    'Position', [0.47 0.02 0.08 0.03]);
Hsigma=uicontrol('Style', 'Edit','Parent',Fig_remove_background,'units','normalized','tag','Hsigma',...
    'String', '','Tooltip','Enter a positive scalar','BackgroundColor',[1 1 1],...
    'Position', [0.56 0.02 0.04 0.03]);

uicontrol('Style', 'pushbutton','Parent',Fig_remove_background,'units','normalized',...
    'String', 'Preview', 'Position',[0.68 0.01 0.09 0.04],'Callback',@PreviewChanges);

uicontrol('Style', 'pushbutton','Parent',Fig_remove_background,'units','normalized',...
    'String', 'Apply','Position', [0.79 0.01 0.09 0.04],'Callback',@ApplyChanges);

uicontrol('Style', 'pushbutton','Parent',Fig_remove_background,'units','normalized',...
    'String', 'Cancel','Position', [0.90 0.01 0.09 0.04],'Callback',@(hObject,eventdata)(close(Fig_remove_background)));


ax1=subplot(2,1,1);
Itmp = Ismall;
Itmp(~indx_mask) = nan;
imshow(Itmp)
ax2=subplot(2,1,2);
Itemp=Ismall;
HItemp=imshow(Itmp);
ax2.Visible='off';
linkaxes([ax1,ax2])
waitfor(Fig_remove_background)
clear Itemp;


    function PreviewChanges(hObject_l,eventdata_l)
        sig=[str2double(get(Vsigma,'String')) str2double(get(Hsigma,'String'))];
        Itemp=imadjust(imsubtract(Ismall,imgaussfilt(Ismall,sig)));
        Itmp = Itemp;
        Itmp(~indx_mask) = nan;
        set(HItemp,'CData',Itmp);
    end

    function ApplyChanges(hObject_l,eventdata_l)
        
        PreviewChanges;
        Iundo2=Ismall;
        Ismall=Itemp;
        update_plot_image;
        delete(Fig_remove_background);
        set(hundo,'Enable','on');
        setappdata(hundo,'Iundo2',Iundo2);
        
    end

end

function zoom2seismogram(hObject, eventdata)
global Ismall 
hf = gcf;
hax = findobj(hf,'type','axes');
%hax = get(himage,'parent');
set(hax,'Xlim',[1 size(Ismall,2)],'Ylim',[1,size(Ismall,1)],'Xaxislocation','Top')
drawnow;
% Reset_MouseMotionMon;
end

function viewOrigImage(hobj,eventdata)
% this function switches between original and processed image for H.ax1
% space

global himage Ismall Ismall0

cstate = get(hobj,'state');

if strcmpi(cstate,'on')  % turn on the original image
    set(himage,'Cdata',Ismall0);
else  % turn on the processed image
    set(himage,'Cdata',Ismall);
end  % if strcmpi(cstate,'on')...

end  % function viewOrigImage


function recoverOrig(hobj,eventdata)
% this function allows user to select a rectangular region and recovers the
% original image in that region

global Ismall Ismall0 himage

% turn off zoon and pan
zoom off; pan off;

hf = gcbf;
htool = findall(hf,'type','uitoolbar');
hundo = findobj(htool,'tag','undo');
horig = findobj(htool,'tag','Original Image');
hax = findobj(hf,'type','axes');

% check to make sure Ismall0 exists
if isempty(Ismall0)
    return
end  % exit function

if any(size(Ismall)~=size(Ismall0))  % if sizes are different
    disp('ERROR: size of Ismall and Ismall0 different');
    return  % exit function
end  % if any(size(Ismall)~=size(Ismall0))

setappdata(hundo,'Iundo2',Ismall);
%Iundo = I0;  % set up value for allowing undo

% select rectangular region
%zoom(H.f1,'enable','off'); pan(H.f1,'enable','off');
% hchil = get(0,'children');
% set(0,'children',[H.f1; hchil(hchil~=H.f1)]);
hrec = imrect(hax);  % create rectangle
pos = wait(hrec);  % get positions of the rectangle
delete(hrec);

% get nx and ny ranges
[nrow ncol] = size(Ismall);
ncs = max(round(pos(1)),1);  % starting column number
nce = min(ncs+round(pos(3)),ncol);  % ending column number
nrs = max(round(pos(2)),1);  % starting row number
nre = min(round(pos(2)+pos(4)),nrow);  % ending row number

Ismall(nrs:nre,ncs:nce) = Ismall0(nrs:nre,ncs:nce);
if strcmpi(get(horig,'state'),'off');
    set(himage,'CDATA',Ismall); drawnow
end

set(hundo,'Enable','on');  % turn undo on

end  % function recoverOrig