function DigitSeis 
% for opening older analysis if replace the name of the function with DigitizeSeis (not the name of the m-file)
% THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS OR
% IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
% WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
% ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY
% DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
% DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
% GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
% WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
% OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
% EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
%
%  DigitSeis uses rgb2hsv_fast.m that can be found at
%  http://www.mathworks.com/matlabcentral/fileexchange/15985-fast-rgb2hsv/content/rgb2hsv_fast.m
%  DigitSeis uses writesac.m function that can be found at SAC
%  distributions: %  http://ds.iris.edu/ds/nodes/dmc/forms/sac/
%
%  Digitseis  is on the Github
%  https://github.com/PetrosBogiatzis/DigitSeis
%  
%  for version history, see DigitSeis.readme
%
%  Writtey by: Petros Bogiatzis & Miaki Ishii
%
% 
% 
% current matlab scripts needed for DigitSeis to work
%   DigitSeis.m
%   digitize_trace_and_tickmarks.m
%   digitize_trace_without_tickmarks.m
%   ginput.m
%   modImage.m
%   rgv2hsv_fast.m
%   writesac.m
% and a directory "icon" where all image files for icons can be found





clear all
%global H HIM I0 output_message ptrace pt_traces pt_traceLABEL Time ftrend ftrend_tick S sumI ww pt_start_time pt_end_time p_tick_1st p_tick_last tick_length_lim
global H I0 Iorig output_message tick_length_lim DigitSeisVersion posD
global ptrace ptraceSTD pt_traces
global S 
global nSizeDefaultUicontrolFontSize
global liris piris

% declaration for easier global variable passing
S = [];
ptrace = [];
ptraceSTD = [];
pt_traces = [];

tick_length_lim=0.2; % 20per cent  of the input length, change according the case % to be included as an option in future versions
DigitSeisVersion = 1.1;  % this version of DigitSeis

% H:  Structure with handles for figure, uicontrols and axes.
% I0: The current version of the seismogram image.
% pt_traces: Handle to the lines along traces
% pt_tticks: handle to lines along time tickmarks. INACTIVE
% ftrend: Fit object that describes the trend of the main traces
% ftrend_tick: fit object that describes the trend of time tickmarks.
% output_message messages list for the output window

I0=[];
Iorig = [];

output_message={};

% set up IRIS parameters
liris = true;
if liris && exist('load_piris')==2  % load_piris exists as a .m file
    load_piris;
else
    liris = false;   % turn IRIS option off
end  % if exist('load_priis')==2

% possibly set up default font size based upon screen size
% 'defaultUicontrolFontSize'
% 'defaultUitableFontSize'
% 'defaultAxesFontSize'
% 'defaultTextFontSize'
% 'defaultUipanelFontSize'
pScreen = get(0,'ScreenSize');
nSizeDefaultUicontrolFontSize = get(0,'defaultUicontrolFontSize');  % save original size
if pScreen(3)>2500 && pScreen(4)>1400  % screen with certain size
    set(0,'defaultUicontrolFontSize',16);
end  % if pScreen(3)>2500 && pScreen(4)>1400

% setup position vectors for various objects in DigitSeis window
% field name corresponds to the field name of the H variable
% main display
posD.textfilename = [0.2 0.96 0.502 0.04];
posD.ax1 = [0.12 0.11 0.802 0.83];
posD.ax2 = [0.925 0.11 0.07 0.83];
posD.axwait = [0.905 0.03 0.11 0.05];  % where digitization circle appears
posD.output_mess = [0.12 0.001 0.802 0.1];
% left column selection/control objects
colWidth = 0.095;  % original 0.08
editWidth = 0.04;  % width of edit boxes
optWidth = 0.015;  % width for option objects
posD.imageOption = [0.002 0.955 0.015 0.03];
posD.image = [0.017 0.95 colWidth-0.015 0.04];
posD.edit_DT = [0.002 0.91 colWidth/4-0.001 0.035];
posD.edit_DTtrace = [0.003+posD.edit_DT(3) 0.91 colWidth/4-0.001 0.035];
posD.edit_DP = [0.004+2*(posD.edit_DT(3)) 0.91 colWidth/4-0.001 0.035];
posD.edit_tick_xlength = [0.005+3*(posD.edit_DP(3)) 0.91 colWidth/4-0.001 0.035];
posD.idTrace_options = [0.002 0.855 0.015 0.03];
posD.auto_num_of_traces = [0.017 0.85 colWidth-0.015 0.04];
posD.adjust_traces = [0.002 0.81 colWidth 0.04];
posD.togle_traces_visibility = [0.002 0.78 colWidth 0.03];
posD.text_luminance = [0.002 0.7 colWidth-editWidth 0.04];
%posD.edit_luminance = [colWidth-editWidth 0.7 editWidth 0.04];
posD.edit_luminance = [0.002 0.71 optWidth 0.04];
posD.correct_classification = [0.002+posD.edit_luminance(3) 0.71 colWidth-posD.edit_luminance(3) 0.04]; % edit classification
dy = 0.015;  % vertical shift for digitization group
%posD.text_offset = [0.002 0.6 colWidth-editWidth 0.04];
%posD.edit_offset = [colWidth-editWidth 0.6 editWidth 0.04];
posD.digitize_options = [0.002 0.625+dy optWidth 0.03];
posD.digitize = [0.017 0.62+dy colWidth-optWidth 0.04];
posD.togle_digital_traces_visibility = [0.002 0.588+dy colWidth 0.03];
posD.togle_digital_traces_STD_visibility = [0.002 0.566+dy colWidth 0.03];
posD.correct_trace_options = [0.002 0.524+dy optWidth 0.03];
posD.digitize_1 = [0.017 0.519+dy colWidth-optWidth 0.04];  % correct trace
% posD.mark_start = [0.002 0.42 colWidth-0.03 0.04];
% posD.pbedit_start = [colWidth-0.03+0.002 0.42 0.028 0.04];
dy = 0.015;  % vertical shift for timing group
posD.mark_start = [0.002+editWidth/2+0.001 0.44+dy colWidth-editWidth/2-0.001 0.04];
posD.pbedit_start = [0.002 0.44+dy editWidth/2 0.04];
%posD.hp1 = [.001 .245+dy colWidth .18];  % panel for time information
dy2 = 0.055;
%posD.timeText = [0.01 0.5 0.5 0.2];  % text object with string "# of time marks"
posD.edit_num_of_ticks = [colWidth-editWidth/2 0.335+dy+dy2 editWidth/2 0.04];
posD.cb_timeEnd = [0.002 0.341+dy+dy2 colWidth-editWidth/2-0.001 0.04];
% posD.mark_1st = [0.02 0.23 0.6 0.22];
% posD.edit_mark_1st = [0.63 0.23 0.34 0.22];
posD.mark_1st = [0.002+editWidth/2+0.001 0.29+dy+dy2 colWidth-editWidth/2-0.001 0.04];
posD.edit_mark_1st = [0.002 0.29+dy+dy2 editWidth/2 0.04];
% posD.mark_last = [0.02 0.03 0.6 0.22];
% posD.edit_mark_last = [0.63 0.03 0.34 0.22];
posD.mark_last = [0.002+editWidth/2+0.001 0.253+dy+dy2 colWidth-editWidth/2-0.001 0.04];
posD.edit_mark_last = [0.002 0.253+dy+dy2 editWidth/2 0.04];
posD.edit_t0 = [0.002 0.255+dy colWidth 0.04];
posD.Create_Time = [0.002 0.20+dy colWidth 0.04];
posD.togle_timebounds_visibility = [0.002 0.17+dy colWidth 0.03];
dy = 0.015;  % vertical shift for extract SAC button
posD.saveSAC = [0.002 0.1+dy colWidth 0.04];
posD.find_gaps = [0.002 0.03 colWidth 0.02];
posD.find_vert = [0.002 0.01 colWidth 0.02];


% ask user for input choice

hf = figure('Name','File Type Selection','NumberTitle','off','Color','w','Units','Pixels',...
    'menu','none','toolbar','none','renderer','OpenGL',...
    'OuterPosition',[427   506   512   200]);

sizeImage = 25;  % pixel number of icons to be shown
wid = 0.55;  % width of pushbuttons
hgt = 0.2;  % height of pushbuttons
pf = get(hf,'position');
% calculate number of pixels corresponding to x1
xiconW = sizeImage/pf(3);  % icon width in normailzed values
yiconH = sizeImage/pf(4);  % icon height
x1 = (1.0-xiconW-0.01-wid)/2+xiconW+0.01;  % pushbutton starting position

htext = uicontrol('Style', 'text','Parent',hf,'units','normalized',...
    'String','Choose starting file type','Position', [0.01 0.75 0.98 0.15],...
    'BackgroundColor',[1 1 1],'horizontalAlignment','center',...
    'fontsize',18);

% calculate positions for image & pushbutton
hax1 = axes('Units','Normalized','Position',[x1-xiconW-0.01 0.53+hgt/2-yiconH/2 xiconW yiconH],... 
    'Xtick',[],'Ytick',[],'box','off','Parent',hf,'CLimMode','manual','Clim',[0 255]);
Ianalysis = imread('icons/file.png');
hIanalysis = image(Ianalysis,'parent',hax1);
set(hax1,'visible','off');

hpb_image = uicontrol('Style', 'pushbutton','Parent',hf,'units','normalized',...
    'String', 'Open an Image','Position', [x1 0.53 0.55 0.2],...
    'Callback',@sel_input,'Enable','on','fontsize',18,'tag','image');

% calculate positions for image & pushbutton
hax2 = axes('Units','Normalized','Position',[x1-xiconW-0.01 0.33+hgt/2-yiconH/2 xiconW yiconH],... 
    'Xtick',[],'Ytick',[],'box','off','Parent',hf,'CLimMode','manual','Clim',[0 255]);
Ianalysis = imread('icons/load_analysis.png');
hIanalysis = image(Ianalysis,'parent',hax2);
set(hax2,'visible','off');

hpb_analysis = uicontrol('Style', 'pushbutton','Parent',hf,'units','normalized',...
    'String', 'Load Existing Analysis','Position', [x1 0.33 0.55 0.2],...
    'Callback',@sel_input,'Enable','on','fontsize',18,'tag','analysis');

hpb_cancel = uicontrol('Style', 'pushbutton','Parent',hf,'units','normalized',...
    'String', 'Quit DigitSeis','Position', [x1 0.05 0.55 0.2],...
    'Callback',@sel_input,'Enable','on','fontsize',18,'tag','quit');

    function sel_input(hObj,eventdata)
        % this function is called when one of the pushbuttons is pressed
        
        set(hf,'visible','off');  % immediately make the figure invisible
        % so that this window isn't available for user to click on while
        % things are happening
        
        cstr = get(hObj,'tag');  % get label
        if strcmpi(cstr,'image')  % load image
            disp('loading image')
            Browse_and_Load;  % load image
        elseif strcmpi(cstr,'analysis')  % load analysis
            disp('loading analysis')
            load_analysis;  % load analysis
        else  % quit
            disp('quitting DigitSeis')
%             delete(H.f1);  % closing DigitSeis
        end  % if strcmpi(cstr,'image')...
        
        delete(hf)  % close figure
    end

end


function change_I0_polarity(hObject, eventdata)
global I0 Iorig
I0=imcomplement(I0);
if ~isempty(Iorig)
    Iorig = imcomplement(Iorig);
end
update_plot_image;
end


function Get_DATE_TIME(hObject, eventdata)
ho=hObject;
cstr = get(ho,'string');

try  % if cstr is in the date format
    [yr0 mon0 day0 hr0 min0 sec0] = datevec(cstr,'yyyymmdd HH:MM');
    % if there are no errors, assume valid date
    yr0 = cstr(1:4);
    mon0 = cstr(5:6);
    day0 = cstr(7:8);
    time0 = cstr(10:14);
catch  % if cstr is in the default format without dates
    yr0 = '1938';
    mon0 = '01';
    day0 = '01';
    time0 = datestr(now,'HH:MM');
end

ctitle = 'Select Reference Time';
cprompt = {'Enter year:','Enter month:','Enter day','Enter time in HH:MM format'};
cdefault = {yr0, mon0, day0, time0};
cans = inputdlg(cprompt,ctitle,1,cdefault);

if ~isempty(cans)  % change made
    cstr = [cans{1} cans{2} cans{3} ' ' cans{4}];  % new string
    % update label
    set(ho,'string',cstr);
end  % if ~isempty(cans)



% try
%     cstr = get(ho,'string');
%     date0 = datenum(cstr(1:8),'yyyymmdd');
%     outtime = cstr(10:17);
%     ldexist = true;
% catch
% %     hfc=uicalendar('Weekend', [1 0 0 0 0 0 1],'SelectionType', 1,...
% %         'WindowStyle','Modal','OutputDateFormat','yyyymmdd');
%     date0 = today;
%     outtime = datestr(now,'HH:MM:SS');
%     ldexist = false;
% end
% hfc=uicalendar('Weekend', [1 0 0 0 0 0 1],'SelectionType', 1,...
%     'WindowStyle','Modal','OutputDateFormat','yyyymmdd',...
%     'InitDate',date0);
% set(hfc,'Name','Select date and time');
% delete(findobj(hfc,'String','Clear all'))
% %outtime=datestr(now,'HH:MM:SS');
% he=uicontrol('Style', 'edit','Parent',hfc,'units','normalized','BackgroundColor',[1 1 1],...
%     'String',outtime,'Position',[0.03 0.027 0.4 0.09]);
% set(findobj(hfc,'String','OK'),'Callback',@getoutput)
% waitfor(hfc);
% 

%     function getoutput(hObject, eventdata)
%         sd=getappdata(hfc,'selectedSquares');
%         if isempty(sd.date)  % date empty
%             if ldexist  % input date exists
%                 outdate = datestr(date0,'yyyymmdd');
%             else  % input date does not exist
%                 outdate = 'yyyymmdd';
%             end  % if ldexist ...
%         else
%             outdate = datestr([fliplr(sd.date) sd.day{1} 0 0 0],'yyyymmdd');
%         end
%         outtime= get(he,'String');
%         set(ho,'string',[outdate ' ' outtime]);
%         close(hfc);
%     end
end




%function Evaluate_ticklength_input(hObject, eventdata)
function Check_length_input(hObject, eventdata)
global H

rtmp(1) = str2double(get(H.edit_DT,'string'));  % time mark separation time (seconds)
rtmp(2) = str2double(get(H.edit_DTtrace,'string'));  % line duration (hours)
rtmp(3) = str2double(get(H.edit_DP,'string'));  % pixel distance between time marks
rtmp(4) = str2double(get(H.edit_tick_xlength,'string'));  % pixel time mark distance

if isempty(find(isnan(rtmp)))  % all contain numbers -> turn zero trace calculation option on
    set(H.auto_num_of_traces,'Enable','on');
    set(H.idTrace_options,'enable','on');
    % note that this is off by default
end  
% if isnan(str2double(get(H.edit_tick_xlength,'String')))
%     set(H.auto_num_of_traces,'Enable','off');
% else
%     set(H.auto_num_of_traces,'Enable','on');
% %     if str2double(get(H.edit_tick_xlength,'String'))<0
% %         set(H.mark_1st,'Enable','off')
% %         set(H.edit_mark_1st,'Enable','off')
% %         set(H.mark_last,'Enable','off')
% %         set(H.edit_mark_last,'Enable','off')
% %     else
% %         set(H.mark_1st,'Enable','on')
% %         set(H.edit_mark_1st,'Enable','on')
% %         set(H.mark_last,'Enable','on')
% %         set(H.edit_mark_last,'Enable','on')
% %     end
%     
% end

end




%function Browse_and_Load(hObject,eventdata)
function Browse_and_Load
global H

[filename, pathname]=uigetfile({'*.jpg;*.tif;*.png;*.gif','All Image Files';...
    '*.*','All Files' },'Select seismogram');
if isequal(filename,0)
    return
end
% set(H.textfilename,'String',filename,'Userdata',pathname)

Load_Image(filename,pathname);
end


%function Load_Image(hObject, eventdata)
function Load_Image(filename, pathname)
global H I0 Iorig pt_traces pt_traceLABEL output_message HIM 
global p_tick_1st p_tick_last pt_start_time pt_end_time
global posD

% turn the current window invisible while working on it and remove loading
% file and analysis icons off

% message to let user know something is happening
hmess = dialog('Name','Loading Image');
cstr = sprintf('%s\n\n%s','Loading Image:',...
    'Please be patient as this may take some time.  This window will close when image becomes available.');
ctxt = uicontrol('parent',hmess,'style','text','units','normalized',...
    'position',[0.1 0.3 0.8 0.4],'string', cstr,...
    'horizontalAlignment','left','fontSize',18);
drawnow

%tic

% read image
I0=[];
Iorig = [];
ffile=fullfile(pathname,filename);
[~,~,ext] = fileparts(ffile);
if strcmpi(ext,'.MAT') % is already I0
    load(ffile,'I0');
else
    button=[];button = questdlg('Press Yes to consider the complement of the scanned image','Image polarity','Yes','No','Yes');

%     output_message{end+1}=['Reading image from file: ' ffile ,[' please wait...']];
%     set(H.output_mess,'String',output_message,'Value',length(output_message)); drawnow;
    cstr = sprintf('%s\n%s%s\n%s','Loading Image:','Reading image from file: ', ffile,...
        'Please be patient as this may take some time.  This window will close when image is fully loaded');
    set(ctxt,'string',cstr);
    drawnow
    I0=imread(ffile);
    % Adjust histogram to improve contrast
%     output_message{end+1}='Converting to 8-bit, HSV, please wait...';
%     set(H.output_mess,'String',output_message,'Value',length(output_message));drawnow;
    cstr = sprintf('%s\n%s','Loading Image:','Converting to 8-bit, HSV, please wait...');
    set(ctxt,'string',cstr);
    drawnow
    %check if it is already 1 channel (e.g., b&w)
    if size(I0,3)==3
        I0=im2uint8(rgb2hsv_fast(I0,'Single','V'));
    else
        I0=im2uint8(I0);
    end
    
%     button=[];button = questdlg('Press Yes to consider the complement of the scanned image','Image polarity','Yes','No','Yes');
    if strcmp(button,'Yes')
%         output_message{end}='Converting to 8-bit, HSV, complement, please wait...';
%         set(H.output_mess,'String',output_message,'Value',length(output_message));drawnow;
        cstr = sprintf('%s\n%s','Loading Image:','Calculating the complement image...');
        set(ctxt,'string',cstr);
        drawnow
        I0= imcomplement(I0);
    end
    
%     button=[];button = questdlg('Press Yes automatically adjust image histogram','Image Histogram Adjustment','Yes','No','Yes');
%     if strcmp(button,'Yes')
%         output_message{end}='Converting to 8-bit, HSV, complement, adjusting histogram, please wait...';
%         set(H.output_mess,'String',output_message,'Value',length(output_message));drawnow;
%         I0=imadjust(I0);
%         
%     end
end

Iorig = I0;

% set up the window

output_message={};
H.f1=figure('Name','DigitSeis','NumberTitle','off','Color','w','Units','Pixels',...
    'OuterPosition',get(0,'screensize'),'menu','none','toolbar','figure','Visible','off','renderer','OpenGL');
set(H.f1,'DeleteFcn',@closeDigitSeis);  % function to be executed when the figure closes


% modify tool bar
temp= findall(findall(H.f1,'Type','uitoolbar'));
%    ' 1'     'FigureToolBar'
%     ' 2'    'Plottools.PlottoolsOn'
%     ' 3'    'Plottools.PlottoolsOff'
%     ' 4'    'Annotation.InsertLegend'
%     ' 5'    'Annotation.InsertColorbar'
%     ' 6'    'DataManager.Linking'
%     ' 7'    'Exploration.Brushing'
%     ' 8'    'Exploration.DataCursor'
%     ' 9'    'Exploration.Rotate'
%     '10'    'Exploration.Pan'
%     '11'    'Exploration.ZoomOut'
%     '12'    'Exploration.ZoomIn'
%     '13'    'Standard.EditPlot'
%     '14'    'Standard.PrintFigure'
%     '15'    'Standard.SaveFigure'
%     '16'    'Standard.FileOpen'
%     '17'    'Standard.NewFigure'
%     '18'    ''

set(temp(15),'ClickedCallback',@Saveresults,'TooltipString','Save results')
set(temp(8),'ClickedCallback',@dataCursorFnc)
set(temp(16),'ClickedCallback',@Browse_and_Load)

htool.save = temp(15);
htool.datacursor = temp(8);
htool.pan = temp(10);
htool.zoomOut = temp(11);
htool.zoomIn = temp(12);
htool.fileOpen = temp(16);

delete(temp([2:7 9 13 14 16 17]));
% uipushtool(temp(1),'Tag','Reload image from file','Cdata',imread('reload.png','BackgroundColor',[1 1 1]),...
%     'Separator','on','TooltipString','Reload image from file',...
%     'HandleVisibility','on','ClickedCallback',@Load_Image);

% uipushtool(temp(1),'Tag','View and edit image file','Cdata',imread('imtool.png','BackgroundColor',[1 1 1]),...
%     'Separator','on','TooltipString','View info and edit image with imtool',...
%     'HandleVisibility','on','ClickedCallback',@viewimage);
htool.editImage = uipushtool(temp(1),'Tag','View and edit portion of the image','Cdata',imread('icons/imtool.png','BackgroundColor',[1 1 1]),...
    'Separator','on','TooltipString','View and edit portion of the image',...
    'HandleVisibility','on','ClickedCallback',@viewimage);

%temp= findall(findall(H.f1,'Type','uitoolbar'));
% set(temp(end),'ClickedCallback',@Browse_and_Load)

% Reorder tools
% set(temp(1),'Children',[temp(4:end-2);temp(2:3);temp(end)])

% htool.loadAnalysis = uipushtool(temp(1),'Tag','Load existing analysis','Cdata',imread('icons/load_analysis.png','BackgroundColor',[1 1 1]),...
%     'Separator','on','TooltipString','Load existing analysis from file',...
%     'HandleVisibility','on','ClickedCallback',@load_analysis);

% add some tools
htool.whole = uipushtool(temp(1),'Tag','Show whole seismogram','Cdata',imread('icons/whole_seismogram.png','BackgroundColor',[1 1 1]),...
    'Separator','on','TooltipString','Show whole seismogram',...
    'HandleVisibility','on','ClickedCallback',@zoom2seismogram);

htool.crop = uipushtool(temp(1),'Tag','Crop Image','Cdata',imread('icons/crop.png','BackgroundColor',[1 1 1]),...
    'Separator','on','TooltipString','Crop image',...
    'HandleVisibility','on','ClickedCallback',@crop_image);
htool.contrast = uipushtool(temp(1),'Tag','Adjust contrast','Cdata',imread('icons/contrast-icon.png','BackgroundColor',[1 1 1]),...
    'Separator','on','TooltipString','Adjust contrast',...
    'HandleVisibility','on','ClickedCallback',@adjust_contrast);

% htool.removeBG = uipushtool(temp(1),'Tag','Remove background','Cdata',imread('icons/Removebackground.png','BackgroundColor',[1 1 1]),...
%     'Separator','on','TooltipString','Click remove large background stains-colorization',...
%     'HandleVisibility','on','ClickedCallback',@remove_background_withGAUSSFILT);

% Create button image & Button
tempim=imnoise(zeros([16,16,3],'uint8'),'salt & pepper',0.2);
for ii=2:16, tempim(ii-1:ii,ii-1:ii,1)=255; tempim(ii-1:ii,ii-1:ii,2:3)=0 ;tempim([ii-1:ii],17-[ii-1:ii],1)=255; tempim([ii-1:ii],17-[ii-1:ii],2:3)=0; end
htool.saltPepper = uipushtool(temp(1),'Cdata',tempim,'Tag','Salt & Pepper Removal',...
    'Separator','off','TooltipString','Remove salt & pepper noise',...
    'HandleVisibility','on','Enable','on','ClickedCallback',@remove_SaltandPepper);

% htool.rotate = uipushtool(temp(1),'Tag','Correct rotation','Cdata',imread('icons/correct_rotation.png','BackgroundColor',[1 1 1]),...
%     'Separator','on','TooltipString','Cprrect for rotation',...
%     'HandleVisibility','on','ClickedCallback',@estimate_rotation);
htool.recoverRegion = uipushtool(temp(1),'Tag','Recover Region','Cdata',imread('icons/retrieveR.png','BackgroundColor',[1 1 1]),...
    'Separator','on','TooltipString','Draw a region to recover from the original image',...
    'HandleVisibility','on','ClickedCallback',@recoverOrig,'enable','on');
htool.removeRegion = uipushtool(temp(1),'Tag','Remove Region','Cdata',imread('icons/removeS.png','BackgroundColor',[1 1 1]),...
    'Separator','on','TooltipString','Draw a region to remove',...
    'HandleVisibility','on','ClickedCallback',@remove_region);
htool.undo = uipushtool(temp(1),'Tag','Undo last','Cdata',imread('icons/undo.jpeg'),...
    'Separator','on','TooltipString','Undo last action',...
    'HandleVisibility','on','Enable','off','ClickedCallback',@Undo_remove);
htool.measureLength = uipushtool(temp(1),'Tag','Measure tick length','Cdata',imread('icons/tick_length.png','BackgroundColor',[1 1 1]),...
    'Separator','on','TooltipString','Measure tick length',...
    'HandleVisibility','on','ClickedCallback',@meassure_pix_dist0);
% Image complement
% polarity_ic=zeros([16,16,3]); for i=1:16, polarity_ic(1:i,i:16,:)=1;end
% htool.IComplement = uipushtool(temp(1),'Tag','Measure tick length','Cdata',polarity_ic,...
%     'Separator','on','TooltipString','Click to change image polarity',...
%     'HandleVisibility','on','ClickedCallback',@change_I0_polarity);
% clear polarity_ic;

% Time marks up or down
% icon_m=zeros([16,16,3]); icon_m(8,:,:)=1; icon_m(4,6:10,:)=1; icon_m(13,6:10,1)=1;
icon_m=zeros([16,16,3]); icon_m(11:12,[1:6, 11:16],:)=1; icon_m(6:7,7:10,2)=1;
H.timemarkpos=uitoggletool(temp(1),'Tag','Time marks relative position','Cdata',icon_m,...
    'Separator','on','TooltipString','Click to indicate that time mark offset occurs downward',...
    'HandleVisibility','on', 'State','off','ClickedCallback',@check_timemarkpos);
clear icon_m

% CurvedTraces
H.curvButton=uitoggletool(temp(1),'Tag','Measure tick length','Cdata',imread('icons/CurvedTraces.png'),...
    'Separator','on','TooltipString','Click to model and correct curved offsets',...
    'HandleVisibility','on','ClickedCallback',@extract_curvature_correction,'state','off');

% original image
H.origImage=uitoggletool(temp(1),'Tag','Original Image','Cdata',imread('icons/origImage.png'),...
    'Separator','on','TooltipString','Switch between original and processed images',...
    'HandleVisibility','on', 'State','off','ClickedCallback',@viewOrigImage);

htool.tmPos = H.timemarkpos;
htool.curvButton = H.curvButton;
htool.origImage = H.origImage;

temp= findall(findall(H.f1,'Type','uitoolbar'));
% reorder tools 
% htmp = [htool.whole; htool.pan; htool.zoomOut; htool.zoomIn; 
%     htool.datacursor; htool.undo; 
%     htool.tmPos; htool.curvButton; htool.measureLength;
%     htool.removeRegion; htool.editImage; 
%     htool.removeBG; htool.saltPepper; 
%     htool.IComplement; htool.rotate; htool.contrast; htool.crop;
%     htool.fileOpen; htool.loadAnalysis; htool.save
%     ];
% htmp = [htool.whole; htool.pan; htool.zoomOut; htool.zoomIn; 
%     htool.datacursor; htool.undo; 
%     htool.tmPos; htool.curvButton; htool.measureLength;
%     htool.removeRegion; htool.editImage; 
%     htool.removeBG; htool.saltPepper; 
%     htool.IComplement; htool.rotate; htool.contrast; htool.crop;
%     htool.save
%     ];
htmp = [htool.whole; htool.pan; htool.zoomOut; htool.zoomIn; 
    htool.datacursor; htool.undo; 
    htool.tmPos; htool.curvButton; htool.measureLength;
    htool.removeRegion; htool.recoverRegion; htool.origImage; htool.editImage; 
    htool.saltPepper; 
    htool.contrast; htool.crop;
    htool.save
    ];
set(temp(1),'Children',htmp);

H.tbar=findall(findall(H.f1,'Type','uitoolbar'));
H.tbar_UNDO=findobj(temp(1),'Tag','Undo last');


H.textfilename=uicontrol('Style', 'text','Parent',H.f1,'units','normalized',...
    'String', 'seismogram file','Position', posD.textfilename,'BackgroundColor',[1 1 1],...
    'FontSize',14);
% H.CursorPosText=uicontrol('Style', 'text','Parent',H.f1,'units','normalized',...
%     'String', 'Cursor coordinates','Position', [0.7 0.95 0.2 0.04],'BackgroundColor',[1 1 1],...
%     'FontSize',14);

H.ax1=axes('Units','Normalized','Position',posD.ax1,... % originally height = 0.72
    'Xtick',[],'Ytick',[],'box','on','Parent',H.f1,'Xaxislocation','Top','CLimMode','manual','Clim',[0 255]);
H.ax2=axes('Units','Normalized','Position',posD.ax2,...
    'Xtick',[],'Ytick',[],'box','on','Parent',H.f1,'Ydir','Reverse');
% H.ax3=axes('Units','Normalized','Position',[0.12 0.08 0.802 0.12],...
%     'Xtick',[],'Ytick',[],'box','on','Parent',H.f1);
H.axwait=axes('Units','Normalized','Position',posD.axwait,'Parent',H.f1,'box', 'off','visible','off');
%pie(H.axwait,[0.99 0.01],{'','1234567'})%,

H.output_mess=uicontrol('Style', 'listbox','Parent',H.f1,'units','normalized',...
    'String', '','Position', posD.output_mess);

% advanced link axes
hlink1 = linkprop([H.ax2 H.ax1],'ylim');
% Store link object on first subplot axes
setappdata(H.ax2,'graphics_linkprop',hlink1);
% advanced link axes
% hlink2 = linkprop([H.ax3 H.ax1],'xlim');
% Store link object on first subplot axes
% setappdata(H.ax3,'graphics_linkprop',hlink2);

H.imageMod_options = uicontrol('style','pushbutton','parent',H.f1,...
    'units','normalized','position',posD.imageOption,'string','+',...
    'Tooltip','Additional parameters/algorithms for image processing',...
    'enable','on','callback',@imageMod_options);
H.imageMod=uicontrol('Style', 'pushbutton','Parent',H.f1,'units','normalized',...
    'String', 'Piecewise Contrast','Position', posD.image,...
    'Callback',@imageMod,'Enable','on',...
    'Tooltip','Modify image by adjusting contrast in small regions');

%DT
H.edit_DT=uicontrol('Style', 'edit','Parent',H.f1,'units','normalized',...
    'TooltipString','Time difference between timemarks (s)','String','60',...
    'Position',posD.edit_DT,'BackgroundColor',[1 1 1],...
    'Enable','on','callback',@Check_length_input);
H.edit_DTtrace=uicontrol('Style', 'edit','Parent',H.f1,'units','normalized',...
    'TooltipString','time difference between succeding traces (in hours)','String','1',...
    'Position',posD.edit_DTtrace,'BackgroundColor',[1 1 1],...
    'Enable','on','callback',@Check_length_input);

H.edit_DP=uicontrol('Style', 'edit','Parent',H.f1,'units','normalized',...
    'TooltipString','pixel dist between timemarks (px)','String', '',...
    'Position',posD.edit_DP,'BackgroundColor',[1 1 1],...
    'Enable','on','callback',@Check_length_input);
H.edit_tick_xlength=uicontrol('Style', 'edit','Parent',H.f1,'units','normalized',...
    'TooltipString','Approximate time mark length (px)','String', '25',...
    'Position',posD.edit_tick_xlength,'BackgroundColor',[1 1 1],...
    'Callback',@Check_length_input);

H.idTrace_options = uicontrol('style','pushbutton','parent',H.f1,...
    'units','normalized','position',posD.idTrace_options,'string','+',...
    'Tooltip','Additional parameters that can be set for identify trace routine',...
    'enable','off','callback',@idTrace_options);
H.auto_num_of_traces=uicontrol('Style', 'pushbutton','Parent',H.f1,'units','normalized',...
    'String', 'Identify traces','Position', posD.auto_num_of_traces,...
    'Callback',@find_traces,'Enable','off');
H.adjust_traces=uicontrol('Style', 'pushbutton','Parent',H.f1,'units','normalized',...
    'String', 'Adjust traces','Position', posD.adjust_traces,...
    'Callback',@adjust_traces,'enable','off');

H.togle_traces_visibility=uicontrol('Style', 'checkbox','Parent',H.f1,'units','normalized',...
    'String', 'Traces 0-line visible','Enable','off','Position', posD.togle_traces_visibility,...
    'BackgroundColor','w','Callback',@traces_visibility);

% H.text_luminance=uicontrol('Style', 'text','Parent',H.f1,'units','normalized',...
%     'String',{'Intensity';'threshold'},'Position', posD.text_luminance,'BackgroundColor',[1 1 1]);
H.edit_luminance=uicontrol('Style', 'edit','Parent',H.f1,'units','normalized',...
    'TooltipString','Luminance thershold, should be between 1 and 99',...
    'String','10','Position', posD.edit_luminance,'BackgroundColor',[1 1 1],...
    'callback',@check_edit_luminance);

H.correct_classification=uicontrol('Style', 'pushbutton','Parent',H.f1,'units','normalized',...
    'String', 'Edit classification','Position', posD.correct_classification,...
    'Callback',@correct_classification,'Enable','off');


% H.text_offset=uicontrol('Style', 'text','Parent',H.f1,'units','normalized',...
%     'String',{'Offset from trace';'to digitize'},'Position', posD.text_offset,...
%     'BackgroundColor',[1 1 1]);
% H.edit_offset=uicontrol('Style', 'edit','Parent',H.f1,'units','normalized',...
%     'TooltipString',...
%     'Offset bellow and over the trace as a portion of the average trace distance, typical value: [0.3 0.3]',...
%     'String','[-0.3 0.3]',...
%     'Position', posD.edit_offset,'BackgroundColor',[1 1 1]);
H.edit_offset = '[-0.3 0.3]';  % initial value
H.digitize_options = uicontrol('style','pushbutton','parent',H.f1,...
    'units','normalized','position',posD.digitize_options,'string','+',...
    'Tooltip','Additional parameters that can be set for digitization routine',...
    'enable','off','callback',@digitize_options);
H.digitize=uicontrol('Style', 'pushbutton','Parent',H.f1,'units','normalized',...
    'String', 'Digitize','Position', posD.digitize,'Callback',@digitize_traces,'Enable','Off');

H.togle_digital_traces_visibility=uicontrol('Style', 'checkbox','Parent',H.f1,'units','normalized',...
    'String', 'Traces visible','Enable','off','Position', posD.togle_digital_traces_visibility,...
    'BackgroundColor','w','Callback',@digital_traces_visibility);
H.togle_digital_traces_STD_visibility=uicontrol('Style', 'checkbox','Parent',H.f1,'units','normalized',...
    'String', 'STD traces visible','Enable','off','Position', posD.togle_digital_traces_STD_visibility,...
    'BackgroundColor','w','Callback',@digital_traces_visibility);
% set(H.togle_digital_traces_visibility,'visible','off');
% set(H.togle_digital_traces_STD_visibility,'visible','off');

H.correct_trace_options = uicontrol('style','pushbutton','parent',H.f1,...
    'units','normalized','position',posD.correct_trace_options,'string','+',...
    'Tooltip','Additional parameters that can be set for correct trace routine',...
    'enable','off','callback',@correct_trace_options);
H.digitize_1=uicontrol('Style', 'pushbutton','Parent',H.f1,'units','normalized',...
    'String', 'Correct trace','Position', posD.digitize_1,...
    'Enable','off','Callback',@correct_trace,'backgroundcolor',[0.8 1 0.8]);


H.mark_start=uicontrol('Style', 'pushbutton','Parent',H.f1,'units','normalized',...
    'String', 'Ends of traces','Position', posD.mark_start,...
    'Callback',@(h,e) mark_start(h,e,0),'enable','off',...
    'Tooltip',['Identify the start and end of each trace ',...
    '(start first, double click to finalize and move on to end, ',...
    'double click to finish)']);
H.pbedit_start=uicontrol('Style', 'pushbutton','Parent',H.f1,'units','normalized',...
    'String', 'Edit','Position', posD.pbedit_start,...
    'Callback',@(h,e) mark_start(h,e,1),'Enable','off');
% H.mark_end=uicontrol('Style', 'pushbutton','Parent',H.f1,'units','normalized',...
%     'String', 'End of traces','Position', [0.002 0.69 0.05 0.04],...
%     'Callback',@(h,e) mark_end(h,e,0));
% H.pbedit_end=uicontrol('Style', 'pushbutton','Parent',H.f1,'units','normalized',...
%     'String', 'Edit','Position', [0.052 0.69 0.028 0.04],...
%     'Callback',@(h,e) mark_end(h,e,1),'Enable','off');


% H.hp1 = uipanel('Title','Timing Information',...
%     'BackgroundColor','white',...
%     'Position',posD.hp1,'Parent',H.f1);

H.edit_t0=uicontrol('Style', 'pushbutton','Parent',H.f1,'units','normalized',...
    'TooltipString','Date & Time of the 1st reference time mark (seismogram with time marks), or of the begining of the 1st trace (seismigram without time marks)',...
    'string', 'yyyymmdd HH:MM','Position',posD.edit_t0,...
    'BackgroundColor',[1 1 1],'Callback',@Get_DATE_TIME,'enable','off');

% uicontrol('Style', 'Text','Parent',H.hp1,'units','normalized',...
%     'String','# of time marks',...
%     'Position',posD.timeText,'BackgroundColor',[1 1 1]);
H.edit_num_of_ticks=uicontrol('Style', 'edit','Parent',H.f1,'units','normalized',...
    'TooltipString','Number of time marks between 1st and last (including 1st and last)',...
    'string','','Position',posD.edit_num_of_ticks,'BackgroundColor',[1 1 1],'enable','off');
H.cb_timeEnd = uicontrol('style','checkbox','parent',H.f1,'units','normalized',...
    'TooltipString','Select if minute starts at the beginning of a time mark',...
    'position',posD.cb_timeEnd,'string','time left','backgroundcolor',[1 1 1],...
    'horizontalAlignment','left','enable','off');
H.mark_1st=uicontrol('Style', 'pushbutton','Parent',H.f1,'units','normalized',...
    'Tooltip','Mark the ending pixels of the 1st time marks',...
    'String', '1st mark','Position',posD.mark_1st,...
    'Callback',@(h,e) mark_1st_and_last(h,e,'FIRST',false),...
    'enable','off');
H.edit_mark_1st=uicontrol('Style', 'pushbutton','Parent',H.f1,'units','normalized',...
    'Tooltip','Edit the ending pixels of the 1st time marks',...
    'String', 'Edit','Position',posD.edit_mark_1st,...
    'Callback',@(h,e) mark_1st_and_last(h,e,'FIRST',true),'Enable','off');

H.mark_last=uicontrol('Style', 'pushbutton','Parent',H.f1,'units','normalized',...
    'Tooltip','Mark the last pixel of the last time marks',...
    'String', 'Last mark','Position', posD.mark_last,...
    'Callback',@(h,e) mark_1st_and_last(h,e,'LAST',false),...
    'enable','off');
H.edit_mark_last=uicontrol('Style', 'pushbutton','Parent',H.f1,'units','normalized',...
    'Tooltip','Edit ending pixels of the last time marks',...
    'String', 'Edit','Position', posD.edit_mark_last,...
    'Callback',@(h,e) mark_1st_and_last(h,e,'LAST',true),'Enable','off');


H.Create_Time=uicontrol('Style', 'pushbutton','Parent',H.f1,'units','normalized',...
    'String', 'Calculate Timing','Position', posD.Create_Time,'Callback',@Create_Time,'Enable','off');

H.togle_timebounds_visibility=uicontrol('Style', 'checkbox','Parent',H.f1,'units','normalized',...
    'String', 'Time boundaries visible','Enable','off','BackgroundColor','w',...
    'Position', posD.togle_timebounds_visibility,'Callback',@togle_timebounds_visibility);


H.saveSAC=uicontrol('Style', 'pushbutton','Parent',H.f1,'units','normalized',...
    'String', 'Extract SAC','Position', posD.saveSAC,'Callback',@saveSAC,'Enable','off');

H.find_gaps=uicontrol('Style', 'checkbox','Parent',H.f1,'units','normalized',...
    'String', 'Check for gaps','Enable','off','Position', posD.find_gaps,...
    'BackgroundColor','w','Callback',@find_gaps,...
    'Tooltip','Check for any gaps in the digitized trace');
H.find_vert=uicontrol('Style', 'checkbox','Parent',H.f1,'units','normalized',...
    'String', 'Check for vertical','Enable','off','Position', posD.find_vert,...
    'BackgroundColor','w','Callback',@find_gaps,...
    'Tooltip','Check for any nearly vertical lines in the digitized trace');


%H.strMousePos=uicontrol('Style', 'pushbutton','Parent',H.f1,'units','normalized',...
%    'String', '','FontSize',11,'Tooltip','Push to reset','Position', [0.002 0.002 0.08 0.04],'Callback',set (H.f1, 'WindowButtonMotionFcn', @mouseMove));

%set (H.f1, 'WindowButtonMotionFcn', @mouseMove); % Display continiously mouse location
%set(H.f1,'Visible','on')

%set(H.f1,'visible','off');
set(H.textfilename,'String',filename,'Userdata',pathname)
%ffile=fullfile(get(H.textfilename,'Userdata'),get(H.textfilename,'string'));
set(H.f1,'Name', ['DigitSeis: ' ffile])

% % modify tool bar
% temp= findall(findall(H.f1,'Type','uitoolbar'));
% % remove the load analysis and open file icons
% try
%     htmp = findobj(temp,'tag','Standard.FileOpen');
%     delete(temp(temp==htmp));
% end
% try
%     htmp = findobj(temp,'tag','Load existing analysis');
%     delete(temp(temp==htmp));
% end
% % delete(temp([end-2:end-1]));

% try
% cla([H.ax1, H.ax2, H.ax3]);
% catch
%     cla([H.ax1 H.ax2]);
% end
% 
% try delete(pt_traces); end
% try delete(p_tick_1st); end
% try delete(p_tick_last); end
% try delete(pt_start_time); end
% try delete(pt_end_time); end
% try delete(pt_traces); end
% try delete(pt_traceLABEL); end
% 
% 
% 

% HIM=image(I0,'Parent',H.ax1);
%HIM=imagesc(I0,'Parent',H.ax1,'AlphaDataMapping','None','CDataMapping','direct');  colormap(H.ax1,gray(256);
HIM=image(I0,'Parent',H.ax1); colormap(H.ax1,gray(256));
set(H.ax1,'Xaxislocation','Top','CLimMode','manual','Clim',[0 255])
drawnow

% % put 1st and last tickmark at null positions
% hold(H.ax1,'on')
% p_tick_1st=plot(H.ax1,0,0,'gv','MarkerSize',20);set(p_tick_1st,'Visible','off');
% p_tick_last=plot(H.ax1,0,0,'r^','MarkerSize',20);set(p_tick_last,'Visible','off');
% hold(H.ax1,'off')
% set([p_tick_1st,p_tick_last],'Visible','off')

% output_message{end+1}='Ready.';
% set(H.output_mess,'String',output_message,'Value',length(output_message));drawnow;

set(H.f1,'visible','on');

delete(hmess);

end




function update_plot_image(hObject, eventdata)
global H I0 HIM
xyl=axis(H.ax1);
set(HIM,'Cdata',I0)
axis(H.ax1,xyl);
%title(H.ax1,get(H.textfilename,'string'),'Interpreter','none')
%set(H.ax1,'Xaxislocation','Top')
drawnow;
end


function zoom2seismogram(hObject, eventdata)
global I0 H
set(H.ax1,'Xlim',[1 size(I0,2)],'Ylim',[1,size(I0,1)],'Xaxislocation','Top')
drawnow;
% Reset_MouseMotionMon;
end


function adjust_contrast(hObject, eventdata)
global H I0 Iundo
Iundo=I0;
hFig = figure('Color','w','Toolbar','none','NumberTitle','off',...
    'Menubar','none','Visible','off','closerequestfcn','','Name',H.textfilename.String);
hIm = imshow(I0);
hSP = imscrollpanel(hFig,hIm);
set(hSP,'Units','normalized',...
    'Position',[0.0 0.05 1 .95])
hMagBox = immagbox(hFig,hIm);
pos = get(hMagBox,'Position');
set(hMagBox,'Position',[0 0 pos(3) pos(4)])
%imoverview(hIm0);
hImCon=imcontrast(hIm);
set(hFig,'Visible','on')
waitfor(hImCon);
I0=get(hIm,'Cdata');
update_plot_image;
delete(hFig);
% enable undo
set(H.tbar_UNDO,'Enable','on');
end


function remove_SaltandPepper(hObject,eventdata)
global I0 Iundo H

Fig_remove_SaltandPepper = figure('Units','Normalized','Position',[0.1 0.1 0.5 0.7],...
    'color','w','WindowStyle','Normal','NumberTitle','Off','Name','Remove bacground long wavelength noise of the image with Gaussian filtering');

hbg=uibuttongroup('Visible','on','BackgroundColor',[1 1 1],'Position',[0.01 0.01 0.25 0.04],'Title','Filter type');
uicontrol(hbg,'Style','radiobutton','units','normalized',...
    'String', '2-D Median','BackgroundColor',[1 1 1],...
    'Position', [0.05 0.01 0.4 1]);
uicontrol(hbg,'Style','radiobutton','units','normalized',...
    'String', 'Wiener','BackgroundColor',[1 1 1],...
    'Position', [0.55 0.01 0.4 1]);

uicontrol('Style', 'Text','Parent',Fig_remove_SaltandPepper,'units','normalized',...
    'String', 'm x n neighborhood:','BackgroundColor',[1 1 1],...
    'Position', [0.30 0.015 0.08 0.02],'HorizontalAlignment','right');
Mvalue=uicontrol('Style', 'Edit','Parent',Fig_remove_SaltandPepper,'units','normalized','tag','Vsigma',...
    'String', '4','Tooltip','Enter a positive scalar','BackgroundColor',[1 1 1],...
    'Position', [0.39 0.02 0.06 0.02]);
uicontrol('Style', 'Text','Parent',Fig_remove_SaltandPepper,'units','normalized',...
    'String', 'x','BackgroundColor',[1 1 1],...
    'Position', [0.455 0.015 0.01 0.02]);
Nvalue=uicontrol('Style', 'Edit','Parent',Fig_remove_SaltandPepper,'units','normalized','tag','Vsigma',...
    'String', '4','Tooltip','Enter a positive scalar','BackgroundColor',[1 1 1],...
    'Position', [0.468 0.02 0.06 0.02]);

uicontrol('Style', 'pushbutton','Parent',Fig_remove_SaltandPepper,'units','normalized',...
    'String', 'Preview', 'Position',[0.68 0.01 0.09 0.04],'Callback',@PreviewChanges);

uicontrol('Style', 'pushbutton','Parent',Fig_remove_SaltandPepper,'units','normalized',...
    'String', 'Apply','Position', [0.79 0.01 0.09 0.04],'Callback',@ApplyChanges);

uicontrol('Style', 'pushbutton','Parent',Fig_remove_SaltandPepper,'units','normalized',...
    'String', 'Cancel','Position', [0.90 0.01 0.09 0.04],'Callback',@(hObject,eventdata)(close(Fig_remove_SaltandPepper)));


%%
ax1=subplot(2,1,1);
imshow(I0)
ax2=subplot(2,1,2);
Itemp=I0;
HItemp=imshow(Itemp);
ax2.Visible='off';
linkaxes([ax1,ax2])
waitfor(Fig_remove_SaltandPepper)
clear Itemp;


    function PreviewChanges(hObject_l,eventdata_l)
        mn=[str2double(get(Mvalue,'String')) str2double(get(Nvalue,'String'))];
        if strcmp(get(hbg,'SelectedObject'),'Wiener')
            Itemp=wiener2(I0,mn);
        else
            Itemp=medfilt2(I0,mn);
        end
        set(HItemp,'CData',Itemp);
        drawnow;
        disp('ok')
    end

    function ApplyChanges(hObject_l,eventdata_l)
        PreviewChanges;
        Iundo=I0;
        I0=Itemp;
        update_plot_image;
        delete(Fig_remove_SaltandPepper);
        set(H.tbar_UNDO,'Enable','on');
        
    end

end




function remove_background_withGAUSSFILT(hObject,eventdata)
global I0 Iundo H

Fig_remove_background = figure('Units','Normalized','Position',[0.1 0.1 0.5 0.7],...
    'color','w','WindowStyle','Normal','NumberTitle','Off','Name','Remove bacground long wavelength noise of the image with Gaussian filtering');

uicontrol('Style', 'Text','Parent',Fig_remove_background,'units','normalized',...
    'String', 'Gaussian distribution standard deviation,','BackgroundColor',[1 1 1],...
    'Position', [0.001 0.02 0.31 0.03]);

uicontrol('Style', 'Text','Parent',Fig_remove_background,'units','normalized',...
    'String', 'Vertical:','BackgroundColor',[1 1 1],...
    'Position', [0.32 0.02 0.08 0.03]);


Vsigma=uicontrol('Style', 'Edit','Parent',Fig_remove_background,'units','normalized','tag','Vsigma',...
    'String', '','Tooltip','Enter a positive scalar','BackgroundColor',[1 1 1],...
    'Position', [0.4 0.02 0.04 0.03]);
uicontrol('Style', 'Text','Parent',Fig_remove_background,'units','normalized',...
    'String', 'Horizontal:','BackgroundColor',[1 1 1],...
    'Position', [0.47 0.02 0.08 0.03]);
Hsigma=uicontrol('Style', 'Edit','Parent',Fig_remove_background,'units','normalized','tag','Hsigma',...
    'String', '','Tooltip','Enter a positive scalar','BackgroundColor',[1 1 1],...
    'Position', [0.56 0.02 0.04 0.03]);

uicontrol('Style', 'pushbutton','Parent',Fig_remove_background,'units','normalized',...
    'String', 'Preview', 'Position',[0.68 0.01 0.09 0.04],'Callback',@PreviewChanges);

uicontrol('Style', 'pushbutton','Parent',Fig_remove_background,'units','normalized',...
    'String', 'Apply','Position', [0.79 0.01 0.09 0.04],'Callback',@ApplyChanges);

uicontrol('Style', 'pushbutton','Parent',Fig_remove_background,'units','normalized',...
    'String', 'Cancel','Position', [0.90 0.01 0.09 0.04],'Callback',@(hObject,eventdata)(close(Fig_remove_background)));


ax1=subplot(2,1,1);
imshow(I0)
ax2=subplot(2,1,2);
Itemp=I0;
HItemp=imshow(Itemp);
ax2.Visible='off';
linkaxes([ax1,ax2])
waitfor(Fig_remove_background)
clear Itemp;


    function PreviewChanges(hObject_l,eventdata_l)
        sig=[str2double(get(Vsigma,'String')) str2double(get(Hsigma,'String'))];
        Itemp=imadjust(imsubtract(I0,imgaussfilt(I0,sig)));
        set(HItemp,'CData',Itemp);
    end

    function ApplyChanges(hObject_l,eventdata_l)
        
        PreviewChanges;
        Iundo=I0;
        I0=Itemp;
        update_plot_image;
        delete(Fig_remove_background);
        set(H.tbar_UNDO,'Enable','on');
        
    end

end





function meassure_pix_dist0(hObject, eventdata)
global H
hd0 = imline(H.ax1);
setColor(hd0,[1 1 0]);
pos=getPosition(hd0);
ht0=text('position',mean(pos),'string',num2str(round(abs(pos(2,1)- pos(1,1)))),'backgroundColor',[.9 .9 .8]);
addNewPositionCallback(hd0,@(pos)show_xdist(pos));
pos=wait(hd0);
delete(hd0);
% set(H.edit_tick_xlength,'String',num2str(round(abs(pos(2,1)- pos(1,1)))))

    function show_xdist(pos)
        set(ht0,'position',mean(pos),'string',num2str(round(abs(pos(2,1)- pos(1,1)))),'backgroundColor',[.9 .9 .8])
    end
delete(ht0);

% Evaluate_ticklength_input;


end

function remove_region(hObject, eventdata)
global H Iundo I0

% turn off zoon and pan
zoom off; pan off;

Iundo=I0; % save an undo copy
h=imfreehand(H.ax1);
h.Deletable = false;
try
    I0(h.createMask)=0;
    delete(h);
catch
    delete(h);
    return
end
update_plot_image;
set(H.tbar_UNDO,'Enable','on');
end

function Undo_remove(hObject, eventdata)
global H Iundo I0 Iorig IundoOrig
I0=Iundo;
if any(size(Iundo)~=size(I0))  % cropping has taken place - need to update if rotation is taken into account
    Iorig = IundoOrig;
end  % if any(size(Iundo~=size(I0))

clear Iundo;
set(H.tbar_UNDO,'Enable','off');
update_plot_image;
end


function viewimage(hObj,eventdata)
%  This function asks the user to select a rectangular region and allows
%  image modification on that section only.  The modified image is then
%  put back into the original image.

global H I0 Iundo HIM Iorig

% turn off zoon and pan
zoom off; pan off;

Iundo = I0;

lorig = true;  % parameter that is true if the original image exists
if isempty(Iorig) || any(size(I0)~=size(Iorig))  % if sizes are different
    lorig = false;  % original data not available
end  % if any (size(I0)~=size(Iorig))

hf1 = H.f1;  % work figure window
hax1 = H.ax1;  % axes where image is found
lrect = false;  % true if using rectangular region for analysis
dedge = 2;  % number of pixels to remove from the edges

Inew = [];
setappdata(hf1,'Inew',Inew);

waitforbuttonpress;
cclick = get(hf1,'SelectionType');  % get type of mouse click
switch cclick
    case 'alt'  % left mouse button click -> freehand
        lrect = true;
end

% disp(['lect: ',num2str(lrect)]);
% size(I0)

if lrect  % if rectangles are used for drawing
    
    him = imrect(hax1);
    position = wait(him);
    
    position = round(position);  % obtain integer values
    
    [nrow ncol] = size(I0);
    
    % ask if the region inside or outside should be used
    %cans = questdlg('Modify region outside of the selected area?','Region Selection','Yes','No','Cancel','No');
    cans = 'No';
    if (strcmpi(cans,'Yes'))  % select region outside
        % work as irregularly shaped selection
        
        % get indices inside
        % define polygon
        x = [position(1) position(1)+position(3) position(1)+position(3) position(1)];  % vertices locations
        y = [position(2) position(2) position(2)+position(4) position(2)+position(4)];
        indx = ~poly2mask(x,y,nrow,ncol);
        lrect = false;
        
        indx2 = reshape(indx,size(I0));
        Itmp = I0;
        Itmp(~indx2) = nan;
        %     figure;
        %     imshow(Itmp)
        
        % keeping track of needed indices
        
        [nrow ncol] = size(indx2);
        
        % find smallest rectangular area of image to be used for the analysis
        [rval1 iposR1] = max(indx2);  % finding starting "true" row number
        [rval2 iposR2] = max(flipud(indx2));  % finding ending "true" row number (position needs to be modified)
        iposR2 = nrow-iposR2+1;
        
        icol = find((iposR2-iposR1)>0 & rval1==1);
        ncolR(1) = min(icol); ncolR(2) = max(icol);  % range of column values
        
        [rval1 iposC1] = max(indx2');  % finding starting "true" column number
        [rval2 iposC2] = max(flipud(indx2'));  % finding ending "true" column number (position needs to be modified)
        iposC2 = ncol - iposC2 + 1;
        
        irow = find((iposC2-iposC1)>0 & rval1==1);
        nrowR(1) = min(irow); nrowR(2) = max(irow);
        
        %     figure;
        %     imshow(Itmp(nrowR(1):nrowR(2),ncolR(1):ncolR(2)));
        
        if (nrowR(2)-nrowR(1))>dedge & (ncolR(2)-ncolR(1))>dedge  % region large enough
            nrowT = nrowR(2)-nrowR(1)+1;
            ncolT = ncolR(2)-ncolR(1)+1;
            if lorig  % if original image exists
                htmp = modImage(Itmp(nrowR(1):nrowR(2),ncolR(1):ncolR(2)),...
                    Iorig(nrowR(1):nrowR(2),ncolR(1):ncolR(2)),hf1,lrect);
            else  % if original image does not exist
                htmp = modImage(Itmp(nrowR(1):nrowR(2),ncolR(1):ncolR(2)),...
                    [],hf1,lrect);
            end  % if lorig
            waitfor(htmp);
            Inew = getappdata(hf1,'Inew');
            %     disp('back in orig');
            %     size(Inew)
            
            if ~isempty(Inew)
                %             figure;
                %             imshow(Inew)
                hwait = waitbar(0,'Updating Figure');
                
                % find indices of images that are used for updating
                nz = max(dedge-nrowR(1)+1,0);
                nz2 = max((nrowR(2)+dedge)-nrow,0);
                itmp = [zeros(nz,ncol); ...
                    indx2(nrowR(1)-(dedge-nz):nrowR(2)+(dedge-nz2),ncolR(1):ncolR(2));...
                    zeros(nz2,ncol)];
                waitbar(1*0.1/8.8,hwait);
                nz3 = max(dedge-ncolR(1)+1,0);
                nz4 = max((ncolR(2)+dedge)-ncol,0);
                itmp2 = [zeros(nz,dedge); ...
                    zeros(2*dedge+nrowT-nz-nz2,nz3) indx2(nrowR(1)-(dedge-nz):nrowR(2)+(dedge-nz2),ncolR(1)-(dedge-nz3):ncolR(1)-1);
                    zeros(nz2,dedge)];
                waitbar(2*0.1/8.8,hwait);
                itmp = [itmp2 itmp];
                itmp2 = [zeros(nz,dedge); ...
                    indx2(nrowR(1)-(dedge-nz):nrowR(2)+(dedge-nz2),ncolR(2)+1:ncolR(2)+(dedge-nz4)) zeros(nrowT,nz4);
                    zeros(nz2,dedge)];
                itmp = [itmp itmp2];
                waitbar(3*0.1/8.8,hwait);
                rposM = zeros(nrowT,ncolT);  % temporary matrix
                for k = 1:dedge  % loop over edges
                    rposM = rposM + itmp(k:k+nrowT-1,1+dedge:end-dedge);
                    rposM = rposM + itmp(end-k-nrowT+2:end-k+1,1+dedge:end-dedge);
                    rposM = rposM + itmp(1+dedge:end-dedge,k:k+ncolT-1);
                    rposM = rposM + itmp(1+dedge:end-dedge,end-k-ncolT+2:end-k+1);
                    waitbar((3+k)*0.1/8.8,hwait);
                end  % for k = 1:dedge
                waitbar(0.1/1.1,hwait);
                
                %             if nrosR(1)>dedge  % there are plenty of rows above
                %                 itmp = 0;
                %             else  % need to add zeros at top
                %             end  % if rowR(1)>dedge
                %             rposM = zeros(nrowT,ncolT);  % temporary matrix
                %             for k = 1:dedge  % loop over edges
                %                 rtmp = [indx2(k+1:end,:); zeros(k,ncol)];
                %                 rposM = rposM + rtmp(nrowR(1):nrowR(2),ncolR(1):ncolR(2));
                %                 rtmp = [zeros(k,ncol); indx2(1:end-k,:)];
                %                 rposM = rposM + rtmp(nrowR(1):nrowR(2),ncolR(1):ncolR(2));
                %                 rtmp = [indx2(:,k+1:end), zeros(nrow,k)];
                %                 rposM = rposM + rtmp(nrowR(1):nrowR(2),ncolR(1):ncolR(2));
                %                 rtmp = [zeros(nrow,k), indx2(:,1:end-k)];
                %                 rposM = rposM + rtmp(nrowR(1):nrowR(2),ncolR(1):ncolR(2));
                %             end  % for k = 1:dedge
                % at this point pixels in the small rectangle used for
                % modification has value of 4*dedge if they are valid
                
                % set up indices for image transfer
                indxRt = find(rposM==4*dedge);  % indices for Inew
                indxR = [];  % indices for original image
                for k = 1+dedge:ncolT-dedge  % loop over rows
                    itmp = (rposM(:,k)==4*dedge);
                    idx = find(itmp) + nrowR(1)-1;  % put back into original image row indices
                    I0(idx,ncolR(1)+k-1) = Inew(itmp,k);
                    % %                 idx = (k-1)*ncol + idx;  % take into account column position
                    % %                 indxR = [indxR; idx];
                    waitbar((0.1+k/(ncolT-2*dedge))/1.1, hwait)
                end  % for k = 1:nrowT
                
                %            I0(indxR) = Inew(indxRt);
                delete(hwait);
                set(HIM,'CData',I0);
            end
        end  % if region large enough
        
    elseif strcmpi(cans,'No')  % rectangular region to analyze
        
        pcol(1) = position(1);  % starting column
        pcol(2) = position(1)+position(3);  % ending column
        pcol(find(pcol<1)) = 1;  % make sure we are not outside of the figure
        pcol(find(pcol>ncol)) = ncol;
        
        prow(1) = position(2);  % starting row
        prow(2) = position(2)+position(4);  % ending row
        prow(find(prow<1)) = 1;
        prow(find(prow>nrow)) = nrow;

        if (pcol(2)-pcol(1))>dedge && (prow(2)-prow(1)) > dedge  % image portion exists
%          htmp = figure;
%         set(htmp,'Name','Before Calling modImage');
%         imshow(I0(prow(1):prow(2),pcol(1):pcol(2)));

if lorig  % if original image exists
    htmp = modImage(I0(prow(1):prow(2),pcol(1):pcol(2)),...
        Iorig(prow(1):prow(2),pcol(1):pcol(2)),hf1);
else  % if original image does not exist
    htmp = modImage(I0(prow(1):prow(2),pcol(1):pcol(2)),...
        [],hf1);
end  % if lorig
            waitfor(htmp);
            Inew = getappdata(hf1,'Inew');
            %     disp('back in orig');
            %     size(Inew)
            if ~isempty(Inew)
                %         figure;
                %         imshow(Inew)
                I0(prow(1)+dedge:prow(2)-dedge,pcol(1)+dedge:pcol(2)-dedge) = Inew(3:end-dedge,3:end-dedge);
                % do not include edges
                set(HIM,'CData',I0);
            end
        end  % if (pcol(2)-pcol(1))> 0 && (prow(2)-prow(1)) > 0
        
    end  % if (strcmpi(cans,'Yes'))
    
else  % if freehand is used to select region
    % clean up all the button functions that may interfere with imfreehand
    try, set(hf1,'WindowButtonDownFcn',''), end
    try, set(hf1,'WindowButtonMotionFcn',''), end
    try, set(hf1,'WindowButtonUpFcn',''), end
    try, set(hf1,'WindowKeyPressFcn',''), end
    try, set(hf1,'WindowKeyReleaseFcn',''), end
    try, set(hf1,'WindowScrollWheelFcn',''), end
        
    him = imfreehand(hax1);
    indx = him.createMask;  % this is a vector rather than 2-D matrix
    
    % ask if the region inside or outside should be used
    cans = questdlg('Modify region outside of the selected area?');
    if (strcmpi(cans,'Yes'))  % select region outside
        indx = ~indx;  % complement selection
    end  % if (strcmpi(cans,'Yes'))
    
    if (~strcmpi(cans,'Cancel'))
        indx2 = reshape(indx,size(I0));
        Itmp = I0;
        Itmp(~indx2) = nan;
        %     figure;
        %     imshow(Itmp)
        
        % keeping track of needed indices
        
        [nrow ncol] = size(indx2);
        
        % find smallest rectangular area of image to be used for the analysis
        [rval1 iposR1] = max(indx2);  % finding starting "true" row number
        [rval2 iposR2] = max(flipud(indx2));  % finding ending "true" row number (position needs to be modified)
        iposR2 = nrow-iposR2+1;
        
        icol = find((iposR2-iposR1)>0 & rval1==1);
        ncolR(1) = min(icol); ncolR(2) = max(icol);  % range of column values
        
        [rval1 iposC1] = max(indx2');  % finding starting "true" column number
        [rval2 iposC2] = max(flipud(indx2'));  % finding ending "true" column number (position needs to be modified)
        iposC2 = ncol - iposC2 + 1;
        
        irow = find((iposC2-iposC1)>0 & rval1==1);
        nrowR(1) = min(irow); nrowR(2) = max(irow);
        
        %     figure;
        %     imshow(Itmp(nrowR(1):nrowR(2),ncolR(1):ncolR(2)));
        
        if (nrowR(2)-nrowR(1))>dedge & (ncolR(2)-ncolR(1))>dedge  % region large enough
            nrowT = nrowR(2)-nrowR(1)+1;
            ncolT = ncolR(2)-ncolR(1)+1;
            if lorig  % if original image exists
                htmp = modImage(I0(nrowR(1):nrowR(2),ncolR(1):ncolR(2)),...
                    Iorig(nrowR(1):nrowR(2),ncolR(1):ncolR(2)),hf1,...
                    indx2(nrowR(1):nrowR(2),ncolR(1):ncolR(2)));
            else  % if original image does not exist
                htmp = modImage(I0(nrowR(1):nrowR(2),ncolR(1):ncolR(2)),...
                    [],hf1,indx2(nrowR(1):nrowR(2),ncolR(1):ncolR(2)));
            end  % if lorig
            waitfor(htmp);
            Inew = getappdata(hf1,'Inew');
            %     disp('back in orig');
            %     size(Inew)
            
            if ~isempty(Inew)
                %             figure;
                %             imshow(Inew)
                hwait = waitbar(0,'Updating Figure');
                
                % find indices of images that are used for updating
                nz = max(dedge-nrowR(1)+1,0);
                nz2 = max((nrowR(2)+dedge)-nrow,0);
                itmp = [zeros(nz,ncol); ...
                    indx2(nrowR(1)-(dedge-nz):nrowR(2)+(dedge-nz2),ncolR(1):ncolR(2));...
                    zeros(nz2,ncol)];
                waitbar(1*0.1/8.8,hwait);
                nz3 = max(dedge-ncolR(1)+1,0);
                nz4 = max((ncolR(2)+dedge)-ncol,0);
                itmp2 = [zeros(nz,dedge); ...
                    zeros(2*dedge+nrowT-nz-nz2,nz3) indx2(nrowR(1)-(dedge-nz):nrowR(2)+(dedge-nz2),ncolR(1)-(dedge-nz3):ncolR(1)-1);
                    zeros(nz2,dedge)];
                itmp = [itmp2 itmp];
                waitbar(2*0.1/8.8,hwait);
                itmp2 = [zeros(nz,dedge); ...
                    indx2(nrowR(1)-(dedge-nz):nrowR(2)+(dedge-nz2),ncolR(2)+1:ncolR(2)+(dedge-nz4)) zeros(nrowT,nz4);
                    zeros(nz2,dedge)];
                itmp = [itmp itmp2];
                waitbar(3*0.1/8.8,hwait);
                rposM = zeros(nrowT,ncolT);  % temporary matrix
                for k = 1:dedge  % loop over edges
                    rposM = rposM + itmp(k:k+nrowT-1,1+dedge:end-dedge);
                    rposM = rposM + itmp(end-k-nrowT+2:end-k+1,1+dedge:end-dedge);
                    rposM = rposM + itmp(1+dedge:end-dedge,k:k+ncolT-1);
                    rposM = rposM + itmp(1+dedge:end-dedge,end-k-ncolT+2:end-k+1);
                    waitbar((3+k)*0.1/8.8,hwait);
                end  % for k = 1:dedge
                waitbar(0.1/1.1,hwait);
                
                %             if nrosR(1)>dedge  % there are plenty of rows above
                %                 itmp = 0;
                %             else  % need to add zeros at top
                %             end  % if rowR(1)>dedge
                %             rposM = zeros(nrowT,ncolT);  % temporary matrix
                %             for k = 1:dedge  % loop over edges
                %                 rtmp = [indx2(k+1:end,:); zeros(k,ncol)];
                %                 rposM = rposM + rtmp(nrowR(1):nrowR(2),ncolR(1):ncolR(2));
                %                 rtmp = [zeros(k,ncol); indx2(1:end-k,:)];
                %                 rposM = rposM + rtmp(nrowR(1):nrowR(2),ncolR(1):ncolR(2));
                %                 rtmp = [indx2(:,k+1:end), zeros(nrow,k)];
                %                 rposM = rposM + rtmp(nrowR(1):nrowR(2),ncolR(1):ncolR(2));
                %                 rtmp = [zeros(nrow,k), indx2(:,1:end-k)];
                %                 rposM = rposM + rtmp(nrowR(1):nrowR(2),ncolR(1):ncolR(2));
                %             end  % for k = 1:dedge
                % at this point pixels in the small rectangle used for
                % modification has value of 4*dedge if they are valid
                
                % set up indices for image transfer
                indxRt = find(rposM==4*dedge);  % indices for Inew
                indxR = [];  % indices for original image
                for k = 1+dedge:ncolT-dedge  % loop over rows
                    itmp = (rposM(:,k)==4*dedge);
                    idx = find(itmp) + nrowR(1)-1;  % put back into original image row indices
                    I0(idx,ncolR(1)+k-1) = Inew(itmp,k);
                    % %                 idx = (k-1)*ncol + idx;  % take into account column position
                    % %                 indxR = [indxR; idx];
                    waitbar((0.1+k/(ncolT-2*dedge))/1.1, hwait)
                end  % for k = 1:nrowT
                
                %            I0(indxR) = Inew(indxRt);
                delete(hwait);
                set(HIM,'CData',I0);
            end
        end  % if region large enough
    end  % if strcmpi(cans,'Cancel')
end  % if ltype (rectangular region vs freehand region)

delete(him);

rmappdata(hf1,'Inew');
set(H.tbar_UNDO,'Enable','on');

    try, set(hf1,'WindowButtonDownFcn',''), end  % clean up
    try, set(hf1,'WindowButtonMotionFcn',''), end
    try, set(hf1,'WindowButtonUpFcn',''), end
    try, set(hf1,'WindowKeyPressFcn',''), end
    try, set(hf1,'WindowKeyReleaseFcn',''), end
    try, set(hf1,'WindowScrollWheelFcn',''), end

end

%
function crop_image(hObject, eventdata)
global H I0 Iorig Iundo IundoOrig
%h=figure('Menu','none','Toolbar','none');%'WindowStyle','Modal');
[temp rect]=imcrop(H.ax1);
if isempty(temp)
    return
end
Iundo=I0;
IundoOrig = Iorig;
I0=temp;
if ~isempty(Iorig)
    Iorig = imcrop(Iorig,rect);
end
axis(H.ax1,[1,size(I0,2),1,size(I0,1)]);
set(H.tbar_UNDO,'Enable','on');
update_plot_image;
end


function estimate_rotation(hObject, eventdata)
global H Iundo I0 Iorig output_message
Iundo=I0;
% Take radon transform for +- 5 degrees with the horizontal

Iorig = [];  % if rotation is applied do not keep track of orignal image
% need to add codes to keep track of Iorig
output_message{end+1}='Searching for optimum rotation... ';
set(H.output_mess,'String',output_message,'Value',length(output_message)); drawnow;

theta0=[82:1:98];
R=zeros(size(theta0));
for i=1:numel(theta0)
    
    output_message{end}=['Testing... ' num2str(theta0(i)-90) ' degrees'];
    set(H.output_mess,'String',output_message,'Value',length(output_message));
    R(i)=min(sum(imrotate(I0,theta0(i),'bilinear','crop'),2));
end
[~,idx]=min(R);
ang0=theta0(idx)-90;

%refine search
output_message{end}='Refining search around mimimum';
set(H.output_mess,'String',output_message,'Value',length(output_message));


theta=[theta0(idx)-1.1:0.1:theta0(idx)+1.1];
RR=zeros(size(theta));
for i=1:numel(theta)
    output_message{end}=['Testing... ' num2str(theta(i)-90) ' degrees'];
    set(H.output_mess,'String',output_message,'Value',length(output_message));
    
    RR(i)=min(sum(imrotate(I0,theta(i),'bilinear','crop'),2));
end
[~,idx]=min(RR);
ang=theta(idx)-90;

hf=figure('WindowStyle','modal','Color','w','Name','Estimate Rotation','NumberTitle','off');
subplot(211)
plot(theta0-90,R,'k'), hold on
plot([ang0 ang0],get(gca,'ylim'),'r');
title(['Initial Estimation: ' num2str(ang0,'%3.2f') '^o'])
ylabel('Min(sum(intensity))')

subplot(212)
plot(theta-90,RR,'k'), hold on
plot([ang ang],get(gca,'ylim'),'r');
axis tight
xlabel('Rotation from horizontal (degrees)'),ylabel('Min(sum(intensity))')
title(['Refined Estimation: ' num2str(ang,'%3.2f') '^o'])
output_message{end}=['Rotation estimation: ' num2str(ang,'%3.2f') ' degrees'];
set(H.output_mess,'String',output_message,'Value',length(output_message));
uiwait(gcf)

%% Rotate Image
rot_ang = inputdlg('Rotate image','Enter rotation angle',1,{num2str(ang)});
if isempty(rot_ang)
    return
end
rot_ang = str2num(rot_ang{1});

if abs(rot_ang)<0.01
    warndlg('Angle is negligible','No rotation applied')
    return
end
% Rotate image accordingly
I0=imrotate(I0,rot_ang,'bilinear','crop');

output_message{end+1}=['Image rotated: ' num2str(rot_ang,'%3.2f') '^o.'];
set(H.output_mess,'String',output_message,'Value',length(output_message)); drawnow;
set(H.tbar_UNDO,'Enable','on')


update_plot_image;
end




function idTrace_options(hObject, eventdata)
% this function allows options for digitization to be set

global H

clear ito;  % initial cleaning

% default value
ito.nv = 15;  % default of the # of vertical divisions

% positions of the properties window
r_wh = 396/564;  % width to height ratio
p = get(0,'ScreenSize');  % get reference screen size
% set up max values for box setup (in pixels)
rval_max = [floor(p(3)/2) floor(p(4)/2) p(3) p(4)];
% height = p(4)/2;  % height of the box
% width = r_wh*height;
height = 564;
width = 396;
x = p(3)-width;
y = p(4)-height;
pos.hf = [x y width height];
pos.nvEdit = [0.063 0.849 0.1 0.04]; % edit box position for digitization strip definition
pos.nvText = [0.165 0.849 0.8 0.04];  % text explaining edit box content
% positions of the execution buttons
% pos.Apply = [0.32 0.1 0.15 0.05];
%pos.Close = [0.53 0.1 0.15 0.05];
pos.Close = [0.425 0.1 0.15 0.05];

lnewFig = true;  % will become false if new figure window is not going to be created

% check to see if the option window already exists
if isappdata(H.f1,'ito');
    ito = getappdata(H.f1,'ito');  % get old value of cto
    
    if isvalid(ito.hf)  % if figure still exists
        hchil = get(0,'children');
        set(ito.hf,'visible','on');  % make sure this window is visibled
        set(0,'children',[ito.hf; hchil(hchil~=ito.hf)]);  % bring relevant figure to front
        lnewFig = false;
    end  % if any(ishandle(ito.figure))    
end  % if isappdata(H.f1,'ito');

if lnewFig  % if new figure is going to be created
    ito.hf = figure('Name','Trace Identification Properties','NumberTitle','off',...
        'Units','pixels','menu','none','toolbar','none',...
        'OuterPosition',pos.hf);
    
    cstr = 'number of vertical strips to be considered';
    ito.hnvText = uicontrol('style','text','parent',ito.hf,'units','normalized',...
        'position',pos.nvText,'string',cstr,'horizontalAlignment','left',...
        'fontsize',12);
    ito.hnvEdit = uicontrol('style','edit','parent',ito.hf,'units','normalized',...
        'position',pos.nvEdit,'string',num2str(ito.nv),...
        'fontsize',12,'callback',@itoCheckValue);

    
%     ito.hApply = uicontrol('style','pushbutton','parent',ito.hf,'units','normalized',...
%         'position',pos.Apply,'string','Apply','tooltipstring','Apply changes',...
%         'fontsize',14,'callback',@itoApply);
    ito.hClose = uicontrol('style','pushbutton','parent',ito.hf,'units','normalized',...
        'position',pos.Close,'string','Close','tooltipstring','Close this window',...
        'fontsize',14,'callback',@itoClose);
    
end  % if lnewFig

setappdata(H.f1,'ito',ito);  % update variable

%     function itoApply(hobj,eventdata)
%         % this function will copy the values in the edit fields into
%         % H.edit_offset so that they can be used by the main program
%         
%         ito.nv = str2num(get(ito.hnvEdit,'string'));
%         setappdata(H.f1,'ito',ito);
%     end  % function ctoApply

    function itoCheckValue(hobj,eventdata)
        % this function checks for the validity of the input values and
        % sets back to previous value if unallowed
        
        cstr = num2str(ito.nv);  % default value
        
        rtmp = str2num(get(hobj,'string'));
        if ~isempty(rtmp) & mod(rtmp,1)==0 & rtmp > 0
            cstr = get(hobj,'string');
        end  % if ~isempty(roffset)

        set(hobj,'string',cstr);
        
        % update information
        ito.nv = str2num(get(ito.hnvEdit,'string'));
        setappdata(H.f1,'ito',ito);
    end  % function ctoCheckValue

    function itoClose(hobj,eventdata)
        % this function closes the property setup window
        delete(ito.hf);
    end  % function ctoClose

end  % function digitize_options


function find_traces(hObject, eventdata)
global H I0 ftrend sumI ww pt_start_time pt_end_time pt_traces pt_traceLABEL output_message
global pt_tracesY0

output_message{end+1}='Detecting traces, please wait...';
set(H.output_mess,'String',output_message,'Value',length(output_message)); drawnow;

pt_tracesY0 = {};

% DEBUG MODE --> COMMENT
% try delete(pt_start_time); end
% try delete(pt_end_time); end
% pt_start_time=[];
% pt_end_time=[];

num_of_v_divisions=15;  % default value for number of vertical divisions
if isappdata(H.f1,'ito')  % number may have been changed
    ito = getappdata(H.f1,'ito');
    num_of_v_divisions = ito.nv;
end  % if isappdata(H.f1,'ito')
[rows, cols]=size(I0);

ww=zeros(num_of_v_divisions,2);
d=ceil(cols/num_of_v_divisions);
sumI=zeros(rows,num_of_v_divisions);
for it=1:num_of_v_divisions
    win=max(1,floor((it-1)*d-d/3)):min(cols,ceil(it*d+d/3));
    sumI(:,it)=sum(I0(:,win),2);
    sumI(:,it)=sumI(:,it)/max(sumI(:,it));
    ww(it,:)=[win(1) win(end)];
end

hold(H.ax2, 'off')
%hold(H.ax1,'on')
%cm=jet(10);
imagesc(linspace(1,cols,size(sumI,2)),linspace(1,rows,size(sumI,1)),sumI,'Parent',H.ax2)
hold(H.ax2,'on')
% for it=1:num_of_v_divisions
% %    plot(H.ax1,mean(ww(it,:))+0.3*d*sumI(:,it),[1:rows]','color',cm(it,:))
%     plot(H.ax2,mean(ww(it,:))+1.5*d*sumI(:,it),[1:rows]')
%     hold(H.ax2,'on')
% end
%axis(H.ax2,'tight')
set(H.ax2,'yticklabel','','ylim',[1 size(I0,1)],'Ydir','Reverse');

%cross-correlate and find trend
maxlags=round(rows/3);
shift=zeros(1,num_of_v_divisions-1);
imaxc=zeros(1,num_of_v_divisions-1);
for it=2:num_of_v_divisions
    [c_ww{it-1},lags{it-1}] = xcorr(sumI(:,it-1),sumI(:,it),maxlags,'coeff');
    [~,imaxc(it-1)]=max(c_ww{it-1});
    shift(it-1)=lags{it-1}(imaxc(it-1));
end
ftrend = fit( mean(ww,2), [0, -cumsum(shift)]', 'poly2');
%plot(H.ax1,1:cols,rows/2+ftrend(1:cols),'m--','LineWidth',4);
plot(H.ax2,1:cols,rows/2+ftrend(1:cols),'-','LineWidth',4,'Color',[0. 0.7 1]);


tol=0.2;
lenpks=zeros(num_of_v_divisions,1);
for it=1:num_of_v_divisions
    [pks{it}, locs{it}]=findpeaks(smooth(sumI(:,it),round(rows/220)),'MinPeakHeight',tol);
    lenpks(it)=numel(pks{it});
end

numpeaks=mode(lenpks); % find dominant (most frequent) number
try delete(pt_traces), end
try delete(pt_traceLABEL), end


iok=1;
for i=1:length(locs)
    if length(locs{i})==numpeaks
        iok=i;
        break
    end
    
end
locs=locs{iok};

pt_traces=zeros(numpeaks,1);
pt_traceLABEL=zeros(numpeaks,1);
%std_lenpks=std(lenpks);
hold(H.ax1,'on')
for i=1:numpeaks
    pt_traces(i)=plot(H.ax1,1:cols,locs(i)+ftrend(1:cols)-ftrend(1),'c--','LineWidth',1);
    pt_tracesY0{i} = get(pt_traces(i),'Ydata');  % original Y data
    pt_traceLABEL(i)=text(100,locs(i)+ftrend(100)-ftrend(1),num2str(i),'BackgroundColor',[1 1 .8],'Parent',H.ax1);
end
hold(H.ax1,'off')

%return % DEBUG MODE --> COMMENT in normal mode

% find_tickmarks;



% populate H.ax3 panel
% output_message{end}='Updating figure, please wait...';
% set(H.output_mess,'String',output_message,'Value',length(output_message)); drawnow;
% 
% num_of_v_divisions=11;
% 
% ww=zeros(num_of_v_divisions,2);
% d=ceil(rows/num_of_v_divisions);
% sumI=zeros(num_of_v_divisions,cols);
% for it=1:num_of_v_divisions
%     win=max(1,floor((it-1)*d-d/3)):min(rows,ceil(it*d+d/3));
%     sumI(it,:)=sum(I0(win,:),1);
%     sumI(it,:)=sumI(it,:)/max(sumI(it,:));
%     ww(it,:)=[win(1) win(end)];
% end
% 
% hold(H.ax3, 'off')
% imagesc(linspace(1,cols,size(sumI,2)),linspace(1,rows,size(sumI,1)),sumI,'Parent',H.ax3)
% set(H.ax3,'yticklabel','','xticklabel','','xlim',[1 size(I0,2)],'Ydir','reverse');


set(H.togle_traces_visibility,'Enable','on','value',1);
set(H.adjust_traces,'Enable','on');
set(H.correct_classification,'Enable','on')


output_message{end+1}='Ready';
set(H.output_mess,'String',output_message,'Value',length(output_message)); drawnow;

end


function traces_visibility(hObject, eventdata)
global H pt_traces pt_traceLABEL
try
    if get(H.togle_traces_visibility,'value')
        set([pt_traces; pt_traceLABEL],'visible','on')
    else
        set([pt_traces; pt_traceLABEL],'visible','off')
    end
catch
    pt_traces=flipud(findobj(H.ax1,'Color','c','Linestyle','--'));
    pt_traceLABEL=flipud(findobj(H.ax1,'type','text','FontSize',10,'BackgroundColor',[1 1 0.800]));
    if get(H.togle_traces_visibility,'value')
        set([pt_traces; pt_traceLABEL],'visible','on')
    else
        set([pt_traces; pt_traceLABEL],'visible','off')
    end
end
end


function digital_traces_visibility(hObject, eventdata)
global H ptrace ptraceSTD % pt_traces

if get(H.togle_digital_traces_visibility,'value')
    set(ptrace,'visible','on')
else
    set(ptrace,'visible','off')
end


try
    if get(H.togle_digital_traces_STD_visibility,'value')
        set(ptraceSTD,'visible','on')
    else
        set(ptraceSTD,'visible','off');
    end
catch
   errordlg('Could not find handles for digitized traces STD. If this file is from an older version of DigitSeis without this feature, ignore this message');
   set(ptraceSTD,'visible','off');
   set(H.togle_digital_traces_STD_visibility,'value',0);
end


end




function mark_1st_and_last(hObject, eventdata,typeflag,edit_flag)
global H S pt_traces ftrend ftrend_tick p_tick_1st p_tick_last
% marks the 1st time mark

% remove any objects that end up having no pixels (this shouldn't
% be necessary if fixed version (01/15/2017) of correct trace has
% been used
indx = find(cellfun('isempty',{S.PixelIdxList}));
S(indx) = [];  % remove empty pixels
        
ltend = true;  % default (minute starts at the end of time marks)
if (get(H.cb_timeEnd,'value')) % minute starts at the beginning of time marks
    ltend = false;
end  % if (get(H.ch_timeEnd,'value'))

ltm = strcmpi(get(H.timemarkpos,'state'),'off');  % true if time marks are above
ltype = strcmpi(typeflag,'FIRST');  % true if first line, else assume last

if edit_flag
    if strcmpi(typeflag,'FIRST')
        X=get(p_tick_1st,'Xdata');
        Y=get(p_tick_1st,'Ydata');
    elseif strcmpi(typeflag,'LAST')
        X=get(p_tick_last,'Xdata');
        Y=get(p_tick_last,'Ydata');
    else
        error('mark_1st_and_last:Uknown typeflag');
    end
    
elseif (min(H.version)>=1.0)  % all objects have been assigned trace numbers
    
    X=zeros(length(pt_traces),1);
    Y=zeros(length(pt_traces),1);
    
    for k = 1:numel(pt_traces)  % loop over traces
        % get time mark positions for this trace
        indx = find([S.TraceNum]==k & ([S.ID]==1|[S.ID]==3));
        
        if ~isempty(indx)  % time marks exist
            tmBoxes = cell2mat({S(indx).BoundingBox}');  % time mark bounding boxes
            
            if ltype  % if "FIRST" time mark sought
                [~, itmp] = min(tmBoxes(:,1)+tmBoxes(:,3)); % x position
            else  % "LAST" time mark sought
                [~, itmp] = max(tmBoxes(:,1)+tmBoxes(:,3)); % x position
            end  % if ltype
            itmp = itmp(1);  % just in case there are multiple objects
            
            if (ltend)  % minute starts at the end of a time mark
                X(k) = tmBoxes(itmp,1)+tmBoxes(itmp,3);
            else  % minute starts at the beginning of a time mark
                X(k) = tmBoxes(itmp,1);
            end  % if (ltend)...
            Y(k) = tmBoxes(itmp,2)+tmBoxes(itmp,4)/2;  % y position
        else  % time marks do not exist
            if ltype  % if "FIRST" time mark sought
                rtmp = get(pt_traces(k),'XDATA');
                X(k) = rtmp(1);
                rtmp = get(pt_traces(k),'YDATA');
                Y(k) = rtmp(1);
            else  % if "LAST" time mark sought
                rtmp = get(pt_traces(k),'XDATA');
                X(k) = rtmp(end);
                rtmp = get(pt_traces(k),'YDATA');
                Y(k) = rtmp(end);
            end  % if ltype
        end  % if ~isempty(indx)...
    end  % for k = 1:numel(pt_traces)
    
else % select with mouse
    
    % obtain time mark information
    itm = find([S.ID]==1|[S.ID]==3);  % time mark indices
    tmBoxes = cell2mat({S(itm).BoundingBox}');  % time mark bounding boxes
    
    % find min/max position for the time marks
%     [~, itmp] = min(tmBoxes(:,1)+tmBoxes(:,3));
%     x1 = tmBoxes(itmp(1),1)+tmBoxes(itmp(1),3);  % x position
%     [~, itmp] = max(tmBoxes(:,1)+tmBoxes(:,3));
%     x2 = tmBoxes(itmp(1),1)+tmBoxes(itmp(1),3);
    
    % get number of eligible traces to consider
    if ltm  % if time marks are above
%        dy = +0.0;  % factor for shifting y position
         npts_traces_togo = length(pt_traces);
    else  % time marks below
%        dy = -1.0;  % factor for shifting y position
         npts_traces_togo = length(pt_traces)-1;
    end  % if strcmpi(get(H.timemarkpos,'state'),'off')...
%     npts_traces_togo = length(pt_traces);

    X=zeros(length(pt_traces),1);
    Y=zeros(length(pt_traces),1);

    pt_traces_Xdata = get(pt_traces,'Xdata'); % separate X and Y values
    pt_traces_Ydata = get(pt_traces,'Ydata');
    
    for i = 1:npts_traces_togo  % loop over eligible traces
        % find time mark boxes corresponding to the current trace
        if ltm  % time marks are above
            ic= tmBoxes(:,2)+tmBoxes(:,4)/2<=pt_traces_Ydata{i}(1)+ftrend(tmBoxes(:,1)) &...
                tmBoxes(:,1)>=pt_traces_Xdata{i}(1) & tmBoxes(:,1)+tmBoxes(:,3)<=pt_traces_Xdata{i}(end);
        else  % time marks are below
            ic= tmBoxes(:,2)+tmBoxes(:,4)/2<=pt_traces_Ydata{i+1}(1)+ftrend(tmBoxes(:,1)) &...
                tmBoxes(:,1)>=pt_traces_Xdata{i}(1) & tmBoxes(:,1)+tmBoxes(:,3)<=pt_traces_Xdata{i}(end);
        end  % if ltm...

%         ic= tmBoxes(:,2)+dy*tmBoxes(:,4)<=pt_traces_Ydata{i}(1)+ftrend(tmBoxes(:,1)) &...
%             tmBoxes(:,1)>=pt_traces_Xdata{i}(1) & tmBoxes(:,1)+tmBoxes(:,3)<=pt_traces_Xdata{i}(end);
        
%         if ltype  % 'FIRST' time marks sought
%             ic= tmBoxes(:,2)+tmBoxes(:,4)/2<=pt_traces_Ydata{i}(1)+ftrend(tmBoxes(:,1)) &...
%                 tmBoxes(:,1)>=pt_traces_Xdata{i}(1) & tmBoxes(:,1)+tmBoxes(:,3)<=pt_traces_Xdata{i}(end);
%         else  % 'LAST' time marks
%             ic= tmBoxes(:,2)+tmBoxes(:,4)/2<=max(pt_traces_Ydata{i}(1)) &...
%                 tmBoxes(:,1)>=pt_traces_Xdata{i}(1) & tmBoxes(:,1)+tmBoxes(:,3)<=pt_traces_Xdata{i}(end);
%         end  % if ltype...
        
        if numel(find(ic))>0  % there are time marks on this trace
            
            % find time marks of current trace
            tmBoxesCurrent = tmBoxes(ic,:);
            tmBoxes(ic,:) = [];  % remove used boxes, i.e., current trace boxes
            
            % find X and Y value
            if ltype  % if 'FIRST' time mark sought
                [~, indx] = min(tmBoxesCurrent(:,1)+tmBoxesCurrent(:,3));  % look at the x position
            else  % if 'LAST' time mark sought
                [~, indx] = max(tmBoxesCurrent(:,1)+tmBoxesCurrent(:,3));
            end  % if ltype
            
            itmp = indx(1);  % use only one in case there are multiple minima
            if (ltend)  % minute starts at the end of a time mark
                X(i) = tmBoxesCurrent(itmp,1)+tmBoxesCurrent(itmp,3);  % x position at the end of the time mark
            else  % minute starts at the beginning of a time mark
                X(i) = tmBoxesCurrent(itmp,1);
            end  % if (ltend)...
            Y(i) = tmBoxesCurrent(itmp,2)+tmBoxesCurrent(itmp,4)/2;  % y position with half the
            
        else  % time marks do not exist for this trace
            
            % set X and Y values using default x locations (x1 and x2)
            if ltype  % if 'FIRST' time mark sought
                X(i) = pt_traces_Xdata{i}(1);  
                Y(i) = pt_traces_Ydata{i}(1);
            else  % if 'LAST' time mark sought
                X(i) = pt_traces_Xdata{i}(end);
                Y(i) = pt_traces_Ydata{i}(end);
            end  % if ltype
%             [vmin imin] = min(abs(pt_traces_Xdata{i}-xtmp));  % get x position index
%             X(i) = xtmp;
%             Y(i) = pt_traces_Ydata{i}(imin(1));
%             if vmin > 2  % pixels, i.e., x1 value not valid
%                 X(i) = nan;
%                 Y(i) = nan;
%             else  % zero position
%                 X(i) = xtmp;
%                 Y(i) = pt_traces_Ydata{i}(imin(1));
%             end  % if vmin > 2
            
        end  % if numel(ic)>0...
        
    end  % for i = 1:npts_traces_togo
    
    if ~ltm  % if time marks are below - take care of the bottommost trace
        
        i = npts_traces_togo + 1;  % last trace number
        
        if numel(tmBoxes)>0  % there are time marks left
            
            % find time marks of current trace
            tmBoxesCurrent = tmBoxes;
            
            % find X and Y value
            if ltype  % if 'FIRST' time mark sought
                [~, indx] = min(tmBoxesCurrent(:,1)+tmBoxesCurrent(:,3));  % look at the x position
            else  % if 'LAST' time mark sought
                [~, indx] = max(tmBoxesCurrent(:,1)+tmBoxesCurrent(:,3));
            end  % if ltype
            
            itmp = indx(1);  % use only one in case there are multiple minima
            if (ltend)  % minute starts at the end of a time mark
                X(i) = tmBoxesCurrent(itmp,1)+tmBoxesCurrent(itmp,3);  % x position at the end of the time mark
            else  % minute starts at the beginning of a time mark
                X(i) = tmBoxesCurrent(itmp,1);  % x position at the beginning of the time mark
            end  % if (ltend)...
            Y(i) = tmBoxesCurrent(itmp,2)+tmBoxesCurrent(itmp,4)/2;  % y position with half the
            
        else  % time marks do not exist for this trace
            
            % set X and Y values using default x locations (x1 and x2)
            if ltype  % if 'FIRST' time mark sought
                X(i) = pt_traces_Xdata{i}(1);
                Y(i) = pt_traces_Ydata{i}(1);
            else  % if 'LAST' time mark sought
                X(i) = pt_traces_Xdata{i}(end);
                Y(i) = pt_traces_Ydata{i}(end);
            end  % if ltype
            
%             [vmin imin] = min(abs(pt_traces_Xdata{i}-xtmp));  % get x position index
%             if vmin > 2  % pixels, i.e., x1 value not valid
%                 X(i) = nan;
%                 Y(i) = nan;
%             else  % zero position
%                 X(i) = xtmp;
%                 Y(i) = pt_traces_Ydata{i}(imin(1));
%             end  % if vmin > 2
            
        end  % if numel(ic)>0...
        
    end  % if ~ltm
    
%     [x,~]=ginput(1);
%     
%     x=round(x);
%     ptrX=get(pt_traces,'Xdata');
%     ptrY=get(pt_traces,'Ydata');
%     
%     X=zeros(length(pt_traces),1);
%     Y=zeros(length(pt_traces),1);
%     
%     for i=1:length(pt_traces)
%         
%         [vmin,imin]=min(abs(ptrX{i}-x));
%         if vmin>2 % pixels
%             X(i)=nan;
%             Y(i)=nan;
%         else
%             Y(i)=ptrY{i}(imin);
%             X(i)=ftrend_tick(Y(i));
%         end
%     end
%     
%     temp=X(~isnan(X));
%     X=X+x-temp(1);
    
end

if strcmpi(typeflag,'FIRST')
    try  delete(p_tick_1st); end;
    set(H.edit_mark_1st,'Enable','on');
else strcmpi(typeflag,'LAST')
    try delete(p_tick_last);  end;
    set(H.edit_mark_last,'Enable','on');
end

% hpoly=impoly(H.ax1,[X(:),Y(:)],'Closed',false);
% hpoly.Deletable = false;
% pos=round(wait(hpoly));
% delete(hpoly);

hline = line(X,Y);  % line connecting points
set(hline,'parent',H.ax1,'color','y');

hpoints = [];  % vector containing handles of draggable points
ccol0 = [];  % default colours
for k = 1:numel(X)  % create draggable points
    htmp = impoint(H.ax1,X(k),Y(k));
    set(htmp,'tag',sprintf('%2.2i',k));
    % change size of the cross and circle
    hchil = get(htmp,'children');
    set(hchil(1),'markersize',25);  % change marker size for the cross
    set(hchil(2),'markersize',15);  % change marker size for circle
    addNewPositionCallback(htmp,@updatePointTM);
    htmp.Deletable = false;
    setColor(htmp,'y')
    ccol = getColor(htmp);
    ccol0 = [ccol0; ccol];
    hpoints = [hpoints htmp];
end  % for k = 1:numel(X)
itmp = numel(X);
while isnan(X(itmp))
    itmp = itmp - 1;
end  % while ~isnan(X(itmp))
htmp = hpoints(itmp);
rtmp = wait(htmp);  % let the updating considered finished if the last point is double-clicked

% now, update the X and Y values to allow for nan entries (indicated by
% changed colours)
X = round(get(hline,'Xdata'));
Y = round(get(hline,'Ydata'));
rcol = [];
for k = 1:numel(X)
    h = hpoints(k);
    ccol = getColor(h);
    rcol(k) = sum(ccol0(k,:)-ccol);
    delete(h);  % delete draggable point object
end  % for k = 1:numel(X)
indx = find(rcol~=0);
X(indx) = nan;
Y(indx) = nan;

delete(hline);  % remove line used for display

hold(H.ax1,'on');
if strcmpi(typeflag,'FIRST')
    p_tick_1st=plot(X,Y,'+-','Color',[0 0.8 0]);
else strcmpi(typeflag,'LAST')
    p_tick_last=plot(X,Y,'+-','Color',[0.8 0 0]);
end
hold(H.ax1,'off');

    function updatePointTM(pos)
%         disp(pos)
        ho = gco;
        ctag = get(ho,'tag');
        k = str2double(ctag);
        X(k) = pos(1);
        Y(k) = pos(2);
        set(hline,'Xdata',X);
        set(hline,'Ydata',Y);
    end

set(H.Create_Time,'enable','on');
set(H.togle_timebounds_visibility,'enable','on');
set(H.togle_timebounds_visibility,'value',1);
end








function mark_start(hObject, eventdata,edit_flag)
global H I0 pt_start_time pt_end_time pt_traces S ftrend
global ptrace ptraceSTD
global pt_tracesY0

%[t0(1), t0(2)]=ginput(1);
% set(pt_start_time,'Xdata',t0(1),'Ydata',t0(2))
% set(H.text_t0,'String',sprintf('xp=%.0f yp=%.0f',round(t0(1)), round(t0(2))))
% drawnow

if edit_flag
    X=get(pt_start_time,'Xdata');  % start time data
    Y=get(pt_start_time,'Ydata');
    
    Xe=get(pt_end_time,'Xdata');  % end time data
    Ye = get(pt_end_time,'Ydata');
    
    delete(pt_start_time);
    delete(pt_end_time);
elseif ~exist('S')  % structure S containing objects do not exist
    delete(pt_start_time);  % clean up lines
    delete(pt_end_time);

    offset=500;
    [x1,~]=ginput(1);  % get input for the start line
    
    x1=round(x1);
    
    [rows,cols]=size(I0);
    
    X=zeros(length(pt_traces),1);
    Y=cell2mat(cellfun(@(v) v(1),get(pt_traces,'Ydata'),'UniformOutput',false));
    for i=1:length(pt_traces)
        [~,X(i)]=max(abs(rangefilt(mean(I0(max(1,round(Y(i))-offset):min(rows,round(Y(i))+offset),1:2*x1)),true(1,7))));
    end
    
    [x1,~]=ginput(1);
    x1=round(x1);
    Xe=zeros(length(pt_traces),1);
    Ye=cell2mat(cellfun(@(v) v(end),get(pt_traces,'Ydata'),'UniformOutput',false));
    
    jstart=round(max(cols/4,3*x1-2*cols)):cols;
    for i=1:length(pt_traces)
        [~,Xe(i)]=max(abs(rangefilt(mean(...
            I0(max(1,round(Y(i))-offset):min(rows,round(Ye(i))+offset),jstart)),true(1,7))));
    end
    Xe=Xe+jstart(1)-1;
elseif min(H.version)>=1.0  % all objects have associated trace numbers
    %disp('taking advantage of trace numbers');
    
    delete(pt_start_time);  % clean up
    delete(pt_end_time);
    
    pt_traces_Xdata = get(pt_traces,'Xdata');
    pt_traces_Ydata = get(pt_traces,'Ydata');
    
    X=zeros(length(pt_traces),1);
    Y=zeros(length(pt_traces),1);
    Xe=zeros(length(pt_traces),1);
    Ye=zeros(length(pt_traces),1);
   
    for k = 1:length(pt_traces)  % loop over traces
        indx = find([S.TraceNum]==k);  % find all trace objects
        
        if ~isempty(indx)  % trace objects exist
            tmBoxes = cell2mat({S(indx).BoundingBox}');
            
            % find starting position
            X(k) = floor(min(tmBoxes(:,1)));  % minimum x value available
            itmp = find(pt_traces_Xdata{k}(:)==X(k));
            if ~isempty(itmp)  % selected point within trace-zero line domain
                Y(k) = pt_traces_Ydata{k}(itmp);
            else  % selected point outside of trace-zero line domain
                % extend y value
                extrapoints = X(k):pt_traces_Xdata{k}(1)-1;
                if ~iscell(pt_tracesY0) || isempty(pt_tracesY0) % reference trace Y values do not exist
                    rtmp = interp1(pt_traces_Xdata{k},pt_traces_Ydata{k},extrapoints,'linear','extrap');
                else  % pt_tracesY available
                    if ~isempty([pt_tracesY0{k}])  % pt_tracesY0 values available
                        rtmp = get(pt_tracesY0{k});
                        rtmp = rtmp(extrapoints);
                    else  % if pt_tracesY0 values not available
                        rtmp = interp1(pt_traces_Xdata{k},pt_traces_Ydata{k},extrapoints,'linear','extrap');
                    end  % if ~isempty([pt_tracesY0{k}])...
                end  % if ~exist(pt_tracesY0) || isempty([pt_tracesY0{k}])...
                Y(k) = rtmp(1);
            end  % if ~isempty(itmp)
            
            % find ending position
            Xe(k) = ceil(max(tmBoxes(:,1)+tmBoxes(:,3)));  % maximum x value available
            itmp = find(pt_traces_Xdata{k}(:)==Xe(k));
            if ~isempty(itmp)  % selected point within trace-zero line domain
                Ye(k) = pt_traces_Ydata{k}(itmp);
            else  % selected point outside of trace-zero line domain
                % extend y value
                extrapoints = pt_traces_Xdata{k}(end)+1:Xe(k);
                if ~iscell(pt_tracesY0) || isempty(pt_tracesY0) % reference trace Y values do not exist
                    rtmp = interp1(pt_traces_Xdata{k},pt_traces_Ydata{k},extrapoints,'linear','extrap');
                else  % pt_tracesY0 available
                    if ~isempty([pt_tracesY0{k}])  % pt_tracesY0{k} values available
                        rtmp = get(pt_tracesY0{k});
                        rtmp = rtmp(extrapoints);
                    else
                        rtmp = interp1(pt_traces_Xdata{k},pt_traces_Ydata{k},extrapoints,'linear','extrap');
                    end  % if ~isempty([pt_tracesY0{k}])...
                end  % if ~exist(pt_tracesY) || isempty([pt_tracesY{k}])...
                Ye(k) = rtmp(end);
            end  % if ~isempty(itmp)
        else  % trace objects do not exist
            
            X(k) = pt_traces_Xdata{k}(1);
            Y(k) = pt_traces_Ydata{k}(1);
            
            Xe(k) = pt_traces_Xdata{k}(end);
            Ye(k) = pt_traces_Ydata{k}(end);
        end  % if ~isempty(indx)
    end  % for k = 1:length(pt_traces)
    
else  % structure S containing objects exist
    delete(pt_start_time);  % clean up lines
    delete(pt_end_time);
    
    % get range above and below the zero-trace line
%    over_under = str2num(get(H.edit_offset,'string'));  
    over_under = str2num(H.edit_offset);  
    
    fX=ftrend(cell2mat(cellfun(@(v) v(1),get(pt_traces,'Xdata'),'UniformOutput',false)));
    rtmpY=cell2mat(cellfun(@(v) v(1),get(pt_traces,'Ydata'),'UniformOutput',false));
    meanOffset=mode(abs(diff(rtmpY)));

    % obtain time mark/trace information
    itrace = find([S.ID]==0|[S.ID]==2);  % main trace
    itm = find([S.ID]==1|[S.ID]==3);  % time marks
    
    % get boxes around these objects
    traceBoxes = cell2mat({S(itrace).BoundingBox}');  % box around trace
    ltmExist = true;
    if ~isempty(itm)
        tmBoxes = cell2mat({S(itm).BoundingBox}');  % box around time marks
    else
        ltmExist = false;
        tmBoxes = [];  % no time marks
    end  % if ~isempty(itm)
    
    ltm = strcmpi(get(H.timemarkpos,'state'),'off');  % true if time marks are above
    
    % get number of eligible traces to consider
    if ltm % if time marks are above
        npts_traces_togo = length(pt_traces);
    else  % time marks below
        npts_traces_togo = length(pt_traces)-1;
    end  % if ltm
    
    % default (initialize to zero)
    X = zeros(length(pt_traces),1);
    Y = zeros(length(pt_traces),1);
    Xe = X;
    Ye = Y;
    
    % separate X and Y values
    pt_traces_Xdata = get(pt_traces,'Xdata');
    pt_traces_Ydata = get(pt_traces,'Ydata');
    
    for i = 1:npts_traces_togo  % loop over eligible traces
        % find trace boxes corresponding to the current trace
        zerotrace_i = rtmpY(i)-fX(i)+ftrend(traceBoxes(:,1)+traceBoxes(:,3)/2);
        itrace_i = traceBoxes(:,2)+traceBoxes(:,4)/2 > over_under(1)*meanOffset+zerotrace_i & ...
            traceBoxes(:,2) + traceBoxes(:,4)/2 < over_under(2)*meanOffset+zerotrace_i;
        
        % find time mark boxes corresponding to the current trace
        ic = [];
        if ltmExist  % if there are time marks
            if ltm  % time marks are above
                ic= tmBoxes(:,2)+tmBoxes(:,4)/2<=pt_traces_Ydata{i}(1)+ftrend(tmBoxes(:,1)) &...
                    tmBoxes(:,1)>=pt_traces_Xdata{i}(1) & tmBoxes(:,1)+tmBoxes(:,3)<=pt_traces_Xdata{i}(end);
            else  % time marks are below
                ic= tmBoxes(:,2)+tmBoxes(:,4)/2<=pt_traces_Ydata{i+1}(1)+ftrend(tmBoxes(:,1)) &...
                    tmBoxes(:,1)>=pt_traces_Xdata{i}(1) & tmBoxes(:,1)+tmBoxes(:,3)<=pt_traces_Xdata{i}(end);
            end  % if ltm...
        end % if ltmExist
        
        rpos = [nan nan];  % initial position information (trace start)
        rpose = rpos;  % initial position (trace end)
%         ypos = rpos;
%         ypose = rpose;
        if (numel(find(itrace_i))>1)  % there are trace boxes availalbe
            [rtmp itmp] = min(traceBoxes(itrace_i,1));  % x position of the boxes
            rpos(1) = rtmp(1);  % minimum starting position
%             idx = find(itrace_i);
%             ypos(1) = traceBoxes(idx(itmp(1)),2)+traceBoxes(idx(itmp(1)),4)/2;  % y position
            [rtmp itmp] = max(traceBoxes(itrace_i,1)+traceBoxes(itrace_i,3));
            rpose(1) = rtmp(1);
%             ypose(1) = traceBoxes(idx(itmp(1)),2)+traceBoxes(idx(itmp(1)),4)/2;  % y position
        elseif (numel(find(itrace_i))>0)  % there is only one trace box available
            rpos(1) = traceBoxes(itrace_i,1);
            rpose(1) = traceBoxes(itrace_i,1)+traceBoxes(itrace_i,3);
        end  % if (numel(itrace_i)>0)
        if (numel(find(ic))>1)  % there are time marks available
            [rtmp itmp] = min(tmBoxes(ic,1));  % x position of the time marks
            rpos(2) = rtmp(1);  % minimum starting position
%             idx = find(ic);
%             ypos(2) = tmBoxes(idx(itmp(1)),2)+tmBoxes(idx(itmp(1)),4)/2;  % y position
            [rtmp itmp] = max(tmBoxes(ic,1)+tmBoxes(ic,3));
            rpose(2) = rtmp(1);
%             ypose(2) = tmBoxes(idx(itmp(1)),2)+tmBoxes(idx(itmp(1)),4)/2;  % y position
        elseif (numel(find(ic))>0)  % there is only one time mark
            rpos(2) = tmBoxes(ic,1);
            rpose(2) = tmBoxes(ic,1)+tmBoxes(ic,3);
        end  % if (numel(ic)>0)
        if numel(find(~isnan(rpos)))>0  % finite value entry available
            [~, itmp] = min(rpos);
            X(i) = floor(min(rpos));
%             Y(i) = ypos(itmp);
            itmp = find(pt_traces_Xdata{i}(:)==X(i));
            if ~isempty(itmp)  % selected point within trace-zero line domain
                Y(i) = pt_traces_Ydata{i}(itmp);
            else  % selected point outside of trace-zero line domain
                % extend y value
                extrapoints = X(i):pt_traces_Xdata{i}(1)-1;
                if ~iscell(pt_tracesY0) || isempty(pt_tracesY0)  % reference trace Y values do not exist
                    rtmp = interp1(pt_traces_Xdata{i},pt_traces_Ydata{i},extrapoints,'linear','extrap');
                else  % pt_tracesY0 available
                    if ~isempty([pt_tracesY0{i}])
                        rtmp = get(pt_tracesY0{i});
                        rtmp = rtmp(extrapoints);
                    else
                        rtmp = interp1(pt_traces_Xdata{i},pt_traces_Ydata{i},extrapoints,'linear','extrap');
                    end  % if ~isempty([pt_tracesY0{i}])
                end  % if ~exist(pt_tracesY) || isempty([pt_tracesY{k}])...
                Y(i) = rtmp(1);
            end  % if ~isempty(itmp)
        else  % neither trace nor time mark can be found
            X(i) = pt_traces_Xdata{i}(1);
            Y(i) = pt_traces_Ydata{i}(1);
        end  % if (numel(rpos(find(~isnan(rpos)))) > 0
        if numel(find(~isnan(rpose)))>0  % finite value entry available
            [~, itmp] = max(rpos);
            Xe(i) = ceil(max(rpose));
%             Ye(i) = ypose(itmp);
            itmp = find(pt_traces_Xdata{i}(:)==Xe(i));
            if ~isempty(itmp)  % selected point within trace-zero line domain
                Ye(i) = pt_traces_Ydata{i}(itmp);
            else  % selected point outside of trace-zero line domain
                %extrapolate
                extrapoints = pt_traces_Xdata{i}(end)+1:Xe(i);
                if ~iscell(pt_tracesY0) || isempty(pt_tracesY0)  % reference trace Y values do not exist
                    rtmp = interp1(pt_traces_Xdata{i},pt_traces_Ydata{i},extrapoints,'linear','extrap');
                else  % pt_tracesY available
                    if ~isempty([pt_tracesY0{k}])
                        rtmp = get(pt_tracesY0{k});
                        rtmp = rtmp(extrapoints);
                    else
                        rtmp = interp1(pt_traces_Xdata{i},pt_traces_Ydata{i},extrapoints,'linear','extrap');
                    end  % if ~isempty([pt_tracesY0{k}])...
                end  % if ~exist(pt_tracesY) || isempty([pt_tracesY{k}])...
                Ye(i) = rtmp(end);
            end  % if ~isempty(itmp)
        else  % neither trace nor time mark can be found
            Xe(i) = pt_traces_Xdata{i}(end);
            Ye(i) = pt_traces_Ydata{i}(end);
        end  % if (numel(find(~isnan(rpose)))>0
        
        % remove already processed boxes
        traceBoxes(itrace_i,:) = [];
        tmBoxes(ic,:) = [];

    end  % for i = 1:npts_traces_togo
    
    if ~ltm  % if time marks are below - take care of the bottommost trace
        
        i = npts_traces_togo + 1;  % last trace number
        
        % find trace boxes corresponding to the current trace
        zerotrace_i = rtmpY(i)-fX(i)+ftrend(traceBoxes(:,1)+traceBoxes(:,3)/2);
        itrace_i = traceBoxes(:,2)+traceBoxes(:,4)/2 > over_under(1)*meanOffset+zerotrace_i & ...
            traceBoxes(:,2) + traceBoxes(:,4)/2 < over_under(2)*meanOffset+zerotrace_i;
        
        rpos = [nan nan];  % initial position information
        rpose = [nan nan];
        if (numel(itrace_i)>0)  % there are trace boxes availalbe
            rtmp = min(traceBoxes(itrace_i,1));  % x position of the boxes
            rpos(1) = rtmp(1);  % minimum starting position
            rtmp = max(traceBoxes(itrace_i,1)+traceBoxes(itrace_i,3));
            rpose(1) = rtmp(1);
        end  % if (numel(itrace_i)>0)
        if (size(tmBoxes,1)>0)  % there are time marks available
            rtmp = min(tmBoxes(:,1));  % x position of the time marks
            rpos(2) = rtmp(1);  % minimum starting position
            rtmp = max(tmBoxes(:,1)+tmBoxes(:,3));
            rpose(2) = rtmp(1);
        end  % if (numel(ic)>0)
        if numel(find(~isnan(rpos)))>0  % finite value entry available
            X(i) = floor(min(rpos));
            itmp = find(pt_traces_Xdata{i}(:)==X(i));
            Y(i) = pt_traces_Ydata{i}(itmp);
        else  % neither trace nor time mark can be found
            X(i) = pt_traces_Xdata{i}(1);
            Y(i) = pt_traces_Ydata{i}(1);
        end  % if (numel(rpos(find(~isnan(rpos)))) > 0
        if numel(find(~isnan(rpose)))>0  % finite value entry available
            Xe(i) = ceil(max(rpose));
            itmp = find(pt_traces_Xdata{i}(:)==Xe(i));
            Ye(i) = pt_traces_Ydata{i}(itmp);
        else  % neither trace nor time mark can be found
            Xe(i) = pt_traces_Xdata{i}(end);
            Ye(i) = pt_traces_Ydata{i}(end);
        end  % if (numel(rpos(find(~isnan(rpos)))) > 0
        
    end  % if ~ltm
    
end


% delete(pt_start_time);
hpoly=impoly(H.ax1,[X(:),Y(:)],'Closed',false);
hpoly.Deletable = false;
pos=round(wait(hpoly));
delete(hpoly);



% % pt_tracesX=[];
% % pt_tracesY=[];
% % load('to_delete_temp.mat','pt_tracesX','pt_tracesY')
% %
% % pos(:,1)=pt_tracesX; pos(:,2)=pt_tracesY;
% %
hold(H.ax1,'on');
pt_start_time=plot(pos(:,1),pos(:,2),'>g-');
hold(H.ax1,'off');

% update end time
% delete(pt_end_time);
hpoly=impoly(H.ax1,[Xe(:),Ye(:)],'Closed',false);
hpoly.Deletable = false;
pose=round(wait(hpoly));
delete(hpoly);

hold(H.ax1,'on');
pt_end_time=plot(pose(:,1),pose(:,2),'<r-');
hold(H.ax1,'off');



for i=1:length(pt_traces)  % update pt_traces, ptrace, and ptraceSTD
    X=get(pt_traces(i),'Xdata');
    Y=get(pt_traces(i),'Ydata');
    
    if pos(i,1)>X(1) % truncate
        idok= X>pos(i,1)-1-eps;
        set(pt_traces(i),'Xdata',X(idok),'Ydata',Y(idok));
    elseif pos(i,1)<X(1) % extrapolate
        extrapoints=pos(i,1):X(1)-1;
        if ~iscell(pt_tracesY0) || isempty(pt_tracesY0) % reference trace Y values do not exist
            rtmp = interp1(X,Y,extrapoints,'linear','extrap');
        else  % pt_tracesY available
            if ~isempty([pt_tracesY0{i}])
                rtmp = get(pt_tracesY0{i});
                rtmp = rtmp(extrapoints);
            else
                rtmp = interp1(X,Y,extrapoints,'linear','extrap');
            end  % if ~isempty([pt_tracesY0{i}])
        end  % if ~exist(pt_tracesY) || isempty([pt_tracesY{k}])...
        set(pt_traces(i),'Xdata',[extrapoints, X],'Ydata',[rtmp, Y]);
    end

    X=get(pt_traces(i),'Xdata');
    Y=get(pt_traces(i),'Ydata');
    
    if pose(i,1)<X(end) % truncate
        idok=X<pose(i,1)+eps+1;
        set(pt_traces(i),'Xdata',X(idok),'Ydata',Y(idok));
    elseif pose(i,1)>X(end) % extrapolate
        extrapoints=X(end)+1:pose(i,1);
        if ~iscell(pt_tracesY0) || isempty(pt_tracesY0)  % reference trace Y values do not exist
            rtmp = interp1(X,Y,extrapoints,'linear','extrap');
        else  % pt_tracesY available
            if ~isempty([pt_tracesY0{i}])
                rtmp = get(pt_tracesY0{i});
                rtmp = rtmp(extrapoints);
            else
                rtmp = interp1(X,Y,extrapoints,'linear','extrap');
            end  % if ~isempty([pt_tracesY0{i}])...
        end  % if ~exist(pt_tracesY) || isempty([pt_tracesY{k}])...
        set(pt_traces(i),'Xdata',[X extrapoints],'Ydata',[Y, rtmp]);
    end
    
end
drawnow;

set(H.pbedit_start,'Enable','on');
set(H.togle_timebounds_visibility,'Enable','on')
set(H.togle_timebounds_visibility,'value',1);
togle_timebounds_visibility;
set(H.Create_Time,'Enable','on');
set(H.edit_t0,'Enable','on');
tmLength = str2num(get(H.edit_tick_xlength,'string'));  % time mark length
if ~isempty(tmLength) & (tmLength>0)  % time marks exist
    set(H.cb_timeEnd,'enable','on');
    set(H.mark_1st,'enable','on');
    set(H.mark_last,'enable','on');
    ntm = getntm(tmLength);
    if ~isempty(ntm)  % if number of estimated time marks exists
        set(H.edit_num_of_ticks,'string',num2str(ntm),'enable','on');
    end  % if ~isempty(ntm)
end  % if ~isempty(rtmp) & (rtmp > 0)


    function nout = getntm(tmLength)  % find number of likely time marks
%         global pt_traces
        
        nout = [];
        traceLength = str2double(get(H.edit_DP,'string'));
        if ~isnan(traceLength) & (traceLength > tmLength)
            posS=cell2mat(cellfun(@(v) v(1),get(pt_traces,'Xdata'),'UniformOutput',false));
            posE=cell2mat(cellfun(@(v) v(end),get(pt_traces,'Xdata'),'UniformOutput',false));
            rtmp = max(posE)-min(posS);
            nout = round(rtmp/(tmLength+traceLength));
        end  % if ~isnan(traceLength) & (traceLength > tmLength)
    end

end


function togle_timebounds_visibility(hObject, eventdata)
global H pt_start_time pt_end_time p_tick_1st p_tick_last hresult_time_marks

if get(H.togle_timebounds_visibility,'value')
    if ishandle(pt_start_time)
        set(pt_start_time,'visible','on')
    end
    if  ishandle(pt_end_time)
        set(pt_end_time,'visible','on')
    end
    if ishandle(p_tick_1st)
        set(p_tick_1st,'visible','on')
    end
    if ishandle(p_tick_last)
        set(p_tick_last,'visible','on')
    end
    if ishandle(hresult_time_marks)
        set(hresult_time_marks,'visible','on')
    end
    
    if get(H.find_gaps,'value')  % gap objects are on
        set(H.find_gaps,'value',0);  % reset
        find_gaps(H.find_gaps);  % remove objects
    end  % if (H.find_gaps,'value')

    if get(H.find_vert,'value')  % gap objects are on
        set(H.find_vert,'value',0);  % reset
        find_gaps(H.find_vert);  % remove objects
    end  % if (H.find_gaps,'value')
else
    if ishandle(pt_start_time)
        set(pt_start_time,'visible','off')
    end
    if  ishandle(pt_end_time)
        set(pt_end_time,'visible','off')
    end
    if ishandle(p_tick_1st)
        set(p_tick_1st,'visible','off')
    end
    if ishandle(p_tick_last)
        set(p_tick_last,'visible','off')
    end
    
    if ishandle(hresult_time_marks)
        set(hresult_time_marks,'visible','off')
    end
    
end


end



%%
function correct_classification(hObject,eventdata)
global H I0 S BW 
global I_RGB ax HIMRGB

set (H.f1, 'WindowButtonMotionFcn', []);

leveldef = 10;  % default bw threshold value

% check to make sure no other Edit Classification windows are open
if isappdata(H.f1,'Hfdistinguish')
    Hfdistinguish = getappdata(H.f1,'Hfdistinguish');
    if isvalid(Hfdistinguish)  % if the figure window still exists
        delete(Hfdistinguish);
    end  % if any(ishandle(Hfdistinguish))
    % rmappdata(H.f1,'Hfdistinguish');  % remove data
end  % if isappdata(H.f1,'Hfdistinguish')

level=str2double(get(H.edit_luminance,'String'));
% check to make sure this is a valid value (needs to be between 1 and 99)
lok = false;  % lok becomes true if the value is valid
if ~isnan(level)  % numerical value
    if level>0 & level<100  % valid value
        lok = true;
    end  % if level>0 & level<100
end  % if ~isnan
if ~lok  % if invalid value -> set back to default
    disp(['Warning (correct_classification): threshold value = ', num2str(level)]);
    disp(['Setting back to the default value: ', num2str(leveldef)]);
    level = leveldef;
    % set the original value back to default
    set(H.edit_luminance,'string',num2str(leveldef));
end  % if ~lok
tick_length=str2double(get(H.edit_tick_xlength,'String'));
minPix = 5;  % minimum number of pixels needed to be considered as an object
%%

%S = [];
if exist('S')==0 | isempty(S)  % structure S doesn't exist or is empty
    %disp('in initial classification mode');
    % needs to set up binary image and obtain objects for the first time
    hmess = dialog('Name','Classification');
    cstr = sprintf('%s\n\n%s','Calculating Classification',...
        'Please be patient as this may take some time.  This window will close when initial classification completes.');
    ctxt = uicontrol('parent',hmess,'style','text','units','normalized',...
        'position',[0.1 0.3 0.8 0.4],'string', cstr,...
        'horizontalAlignment','center','fontSize',18);
    drawnow
   
    BW=im2bw(I0,level/100);  % binary image
    
%     output_message{end}=sprintf('Removing objects with less than %d pixels',minPix);
%     set(H.output_mess,'String',output_message,'Value',length(output_message)); drawnow
    BW = bwareaopen(BW,round(minPix)); %removes all connected components (objects) that have fewer than P pixels
    
%     output_message{end+1}='Classifying binary image...';
%     set(H.output_mess,'String',output_message,'Value',length(output_message)); drawnow
    
    Classification_Recalculate;
    delete(hmess);
end  % if exist('S')

Hfdistinguish=figure ('WindowStyle','normal','menu','none','toolbar','Figure','Color','w','Name','View-Edit classification','NumberTitle','off',...
    'Units','Normalized','Position',[0.1 0.4 0.85 0.5],'Resizefcn','');
set(Hfdistinguish,'DeleteFcn',@deleteFigure);
ax=axes('Units','Normalized','Position',[0.06 0.08 0.89 0.89],...
    'box','off','Parent',Hfdistinguish,'Visible','off');

% modify tool bar
tempbar= findall(findall(Hfdistinguish,'Type','uitoolbar'));
delete(tempbar([2:7 8 9 13 14 15 16 17]));
% % Reorder tools
% set(temp(1),'Children',[temp(4:end-1);temp(2:3);temp(end)])

%htools.datacursor = tempbar(8);
htools.Pan = tempbar(10);
htools.zoomOut = tempbar(11);
htools.zoomIn = tempbar(12);
%htools.save = tempbar(15);

set(htools.zoomIn,'clickedcallback',{@new_zoom 1});
set(htools.zoomOut,'clickedcallback',{@new_zoom 2});
set(htools.Pan,'clickedcallback',{@new_zoom 3});

bColorScheme=uitoggletool(tempbar(1),'Tag','ChangeColor','Cdata',imread('icons/ColorScheme.png','BackgroundColor',[1 1 1]),...
    'Separator','on','TooltipString','Change color scheme',...
    'HandleVisibility','on','ClickedCallback',@changeColorScheme,'state','off');

htools.Color = bColorScheme;

% htools.image = uipushtool(tempbar(1),'Tag','Seperate','Cdata',imread('ClassUseOtherImage.png','BackgroundColor',[1 1 1]),...
%     'Separator','on','TooltipString','Load different grayscale image, of same size for classification purpose',...
%     'HandleVisibility','on','ClickedCallback',@BrowseClass);


bremove_along_path=uitoggletool(tempbar(1),'Tag','Separate','Cdata',imread('icons/seperateS.png','BackgroundColor',[1 1 1]),...
    'Separator','on','TooltipString','Remove pixels along the drawn path',...
    'HandleVisibility','on','ClickedCallback',@remove_along_path,'state','off');

% bdraw_along_path=uitoggletool(tempbar(1),'Tag','Seperate','Cdata',imread('drawS.png','BackgroundColor',[1 1 1]),...
%     'Separator','off','TooltipString','add pixels along the drawn path',...
%     'HandleVisibility','on','ClickedCallback',@draw_along_path,'state','off');
% 
bremove_bregion=uitoggletool(tempbar(1),'Tag','RemRegion','Cdata',imread('icons/removeS.png','BackgroundColor',[1 1 1]),...
    'Separator','off','TooltipString','Remove binary region',...
    'HandleVisibility','on','ClickedCallback',@remove_bregion,'state','off');

htools.cutLine = bremove_along_path;
% htools.addPoints = bdraw_along_path;
htools.removeRegion = bremove_bregion;

bundo_last_action=uipushtool(tempbar(1),'Tag','Undo last remove or add','Cdata',imread('icons/undo.jpeg'),...
    'Separator','on','TooltipString','Undo last remove path action',...
    'HandleVisibility','on','Enable','off','ClickedCallback',@undo_last_action);

htools.undo = bundo_last_action;

htools.reject = uipushtool(tempbar(1),'Tag','s2Reject','Cdata',imread('icons/reject.png','BackgroundColor',[1 1 1]),...
    'Separator','off','TooltipString','Select object to classify as rejected',...
    'HandleVisibility','on','ClickedCallback',@eval_mark,'Enable','on');
htools.timeMark = uipushtool(tempbar(1),'Tag','s2Timemark','Cdata',imread('icons/timemark.png','BackgroundColor',[1 1 1]),...
    'Separator','off','TooltipString','Select object to classify as timemark',...
    'HandleVisibility','on','ClickedCallback',@eval_mark,'Enable','on');
htools.trace = uipushtool(tempbar(1),'Tag','s2Trace','Cdata',imread('icons/trace.png','BackgroundColor',[1 1 1]),...
    'Separator','off','TooltipString','Select object to classify as trace',...
    'HandleVisibility','on','ClickedCallback',@eval_mark,'Enable','on');

htools.whole = uipushtool(tempbar(1),'Tag','Show whole seismogram','Cdata',imread('icons/whole_seismogram.png','BackgroundColor',[1 1 1]),...
    'Separator','off','TooltipString','Show whole seismogram',...
    'HandleVisibility','on','ClickedCallback',@zoom2seismogramCLASS);

% reorder icons
htmp = [htools.whole; htools.Pan; htools.zoomOut; htools.zoomIn; 
    htools.trace; htools.timeMark; htools.reject; 
    htools.undo; htools.removeRegion; htools.cutLine; 
    htools.Color];
set(tempbar(1),'children',htmp);

htools.zoomRegion = uipushtool(tempbar(1),'Tag','s2Trace','Cdata',imread('icons/zoom2region.png','BackgroundColor',[1 1 1]),...
    'Separator','on','TooltipString','Zoom DigitSeis main axis to the region displayed in the classification figure',...
    'HandleVisibility','on','ClickedCallback',@zoom2region,'Enable','on');



% bReAssign_Objects_to_Traces=uipushtool(tempbar(1),'Tag','s2Trace','Cdata',imread('AssignObjectsToTraces.png','BackgroundColor',[1 1 1]),...
%     'Separator','on','TooltipString','Assign objects to traces',...
%     'HandleVisibility','on','ClickedCallback',@ReAssign_Objects_to_Traces,'Enable','on');
% 
% htools.assignObjects = bReAssign_Objects_to_Traces;

hToolbar = findall(Hfdistinguish,'tag','FigureToolBar');


drawnow % Needed before jToolbar line
% since Matlab only allows only uipushtools & uitoggletools to toolbars I'm using a Java component.
jToolbar = get(get(hToolbar,'JavaContainer'),'ComponentPeer');
drawnow

jModel = javax.swing.SpinnerNumberModel(level,1,99,1);
jSpinner = javax.swing.JSpinner(jModel);
jSpinner =handle(jSpinner,'callbackproperties');
jSpinner.setToolTipText('Change binary threshold');
jSpinner.setSize(70,25);
jSpinner(1).setPreferredSize(java.awt.Dimension(70,25));
jSpinner.setMaximumSize(java.awt.Dimension(100,25));


% JCheckBox=javax.swing.JCheckBox();
% JCheckBox.setSize(30,25);
% hJCheckBox = handle(JCheckBox,'callbackproperties');
% hJCheckBox.ActionPerformedCallback = @Show_Object_assignments;
% 
Jbox=javax.swing.Box.createHorizontalGlue();
Jbox.setSize(300,25);


temp=zeros(16,16,3,'uint8');
temp(:,:,1)=255;
uipushtool(tempbar(1),'Tag','recalcCLASS','Cdata',temp,...
    'Separator','on','TooltipString','Recalculate Classification',...
    'HandleVisibility','on','ClickedCallback',@recalculate_callback);

%Create button image & Button
temp=zeros([16,16,3],'uint8'); rng(1980) ;
for ii=1:10
    temp(randi([2,15],6,1),randi([2 15],1),1)=255;
end
for ii=2:16, temp(ii-1:ii,ii-1:ii,:)=255; temp([ii-1:ii],17-[ii-1:ii],:)=255; end
uipushtool(tempbar(1),'Tag','DeleteReds','Cdata',temp,...
    'Separator','off','TooltipString','Delete noise objects (saves memory)',...
    'HandleVisibility','on','Enable','on','ClickedCallback',@deleteRedObjects);


% Add here 1 for each new button.
n = numel(fieldnames(htools));
jToolbar.add(Jbox,n+4);
%jToolbar.add(JCheckBox,27);
%jToolbar.add(jSpinner,26);
jToolbar.add(jSpinner,n+7);


%@bShow_Object_assignments

jToolbar.repaint;
jToolbar.revalidate;


htext=text(mean(get(ax,'xlim')),mean(get(ax,'ylim')), 'please wait',...
    'HorizontalAlignment','Center','Parent',ax,'FontSize',32,'BackgroundColor','w');
drawnow


%%
% retrieve some info for classification
[rows,cols]=size(BW);

G=zeros(rows,cols,'uint8'); R=G; B=G;

itickL=([S.ID]==1|[S.ID]==3);
itraceL=([S.ID]==0|[S.ID]==2);

irej=cell2mat({S([S.ID]==-1).PixelIdxList}');
itick=cell2mat({S(itickL).PixelIdxList}');
itrace=cell2mat({S(itraceL).PixelIdxList}');


if strcmpi(get(bColorScheme,'state'),'off')
    R([irej;itrace])=255;
    G([itick; itrace])=255;
    B(itrace)=255;
else
    R([irej;itrace])=255;
    B([itick;itrace])=255;
    G(itick)=220;
    G(itrace)=255;
end

I_RGB=zeros(rows,cols,3,'uint8');
I_RGB(:,:,1)=R; I_RGB(:,:,2)=G; I_RGB(:,:,3)=B;


HIMRGB=image(I_RGB);
xlim = get(ax,'xlim');
ylim = get(ax,'ylim');
axesCorig = [xlim ylim];
setappdata(ax,'axesCorig',axesCorig);
setappdata(ax,'zoom_zoomOrigAxesLimits',axesCorig);

drawnow
delete(htext)

BW0=BW;
S0=S;


mult_bck_color=[0.7 0.7 0.7]; % NOT WHITE color for textbox when multiple assignments.
iwithnuminS=[];
imultiple_inS=[];
hObjAssignmentLabel=[];
if isfield(S,'TraceNum')
    ineg= cellfun(@isempty,{S.TraceNum});
    [S(ineg).TraceNum]=deal(-1);
  %  Update_AssignementLabelsfromS();
else  % if TraceNum field doesn't exist in S
    [S(:).TraceNum] = deal(-1);
end

setappdata(H.f1,'Hfdistinguish',Hfdistinguish);  % save figure information


    function new_zoom(hobj,eventdata,itype)
        
        %disp('in new zoom function')
%         ctmp1 = get(bremove_bregion,'State');
%         ctmp2 = get(bremove_along_path,'state');
        
        if itype==1  % zoom in
            putdowntextN('zoomin',hobj);
        elseif itype==2  % zoom out
            putdowntextN('zoomout',hobj);
        elseif itype==3  % pan
            putdowntextN('pan',hobj);
        end
        
%         set(bremove_bregion,'state',ctmp1);
%         set(bremove_along_path,'state',ctmp2);
    end



    function  ReAssign_Objects_to_Traces(hObject, eventdata)
        global pt_traces ftrend
        
        try
            delete(hObjAssignmentLabel)
        end
        
%         over_under=str2num(get(H.edit_offset,'string')); %#ok<ST2NM>
        over_under=str2num(H.edit_offset); %#ok<ST2NM>
        [rows,cols]=size(BW);
        
        fX=ftrend(cell2mat(cellfun(@(v) v(1),get(pt_traces,'Xdata'),'UniformOutput',false)));
        Y=cell2mat(cellfun(@(v) v(1),get(pt_traces,'Ydata'),'UniformOutput',false));
        meanOffset=mode(abs(diff(Y)));
        
        Bbox=cell2mat({S.BoundingBox}');
        itickL=([S.ID]==1|[S.ID]==3);
        itraceL=([S.ID]==0|[S.ID]==2);
        
        C=[Bbox(:,2)+Bbox(:,4)/2,Bbox(:,1)+Bbox(:,3)/2, [1:size(Bbox,1)]']; % the last column is just the initial index.
        Ctick=C(itickL,:);
        C=C(itraceL,:);
        
        [S.TraceNum]=deal(-1);
        for i=1:numel(pt_traces)
            
            % prepare stripes
            zerotrace_i= Y(i)-fX(i)+ftrend(C(:,2));
            itrace_i=C(:,1) > over_under(1)*meanOffset+zerotrace_i &...
                C(:,1) < over_under(2)*meanOffset+zerotrace_i;
            zerotick_i=Y(i)-fX(i)+ftrend(Ctick(:,2));
            
            if  strcmpi(get(H.timemarkpos,'State'),'off') % time marks above the trace
                itick_i=Ctick(:,1) > -abs(over_under(1))*meanOffset+zerotick_i-meanOffset/2 &...
                    Ctick(:,1) < zerotick_i;
            elseif strcmpi(get(H.timemarkpos,'State'),'on')  % time marks bellow the trace
                itick_i=Ctick(:,1) < abs(over_under(2))*meanOffset+zerotick_i+meanOffset/2 &...
                    Ctick(:,1) > zerotick_i;
            end
            
            [S(C(itrace_i,end)).TraceNum]=deal(i);
            [S(Ctick(itick_i,end)).TraceNum]=deal(i);
        end
        
        
        cm=lines(numel(pt_traces))+0.1;
        cm(cm>1)=1;
        iwithnuminS=cellfun(@(x)(x(1)>0),{S.TraceNum}); % find all with 1st TraceNum > 0
        if any(imultiple_inS)
            imultiple_inS=cellfun(@(x) numel(x)>1,{S.TraceNum}); % find  the ones with multiple numbers in TraceNum
        end
        
        
        hObjAssignmentLabel=text(Bbox(iwithnuminS,1)+Bbox(iwithnuminS,3)/2,Bbox(iwithnuminS,2)+Bbox(iwithnuminS,4)/2, num2str([S(iwithnuminS).TraceNum]'),...
            'Color','k','FontUnits','Pixels','FontSize',7,'Interpreter','none','Visible','on','Margin',1,'ButtonDownFcn',@Correct_Object_Trace_Assignment);
        tempTN=[S(iwithnuminS).TraceNum]';
        for i=1:length(hObjAssignmentLabel)
            hObjAssignmentLabel(i).BackgroundColor= cm(tempTN(i),:);
        end
        if any(imultiple_inS)
            hObjAssignmentLabel(imultiple_inS).BackgroundColor=mult_bck_color; % paint all with multiple assignments black
        end
        
        Show_Object_assignments;
        drawnow
        
    end


    function Update_AssignementLabelsfromS(hObject,eventdata)
        global pt_traces
        
        try
            delete(hObjAssignmentLabel)
        end
        
        Bbox=cell2mat({S.BoundingBox}');
        %         iwithnum= ~cellfun(@(x)(x>0),{S.TraceNum});
        %         iwithnuminS= find(iwithnum);
        
        [S(~(itickL | itraceL)).TraceNum]=deal(-1);
        
        cm=lines(numel(pt_traces))+0.1;
        cm(cm>1)=1;
        
        
        iwithnuminS=cellfun(@(x)(x(1)>0),{S.TraceNum}); % find all with 1st TraceNum > 0
        imultiple_inS=cellfun(@(x) numel(x)>1,{S.TraceNum}); % find  the ones with multiple numbers in TraceNum
        
        
        hObjAssignmentLabel=text(Bbox(iwithnuminS,1)+Bbox(iwithnuminS,3)/2,Bbox(iwithnuminS,2)+Bbox(iwithnuminS,4)/2, num2str([S(iwithnuminS).TraceNum]'),...
            'Color','k','FontUnits','Pixels','FontSize',7,'Interpreter','none','Visible','on','Margin',1,'ButtonDownFcn',@Correct_Object_Trace_Assignment);
        
        tempTN=[S(iwithnuminS).TraceNum]';
        
        for i=1:length(hObjAssignmentLabel)
            hObjAssignmentLabel(i).BackgroundColor= cm(tempTN(i),:);
        end
        if any(imultiple_inS)
            hObjAssignmentLabel(imultiple_inS).BackgroundColor=mult_bck_color; % paint all with multiple assignments black
        end
        
        Show_Object_assignments;
        drawnow
        
    end



    function  Correct_Object_Trace_Assignment(hObject, eventdata)
        global pt_traces
        
        oldcolor=hObject.BackgroundColor;
        oldassignment=str2num(hObject.String); %#ok<ST2NM>
        oldmargin=hObject.Margin;
        
        set(hObject,'Margin',4,'BackgroundColor',[1 1 1])
        drawnow;
        hObject.Editing='on';
        waitfor(hObject,'Editing','off')
        try
            newassignment= round(str2num(hObject.String)); %#ok<ST2NM>
        catch
            warndlg(sprintf('Invalid assignment format it should be either an integer or in the form [integer1 integer2 ...]'));
            set(hObject,'Margin',oldmargin,'BackgroundColor',oldcolor,'String',num2str(oldassignment));
            return
        end
        
        if all(newassignment>=1) && all(newassignment<=length(pt_traces))
            iobj=findobj(hObjAssignmentLabel,'Margin',4,'BackgroundColor',[1 1 1]);
            if isempty(iobj)
                return
            end
            i=find(iobj==hObjAssignmentLabel,1);
            index=find(iwithnuminS);
            S(index(i)).TraceNum=newassignment;
            
            if numel(newassignment)==1
                iother=find([S(iwithnuminS).TraceNum]==S(index(i)).TraceNum,1);% this will return empty or maximum 1 result.
                set(hObject,'Margin',oldmargin,'BackgroundColor',hObjAssignmentLabel(iother).BackgroundColor);
            else
                set(hObject,'Margin',oldmargin,'BackgroundColor',mult_bck_color); % set color to mult_bck_color
            end
            drawnow;
        else
            warndlg(sprintf('You should enter integers between 1 and %d',length(pt_traces)));
            set(hObject,'Margin',oldmargin,'BackgroundColor',oldcolor,'String',num2str(oldassignment));
        end
        drawnow
    end




    function Show_Object_assignments(hObject, eventdata)
        
        if exist('JCheckBox')~=0  % if JCheckBox exists
       
        if ~isempty(hObjAssignmentLabel)
            if JCheckBox.isSelected
                set(hObjAssignmentLabel,'Visible','on')
            else
                set(hObjAssignmentLabel,'Visible','off')
            end
        else
            
        end
        end  % if exist('JCheckBox')~=0
    end





    function  changeColorScheme(hObject, ~)
        ht1=text(mean(get(ax,'xlim')),mean(get(ax,'ylim')), 'please wait',...
            'HorizontalAlignment','Center','Parent',ax,'FontSize',32,'BackgroundColor','w');
        drawnow;
        
        G=zeros(rows,cols,'uint8'); R=G; B=G;
        if strcmpi(get(bColorScheme,'state'),'off')
            R([irej;itrace])=255;
            G([itick; itrace])=255;
            B(itrace)=255;
        else
            R([irej;itrace])=255;
            B([itick;itrace])=255;
            G(itick)=220;
            G(itrace)=255;
        end
        I_RGB(:,:,1)=R; I_RGB(:,:,2)=G; I_RGB(:,:,3)=B;
        set(HIMRGB,'CDATA',I_RGB);
        delete(ht1);
        drawnow
        
    end


    function deleteRedObjects(hObject, eventdata)
        
        button = questdlg('Are you sure you want to delete all noise objects?\n This action cannot be undone','Image polarity','Yes','No','No');
        if strcmp(button,'No')
            return;
        end
        
        
        ht=text(mean(get(ax,'xlim')),mean(get(ax,'ylim')), 'Removing noise, please wait',...
            'HorizontalAlignment','Center','Parent',ax,'FontSize',32,'BackgroundColor','w');
        drawnow;
        
        % remove reds
        itoremove=[S.ID]==-1;
        if sum(itoremove)>0
            S([S.ID]==-1)=[];
            update_classification_plot;
        end
        delete(ht)
    end



    function zoom2seismogramCLASS(hObject, eventdata)
        updateSmallImage(0);
        set(ax,'Xlim',[1 size(BW,2)],'Ylim',[1,size(BW,1)])
        drawnow;
        axesCorig = [get(ax,'xlim') get(ax,'ylim')];
        setappdata(ax,'axesCorig',axesCorig);
%         Reset_MouseMotionMon;
    end

    function zoom2region(hObject, eventdata)
        set(H.ax1,'Xlim',get(ax,'Xlim'),'Ylim',get(ax,'Ylim'))
        drawnow;
%         Reset_MouseMotionMon;
    end


    function recalculate_callback(h,e)
        
        % check whether any of the zoom/pan icons is on - if it is, need to
        % reset the image
        if any(strcmpi(get([htools.Pan; htools.zoomOut; htools.zoomIn],'state'),'on'))
            xlim = get(ax,'xlim');
            ylim = get(ax,'ylim');
            axesCorig = [xlim ylim];
            setappdata(ax,'axesCorig',axesCorig);
            updateSmallImage(1);
            zoom off; pan off;
            %set([htools.Pan; htools.zoomOut; htools.zoomIn],'state','off');
        end  % if any of the three icons is on
        
        button = questdlg({'Are you sure you want to recalculate classification using the original image?';'Current classification will be lost!'},'Confirmation','Yes','No','No');
        if strcmpi(button,'No')
            return
        end
        
        tick_length=str2double(get(H.edit_tick_xlength,'String'));
        % make sure tick length is updated in case the user changes on the
        % main window

        ht1=text(mean(get(ax,'xlim')),mean(get(ax,'ylim')), 'please wait',...
            'HorizontalAlignment','Center','Parent',ax,'FontSize',32,'BackgroundColor','w');
        drawnow;
        
        %get(jSpinner,'Value')
        BW=im2bw(I0,get(jSpinner,'Value')/100);%bradleyPetros(I0,[256 256],get(jSpinner,'Value'));
        BW = bwareaopen(BW,round(minPix)); %removes all connected components (objects) that have fewer than minPix pixels
        
        Classification_Recalculate;
        update_classification_plot;
        
        delete(ht1);
        drawnow;
        
        % update threshold level in the main window
        set(H.edit_luminance,'string',num2str(get(jSpinner,'Value')));
        
    end

    function  Classification_Recalculate()%(hObject, eventdata)
        global tick_length_lim
        S=regionprops(BW,{'BoundingBox','PixelIdxList'});
        % add the id field 0 for normal trace, 1 for time mark, -1 for
        % rejected objects
        
        time_length=cell2mat({S.BoundingBox}');
        time_length=time_length(:,3)';% tale width;
        
        %idRej=time_length>tick_length+5 | time_length<tick_length-5; % normal trace or too short objects
        
        if tick_length>0
            idTick=abs(time_length-tick_length)<tick_length_lim*tick_length;
            idTrace=time_length-tick_length>tick_length_lim*tick_length;
        else
            idTick=[];
            idTrace=time_length+tick_length>tick_length_lim*(-tick_length);
        end
        
%         disp('in Classification_Recalculate');
%         disp(['# of Ticks: ',num2str(numel(find(idTick)))]);
%         disp(['# of Traces: ',num2str(numel(find(idTrace)))]);
        
        [S(:).ID] = deal(-1);
        [S(idTrace).ID]=deal(0);
        [S(idTick).ID]=deal(1);
        %idRej=cellfun(@(x) isempty(x),{S.ID});
        %[S(idRej).ID]=deal(-1);
        
        [S(:).TraceNum]=deal(-1);
        
    end




    function idx=Classification_UpdateADD(linearIdx)
        global tick_length_lim
        
        idx=linearIdx;
        
        PixelIdxList=cell2mat({S.PixelIdxList}');
        if isempty(intersect(linearIdx,PixelIdxList))
            
            % create temp binary image and detect connected objects
            [i,j] = ind2sub(size(BW),linearIdx);
            BoundingBox=[min(j)-0.5 min(i)-0.5 max(j)-min(j)+1 max(i)-min(i)+1];
            bwtemp=false(BoundingBox([4 3]));
            b0=floor(BoundingBox);
            [i1,j1]=ind2sub(size(BW),linearIdx);
            bwtemp(sub2ind(size(bwtemp),i1-b0(2),j1-b0(1)))=true;
            Stemp=regionprops(bwtemp,'BoundingBox','PixelList');
            
            nend=length(S);
            for i=1:length(Stemp)
                S(nend+i).PixelIdxList=...
                    sub2ind(size(BW),Stemp(i).PixelList(:,2)+b0(2),Stemp(i).PixelList(:,1)+b0(1));
                S(nend+i).BoundingBox=...
                    [Stemp(i).BoundingBox(1)+b0(1),...
                    Stemp(i).BoundingBox(2)+b0(2),...
                    Stemp(i).BoundingBox(3), Stemp(i).BoundingBox(4)];
                
                
                
                % clasiffy it
                if tick_length>0
                    if abs(S(nend+i).BoundingBox(3)-tick_length)<=tick_length_lim*tick_length;
                        S(nend+i).ID=1;
                    elseif S(nend+i).BoundingBox(3)-tick_length>tick_length_lim*tick_length;
                        S(nend+i).ID=0;
                    else
                        S(nend+i).ID=-1;
                    end
                else
                    if S(nend+i).BoundingBox(3)+tick_length>tick_length_lim*(-tick_length);
                        S(nend+i).ID=0;
                    else
                        S(nend+i).ID=-1;
                    end
                end
                
                S(nend+i).TraceNum = -1;
            end
            return
        else
            
            for iter=1:length(S)
                if ~isempty(intersect(linearIdx,S(iter).PixelIdxList))
                    temp_union=union(S(iter).PixelIdxList,linearIdx);
                    [i,j] = ind2sub(size(BW),temp_union);
                    
                    BoundingBox=[min(j)-0.5 min(i)-0.5 max(j)-min(j)+1 max(i)-min(i)+1];
                    
                    bwtemp=false(BoundingBox([4 3]));
                    b0=floor(BoundingBox);
                    
                    [i1,j1]=ind2sub(size(BW),temp_union);
                    bwtemp(sub2ind(size(bwtemp),i1-b0(2),j1-b0(1)))=true;
                    
                    Stemp=regionprops(bwtemp,'BoundingBox','PixelList');
                    
                    S(iter).PixelIdxList=...
                        sub2ind(size(BW),Stemp(1).PixelList(:,2)+b0(2),Stemp(1).PixelList(:,1)+b0(1));
                    S(iter).BoundingBox=...
                        [Stemp(1).BoundingBox(1)+b0(1),...
                        Stemp(1).BoundingBox(2)+b0(2),...
                        Stemp(1).BoundingBox(3), Stemp(1).BoundingBox(4)];
                    
                    if tick_length>0
                        if abs(S(iter).BoundingBox(3)-tick_length)<=tick_length_lim*tick_length;
                            S(iter).ID=1;
                        elseif S(iter).BoundingBox(3)-tick_length>tick_length_lim*tick_length;
                            S(iter).ID=0;
                        else
                            S(iter).ID=-1;
                        end
                    else
                        if S(iter).BoundingBox(3)+tick_length>tick_length_lim*(-tick_length);
                            S(iter).ID=0;
                        else
                            S(iter).ID=-1;
                        end
                    end
                    
                    S(iter).TraceNum = -1;
                    
                    nend=length(S);
                    
                    idx=[iter, nend+[1:length(Stemp)-1]];
                    
                    
                    for i=2:length(Stemp)
                        S(nend+i-1).PixelIdxList=...
                            sub2ind(size(BW),Stemp(i).PixelList(:,2)+b0(2),Stemp(i).PixelList(:,1)+b0(1));
                        S(nend+i-1).BoundingBox=...
                            [Stemp(i).BoundingBox(1)+b0(1),...
                            Stemp(i).BoundingBox(2)+b0(2),...
                            Stemp(i).BoundingBox(3), Stemp(i).BoundingBox(4)];
                        
                        if tick_length>0
                            if abs(S(nend+i-1).BoundingBox(3)-tick_length)<=tick_length_lim*tick_length;
                                S(nend+i-1).ID=1;
                            elseif S(nend+i-1).BoundingBox(3)-tick_length>tick_length_lim*tick_length;
                                S(nend+i-1).ID=0;
                            else
                                S(nend+i-1).ID=-1;
                            end
                        else
                            if S(nend+i-1).BoundingBox(3)+tick_length>tick_length_lim*(-tick_length);
                                S(nend+i-1).ID=0;
                            else
                                S(nend+i-1).ID=-1;
                            end
                        end
                        
                        S(nend+i-1).TraceNum = -1;
                    end
                    
                    return
                end
            end
        end
        
        
    end

    function res=check_if_inside(xpatch,ypatch,qbox)
        
        res=false;
        
        lx = false;
        if ( ( qbox(1)<=xpatch(1) && qbox(1)+qbox(3) >= xpatch(end) ) || ...
                ( qbox(1) >= xpatch(1) && qbox(1) < xpatch(end) ) || ...
                ( qbox(1)+qbox(3) > xpatch(1) && qbox(1)+qbox(3) <= xpatch(end) ) )
            %         if ((qbox(1) >= xpatch(1) &&  qbox(1) <= xpatch(end)) ||...
            %            ((qbox(1)+ qbox(3) >= xpatch(1) && qbox(1)+ qbox(3) <= xpatch(end))))
            %
            lx=true;
        end
        
        if lx  % x values within range
            
            if ( ( qbox(2)<=ypatch(1) && qbox(2)+qbox(4) >= ypatch(end) ) || ...
                    ( qbox(2) >= ypatch(1) && qbox(2) < ypatch(end) ) || ...
                    ( qbox(2)+qbox(4) > ypatch(1) && qbox(2)+qbox(4) <= ypatch(end) ) )
                res = true;
            end
        end  % if lx
    end


    function idx=Classification_UpdateREMOVE(linearIdx)
        global tick_length_lim
        idx=[];
        
        % narrow down on the potential objects that may be affected
        [rowsp colsp] = ind2sub(size(I0),linearIdx);
        xpatch = [min(colsp) max(colsp)]; ypatch = [min(rowsp) max(rowsp)];
        iok=cellfun(@(x) check_if_inside(xpatch,ypatch,x),{S.BoundingBox});
        iok = find(iok);
        
        % get pixels of the objects to be analyzed and remove them from the
        % BW and coloured images
        ipix = cell2mat({S(iok).PixelIdxList}');
        BW(ipix) = false;
        R(ipix) = 0; G(ipix) = 0; B(ipix) = 0;
         
        % set up and process a temporary image for classification
        bBox = cell2mat({S(iok).BoundingBox}');
        b0(1) = min([floor(min(bBox(:,1))) xpatch(1)])-1;  % dimensions for new image
        b0(2) = min([floor(min(bBox(:,2))) ypatch(1)])-1;
        b0(3) = max([ceil(max(bBox(:,1)+bBox(:,3))-b0(1)) xpatch(2)-b0(1)]);
        b0(4) = max([ceil(max(bBox(:,2)+bBox(:,4))-b0(2)) ypatch(2)-b0(2)]);
        bwtemp = false(b0([4 3]));  % new temporary image
        [i1, j1] = ind2sub(size(I0),ipix);
        bwtemp(sub2ind(size(bwtemp),i1-b0(2),j1-b0(1)))=true;
        bwtemp(sub2ind(size(bwtemp),rowsp-b0(2),colsp-b0(1)))=false;
        Stemp = regionprops(bwtemp,'BoundingBox','PixelList');
        
        % delete S(iok) objects
        S(iok) = [];
        nend = numel(S);
        
        % classify objects and put them back into the main S structure
        idx_noise = [];
        idx_trace = [];
        idx_tm = [];
        for k = 1:numel(Stemp)  % loop over new objects
            S(nend+k).PixelIdxList = sub2ind(size(BW),Stemp(k).PixelList(:,2)+b0(2),Stemp(k).PixelList(:,1)+b0(1));
            S(nend+k).BoundingBox = [Stemp(k).BoundingBox(1)+b0(1),...
                Stemp(k).BoundingBox(2)+b0(2), Stemp(k).BoundingBox(3),...
                Stemp(k).BoundingBox(4)];
            S(nend+k).ID = -1;
            if tick_length>0
                if abs(S(nend+k).BoundingBox(3)-tick_length)<=tick_length_lim*tick_length;
                    S(nend+k).ID=1;
                elseif S(nend+k).BoundingBox(3)-tick_length>tick_length_lim*tick_length;
                    S(nend+k).ID=0;
                end
            else
                if S(nend+k).BoundingBox(3)+tick_length>tick_length_lim*(-tick_length);
                    S(nend+k).ID=0;
                end
            end
            S(nend+k).TraceNum = -1;
            
            % collect pixel indices for late plotting
            if S(nend+k).ID==0  % trace
                idx_trace = [idx_trace; S(nend+k).PixelIdxList];
            elseif S(nend+k).ID==1  % time mark
                idx_tm = [idx_tm; S(nend+k).PixelIdxList];
            else  % noise
                idx_noise = [idx_noise; S(nend+k).PixelIdxList];
            end  % if S(nend+k).ID==0
        end  % for k = 1:numel(Stemp)
        
        % update classification figure
        R([idx_noise;idx_trace;idx_tm]) = 0;
        G([idx_noise;idx_trace;idx_tm]) = 0;
        B([idx_noise;idx_trace;idx_tm]) = 0;
        if strcmpi(get(bColorScheme,'state'),'off')
            R([idx_noise;idx_trace])=255;
            G([idx_tm; idx_trace])=255;
            B(idx_trace)=255;
        else
            R([idx_noise;idx_trace])=255;
            B([idx_tm;idx_trace])=255;
            G(idx_tm)=220;
            G(idx_trace)=255;
        end
        I_RGB(:,:,1)=R; I_RGB(:,:,2)=G; I_RGB(:,:,3)=B;
        updateSmallImage(1);
        %set(HIMRGB,'CDATA',I_RGB);
        %drawnow

        %for iter=1:length(S)
%         for iter = iok
%             if ~isempty(intersect(linearIdx,S(iter).PixelIdxList))
%                 bwtemp=false(S(iter).BoundingBox([4 3]));
%                 b0=floor(S(iter).BoundingBox);
%                 
%                 [i1,j1]=ind2sub(size(BW),setdiff(S(iter).PixelIdxList,linearIdx));
%                 bwtemp(sub2ind(size(bwtemp),i1-b0(2),j1-b0(1)))=true;
%                 
%                 Stemp=regionprops(bwtemp,'BoundingBox','PixelList');
%                 
%                 
%                 S(iter).PixelIdxList=...
%                     sub2ind(size(BW),Stemp(1).PixelList(:,2)+b0(2),Stemp(1).PixelList(:,1)+b0(1));
%                 S(iter).BoundingBox=...
%                     [Stemp(1).BoundingBox(1)+b0(1),...
%                     Stemp(1).BoundingBox(2)+b0(2),...
%                     Stemp(1).BoundingBox(3), Stemp(1).BoundingBox(4)];
%                 
%                 if tick_length>0
%                     if abs(S(iter).BoundingBox(3)-tick_length)<=tick_length_lim*tick_length;
%                         S(iter).ID=1;
%                     elseif S(iter).BoundingBox(3)-tick_length>tick_length_lim*tick_length;
%                         S(iter).ID=0;
%                     else
%                         S(iter).ID=-1;
%                     end
%                 else
%                     if S(iter).BoundingBox(3)+tick_length>tick_length_lim*(-tick_length);
%                         S(iter).ID=0;
%                     else
%                         S(iter).ID=-1;
%                     end
%                 end
%                 S(iter).TraceNum = -1;
%                 
%                 nend=length(S);
%                 
%                 idx=[iter, nend+[1:length(Stemp)-1]];
%                 
%                 
%                 for i=2:length(Stemp)
%                     S(nend+i-1).PixelIdxList=...
%                         sub2ind(size(BW),Stemp(i).PixelList(:,2)+b0(2),Stemp(i).PixelList(:,1)+b0(1));
%                     S(nend+i-1).BoundingBox=...
%                         [Stemp(i).BoundingBox(1)+b0(1),...
%                         Stemp(i).BoundingBox(2)+b0(2),...
%                         Stemp(i).BoundingBox(3), Stemp(i).BoundingBox(4)];
%                     
%                     if tick_length>0
%                         if abs(S(nend+i-1).BoundingBox(3)-tick_length)<=tick_length_lim*tick_length;
%                             S(nend+i-1).ID=1;
%                         elseif S(nend+i-1).BoundingBox(3)-tick_length>tick_length_lim*tick_length;
%                             S(nend+i-1).ID=0;
%                         else
%                             S(nend+i-1).ID=-1;
%                         end
%                     else
%                         if S(nend+i-1).BoundingBox(3)+tick_length>(tick_length_lim*(-tick_length));
%                             S(nend+i-1).ID=0;
%                         else
%                             S(nend+i-1).ID=-1;
%                         end
%                     end
%                     S(nend+i-1).TraceNum = -1;
%                 end
%                 
%                 return
%             end
%         end
        
        
    end



    function update_classification_plot(iter)%hobject,eventdata)%(hObject, eventdata)
        
%         disp('start in update_classifiation_plot');
%         tic
        if nargin==1
            if strcmpi(get(bColorScheme,'state'),'off')
                TickColor=[0 255 0];
            else
                TickColor=[0 220 255];
            end
            
            for it=iter
                [i,j]=ind2sub(size((BW)),[S(it).PixelIdxList]);
                id=sub2ind([rows,cols],i,j);
                if S(it).ID==-1
                    rgb=[255 0 0];
                elseif S(it).ID==1
                    rgb=TickColor;
                else
                    rgb=[255 255 255];
                end
                I_RGB(id)=rgb(1);
                I_RGB(id+numel(BW))=rgb(2);
                I_RGB(id+2*numel(BW))=rgb(3);
            end
        else
            G(:)=0; R(:)=0; B(:)=0;
            irej=cell2mat({S([S.ID]==-1).PixelIdxList}');
            itick=cell2mat({S([S.ID]==1|[S.ID]==3).PixelIdxList}');
            itrace=cell2mat({S([S.ID]==0|[S.ID]==2).PixelIdxList}');
            
            
            if strcmpi(get(bColorScheme,'state'),'off')
                R([irej;itrace])=255;
                G([itick; itrace])=255;
                B(itrace)=255;
            else
                R([irej;itrace])=255;
                B([itick;itrace])=255;
                G(itick)=220;
                G(itrace)=255;
            end
            
            I_RGB(:,:,1)=R; I_RGB(:,:,2)=G; I_RGB(:,:,3)=B;
        end
        updateSmallImage(1);
        %set(HIMRGB,'CDATA',I_RGB);    
        
    end


    function remove_bregion(hObject, eventdata)
        
        % check whether any of the zoom/pan icons is on - if it is, need to
        % reset the image
        if any(strcmpi(get([htools.Pan; htools.zoomOut; htools.zoomIn],'state'),'on'))
            xlim = get(ax,'xlim');
            ylim = get(ax,'ylim');
            axesCorig = [xlim ylim];
            setappdata(ax,'axesCorig',axesCorig);
            updateSmallImage(1);
            zoom off; pan off;
            %set([htools.Pan; htools.zoomOut; htools.zoomIn],'state','off');
        end  % if any of the three icons is on
        
        set(bremove_along_path,'state','off');
        % only one of remove region or remove line feature should be on at
        % any one time
        
        try
        while strcmpi(get(bremove_bregion,'State'),'on')
            
            % make sure window button functions have been cleaned before
            % imfreehand
            try, set(Hfdistinguish,'WindowButtonDownFcn',''), end
            try, set(Hfdistinguish,'WindowButtonMotionFcn',''), end
            try, set(Hfdistinguish,'WindowButtonUpFcn',''), end
            try, set(Hfdistinguish,'WindowKeyPressFcn',''), end
            try, set(Hfdistinguish,'WindowKeyReleaseFcn',''), end
            try, set(Hfdistinguish,'WindowScrollWheelFcn',''), end
                        
            hfh=imfreehand(ax,'closed',true);
%             try  % this should work if the region isn't empty
%                 hfh.Deletable = false;
            try
                mask=hfh.createMask;
%                 delete(hfh);
            catch
                try;; delete(hfh); end;
                break
            end
                
            % create undo point
            S0=S;
            BW0=BW;
            set(bundo_last_action,'Enable','on');
            
            % make sure mask is in the original coordinates
            mask = find(mask);
            [nrp ncp ndp] = size(get(HIMRGB,'Cdata'));
            [i1 i2] = ind2sub([nrp ncp],mask);
            % get reference values
            axesCorig = getappdata(ax,'axesCorig');
            x0 = floor(axesCorig(1));
            y0 = floor(axesCorig(3));
            i1 = i1 + y0;
            i2 = i2 + x0;
            mask = sub2ind(size(BW),i1,i2);
            
            % update plot
            linearIdx=mask;
            BW(mask)=false;
            %             update_BW_plot(linearIdx,false)
            %              drawnow;
            % remove parts in the classification image that has been
            % selected
            R(mask) = 0; G(mask) = 0; B(mask) = 0;
            % update both classification and image
            %idx=Classification_UpdateREMOVE_BREGION(linearIdx);
            idx=Classification_UpdateREMOVE(linearIdx);
%             if ~isempty(idx)
%                 update_classification_plot(idx);
%             end
%             drawnow;
                delete(hfh);

            %end  % try
                        
        end
        
        set(bremove_bregion,'State','off');
        end  % try
    end


    function remove_along_path(hObject, eventdata)
        
        % check whether any of the zoom/pan icons is on - if it is, need to
        % reset the image
        if any(strcmpi(get([htools.Pan; htools.zoomOut; htools.zoomIn],'state'),'on'))
            xlim = get(ax,'xlim');
            ylim = get(ax,'ylim');
            axesCorig = [xlim ylim];
            setappdata(ax,'axesCorig',axesCorig);
            updateSmallImage(1);
            set([htools.Pan; htools.zoomOut; htools.zoomIn],'state','off');
        end  % if any of the three icons is on
        
        set(bremove_bregion,'state','off');
        % only one of remove region or remove line feature should be on at
        % any one time
        %zoom off; pan off;

        try
        while strcmpi(get(bremove_along_path,'State'),'on')
            
            % clean up window button functions before imfreehand
            try, set(Hfdistinguish,'WindowButtonDownFcn',''), end
            try, set(Hfdistinguish,'WindowButtonMotionFcn',''), end
            try, set(Hfdistinguish,'WindowButtonUpFcn',''), end
            try, set(Hfdistinguish,'WindowKeyPressFcn',''), end
            try, set(Hfdistinguish,'WindowKeyReleaseFcn',''), end
            try, set(Hfdistinguish,'WindowScrollWheelFcn',''), end
            
            hfh=imfreehand(ax,'closed',false);
            %hfh.Deletable = false;
            try
                pos=hfh.getPosition;
            catch
                try; delete(hfh); end;
                break
            end
            
            if size(pos,1)<2 || strcmpi(get(bremove_along_path,'State'),'off')
                delete(hfh);
                set(bremove_along_path,'State','off')
                break
            end
            %delete(hfh);
            
            S0=S;
            BW0=BW;
            set(bundo_last_action,'Enable','on');
            
            if range(pos(:,1))>range(pos(:,2))
                [~,iu]=unique(pos(:,1));
                pos=pos(iu,:);
                cx= min(pos(:,1)):max(pos(:,1));
                cy= interp1(pos(:,1),pos(:,2),cx,'linear');
                % make it thick
                cx=[cx(:);cx(:)];
                cy=[cy(:);cy(:)+1];
            else
                [~,iu]=unique(pos(:,2));
                pos=pos(iu,:);
                cy= min(pos(:,2)):max(pos(:,2));
                cx= interp1(pos(:,2),pos(:,1),cy,'linear');
                % make it thick
                cy=[cy(:);cy(:)];
                cx=[cx(:);cx(:)+1];
            end
            
            % get reference values
            axesCorig = getappdata(ax,'axesCorig');
            x0 = floor(axesCorig(1));
            y0 = floor(axesCorig(3));
            i1 = round(cy) + y0;
            i2 = round(cx) + x0;

            linearIdx = sub2ind([rows,cols], i1,i2);
            %linearIdx = sub2ind([rows,cols], round(cy),round(cx));
            BW(linearIdx)=false;
            R(linearIdx) = 0; G(linearIdx) = 0; B(linearIdx) = 0;
            
            %update_BW_plot(linearIdx,false);
            
            idx=Classification_UpdateREMOVE(linearIdx);
%             if ~isempty(idx)
%                 update_classification_plot;
%             end
%             drawnow;
            delete(hfh);

        end
        
        set(bremove_along_path,'State','off');
        end  % try
    end


    function undo_last_action(hObject, eventdata)
        
        BW=BW0;
        S=S0;
        set(bundo_last_action,'Enable','off');
        update_classification_plot;
        drawnow
    end


    function eval_mark(hObject, eventdata)
        
        % check whether any of the zoom/pan icons is on - if it is, need to
        % reset the image
        if any(strcmpi(get([htools.Pan; htools.zoomOut; htools.zoomIn],'state'),'on'))
            xlim = get(ax,'xlim');
            ylim = get(ax,'ylim');
            axesCorig = [xlim ylim];
            setappdata(ax,'axesCorig',axesCorig);
            updateSmallImage(1);
            set([htools.Pan; htools.zoomOut; htools.zoomIn],'state','off');
        end  % if any of the three icons is on
        
        set(bremove_bregion,'State','off');
        set(bremove_along_path,'state','off');
 
        tag=get(hObject,'tag');
        if strcmp(tag,'s2Trace')
            id=0;
        elseif strcmp(tag,'s2Reject')
            id=-1;
        elseif strcmp(tag,'s2Timemark')
            id=1;
        else
            error('Problem in modify classification')
        end
        
        axesCorig = getappdata(ax,'axesCorig');
        x0 = floor(axesCorig(1));
        y0 = floor(axesCorig(3));
        xpatch = axesCorig(1:2);
        ypatch = axesCorig(3:4);
        
        %zoom off; pan off;
        [colSub,rowSub,butt]=ginput(1);
        colSub = colSub + x0;
        rowSub = rowSub + y0;
        
        while butt==1
            if (colSub>0&colSub<=cols) & (rowSub>0&rowSub<=rows)  % within image range
                linearInd = sub2ind([rows,cols], round(rowSub), round(colSub));
                iok = cellfun(@(x) check_if_inside(xpatch,ypatch,x),{S.BoundingBox});
                iok = find(iok);
                %for iter=1:length(S)
                for iter = iok
                    if any(linearInd==S(iter).PixelIdxList)
                        S(iter).ID=id;
                        
                        % update figure
                        if strcmpi(get(bColorScheme,'state'),'off')  % original colour scheme
                            if id==-1  % noise
                                R(S(iter).PixelIdxList) = 255;
                                G(S(iter).PixelIdxList) = 0;
                                B(S(iter).PixelIdxList) = 0;
                            elseif id==0  % trace
                                R(S(iter).PixelIdxList) = 255;
                                G(S(iter).PixelIdxList) = 255;
                                B(S(iter).PixelIdxList) = 255;
                            elseif id==1  % time mark
                                R(S(iter).PixelIdxList) = 0;
                                G(S(iter).PixelIdxList) = 255;
                                B(S(iter).PixelIdxList) = 0;
                            end  % if id==-1
                        else
                            if id==-1  % noise
                                R(S(iter).PixelIdxList) = 255;
                                G(S(iter).PixelIdxList) = 0;
                                B(S(iter).PixelIdxList) = 0;
                            elseif id==0  % trace
                                R(S(iter).PixelIdxList) = 255;
                                G(S(iter).PixelIdxList) = 255;
                                B(S(iter).PixelIdxList) = 255;
                            elseif id==1  % time mark
                                R(S(iter).PixelIdxList) = 0;
                                G(S(iter).PixelIdxList) = 220;
                                B(S(iter).PixelIdxList) = 255;
                            end  % if id==-1
                        end  % if strcmpi(get(bColorScheme,'state'),'off')
                        I_RGB(:,:,1)=R; I_RGB(:,:,2)=G; I_RGB(:,:,3)=B;
                        updateSmallImage(1);
                        %update_classification_plot(iter);
                        %drawnow
                        break;
                    end
                end
            end  % if within image range
            
            [colSub,rowSub,butt]=ginput(1);
            colSub = colSub + x0;
            rowSub = rowSub + y0;
        end
        
    end

    function deleteFigure(hObj,eventdata)
        % this function is called when the Edit Classification window is
        % closed
        
        rmappdata(H.f1,'Hfdistinguish');
    end  % function deleteFigure

%set (H.f1, 'WindowButtonMotionFcn', @mouseMove)
set(H.digitize,'enable','on');
set(H.digitize_options,'enable','on');

end

%%




function adjust_traces(hObject, eventdata)
global H I0 ftrend pt_traces pt_traceLABEL pt_tracesY0

dat=cell(numel(pt_traces),4);
for i=1:numel(pt_traces)
    x=get(pt_traces(i),'XDATA');
    y=get(pt_traces(i),'YDATA');
    dat(i,:)={ y(1),x(1), x(end), true };
end
% sort by ascending 1st y-pixel
firstyat_1=ftrend(1)+[dat{:,1}]'-ftrend([dat{:,2}]);

[~,idat]=sort(firstyat_1);
dat=dat(idat,:);
pt_traces=pt_traces(idat);


f = figure('Units','Normalized','Position',[0.8 0.25 0.18 0.5],'color','w',...
    'WindowStyle','normal','Menubar','None','Toolbar','none',...
    'NumberTitle','Off','Name','Adjust traces');
columnname =   {'First y-pixel','First x-pixel','Last x-pixel','Active'};
columnformat = {'numeric','numeric', 'numeric','logical'};
columneditable =  [true true true true];
t = uitable('Units','normalized','Position',...
    [0.01 0.1 0.98 0.89], 'Data', dat,...
    'ColumnName', columnname,...
    'ColumnFormat', columnformat,...
    'ColumnEditable', columneditable,...
    'RowName',1:numel(pt_traces),'FontSize',18);

apply_changes=uicontrol('Style', 'pushbutton','Parent',f,'units','normalized',...
    'String','Apply','Position', [0.782 0.02 0.2 0.07],'Callback',@apply_table_changes);

add_trace=uicontrol('Style', 'pushbutton','Parent',f,'units','normalized',...
    'String', '+','Position', [0.66 0.02 0.12 0.07],'Callback',@fadd_trace);

shift_trace=uicontrol('Style', 'pushbutton','Parent',f,'units','normalized',...
    'String', '^','Position', [0.54 0.02 0.12 0.07],'Callback',@fshift_trace);


uicontrol('Style', 'pushbutton','Parent',f,'units','normalized',...
        'String', 'All','TooltipString','Select or deselect all',...
        'Position', [0.01 0.02 0.18 0.05],'Callback',@select_trtab);
uicontrol('Style', 'pushbutton','Parent',f,'units','normalized',...
      'String', 'Odd','TooltipString','Select odd traces',...
      'Position', [0.2 0.02 0.18 0.05],'Callback',@select_trtab);
% uicontrol('Style', 'pushbutton','Parent',f,'units','normalized',...
%         'String', 'Even','TooltipString','Click on the even',...
%         'Position', [0.39 0.02 0.18 0.05],'Callback',@select_trtab);
    


uiwait(gcf);

%%%%%%%%% functions for the buttons of selection figure.
    function fshift_trace(hObject, eventdata)
        % This function shifts selected traces by the same amount
        
        % get the shift amount
        cprompt = ['Enter shift value.  Remember that if you want to ',...
            'shift upwards, the value should be negative'];
        cans = inputdlg(cprompt,'Set Vertical Shift',1,{'0'});
        
        if ~isnan(str2double(cans))  % numerical value returned
            dy = str2double(cans);  % shift value
            
            tdata = get(t,'data');  % original table data
            
            % find out which traces need to be shifted
            indx = find([tdata{:,end}]);
            
            for k = indx  % loop over lines to be shifted
                % update zero traces
                rtmp = get(pt_traces(k),'Ydata');
                rtmp = rtmp + dy;
                set(pt_traces(k),'YDATA',rtmp);
                
                % update first y-value information
                tdata(k,1) = {rtmp(1)};
                
                % update zero trace labels
                rtmp = get(pt_traceLABEL(k),'position');
                rtmp(2) = rtmp(2) + dy;
                set(pt_traceLABEL(k),'position',rtmp);
            end  % for k = indx
            
            set(t,'data',tdata);  % update table data
            
        end  % if ~isnan(str2double(cans))
        
    end

    function select_trtab(hObject, eventdata)
        tdata=get(t,'data');
        r=size(tdata,1);
        if strcmpi(get(hObject,'String'), 'All')
            if sum([tdata{:,end}])==r;  % all true
                tdata(1:r,end) = {false};
            else
                tdata(1:r,end) = {true};
            end  % if sum(tdata{:,end})==r
%             tdata(1:r,end)=num2cell(~[tdata{1:r,end}]);
        elseif strcmpi(get(hObject,'String'), 'Odd')
            tdata(1:r,end) = {true};
            tdata(2:2:r,end)={false};
            set(hObject,'string','Even','TooltipString','Select even traces');
        elseif strcmpi(get(hObject,'String'), 'Even')
            tdata(1:r,end) = {true};
            tdata(1:2:r,end) = {false};
            set(hObject,'string','Odd','TooltipString','Select odd traces');
        else
            error('Wrong selection (it should never gets here)')
        end
        set(t,'data',tdata);
        
    end




    function fadd_trace(hObject, eventdata)
        [r,c]=size(I0);
        
        ans_trac = inputdlg({'First y-pixel','First x-pixel','Last x-pixel'},'Add trace',1,{'1','1',num2str(c)});
        if isempty(ans_trac)
            return
        elseif str2double(ans_trac{2})>str2double(ans_trac{3})
            uiwait(msgbox('Operation can not be completed: 1st x > last x','error','modal'));
            return
        end
        
        
        dat=get(t,'data');
        idx2keep= [dat{:,end}];
        dat=dat(idx2keep,:);
        
        numeric=round([str2double(ans_trac{1}),str2double(ans_trac{2}),str2double(ans_trac{3})]);
        dat(size(dat,1)+1,:)={numeric(1),numeric(2),numeric(3),true};
        set(t,'Data',dat,'RowName',1:numel(pt_traces));
        
        apply_table_changes;
    end


    function apply_table_changes(hObject, eventdata)
        %global pt_traces pt_traceLABEL % to be able to delete it.
        % apply changes to uitable
        
        dat=get(t,'data');
        idx2keep= [dat{:,end}];
        dat=dat(idx2keep,:);
        
        try, delete(pt_traces); end
        try, delete(pt_traceLABEL); end
        try, clear pt_tracesY0; end
        
        % sort by ascending 1st y-pixel
        firstyat_1=ftrend(1)+[dat{:,1}]'-ftrend([dat{:,2}]);
        
        
        [~,idat]=sort(firstyat_1);
        dat=dat(idat,:);
        
        pt_traces=zeros(1,size(dat,1));
        pt_traceLABEL=zeros(1,size(dat,1));
        hold(H.ax1,'on');
        for it=1:size(dat,1)
            pt_traces(it)=plot(H.ax1,dat{it,2}:dat{it,3},dat{it,1}+ftrend(dat{it,2}:dat{it,3})-ftrend(dat{it,2}),'c--','LineWidth',1);
            %pt_traceLABEL(it)=text(dat{it,2}+10,10+dat{it,1}+ftrend(dat{it,2}),num2str(it),'BackgroundColor',[1 1 .8],'Parent',H.ax1);
            pt_traceLABEL(it)=text(dat{it,2}+10,10+dat{it,1},num2str(it),'BackgroundColor',[1 1 .8],'Parent',H.ax1);
            pt_tracesY0{it} = get(pt_traces(it),'Ydata');
        end
        hold(H.ax1,'off');
        
        traces_visibility;
        set(t,'Data',dat,'RowName',1:numel(pt_traces));
        drawnow
        
    end


end


function digitize_options(hObject, eventdata)
% this function allows options for digitization to be set

global H ptrace ptraceSTD I0

clear digitize_opt;  % initial cleaning

% positions of the properties window
r_wh = 396/564;  % width to height ratio
p = get(0,'ScreenSize');  % get reference screen size
% set up max values for box setup (in pixels)
rval_max = [floor(p(3)/2) floor(p(4)/2) p(3) p(4)];
% height = p(4)/2;  % height of the box
% width = r_wh*height;
height = 564;
width = 396;
x = p(3)-width;
y = p(4)-height;
pos.hf = [x y width height];
pos.rangeEdit = [0.05 0.849 0.2 0.05]; % edit box position for digitization strip definition
pos.rangeText = [0.255 0.849 0.7 0.055];  % text explaining edit box content
pos.checkTrace = [0.05 0.75 0.44 0.05];  % puxhbutton for checking traces
pos.checkSTD = [0.51 0.75 0.44 0.05];  % pushbutton for checking STD
pos.recalX1 = [0.05 0.68 0.25 0.05];  % edit box for starting x value for recalculating trace
pos.recalX2 = [0.32 0.68 0.25 0.05];  % edit box for ending x value for recalculating trace
pos.recal = [0.6 0.68 0.35 0.05];  % pushbutton to recalculate
% positions of the execution buttons
pos.Apply = [0.32 0.1 0.15 0.05];
pos.Close = [0.53 0.1 0.15 0.05];

lnewFig = true;  % will become false if new figure window is not going to be created

% check to see if the option window already exists
if isappdata(H.f1,'digitize_opt');
    digitize_opt = getappdata(H.f1,'digitize_opt');  % get old value of cto
    
    if ishandle(digitize_opt.hf)  % if figure still exists
        hchil = get(0,'children');
        set(digitize_opt.hf,'visible','on');  % make sure this window is visibled
        set(0,'children',[digitize_opt.hf; hchil(hchil~=digitize_opt.hf)]);  % bring relevant figure to front
        lnewFig = false;
    end  % if any(ishandle(cto.figure))    
end  % if isappdata(H.f1,'hcorrect_trace_options');

if lnewFig  % if new figure is going to be created
    digitize_opt.hf = figure('Name','Digitization Properties','NumberTitle','off',...
        'Units','pixels','menu','none','toolbar','none',...
        'OuterPosition',pos.hf);
    
    cstr = 'Range for defining digitization region (in fraction of vertical spacing)';
    digitize_opt.hRangeText = uicontrol('style','text','parent',digitize_opt.hf,'units','normalized',...
        'position',pos.rangeText,'string',cstr,'horizontalAlignment','left',...
        'fontsize',12);
    digitize_opt.hRangeEdit = uicontrol('style','edit','parent',digitize_opt.hf,'units','normalized',...
        'position',pos.rangeEdit,'string',num2str(H.edit_offset),...
        'fontsize',12,'callback',@doCheckValue);
    
    digitize_opt.hCheckTrace = uicontrol('style','pushbutton',...
        'parent',digitize_opt.hf,'units','normalized','position',pos.checkTrace,...
        'string','Check Traces','horizontalAlignment','center','tag','trace',...
        'fontsize',12,'enable','off','tooltipstring','display each trace separately',...
        'callback',@checkTrace);
    digitize_opt.hCheckSTD = uicontrol('style','pushbutton',...
        'parent',digitize_opt.hf,'units','normalized','position',pos.checkSTD,...
        'string','Check STD','horizontalAlignment','center','tag','std',...
        'fontsize',12,'enable','off','tooltipstring','display each STD separately',...
        'callback',@checkTrace);
    
    % recover traces
    h1=findobj(H.ax1,'Color',[0 0.8 1],'Visible','on'); % light blue
    h2=findobj(H.ax1,'Color',[0 0.6 1],'Visible','on'); % darker blue
    h3=findobj(H.ax1,'Color',[1 0.7 0],'Visible','on'); % orange
    
    if ~isempty(h1) % check for empty content
        im1=cell2mat(cellfun(@(x) [min(x) max(x)], {h1.XData},'UniformOutput',0)'); % get their limit values
    else
        im1=[];
    end
    if ~isempty(h2) % orange may be empty
        im2=cell2mat(cellfun(@(x) [min(x) max(x)], {h2.XData},'UniformOutput',0)');
    else
        im2=[];
    end
    if ~isempty(h3) % orange may be empty
        im3=cell2mat(cellfun(@(x) [min(x) max(x)], {h3.XData},'UniformOutput',0)');
    else
        im3=[];
    end
    im=[im1;im2;im3];
    if ~isempty(im)  % traces exist        
        % deciding where to look along the x direction
        rtmp = abs(min(im(:,1)) - max(im(:,2)));  % valid length
        i1 = round(0.1*rtmp);
        i2 = round(0.9*rtmp);
    else  % if there are no traces available (the options shouldn't be available)
        i1 = 1;
        i2 = size(I0,2);
    end  % if ~isempty(im)

    digitize_opt.hrecalX1 = uicontrol('style','edit',...
        'parent',digitize_opt.hf,'units','normalized','position',pos.recalX1,...
        'string',num2str(i1),'horizontalAlignment','center','tag','x1',...
        'fontsize',12,'enable',get(H.digitize_1,'enable'),...
        'tooltipstring','starting x position for recalculating or recovering digitized trace/STD',...
        'callback',@doCheckValue2);
    digitize_opt.hrecalX2 = uicontrol('style','edit',...
        'parent',digitize_opt.hf,'units','normalized','position',pos.recalX2,...
        'string',num2str(i2),'horizontalAlignment','center','tag','x2',...
        'fontsize',12,'enable',get(H.digitize_1,'enable'),...
        'tooltipstring','ending x position for recalculating or recovering digitized trace/STD',...
        'callback',@doCheckValue2);
    digitize_opt.hrecal = uicontrol('style','pushbutton',...
        'parent',digitize_opt.hf,'units','normalized','position',pos.recal,...
        'string','Recover Traces','horizontalAlignment','center',...
        'fontsize',12,'enable',get(H.digitize_1,'enable'),...
        'tooltipstring','recalculate or recover digitized traes/std',...
        'callback',@recover_traces);

    
    digitize_opt.hApply = uicontrol('style','pushbutton','parent',digitize_opt.hf,'units','normalized',...
        'position',pos.Apply,'string','Apply','tooltipstring','Apply changes',...
        'fontsize',14,'callback',@doApply);
    digitize_opt.hClose = uicontrol('style','pushbutton','parent',digitize_opt.hf,'units','normalized',...
        'position',pos.Close,'string','Close','tooltipstring','Close this window',...
        'fontsize',14,'callback',@doClose);
    
end  % if lnewFig

if any(ishandle(ptrace))  % if trace objects available
    set(digitize_opt.hCheckTrace,'enable','on');
end  % if any(ishandle(ptrace))
if any(ishandle(ptraceSTD))  % if STD objects available
    set(digitize_opt.hCheckSTD,'enable','on');
end  % if any(ishandle(ptraceSTD))

setappdata(H.f1,'digitize_opt',digitize_opt);  % update variable

    function doCheckValue2(hobj,eventdata)
        % this function makes sure that the input value is valid
        
        cstrin = get(hobj,'string');  % input string value
        nin = round(str2num(cstrin));
        
        if strcmpi(get(hobj,'tag'),'x1')  % starting value
            if (nin>1) && (nin<size(I0,2)) && nin<str2num(get(digitize_opt.hrecalX2,'string'))  % nin valid value
                i1 = nin;
            end  % if (nin>1) && (nin<size(I0,2)) && nin<str2num(get(digitize_opt.hrecalX2))
            set(hobj,'string',num2str(i1));
        elseif strcmpi(get(hobj,'tag'),'x2')  % ending value
            if (nin>1) && (nin<size(I0,2)) && nin>str2num(get(digitize_opt.hrecalX1,'string'))  % nin valid value
                i2 = nin;
            end  % if (nin>1) && (nin<size(I0,2)) && nin<str2num(get(digitize_opt.hrecalX2))
            set(hobj,'string',num2str(i2));
        else  % shouldn't get here
            disp('!!Error(doCheckValue2): shouldn''t get here');
            disp(['tag of the callback object: ', get(hobj,'tag')]);
            return  % quit out of function
        end  % if strcmpi(get(hobj,'tag'),'x1')...
    end  % function doCheckValue2


    function recover_traces(hobj,eventdata)
        % this function runs calculation of the trace recovery using the
        % starting and ending x values specified in the
        % digitize_opt.hrecalX1 and digitize_opt.hrecalX2 edit boxes
        
        % get starting and ending x values
        i1 = str2num(get(digitize_opt.hrecalX1,'string'));
        i2 = str2num(get(digitize_opt.hrecalX2,'string'));
        
        % get traces
        h1=findobj(H.ax1,'Color',[0 0.8 1],'Visible','on'); % light blue
        h2=findobj(H.ax1,'Color',[0 0.6 1],'Visible','on'); % darker blue
        h3=findobj(H.ax1,'Color',[1 0.7 0],'Visible','on'); % orange
        hhh=[h1;h2;h3]; % gather them in a vector
        
        % find median (when there are big missing parts and/or big quadratic distortion
        % the mean/median within a specific shorter range may work better)
        if ~isempty(h1)
            Y=cellfun(@(x) double(x), {h1.YData},'UniformOutput',false);
            m1=cellfun(@(x,y) median_nonan(y(x>i1 & x<i2)), {h1.XData},Y);
        else
            m1 = [];
        end
        if ~isempty(h2)
            Y=cellfun(@(x) double(x), {h2.YData},'UniformOutput',false);
            m2=cellfun(@(x,y) median_nonan(y(x>i1 & x<i2)), {h2.XData},Y);
        else
            m2=[];
        end
        if ~isempty(h3)
            Y=cellfun(@(x) double(x), {h3.YData},'UniformOutput',false);
            m3=cellfun(@(x,y) median_nonan(y(x>i1 & x<i2)), {h3.XData},Y);
        else
            m3=[];
        end
        m=[m1(:);m2(:);m3(:)];
        
        [~,idx]=sort(m); % sort the medians
        ptrace=hhh(idx); % replace ptrace handles
        
        % now work on the STD
        hhh=findobj(H.ax1,'Color',[0.3 0.1 1]);
        
        if ~isempty(hhh)
            im=cell2mat(cellfun(@(x) [min(x) max(x)], {hhh.XData},'UniformOutput',0)');
            % i1=max(im(:,1));
            % i2=min(im(:,2));
            
            Y=cellfun(@(x) double(x), {hhh.YData},'UniformOutput',false);
            m=cellfun(@(x,y) median_nonan(y(x>i1 & x<i2)), {hhh.XData},Y);
            
            [~,idx]=sort(m);
            ptraceSTD=hhh(idx);
            
        end
        
        disp('after trace recovery')
    end  % function recover_traces

    function checkTrace(hobj,eventdata)
        % this function makes selected object red one by one so that they
        % are easy to see (mostly used to make sure trace identification
        % performed in load_analysis has worked
        
        cstr = get(hobj,'tag');  % tag of the callback object
        lorder = true;  % set this to false if STD check should be done quickly
        if strcmpi(cstr,'trace')  % examine trace objects
            htrace = ptrace;
            lorder = true;
        elseif strcmpi(cstr,'std')  % examine std objects
            htrace = ptraceSTD;
        else  % shouldn't get here
            disp('in digitize_options, checkTrace: shouldn''t get here');
            disp(['cstr = ', cstr]);
            return
        end  % if strcmpi(cstr,'trace')...
        
        % bring the main window to the front
        htmp = get(0,'children');
        set(0,'children',[H.f1; htmp(htmp~=H.f1)]);
        hchil0 = get(H.ax1,'children');
        hchil = setdiff(hchil0,htrace);  % all the objects that are not traces
        
        % loop through each trace of interest and display one by one
        for k = 1:numel(htrace)
            ccol = get(htrace(k),'Color');
            cwidth = get(htrace(k),'lineWidth');
            
            set(htrace(k),'Color','r','lineWidth',2);
            if lorder  % make sure trace is in front
                %set(H.ax1,'children',[htrace(k); htrace(1:k-1); htrace(k+1:end); hchil]);  % bring target line to front
                set(H.ax1,'children',[htrace(k); hchil0(hchil0~=htrace(k))]);  % bring target line to front
            end % if lorder
             pause(0.5)  % pause so that the user can see
            
            % set back to original
            set(htrace(k),'Color',ccol,'lineWidth',cwidth);
            
        end  % for k = 1:numel(htrace)
        if lorder  % if orders have been changed
            set(H.ax1,'children',hchil0);  % set back to original
        end  % if lorder
    end  % funciton checkTrace

    function doApply(hobj,eventdata)
        % this function will copy the values in the edit fields into
        % H.edit_offset so that they can be used by the main program
        
        H.edit_offset = get(digitize_opt.hRangeEdit,'string');
    end  % function doApply

    function doCheckValue(hobj,eventdata)
        % this function checks for the validity of the input values and
        % sets back to previous value if unallowed
        
        cstr = H.edit_offset;  % default value
        
        roffset = str2num(get(hobj,'string'));
        if ~isempty(roffset)&isfloat(roffset)&(numel(roffset)==2)&...
                (roffset(1)<roffset(2))
            cstr = get(hobj,'string');
        end  % if ~isempty(roffset)

        set(hobj,'string',cstr);
    end  % function doCheckValue

    function doClose(hobj,eventdata)
        % this function closes the property setup window
        delete(digitize_opt.hf);
    end  % function doClose

end  % function digitize_options


function digitize_traces(hObject, eventdata)
global H ptrace ptraceSTD pt_traces itodigitize 
global S I0 ftrend output_message

% remove noise objects
% indx = find([S.ID]==-1);
% S(indx) = []; 

% make sure all objects have trace numbers
indx = find(cellfun('isempty',{S.TraceNum}));
if ~isempty(indx)
    [S(indx).TraceNum] = deal(-1);
end  % if ~isempty(indx)

% remove any objects that end up having no pixels (this shouldn't
% be necessary if fixed version (01/15/2017) of correct trace has
% been used
indx = find(cellfun('isempty',{S.PixelIdxList}));
S(indx) = [];  % remove empty pixels
        
% initialize
tmshift = nan*ones(1,numel(pt_traces));
if isappdata(H.ax1,'tmshift')  % previous version of the tmshift exists
    tmshift = getappdata(H.ax1,'tmshift');
end  % if isappdata(H.ax1,'tmshift')

itodigitize=1:length(pt_traces);
if any(ishandle(ptrace))
    cancel_dig=true;
    %% select traces to be proccesed
    f = dialog('Units','Normalized','Position',[0.8 0.1 0.11 0.7],'Name','Select traces to digitize');
    columnname ='Digitize';
    columnformat = {'logical'};
    columneditable =  [true];
    tdata=false(numel(pt_traces),1); tdata(itodigitize)=true;
    t = uitable('Parent',f,'Units','normalized','Position',...
        [0.0 0.14 1 0.84], 'Data',tdata,...
        'ColumnName', columnname,...
        'ColumnFormat', columnformat,...
        'ColumnEditable', columneditable,...
        'RowName',1:numel(pt_traces));
    
    uicontrol('Style', 'pushbutton','Parent',f,'units','normalized',...
        'String', 'All','TooltipString','Click to select/deselect all traces',...
        'Position', [0.05 0.075 0.3 0.05],'Callback',@select_tr);
    uicontrol('Style', 'pushbutton','Parent',f,'units','normalized',...
        'String', 'Odd','TooltipString','Select odd traces',...
        'Position', [0.35 0.075 0.3 0.05],'Callback',@select_tr);
   uicontrol('Style', 'pushbutton','Parent',f,'units','normalized',...
       'String', 'Reset','TooltipString','Reset trace assignments',...
       'Position', [0.65 0.075 0.3 0.05],'Callback',@reset_TraceNum);
    
    uicontrol('Style', 'pushbutton','Parent',f,'units','normalized',...
        'String', 'Ok','Position', [0.05 0.02 0.35 0.05],'Callback',@select_traces_to_digitize);
    uicontrol('Style', 'pushbutton','Parent',f,'units','normalized',...
        'String', 'Cancel','Position', [0.4 0.02 0.55 0.05],'Callback',@cancel_digitizing);
    %%
    waitfor(f);
    
    if  cancel_dig
        return
    end
    
else
    try
        delete(ptrace)
        delete(ptraceSTD)
    end
    try
        delete(findobj(H.ax1,'Color',[0 .8 1]));
        delete(findobj(H.ax1,'Color',[0 .6 1]));
        delete(findobj(H.ax1,'Color',[1 .7 0]));
        delete(findobj(H.ax1,'Color',[0.3 0.1 1])); % std
    end
    drawnow
    
    
    % create dummy traces update XData YData during digitization.
    ptraceSTD=[];
    ptrace=[];
    bcol=1;col=[0 .6 1];
    hold(H.ax1,'on')
    for i=[itodigitize(:)]'
        ptrace(i)=plot(H.ax1,1,i,'-','color',col,'linewidth',2);
        ptraceSTD(i)=plot(H.ax1,1,i,'-','color',[0.3 0.1 1],'linewidth',2);
        if bcol, bcol=0;col=[0 .8 1]; else bcol=1;col=[0 .6 1]; end
    end
end



% Question
% button = questdlg('Use trace assignment information for digitization?','Digitize traces','Yes','No','Cancel','No');
% if strcmpi(button,'yes')
%     digitize_traces_with_assignments;
% elseif strcmpi(button,'No')
%     digitize_traces_without_assignments;
% else
%     return
% end

% use the next line if resetting all trace associations
% [S(:).TraceNum] = deal(-1); 

% retrieve values from GUI
%luminance_thershold=str2double(get(H.edit_luminance,'string'));
% over_under=str2num(get(H.edit_offset,'string')); %#ok<ST2NM>
over_under=str2num(H.edit_offset); 

[rows,cols]=size(I0);

fX=ftrend(cell2mat(cellfun(@(v) v(1),get(pt_traces,'Xdata'),'UniformOutput',false)));
Y=cell2mat(cellfun(@(v) v(1),get(pt_traces,'Ydata'),'UniformOutput',false));
meanOffset=mode(abs(diff(Y)));

% Bbox=cell2mat({S.BoundingBox}');
% itick=[S.ID]==1;
% itrace=[S.ID]==0;
% 
% indx_trace = find(itrace); % indices of valid traces (position corresponds to row numbers of C)
% indx_tm = find(itick); % indices of valid time marks (order corresponds to row numbers of Ctick)
% 
% C=[Bbox(:,2)+Bbox(:,4)/2,Bbox(:,1)+Bbox(:,3)/2];
% Bbox=[floor(Bbox(:,2)),ceil(Bbox(:,2)+Bbox(:,4))];
% BboxTick=Bbox(itick,:);
% Bbox=Bbox(itrace,:);
% Ctick=C(itick,:);
% C=C(itrace,:);
% 
% Stick=S(itick);
% Strace=S(itrace);
    
hold(H.ax1,'on')
output_message{end+1}='Digitizing, please wait...';
set(H.output_mess,'String',output_message,'Value',length(output_message)); drawnow

set(H.axwait,'Visible','on');
for i=[itodigitize(:)]' %1:numel(pt_traces) trace number
    
    % update axwait
    temp=find(i==itodigitize)/numel(itodigitize); %  oxi I ALLA INDX(I) 
    h=pie(H.axwait,[1-temp+eps, temp],{'',''});
    set(h(3),'EdgeColor','none');
    set(h(1),'EdgeColor','None','FaceColor',[0.3 0.3 0.3]);
    set(h(3),'FaceColor',[1 1 1]);
    text(0,0,[num2str(find(i==itodigitize)) '/' num2str(numel(itodigitize))],...
        'HorizontalAlignment','Center','FontWeight','bold',...
        'Color','w','BackgroundColor',[0.5 0.5 0.5],'Parent',H.axwait);
    drawnow
    %% digitize
    
    % identify objects to be used
    itraceN = find([S.TraceNum]==i);  % objects already associated with this trace
    
    itick0 = [];  % cleaning
    itrace0 = [];
    itrace_i = [];
    itick_i = [];
    indxSat = [];  % index for which the image will be saturated
    
    if ~isempty(itraceN)
        clear Stmp Bbox0 BboxTick0 C0 Ctick0 itrace itick
        Stmp = S(itraceN);
        Bbox0 = cell2mat({Stmp.BoundingBox}');
        itick0 = ([Stmp.ID]==1|[Stmp.ID]==3);
        itrace0 = ([Stmp.ID]==0|[Stmp.ID]==2);
        indxSat = find([Stmp.ID]==2|[Stmp.ID]==3);  % manually traced regions
        if ~isempty(indxSat)
            indxSat = cell2mat({Stmp(indxSat).PixelIdxList}');
        end  % if ~isempty(indxSat)
        C0 = [Bbox0(:,2)+Bbox0(:,4)/2, Bbox0(:,1)+Bbox0(:,3)/2];
        Bbox0=[floor(Bbox0(:,2)),ceil(Bbox0(:,2)+Bbox0(:,4))];
        BboxTick0=Bbox0(itick0,:);
        Bbox0=Bbox0(itrace0,:);
        Ctick0=C0(itick0,:);
        C0=C0(itrace0,:);
    end
    
    indx = find([S.TraceNum]==-1);  % any other objects with no trace association
    
    if ~isempty(indx)
        % prepare strips
        
        clear Stmp Bbox BboxTick C Ctick itrace itick indx_trace indx_tm % initial cleaning
        
        Stmp = S(indx);
        Bbox = cell2mat({Stmp.BoundingBox}');
        itick = ([Stmp.ID]==1);
        itrace = ([Stmp.ID]==0);
        
        indx_trace = find(itrace);  % trace indices
        indx_tm = find(itick);  % time mark indices
        
        C = [Bbox(:,2)+Bbox(:,4)/2, Bbox(:,1)+Bbox(:,3)/2];
        Bbox=[floor(Bbox(:,2)),ceil(Bbox(:,2)+Bbox(:,4))];
        BboxTick=Bbox(itick,:);
        Bbox=Bbox(itrace,:);
        Ctick=C(itick,:);
        C=C(itrace,:);
        
        zerotrace_i= Y(i)-fX(i)+ftrend(C(:,2));
        itrace_i=find( C(:,1) > over_under(1)*meanOffset+zerotrace_i &...
            C(:,1) < over_under(2)*meanOffset+zerotrace_i );
        
        zerotick_i=Y(i)-fX(i)+ftrend(Ctick(:,2));
        if  strcmpi(get(H.timemarkpos,'State'),'off') % time marks above the trace
            itick_i=find( Ctick(:,1) > -abs(over_under(1))*meanOffset+zerotick_i-meanOffset/2 &...
                Ctick(:,1) < zerotick_i );
        elseif strcmpi(get(H.timemarkpos,'State'),'on')  % time marks bellow the trace
            itick_i=find( Ctick(:,1) < abs(over_under(2))*meanOffset+zerotick_i+meanOffset/2 &...
                Ctick(:,1) > zerotick_i );
        else % It should never go to else
            error('Problem with time marks button status')
        end
        
        % assign trace numbers to these objects
        [S(indx(indx_trace(itrace_i))).TraceNum] = deal(i);
        [S(indx(indx_tm(itick_i))).TraceNum] = deal(i);
    end  %if ~isempty(indx)
    
%     whos itrace_i itick_i
%     whos indx indx_trace itrace0 itick0 itraceN
    
    rsV = [];
    reV = [];
    itrace_iA = [];
    itick_iA = [];
    if ~isempty(itrace_i)  % if trace objects available
        rsV = [rsV; Bbox(itrace_i,1)];
        reV = [reV; Bbox(itrace_i,2)];
        itrace_iA = indx(indx_trace(itrace_i));
    end % if ~isempty(itrace_i)
    if ~isempty(itick_i)  % if time mark objects available
        rsV = [rsV; BboxTick(itick_i,1)];
        reV = [reV; BboxTick(itick_i,2)];
        itick_iA = indx(indx_tm(itick_i));
    end  % if ~isempty(itick_i)
    if ~isempty(find(itrace0))
        rsV = [rsV; Bbox0(:,1)];
        reV = [reV; Bbox0(:,2)];
%         whos itrace_iA
%         if ~isempty(itrace_iA)  % if itrace_iA isn't empty
            itrace_iA = [itrace_iA itraceN(itrace0)];
%         else
%             itrace_iA = itraceN(itrace0);
%         end  % if ~isempty(itrace_iA)...
    end  % if ~isempty(itrace0)
    if ~isempty(find(itick0))
        rsV = [rsV; BboxTick0(:,1)];
        reV = [reV; BboxTick0(:,2)];
%         if ~isempty(itick_iA)  % if itick_iA isn't empty
            itick_iA = [itick_iA itraceN(itick0)];
%         else
%             itick_iA = itraceN(itick0);
%         end  % if ~isempty(itick_iA)...
    end  % if ~isempty(itick0)
    
    if isempty(itrace_iA) & isempty(itick_iA)  % no objects available for this trace
        warning('Problem during digitization of trace # %d.  No objects available.',i);
        itrace_iA = [];
        itick_iA = [];
        break
    else
        rstart = max(1,min(rsV));
        rend = min(size(I0,1),max(reV));
    end
    
%     % update the non-assigned objects
%     if ~isempty(indx)
%         itmp = [indx_trace(itrace_i) indx_tm(itick_i)];  % indices of indx to be removed
%         indx(itmp) = [];
%         Bbox(itrace_i,:) = [];
%         BboxTick(itick_i,:) = [];
%         C(itrace_i,:) = [];
%         Ctick(itick_i,:) = [];
%         indx_trace(itrace_i) = [];
%         indx_tm(itick_i) = [];
%     end  % if ~isempty(itrace_i)
    
    buff=rend-rstart+1;
    
    Itemp = I0;
    % if there are objects with ID of 2 or 3 (i.e., manually traced),
    % change the luminescence values within the objects
    if ~isempty(indxSat)  % there are pixels that need to be saturated
        Itemp(indxSat) = uint8(255);
    end  % if ~isempty(indxSat)
    Itemp = Itemp(rstart:rend,:);
   % Itemp=I0(rstart:rend,:);
    
    temp=false(size(Itemp));
    tempNULL=temp;
    
    [i1,j1]=ind2sub([rows,cols],cell2mat({S(itrace_iA).PixelIdxList}'));
    try
        iok=i1>=rstart & i1<=rend;
    catch
        warning('Possible problem during digitization of trace# %d. No assigned objects were found.',i)
        output_message{end}=['skipping ' num2str(find(i==itodigitize)) '/' num2str(numel(itodigitize))];
        set(H.output_mess,'String',output_message,'Value',length(output_message)); drawnow
        continue
    end
    temp(sub2ind(size(temp),i1(iok)-rstart+1,j1(iok)))=true;
    
    [i1,j1]=ind2sub([rows,cols],cell2mat({S(itick_iA).PixelIdxList}'));
    iok=i1>=rstart & i1<=rend;
    tempNULL(sub2ind(size(tempNULL),i1(iok)-rstart+1,j1(iok)))=true;
    
    x=round(get(pt_traces(i),'XDATA'));
    y=round(get(pt_traces(i),'YDATA'))-rstart+1;
    if (y(1))<=0
        y=y -y(1)+1;
    end
    
    
    stripe=zeros(3*buff,numel(x),'uint8');
    bstripe=false(3*buff,numel(x));
    bstripe_tick=false(3*buff,numel(x));
    
    
    for ix=1:numel(x)
        if x(ix)>0 && x(ix)<=size(Itemp,2);
            stripe(max(1,y(ix)+ [1:buff]),ix)=Itemp(:,x(ix));
            bstripe(max(1,y(ix)+ [1:buff]),ix)= temp(:,x(ix));
            bstripe_tick(max(1,y(ix)+ [1:buff]),ix)= tempNULL(:,x(ix));
        else
            warning('Possible problem during digitization. report that: x(ix)=%d (it should be between 1 and %d), trace# %d',x(ix),size(Itemp,2),i)
        end
    end
    
    if isfield(H, 'curvButton')
        % Correct offset curvature
        if ~isempty( get(H.curvButton,'UserData') )
            
            fcurv=get(H.curvButton,'UserData');
            [r,c]=size(stripe);
            
            xi=fcurv(linspace(-r/2,r/2,r));
            
            % create displacement field
            D=zeros(r,c,2);
            D(:,:,1)=repmat(xi-min(xi),1,c);
            % apply displacement field
            stripe=imwarp(stripe,D,'cubic');
            bstripe=imwarp(bstripe,D,'cubic');
            bstripe_tick=imwarp(bstripe_tick,D,'cubic');
            
        end
    end
        
    
    output_message{end}=['Digitizing ' num2str(find(i==itodigitize)) '/' num2str(numel(itodigitize)) ', please wait...'];
    set(H.output_mess,'String',output_message,'Value',length(output_message)); drawnow
    
    if str2double(get(H.edit_tick_xlength,'String'))>0 && numel(itick_iA)>0
        if ~isnan(tmshift(i))  % tmshift value for this trace already exists
            yshift = tmshift(i);
            ry = 0.05;
        elseif numel(find(~isnan(tmshift)))>0  % other values of tmshift exist
            yshift = mean(tmshift(find(~isnan(tmshift))));
            ry = 0.2;  
        else
            yshift = nan;
            ry = 0;
        end  % if ~isnan(tmshift(i))...
        [YDATA,YDATASTD,ytmShift]=digitize_trace_and_tickmarks(stripe,stripe,bstripe,bstripe_tick,yshift,ry); % stripe and stripe NULL are the same
        tmshift(i) = ytmShift;
        %disp(['trace ',num2str(i),': ',num2str(ytmShift)]);
    else
        [YDATA,YDATASTD]=digitize_trace_without_tickmarks(stripe,bstripe);
    end
    
    XDATA=x; % correct for real starting point
    YDATA=YDATA-y + rstart - 1;
    YDATASTD=YDATASTD+YDATA;
    
    set(ptrace(i),'XData',XDATA,'YData',YDATA);
    set(ptraceSTD(i),'XData',XDATA,'YData',YDATASTD); %std
    
end
hold(H.ax1,'off')

setappdata(H.ax1,'tmshift',tmshift);

% delete all objects that were not associated with any traces
%S([S.TraceNum]==-1) = [];

if isappdata(H.ax1,'oldTraceNum')  % old trace numbers exist
    indx = find([S.TraceNum]==-1);
    for k = indx
        S(k).TraceNum = oldTraceNum(k);
    end  % for k = indx
    rmappdata(H.ax1,'oldTraceNum');  % remove variable
end  % if isappdata(H.ax1,'oldTraceNum')

output_message{end+1}='Digitization finished.';
set(H.output_mess,'String',output_message,'Value',length(output_message)); drawnow
output_message{end+1}='Ready.';
set(H.output_mess,'String',output_message,'Value',length(output_message)); drawnow
cla(H.axwait); set(H.axwait,'Visible','off');

set(H.digitize_1,'enable','on');
set(H.correct_trace_options,'enable','on');

% disable trace identification and adjustments
set(H.idTrace_options,'enable','off');
set(H.auto_num_of_traces,'enable','off');
set(H.adjust_traces,'enable','off');

%%%%%%%%% functions for the buttons of selection figure.

    function reset_TraceNum(hObject, eventdata)
        % when this button is pressed, all TraceNum information if first
        % removed and then restored for any objects for which trace number
        % assignment was not done
        
        % confirm
        cstr = sprintf('%s\n%s','This will remove all trace number assignment information.',...
            'Are you sure you want to proceed?');
        cans = questdlg(cstr,'Trace Number Reset Confirmation','Yes','No','No');
        
        if strcmpi(cans,'Yes')
            oldTraceNum = [S.TraceNum];  % keep copy of the old trace number
            
            [S.TraceNum] = deal(-1);  % reset to unknown
            
            % store old Trace Number in the axes handle
            setappdata(H.ax1,'oldTraceNum',oldTraceNum);
        end  % if strcmpi(cans,'Yes')
    end  % function reset_TraceNum


    function select_tr(hObject, eventdata)
        tdata=get(t,'data');
        r=size(tdata,1);
        
        if strcmpi(get(hObject,'String'), 'All')
            if sum([tdata(:,end)])==r;  % all true
                tdata(1:r,end) = false;
            else
                tdata(1:r,end) = true;
            end  % if sum(tdata{:,end})==r
        elseif strcmpi(get(hObject,'String'), 'Odd')
            tdata(1:r,end) = true;
            tdata(2:2:r,end)=false;
            set(hObject,'string','Even','TooltipString','Select even traces');
        elseif strcmpi(get(hObject,'String'), 'Even')
            tdata(1:r,end) = true;
            tdata(1:2:r,end) = false;
            set(hObject,'string','Odd','TooltipString','Select odd traces');
        else
            error('Wrong selection (it should never gets here)')
        end
        set(t,'data',tdata);
        
    end



    function select_traces_to_digitize(hObject, eventdata)
        dat=get(t,'data');
        itodigitize= find(dat);
        cancel_dig=false;
        close(f);
    end
    function cancel_digitizing(hObject, eventdata)
        cancel_dig=true;
        close(f);
    end

set(H.togle_digital_traces_visibility,'enable','on','value',1);
set(H.togle_digital_traces_STD_visibility,'enable','on','value',1);
set(H.find_gaps,'enable','on');
set(H.find_vert,'enable','on');

set(H.mark_start,'enable','on');

% check for time marks
% tmLength = str2double(get(H.edit_tick_xlength,'string'));
% if tmLength>0  % time marks avaiable
%     set(H.mark_1st,'enable','on');
%     set(H.mark_last,'enable','on');
%     ntm = getntm(tmLength);
%     if ~isempty(ntm)  % if number of estimated time marks exists
%         set(H.edit_num_of_ticks,'string',num2str(ntm),'enable','on');
%     end
% end  % if tmlength>0
% 

%     function nout = getntm(tmLength)  % find number of likely time marks
% %         global pt_traces
%         
%         nout = [];
%         traceLength = str2double(get(H.edit_DP,'string'));
%         if ~isnan(traceLength) & (traceLength > tmLength)
%             posS=cell2mat(cellfun(@(v) v(1),get(pt_traces,'Xdata'),'UniformOutput',false));
%             posE=cell2mat(cellfun(@(v) v(end),get(pt_traces,'Xdata'),'UniformOutput',false));
%             rtmp = max(posE)-min(posS);
%             nout = round(rtmp/(tmLength+traceLength));
%         end  % if ~isnan(traceLength) & (traceLength > tmLength)
%         
%     end

end


function  extract_curvature_correction(hObject,eventdata)
global H I0 pt_traces
% initialize
 set(H.curvButton,'UserData',[],'State','off') % In case the user closes the figure;
 
number_of_traces=numel(pt_traces);

% Select trace with mouse
h=msgbox('Click with mouse anywhere on the trace you want to use for modeling offset curvature','Correct curvature','help',[],'WindowStyle','modal');
waitfor(h);

[xc,yc]=ginput(1);

% Find the trace number
d=zeros(number_of_traces,1);
for i=1:number_of_traces
    d(i)=min( (get(pt_traces(i),'XDATA')-xc).^2 + (get(pt_traces(i),'YDATA')-yc).^2 );
end
[~,iTRACE]=min(d);
x=round(get(pt_traces(iTRACE),'XDATA'));

xtemp=get(pt_traces(iTRACE),'XDATA'); ytemp=get(pt_traces(iTRACE),'YDATA');
[~,ii]=min( (xtemp -xc).^2 + (ytemp-yc).^2 );
XC=xtemp(ii); YC=ytemp(ii);

position=[max(1,XC-300) max(1,YC-70) 600 140];
position(3)=min(size(I0,2)-position(1),position(3));
position(4)=min(size(I0,1)-position(2),position(4));

% create rectangle to get the problematic patch
hrec = imrect(gca,position);
setPositionConstraintFcn(hrec,@symetric_fixedCenterY_fcn); 


position=wait(hrec);
delete(hrec);
position=round(position);
stripe=imcrop(I0,position);
[r,c]=size(stripe);

xpatch=position(1)+[0:position(3)];
xpatch=xpatch(xpatch>=x(1) & xpatch<=x(end));
ypatch=position(2)+[0:position(4)];

% Launch GUI to correct curvature
h1stripeC=figure('Color','w','WindowStyle','normal','Name',['Edit trace ' num2str(iTRACE)] ,'NumberTitle','Off',...
    'Menubar','none','ToolBar','figure','Units','Normalized','Position',[0.1 0.5 0.8 0.3]);


% modify tool bar
temp= findall(findall(h1stripeC,'Type','uitoolbar'));
delete(temp([2:7 9 13 16 17]));
% % Reorder tools
% set(temp(1),'Children',[temp(4:end-1);temp(2:3);temp(end)])


uipushtool(temp(1),'Tag','Select region mirrored','Cdata',imread('icons/CurvedTraces.png'),...
    'Separator','on','TooltipString','Define curvature using impoly object',...
    'HandleVisibility','on','ClickedCallback',@DefineCurvature);


bpreview=uicontrol('Style', 'pushbutton','units','normalized',...
    'String', 'Preview','Position', [0.92 0.26 0.07 0.08],...
    'Callback',@preview_curvature_corection,'enable','off');


bpapply=uicontrol('Style', 'pushbutton','units','normalized',...
    'String', 'Save & Close','Position', [0.92 0.17 0.07 0.08],...
    'Callback',@save_curvature_model);

uicontrol('Style', 'pushbutton','units','normalized',...
    'String', 'Cancel','Position', [.92 0.08 0.07 0.08],...
    'Callback',@close_without_saving_curvature_model);



ax1=axes('Position',[0.06 0.55 .8 .44]);
% plot trace
imagesc(stripe,'Parent',ax1); colormap(ax1,gray(256));
ax2=axes('Position',[0.06 0.05 .8 .44]);
hpoly=[];
fcurv=[];

    function constrained_position = symetric_fixedCenterY_fcn(new_position)
       constrained_position=new_position;
       constrained_position(2)=YC-new_position(4)/2;      
    end
    
    function DefineCurvature(ho,ev)
        try
        delete(hpoly)
        end
        delete(findobj([ax1 ax2],'Color','r'))
        delete(findobj([ax1 ax2],'Color','c'))
        delete(findobj([ax1 ax2],'Color','y'))
        hpoly=impoly(ax1,'Closed',false);
        hpoly.Deletable = true;
        set(bpreview,'Enable','on');
        set(bpapply,'Enable','on');
        
    end

    function preview_curvature_corection(ho, ev)
        % delete previous diagnostic lines
        delete(findobj([ax1 ax2],'Color','r'))
        delete(findobj([ax1 ax2],'Color','c'))
        delete(findobj([ax1 ax2],'Color','y'))
       
        % get current position of impoly obj
        pos=getPosition(hpoly);
        pos2=pos(:,2)-r/2;       
        
        [r,c]=size(stripe);
        % fit a 2nd degree eq
        fcurv = fit(pos2,pos(:,1),'poly2');
        
        yi=linspace(min(get(gca,'Ylim')),max(get(gca,'Ylim')),r);
        xi=fcurv(linspace(-r/2,r/2,r));
        
        [~,imin2]=min(xi);
        
         
        hold(ax1,'on')
        plot(ax1,xi,yi,'r');
        [~,imin1]=min(abs(gradient(xi,yi)));
        plot(ax1,get(ax1,'Xlim'),yi(imin1)*[1 1] ,'y--');
        plot(ax1,xi(imin2)*[1 1],get(gca,'Ylim'),'c--');
        plot(ax1,get(ax1,'Xlim'),yi(imin2)*[1 1],'c:');
        hold(ax1,'off')
        
        % create displacement field
        D=zeros(r,c,2);
        D(:,:,1)=repmat(xi-min(xi),1,c);        
        B=imwarp(stripe,D,'cubic');
        
        axes(ax2)
        cla(ax2)
        imagesc(B,'Parent',ax2)
        colormap(ax2,gray(256))
        hold(ax2,'on')
        plot(ax2,xi,yi,'r--');
        plot(ax2,xi(imin2)*[1 1],get(gca,'Ylim'),'c--');
        
        set(ax2,'Xgrid','on')
        linkaxes([ax1 ax2])
        
    end

    function save_curvature_model(ho,ev)
          set(H.curvButton,'UserData',fcurv,'State','on')
          
          close(h1stripeC)
    end


    function close_without_saving_curvature_model(ho,ev)
         set(H.curvButton,'UserData',[],'State','off')
         close(h1stripeC)
    end





end







%%


function correct_trace_options(hObject, eventdata)
global H

clear cto;  % initial cleaning

set(H.mark_start,'enable','off');  % turn trace end marking button off while this window is open

% default values
cto.box = [92 300 185 600];
% [(# of pixels left of selected point) (# of pixels above the selected point)
%   (# of pixels for width of the box) (# of pixels for the height of the box)]
cto.window = [0.107 0.349 0.462 0.432]; % position vector for new correct trace window
% positions of the properties window
r_wh = 396/564;  % width to height ratio
p = get(0,'ScreenSize');  % get reference screen size
% set up max values for box setup (in pixels)
rval_max = [floor(p(3)/2) floor(p(4)/2) p(3) p(4)];
% height = p(4)/2;  % height of the box
% width = r_wh*height;
height = 564;
width = 396;
x = p(3)-width;
y = p(4)-height;
pos.hf = [x y width height];
dx = 0.05;  % shift in x value
% positions of the Box selection objects in the properties box
pos.BoxText = [0.063 0.909 0.891 0.041];  % position of box title
pos.Box_dx = [0.093+dx 0.869 0.114 0.039];  % position of edit box for dx
pos.Box_dxText = [0.217+dx 0.861 0.239 0.041];  % position of text for dx
pos.Box_dy = [0.503+dx 0.869 0.114 0.039];  % position of edit box for dy
pos.Box_dyText = [0.626+dx 0.861 0.239 0.041];  % position of text for dy
pos.Box_Dx = [0.093+dx 0.815 0.114 0.039];  % position of edit box for Dx
pos.Box_DxText = [0.217+dx 0.805 0.239 0.041];  % position of text for Dx
pos.Box_Dy = [0.503+dx 0.815 0.114 0.039];  % position of edit box for Dy
pos.Box_DyText = [0.626+dx 0.805 0.239 0.041];  % position of text for Dy
% positions of the Correct Trace window selection in the property box
pos.WindowText = [0.063 0.718 0.891 0.041];  % position of box title
pos.Window_dx = [0.093+dx 0.677 0.114 0.039];  % position of edit box for dx
pos.Window_dxText = [0.217+dx 0.670 0.239 0.041];  % position of text for dx
pos.Window_dy = [0.503+dx 0.677 0.114 0.039];  % position of edit box for dy
pos.Window_dyText = [0.626+dx 0.670 0.239 0.041];  % position of text for dy
pos.Window_Dx = [0.093+dx 0.619 0.114 0.039];  % position of edit box for Dx
pos.Window_DxText = [0.217+dx 0.619 0.239 0.041];  % position of text for Dx
pos.Window_Dy = [0.503+dx 0.619 0.114 0.039];  % position of edit box for Dy
pos.Window_DyText = [0.626+dx 0.619 0.239 0.041];  % position of text for Dy
% positions for interpolation scheme selection objects
pos.InterpText = [0.063 0.532 0.5 0.041];  % position of interpolation selection title
pos.InterpPopUp = [0.563 0.532 0.4 0.041];  % position of interpolation selection pop up menu
% positions of the execution buttons
pos.Apply = [0.32 0.05 0.15 0.05];
pos.Close = [0.425 0.05 0.15 0.05];
pos.Recover = [0.8 0.05 0.17 0.05];  % button to recover correct trace pushbutton
%pos.Close = [0.53 0.05 0.15 0.05];

lnewFig = true;  % will become false if new figure window is not going to be created

% check to see if the option window already exists
if isappdata(H.f1,'cto');
    cto = getappdata(H.f1,'cto');  % get old value of cto
    
    if ishandle(cto.hf)  % if figure still exists
        hchil = get(0,'children');
        set(cto.hf,'visible','on');  % make sure this window is visibled
        set(0,'children',[cto.hf; hchil(hchil~=cto.hf)]);  % bring relevant figure to front
        lnewFig = false;
    end  % if any(ishandle(cto.figure))    
end  % if isappdata(H.f1,'hcorrect_trace_options');

if lnewFig  % if new figure is going to be created
    cto.hf = figure('Name','Correct Trace Properties','NumberTitle','off',...
        'Units','pixels','menu','none','toolbar','none',...
        'OuterPosition',pos.hf,'DeleteFcn',@closecto);
    
    cstr = 'Box for Trace Selection (in pixels)';
    cto.hBoxText = uicontrol('style','text','parent',cto.hf,'units','normalized',...
        'position',pos.BoxText,'string',cstr,'horizontalAlignment','left',...
        'fontsize',12);
    cto.hBox(1) = uicontrol('style','edit','parent',cto.hf,'units','normalized',...
        'position',pos.Box_dx,'string',num2str(cto.box(1)),...
        'fontsize',12,'callback',@ctoCheckValue);
    cto.hBoxText(1) = uicontrol('style','text','parent',cto.hf,'units','normalized',...
        'position',pos.Box_dxText,'string','left of point',...
        'fontsize',12,'horizontalAlignment','left');
    cto.hBox(2) = uicontrol('style','edit','parent',cto.hf,'units','normalized',...
        'position',pos.Box_dy,'string',num2str(cto.box(2)),...
        'fontsize',12,'callback',@ctoCheckValue);
    cto.hBoxText(2) = uicontrol('style','text','parent',cto.hf,'units','normalized',...
        'position',pos.Box_dyText,'string','above point',...
        'fontsize',12,'horizontalAlignment','left');
    cto.hBox(3) = uicontrol('style','edit','parent',cto.hf,'units','normalized',...
        'position',pos.Box_Dx,'string',num2str(cto.box(3)),...
        'fontsize',12,'callback',@ctoCheckValue);
    cto.hBoxText(3) = uicontrol('style','text','parent',cto.hf,'units','normalized',...
        'position',pos.Box_DxText,'string','width',...
        'fontsize',12,'horizontalAlignment','left');
    cto.hBox(4) = uicontrol('style','edit','parent',cto.hf,'units','normalized',...
        'position',pos.Box_Dy,'string',num2str(cto.box(4)),...
        'fontsize',12,'callback',@ctoCheckValue);
    cto.hBoxText(4) = uicontrol('style','text','parent',cto.hf,'units','normalized',...
        'position',pos.Box_DyText,'string','height',...
        'fontsize',12,'horizontalAlignment','left');
   
    cstr = 'Correct Trace Window Selection (in normalized units)';
    cto.hWindowText = uicontrol('style','text','parent',cto.hf,'units','normalized',...
        'position',pos.WindowText,'string',cstr,'horizontalAlignment','left',...
        'fontsize',12);
    cto.hWindow(1) = uicontrol('style','edit','parent',cto.hf,'units','normalized',...
        'position',pos.Window_dx,'string',num2str(cto.window(1)),...
        'fontsize',12,'callback',@ctoCheckValue);
    cto.hWindowText(1) = uicontrol('style','text','parent',cto.hf,'units','normalized',...
        'position',pos.Window_dxText,'string','bottom left (x)',...
        'fontsize',12,'horizontalAlignment','left');
    cto.hWindow(2) = uicontrol('style','edit','parent',cto.hf,'units','normalized',...
        'position',pos.Window_dy,'string',num2str(cto.window(2)),...
        'fontsize',12,'callback',@ctoCheckValue);
    cto.hWindowText(2) = uicontrol('style','text','parent',cto.hf,'units','normalized',...
        'position',pos.Window_dyText,'string','botom left (y)',...
        'fontsize',12,'horizontalAlignment','left');
     cto.hWindow(3) = uicontrol('style','edit','parent',cto.hf,'units','normalized',...
        'position',pos.Window_Dx,'string',num2str(cto.window(3)),...
        'fontsize',12,'callback',@ctoCheckValue);
    cto.hWindowText(3) = uicontrol('style','text','parent',cto.hf,'units','normalized',...
        'position',pos.Window_DxText,'string','width',...
        'fontsize',12,'horizontalAlignment','left');
    cto.hWindow(4) = uicontrol('style','edit','parent',cto.hf,'units','normalized',...
        'position',pos.Window_Dy,'string',num2str(cto.window(4)),...
        'fontsize',12,'callback',@ctoCheckValue);
    cto.hWindowText(4) = uicontrol('style','text','parent',cto.hf,'units','normalized',...
        'position',pos.Window_DyText,'string','height',...
        'fontsize',12,'horizontalAlignment','left');

    cstr = 'Interpolation Scheme:';
    cto.hInterpText = uicontrol('style','text','parent',cto.hf,'units','normalized',...
        'position',pos.InterpText,'string',cstr,'horizontalAlignment','left',...
        'fontsize',12);
    cto.hInterpText = uicontrol('style','popupmenu','parent',cto.hf,'units','normalized',...
        'position',pos.InterpPopUp,'string',{'Spline','Linear'},'horizontalAlignment','left',...
        'fontsize',12,'value',1,'callback',@updateInterp);
    cto.Interp = 1;
    
%     cto.hApply = uicontrol('style','pushbutton','parent',cto.hf,'units','normalized',...
%         'position',pos.Apply,'string','Apply','tooltipstring','Apply changes',...
%         'fontsize',14,'callback',@ctoApply);
    cto.hClose = uicontrol('style','pushbutton','parent',cto.hf,'units','normalized',...
        'position',pos.Close,'string','Close','tooltipstring','Close this window',...
        'fontsize',14,'callback',@ctoClose);
    cto.hRecover = uicontrol('style','pushbutton','parent',cto.hf,'units','normalized',...
        'position',pos.Recover,'string','Recover',...
        'tooltipstring','Enable "Correct trace" pushbutton.  Note current correct trace window will be closed',...
        'fontsize',14,'callback',@ctoRecover,'visible','off');
    if strcmpi(get(H.digitize_1,'enable'),'off')  % "Correct trace" pushbutton disabled
        set(cto.hRecover,'visible','on')
    end  % if strcmpi(get(H.digitize_1,'enable'),'off')
    
    
end  % if lnewFig

setappdata(H.f1,'cto',cto);  % update variable

    function updateInterp(hobj,eventdata)
        % update cto.Interp value when interpolation selection has been
        % changed
        
        cto.Interp = get(hobj,'value');
        setappdata(H.f1,'cto',cto);
    end  % function updateInterp

%     function ctoApply(hobj,eventdata)
%         % this function will copy the values in the edit fields into cto
%         % structure so that they can be used by the main program
%         cto.box = str2num(char(get(cto.hBox,'string')));
%         cto.window = str2num(char(get(cto.hWindow,'string')));
%         setappdata(H.f1,'cto',cto);  % update information
%     end  % function ctoApply

    function ctoCheckValue(hobj,eventdata)
        % this function checks for the validity of the input values and
        % sets back to previous value if unallowed
        
        if any(cto.hBox==hobj)  % edit box is for box selection
            % values should be in pixels and range determined by screen
            % size
            
            rval = round(str2double(get(hobj,'string')));
            indx = find(cto.hBox==hobj);
            
            if ( (rval<1) || (rval>rval_max(indx)) )  % if rval out of range
                rval = cto.box(indx);
            end  % if ( (rval<1) | (rval>rval_max(indx)) )
            
        else  % assume window size
            % values should be in normalized units and range between 0 and
            % 1
            
            rval = str2double(get(hobj,'string'));
            indx = find(cto.hWindow==hobj);
            
            if ( (rval<0) || (rval>1.0) )  % if rval out of range
                rval = cto.window(indx);
            end  % if ( (rval<1) | (rval>rval_max(indx)) )
            
        end  % if any(cto.hBox==hobj)...
        
        set(hobj,'string',rval);
        
        % update values
        cto.box = str2num(char(get(cto.hBox,'string')));
        cto.window = str2num(char(get(cto.hWindow,'string')));
        setappdata(H.f1,'cto',cto);  % update information
    end  % function ctoCheckValue


    function ctoRecover(hobj,eventdata)
        % this function allows the user to recover the "Correct trace"
        % pushbutton functionality.  If there is an error during correct
        % trace window opening, the pushbutton "enable" property is set to
        % "off" and the user loses the ability to continue to use the
        % function.  This recover button is aimed to solve the issue.
        
        % check to see if the correct trace window is currently open
        if isappdata(H.f1,'h1stripe')  % check if correct_trace window is still open
            h1stripe = getappdata(H.f1,'h1stripe');
            if ishandle(h1stripe.f)  % figure window exists
                cbut = questdlg('The Correct Trace window will be closed.  Do you want to continue?',...
                    'Confirmation','Yes','No','Yes');
                if strcmpi(cbut,'No')  % get out of this function
                    return
                else  % close the window
                    delete(h1stripe.f);
                    setappdata(H.f1,'h1stripe',h1stripe);
                end  % if strcmpi(cbut,'No')
            end  % if ishandle(h1stripe)
        end  % if isappdata(H.f1,'h1stripe')
        
        % return functionality of the correct trace button
        set(H.digitize_1,'enable','on');
        
        % turn the Recove button invisible
        set(cto.hRecover,'visible','off');
    
    end  % function ctoRecover

    function ctoClose(hobj,eventdata)
        % this function closes the property setup window
        set(H.mark_start,'enable','on');  % make the start/end pushbutton available
        delete(cto.hf);
    end  % function ctoClose

    function closecto(hobj,eventdata)
        set(H.mark_start,'enable','on');  % make the start/end pushbutton available
    end  % function closecto(hobj,eventdata)

end  % function correct_trace_options



function correct_trace(hObject, eventdata)
global H I0 pt_traces ptrace ptraceSTD ftrend
global pt_tracesY0 pt_start_time pt_end_time
global S

% deactivate the correct trace pushbutton so that it cannot be pushed
% multiple times
set(hObject,'enable','off');
% if the correct trace option window is open, make the Recover pushbutton
% visible
if isappdata(H.f1,'cto')  % if correct trace option variable exists
    cto = getappdata(H.f1,'cto');
    if ishandle(cto.hf)  % if window exists
        set(cto.hRecover,'visible','on');
    end  % if ishandle(cto.hf)
end  % if isappdata(H.f1,'cto')

% initialize
BWstripe=[];
BWstripeNULL=[];
BWS=[];
number_of_traces=numel(pt_traces);
leveldef = 10;  % default value for BW threshold

% pre-set values
npix_min = 5;  % number of pixels below which objects are deleted

% Select trace with mouse parameters
dpixels = 10;  % number of pixels around the selected line for determination of the object thickness
% these pixels will be used to calculate the peak luminescence from which
% the cutoff level is determined
% point-dpixels:point+dpixels
% are considered in max calculation

thresLum = 0.5;
% threshold (fraction) of luminescence below which the pixels are
% considered background for the purpose of determining the object thickness

defcol = [0.28 0.28 0.99];
% default colour for impoint objects

% close if there is any previous correct_trace window open
if isappdata(H.f1,'h1stripe')
    h1stripe = getappdata(H.f1,'h1stripe');
    if isvalid(h1stripe.f)  % if figure window still exists
        delete(h1stripe.f);
    end  % if any(ishandle(h1stripe.f))
    clear h1stripe;
    rmappdata(H.f1,'h1stripe');
end  % if isappdata(H.f1,'h1stripe')

% check if this is a curved trace
itsacurvedtrace= 'No';
lcurve = false;
if strcmpi(get(H.curvButton,'state'),'on')  % curvature needs to be considered
    itsacurvedtrace = 'Yes';
    lcurve = true;
end  % if strcmpi(get(H.curvButton,'state'),'on')
% itsacurvedtrace=questdlg('Is this a seismogram with curved traces?','Curved trace option','Yes','No','No');
% lcurve = true;

% Select trace with mouse
% h=msgbox('Click with mouse anywhere on the trace you want to correct.','Correct trace','help',[],'WindowStyle','modal');
% waitfor(h);

[xc,yc]=ginput(1);

% Find the trace number
d=zeros(number_of_traces,1);
for i=1:number_of_traces
    d(i)=min( (get(pt_traces(i),'XDATA')-xc).^2 + (get(pt_traces(i),'YDATA')-yc).^2 );
end
[~,iTRACE]=min(d);

xtemp=get(pt_traces(iTRACE),'XDATA'); ytemp=get(pt_traces(iTRACE),'YDATA');
[~,ii]=min( (xtemp -xc).^2 + (ytemp-yc).^2 );
XC=xtemp(ii); YC=ytemp(ii);

if isappdata(H.f1,'cto')  % modified box values exist
    cto = getappdata(H.f1,'cto');
    dx = cto.box(1);
    dy = cto.box(2);
    Dx = cto.box(3);
    Dy = cto.box(4);
else  % use default
    Dx = 185; Dy = 600; dx = 92; dy = 300;
    %dx = 300; dy = 30;  Dx = 2*dx; Dy = 2*dy; % dx = 300, dy = 70, Dx = 600; Dy = 140; default
end  % if isappdata(H.f1,'cto')...
position=[max(1,XC-dx) max(1,YC-dy) Dx Dy];
position(3)=min(size(I0,2)-position(1),position(3));
position(4)=min(size(I0,1)-position(2),position(4));

% create rectangle to get the problematic patch
hrec = imrect(H.ax1,position);
if strcmpi(itsacurvedtrace,'YES')
    setPositionConstraintFcn(hrec,@symetric_fixedCenterY_fcn); 
end

position=wait(hrec);
delete(hrec);
position=round(position);
% correct position
position(1)=min(size(I0,2),max(1,position(1)));
position(2)=min(size(I0,1),max(1,position(2)));
position(3)=min(size(I0,2)-position(1),position(3));
position(4)=min(size(I0,1)-position(2),position(4));

stripe=imcrop(I0,position);  % original image to be processed
[r,c]=size(stripe);

xpatch=position(1)+[0:position(3)];
%xpatch=xpatch(xpatch>=x(1) & xpatch<=x(end));
ypatch=position(2)+[0:position(4)];


% create a mapping from big image to stripe
[ypa,xpa]=ndgrid(ypatch,xpatch);
patch_indexes=sub2ind(size(I0),ypa(:),xpa(:));

% indxM = find_trace_mask;
% 
%     function indxM = find_trace_mask
%         global S
        ineg= cellfun(@isempty,{S.TraceNum});
        [S(ineg).TraceNum]=deal(-1);
        
        % set initial mask based upon other trace objects in the rectangle
        indx = find([S.TraceNum]~=iTRACE);  % traces and time marks belonging to other traces
        tBoxes = cell2mat({S(indx).BoundingBox}');  % bounding box of all objects
        % check x range, y range and combine to get indices of objects that
        % potentially have parts within the region of interest
        indxX = ((tBoxes(:,1)>xpatch(1) & tBoxes(:,1)<xpatch(end)) | ... % starting position inside x range
            (tBoxes(:,1)+tBoxes(:,3) > xpatch(1) & tBoxes(:,1)+tBoxes(:,3) < xpatch(end)) | ... % ending position inside x range
            (tBoxes(:,1)<xpatch(1) & tBoxes(:,1)+tBoxes(:,3) > xpatch(end))); % starting and ending covering the x range
        indxY = ((tBoxes(:,2)>ypatch(1) & tBoxes(:,2)<ypatch(end)) | ... % starting position inside y range
            (tBoxes(:,2)+tBoxes(:,4) > ypatch(1) & tBoxes(:,2)+tBoxes(:,4) < ypatch(end)) | ... % ending position inside y range
            (tBoxes(:,2)<ypatch(1) & tBoxes(:,2)+tBoxes(:,4) > ypatch(end))); % starting and ending covering the y range
        indxM = indx(indxX & indxY);  % indices satisfying both indxX and indxY
        indxM = intersect(cell2mat({S(indxM).PixelIdxList}'),patch_indexes);  % all pixel indices that are inside
        [ym xm] = ind2sub(size(I0),indxM);
        xm = xm - xpatch(1)+1;  ym = ym - ypatch(1)+1;  % new coordinates
        indxM = sub2ind(size(stripe),ym,xm);  % mask in the new coordinates
        
%     end

%% Launch GUI for single trace analysis and digitization
if isappdata(H.f1,'cto')  % modified box values exist
    pfig = cto.window;
else  % use default
    wid = 0.6;  % width of the correct trace window
    hgt = 0.3;  % height of the correct trace window
    xp = (1.0 - wid)/2.0;
    yp = (1.0 - hgt)/2.0;
    pfig = [xp yp wid hgt];  % default values [0.1 0.5 0.8 0.3]
    pfig = [0.107 0.349 0.462 0.432];
end  % if isappdata(H.f1,'cto')...
h1stripe.f=figure('Color','w','WindowStyle','normal','Name',['Edit trace ' num2str(iTRACE)] ,'NumberTitle','Off',...
    'Menubar','none','ToolBar','figure','Units','Normalized','Position',pfig);
set(h1stripe.f,'DeleteFcn',@deleteFigure);  % function to be called when figure is deleted

% get default pointer properties
defPointer.Pointer = get(h1stripe.f,'Pointer');
defPointer.PointerShapeCData = get(h1stripe.f,'PointerShapeCData');
defPointer.PointerShapeHotSpot = get(h1stripe.f,'PointerShapeHotSpot');

% modify tool bar
temp= findall(findall(h1stripe.f,'Type','uitoolbar'));
htool.pan = temp(10);
htool.zoomOut = temp(11);
htool.zoomIn = temp(12);
delete(temp([2:9 13:17]));
% % Reorder tools
% set(temp(1),'Children',[temp(4:end-1);temp(2:3);temp(end)])

% add some tools
bColorSchemeS=uitoggletool(temp(1),'Tag','Change colors','Cdata',imread('icons/ColorScheme.png','BackgroundColor',[1 1 1]),...
    'Separator','on','TooltipString','Change classification colors',...
    'HandleVisibility','off','ClickedCallback',@update_classplotS);
htool.color = bColorSchemeS;

htool.mask = uitoggletool(temp(1),'Tag','Mask Image','Cdata',imread('icons/mask.png','BackgroundColor',[1 1 1]),...
    'Separator','on','TooltipString','Click to cycle between masked image and original image',...
    'HandleVisibility','on','state','on','ClickedCallback',@mask_OnOff);

htool.tracePoints = uipushtool(temp(1),'Tag','Trace Line','Cdata',imread('icons/fitspline.png','BackgroundColor',[1 1 1]),...
    'Separator','on','TooltipString','Click to activate tracing line through mouse clicks',...
    'HandleVisibility','on','ClickedCallback',@trace_curve);

htool.contrast = uipushtool(temp(1),'Tag','Change Image Contrast','Cdata',imread('icons/contrast-icon.png','BackgroundColor',[1 1 1]),...
    'Separator','on','TooltipString','Adjust contrast',...
    'HandleVisibility','on','ClickedCallback',@adjust_contrastS);

% htool.mirror = uipushtool(temp(1),'Tag','Select region mirrored','Cdata',imread('select_MirrorS.png','BackgroundColor',[1 1 1]),...
%     'Separator','on','TooltipString','Draw a symmetric region',...
%     'HandleVisibility','on','ClickedCallback',@select_region_mirrorS);
% 

hretrieve_region = uitoggletool(temp(1),'Tag','Retrieve Region','Cdata',imread('icons/retrieveR.png','BackgroundColor',[1 1 1]),...
    'Separator','on','TooltipString','Draw a region to retrieve original image',...
    'HandleVisibility','on','ClickedCallback',@retrieve_regionS);
htool.retrieveRegion = hretrieve_region;

hrm_region = uitoggletool(temp(1),'Tag','Remove Region','Cdata',imread('icons/removeS.png','BackgroundColor',[1 1 1]),...
    'Separator','on','TooltipString','Draw a region to remove',...
    'HandleVisibility','on','ClickedCallback',@remove_regionS);
htool.removeRegion = hrm_region;

htool.line = uitoggletool(temp(1),'Tag','Seperate','Cdata',imread('icons/seperateS.png','BackgroundColor',[1 1 1]),...
    'Separator','on','TooltipString','Draw a line to remove',...
    'HandleVisibility','on','ClickedCallback',@remove_along_pathS);
hrm_line = htool.line;

htundo=uipushtool(temp(1),'Tag','Undo last','Cdata',imread('icons/undo.jpeg'),...
    'Separator','on','TooltipString','Undo last action',...
    'HandleVisibility','on','Enable','off','ClickedCallback',@UNDO);

htool.classification = uitoggletool(temp(1),'Tag','Show clasification','Cdata',imread('icons/show_Class.png','BackgroundColor',[1 1 1]),...
    'Separator','on','TooltipString','Calculate and show classification',...
    'HandleVisibility','on','ClickedCallback',@SeperateS,'State','off');
htool.reject = uipushtool(temp(1),'Tag','s2Reject','Cdata',imread('icons/reject.png','BackgroundColor',[1 1 1]),...
    'Separator','on','TooltipString','Select object to ckassify as rejected',...
    'HandleVisibility','on','ClickedCallback',@modify_classification,'Enable','off');
htool.tm = uipushtool(temp(1),'Tag','s2Timemark','Cdata',imread('icons/timemark.png','BackgroundColor',[1 1 1]),...
    'Separator','on','TooltipString','Select object to ckassify as timemark',...
    'HandleVisibility','on','ClickedCallback',@modify_classification,'Enable','off');
htool.trace = uipushtool(temp(1),'Tag','s2Trace','Cdata',imread('icons/trace.png','BackgroundColor',[1 1 1]),...
    'Separator','on','TooltipString','Select object to ckassify as trace',...
    'HandleVisibility','on','ClickedCallback',@modify_classification,'Enable','off');
% hbut_draw=uipushtool(temp(1),'Tag','drawmask','Cdata',zeros(16,16,3,'uint8'),... %imread('drawmask.png','BackgroundColor',[1 1 1])
%     'Separator','on','TooltipString','Click to create a mask for the current trace (currently inactive)',...
%     'HandleVisibility','on','ClickedCallback',@drawmask,'Enable','off');
redbt=zeros(16,16,3,'uint8'); redbt(:,:,1)=255;
htool.recalculate = uipushtool(temp(1),'Tag','s2Trace','Cdata',redbt,...
    'Separator','on','TooltipString','Recalculate classification',...
    'HandleVisibility','on','ClickedCallback',@CaLLClassify_from_scratchS,'Enable','off');
clear redbt;

if lcurve  % curved trace button available
curvButton=uitoggletool(temp(1),'Tag','Measure tick length','Cdata',imread('icons/CurvedTraces.png'),...
    'Separator','on','TooltipString','Click to model and correct curved offsets',...
    'HandleVisibility','on','ClickedCallback',@extract_curvature_correctionS,'state','on');
end

htool.whole = uipushtool(temp(1),'Tag','Show whole image','Cdata',imread('icons/whole_seismogram.png','BackgroundColor',[1 1 1]),...
    'Separator','on','TooltipString','Show whole seismogram',...
    'HandleVisibility','on','ClickedCallback',@zoom2all);

hbdigitize_single=uipushtool(temp(1),'Tag','Digitize trace','Cdata',imread('icons/digitize1.png','BackgroundColor',[1 1 1]),...
    'Separator','on','TooltipString','Digitize trace',...
    'HandleVisibility','on','ClickedCallback',@Digitize_Single,'Enable','off');

h1stripe.tbar=findall(findall(h1stripe.f,'Type','uitoolbar'));  % toolbar items

% order tools
if lcurve  % make curved trace button available
htmp = [htool.color; htool.tracePoints;
    htool.contrast; htool.mask; htool.retrieveRegion; htool.removeRegion; htool.line; 
    htundo;
    htool.classification; htool.reject; htool.tm; htool.trace; htool.recalculate;
    curvButton;
    hbdigitize_single;
    htool.zoomIn; htool.zoomOut; htool.pan; htool.whole;
    ];
else % no curved trace button
htmp = [htool.color; htool.tracePoints;
    htool.contrast; htool.mask; htool.retrieveRegion; htool.removeRegion; htool.line; 
    htundo;
    htool.classification; htool.reject; htool.tm; htool.trace; htool.recalculate;
    hbdigitize_single;
    htool.zoomIn; htool.zoomOut; htool.pan; htool.whole;
    ];
end  % if strcmpi(itsacurvedtrace,'YES')
set(h1stripe.tbar(1),'children',htmp(end:-1:1));
hbuttons = htmp;
%clear htool;

hToolbar = findall(h1stripe.f,'tag','FigureToolBar');  % toolbar handl
drawnow  % needed before jToolbar line
% java for more flexibility
jToolbar = get(get(hToolbar,'JavaContainer'),'ComponentPeer');
drawnow

% set value of the main window threshold
level=str2double(get(H.edit_luminance,'string'));
% check to make sure this is a valid value (needs to be between 1 and 99)
lok = false;  % lok becomes true if the value is valid
if ~isnan(level)  % numerical value
    if level>0 & level<100  % valid value
        lok = true;
    end  % if level>0 & level<100
end  % if ~isnan
if ~lok  % if invalid value -> set back to default
    disp(['Warning (correct_trace): BW threshold value = ', num2str(level)]);
    disp(['Setting back to the default value: ', num2str(leveldef)]);
    level = leveldef;
    % set the original value back to default
    set(H.edit_luminance,'string',num2str(leveldef));
end  % if ~lok

jModel = javax.swing.SpinnerNumberModel(level,1,99,1);
jSpinner = javax.swing.JSpinner(jModel);
jSpinner =handle(jSpinner,'callbackproperties');
jSpinner.setToolTipText('Change binary threshold');
jSpinner.setSize(70,25);
jSpinner(1).setPreferredSize(java.awt.Dimension(70,25));
jSpinner.setMaximumSize(java.awt.Dimension(100,25));

Jbox=javax.swing.Box.createHorizontalGlue();
Jbox.setSize(300,25);

if lcurve
    jToolbar.add(Jbox,37);
    jToolbar.add(jSpinner,38);
else  % no curved trace button available
    jToolbar.add(Jbox,35);
    jToolbar.add(jSpinner,36);
end  % if lcurve

%@bShow_Object_assignments
jToolbar.repaint;
jToolbar.revalidate;

h1stripe.axI=axes('Units','Normalized','Position',[0.05 0.2 0.94 0.79],...
    'Xtick',[],'Ytick',[],'box','on','Parent',h1stripe.f);

h1stripe.close=uicontrol('Style', 'pushbutton','Parent',h1stripe.f,'units','normalized',...
    'String', 'Close','Position', [0.005 .001 0.14 0.12],'Callback',@Closethefigure);
h1stripe.applyb=uicontrol('Style', 'pushbutton','Parent',h1stripe.f,'units','normalized',...
    'String', '<html><center>Apply<br>Skip NaN</center>','Position', [0.155 0.001 0.14 0.12],...
    'tag','skipNaN','Callback',@Apply_to_main,'HorizontalAlignment','center');
h1stripe.applyb2=uicontrol('Style', 'pushbutton','Parent',h1stripe.f,'units','normalized',...
    'String', '<html><center>Apply<br>Include NaN</center>','Position', [0.305 0.001 0.14 0.12],...
    'tag','includeNaN','Callback',@Apply_to_main,'HorizontalAlignment','center');

SCLA=[];

% plot trace
indxM0 = indxM;  % initial mask
stripeM = stripe;
stripeM(indxM) = 0;  % apply mask
himS = image(xpatch,ypatch,stripeM,'parent',h1stripe.axI); colormap(h1stripe.axI,gray(256));
%image(xpatch,ypatch,stripe,'parent',h1stripe.axI); colormap(h1stripe.axI,gray(256));

setappdata(H.f1,'h1stripe',h1stripe);  % save figure information

    function trace_curve(hobj,eventdata)
        % this function allows user to trace out lines
        
        resetZoom;  % make sure all zoom/pan buttons are off
        
        % deactivate some buttons
        set(hrm_line,'state','off','enable','off');
        set(hrm_region,'state','off','enable','off');
        set(htool.retrieveRegion,'state','off','enable','off');
        if strcmpi(get(htool.mask,'state'),'on')  % turn mask off
            set(htool.mask,'state','off','enable','off');
            update_plotS;
        end
        
        % determine the interpolation method
        cmethod = 'spline';  % default value
        if isappdata(H.f1,'cto')
            cto = getappdata(H.f1,'cto');
            if isfield(cto,'Interp')  % if Interp field exists
                if cto.Interp==2  % linear
                    cmethod = 'linear';
                end  % if cto.Interp==2
            end  % if isfield(cto,'Interp');
        end  % if isappdata(H.f1,'cto')
        
        hpinpoint = [];
        xrange = [0 0];
        if isappdata(h1stripe.axI,'hpinpoint')  % if hpinpoint exists
            hpinpoint = getappdata(h1stripe.axI,'hpinpoint');
            
            % find x ranges of existing lines
            xrange = [];
            for k = 1:numel(hpinpoint)
                htmp = hpinpoint(k).points;  % all points
                if ~isempty(htmp)
                    xtmp = [];
                    for m = 1:numel(htmp)
                        if isvalid(htmp(m))
                            rpos = round(getPosition(htmp(m)));
                            xtmp(end+1) = rpos(1);
                        end  % isvalid(htmp(m))
                    end  % for m = 1:numel(htmp)
                    xrange = [xrange; min(xtmp) max(xtmp)];
                else  % if no point
                    xrange = [xrange; -1 -1];
                end  % if ~isempty(hpoints(k).points)
            end  % for k = 1:numel(hpinpoint)
        end
        
        try, set(h1stripe.f,'WindowButtonDownFcn',''), end
        try, set(h1stripe.f,'WindowButtonMotionFcn',''), end
        try, set(h1stripe.f,'WindowButtonUpFcn',''), end
        
        hppNew = [];
        iset = numel(hpinpoint)+1;
        [x y button] = ginput(1);  % initial point
        while (button==1)  % while the left mouse button has been pressed
            x = round(x); y = round(y);
            htmp = impoint(h1stripe.axI,x,y);  % movable point
            anFnc = @(pos) updatePinPoint(htmp,pos);
            addNewPositionCallback(htmp,anFnc);
            setColor(htmp,defcol);  % default colour
            set(htmp,'DeleteFcn',@point_deleted);
            
            % check which group this point belongs to
            indx = find(x > xrange(:,1) & x < xrange(:,2));
            
            if numel(indx)>0  % if within existing sets
                if (numel(indx)>1)  % multiple sets identified
                    rdist = [];
                    for k = indx'
                        rtmp = [];
                        htmp = hpinpoint(k).points;  % all points
                        for m = 1:numel(htmp)
                            rpos = round(getPosition(htmp(m)));
                            rtmp(end+1) = dist([rpos(1) x; rpos(2) y]);
                        end  % for m = 1:numel(htmp)
                        rdist(end+1) = min(rtmp);
                    end
                    [rtmp itmp] = min(rdist);
                    nset = indx(itmp);  % set to which x and y closest
                else
                    nset = indx;
                end  % if (numel(find(indx))>1)
                
                set(htmp,'tag',num2str(nset));
                
                % check to make sure that x values in this set do not
                % overlap
                rtmp = round(hpinpoint(nset).X)-x;  % get x values
                itmp = find(rtmp==0);
                ladd = true;
                if ~isempty(itmp)  % there are overlapping x positions
                    if isempty(find(rtmp==1))  % if there are no existing point one pixel to the right
                        x = x + 1;
                    elseif isempty(find(rtmp==-1)) % availability to the left
                        x = x - 1;
                    else
                        errordlg('You cannot place a point at the same x position with an existing point','Point Addition');
                        ladd = false;
                        delete(htmp);  % remove the point
                    end  % if isempty(rind(rtmp==1))
                end  % if ~isempty(itmp)
                
                if ladd  % if point to be added
                % update points and line
                hpp = [hpinpoint(nset).points; htmp];
                hpinpoint(nset).points = hpp;
                setappdata(h1stripe.axI,'hpinpoint',hpinpoint);
                
                X = []; X0 = [];
                Y = []; Y0 = [];
                for k = 1:numel(hpp)  % build data
                    rpos = round(getPosition(hpp(k)));
                    X0(end+1) = rpos(1);
                    Y0(end+1) = rpos(2);
                end  % for k = 1:numel(hppNew)
                [X indx] = sort(X0);  % sort to make sure X is in ascending order
                Y = Y0(indx);
                
                hpinpoint(nset).X = X0;
                hpinpoint(nset).Y = Y0;
                
                xpix = round(X(1)):round(X(end));
                
                % fit spline
                rtmp = diff(X);
                indx = [(rtmp~=0) true];  % last true to include the end point
                if numel(indx)>1
                    ypix = interp1(X(indx),Y(indx),xpix,cmethod);
                    %ypix = spline(X(indx),Y(indx),xpix);  % cubic spline interpolation
                    
                    set(hpinpoint(nset).line,'XDATA',xpix,'YDATA',ypix);
                else  % delete this set
                    hpinpoint(nset).points = [];
                    delete(hpinpoint(nset).line);
                end  % if numel(indx)>1
                
                end
                
            else  % no set
                
                
                set(htmp,'tag',num2str(iset));
                % change size of the cross and circle
                %     hchil = get(htmp,'children');
                %     set(hchil(1),'markersize',25);  % change marker size for the cross
                %     set(hchil(2),'markersize',15);  % change marker size for circle
                
                hppNew = [hppNew; htmp];  % add this object to new set of points
            end
            
            
            % get next point
            [x y button] = ginput(1);  % point input
        end  % while (button==1)
        
        if numel(hppNew)>1
            X = []; X0 = [];
            Y = []; Y0 = [];
            for k = 1:numel(hppNew)  % build data
                rpos = round(getPosition(hppNew(k)));
                X0(end+1) = rpos(1);
                Y0(end+1) = rpos(2);
            end  % for k = 1:numel(hppNew)
            [X indx] = sort(X0);  % sort to make sure X is in ascending order
            Y = Y0(indx);
            
            xpix = round(X(1)):round(X(end));
            
            % fit spline
            rtmp = diff(X);
            indx = [(rtmp~=0) true];  % last true to include the end point
            
            if numel(indx)>1  % need at least two points for this to be valid
            ypix = interp1(X(indx),Y(indx),xpix,cmethod);
            %ypix = spline(X(indx),Y(indx),xpix);  % cubic spline interpolation
            
            hline = line(xpix,ypix,'color','r','parent',h1stripe.axI);
            
            % put line to the back so that points can be moved
            htmp = get(h1stripe.axI,'children');
            htmp = [htmp(htmp~=hline&htmp~=himS); hline; himS];
            set(h1stripe.axI,'children',htmp);
            
            hpinpoint(iset).points = hppNew;
            hpinpoint(iset).line = hline;
            hpinpoint(iset).X = X0;
            hpinpoint(iset).Y = Y0;
            
            else  % delete this set
                for k = 1:numel(hppNew)
                    set(hppNew(k),'DeleteFcn','');
                end  % for k = 1:numel(hppNew)
                delete(hppNew);
            end  % if numel(indx)>2
        else
            for k = 1:numel(hppNew)
                set(hppNew(k),'DeleteFcn','');
            end  % for k = 1:numel(hppNew)
            delete(hppNew);
        end  % if numel(hppNew)>1
        
        setappdata(h1stripe.axI,'hpinpoint',hpinpoint);  % update info
        
        set([htool.pan; htool.zoomOut; htool.zoomIn],'enable','on');
        
        try, set(h1stripe.f,'WindowButtonDownFcn',''), end
        try, set(h1stripe.f,'WindowButtonMotionFcn',''), end
        try, set(h1stripe.f,'WindowButtonUpFcn',''), end
        
        resetZoom;
    end


    function point_deleted(hobj,eventdata)
        
        try, set(h1stripe.f,'WindowButtonDownFcn',''), end
        try, set(h1stripe.f,'WindowButtonMotionFcn',''), end
        try, set(h1stripe.f,'WindowButtonUpFcn',''), end

        hpinpoint = getappdata(h1stripe.axI,'hpinpoint');
        
        %                 get(hobj,'tag')
        %                 get(hobj,'type')
        iset = str2double(get(hobj,'tag'));
        
        hppNew = hpinpoint(iset).points;
        hline = hpinpoint(iset).line;
        hchil = get(hobj,'children');  % get circle and plus positions
        x = get(hchil(1),'xdata');
        y = get(hchil(1),'ydata');
        
        X = []; X0 = [];
        Y = []; Y0 = [];
        indx = [];
        for k = 1:numel(hppNew)  % build data
            if isvalid(hppNew(k))  % any points other than the deleted one
                rpos = getPosition(hppNew(k));
                X0(end+1) = rpos(1);
                Y0(end+1) = rpos(2);
            else  % point deleted
                indx(end+1) = k;
            end
        end  % for k = 1:numel(hppNew)
        if ~isempty(indx)
            hppNew(indx) = []; % remove impoint object
        end  % if ~isempty(indx)
        [X indx] = sort(X0);  % sort to make sure X is in ascending order
        Y = Y0(indx);
        hpinpoint(iset).X = X0;
        hpinpoint(iset).Y = Y0;
        
        xpix = round(X(1)):round(X(end));
        
        % determine interpolation scheme
        cmethod = 'spline';  % default value
        if isappdata(H.f1,'cto')
            cto = getappdata(H.f1,'cto');
            if isfield(cto,'Interp')  % if Interp field exists
                if cto.Interp==2  % linear
                    cmethod = 'linear';
                end  % if cto.Interp==2
            end  % if isfield(cto,'Interp')
        end  % if isappdata(H.f1,'cto')
        
        % fit spline
        rtmp = diff(X);
        indx = [(rtmp~=0) true];  % last true to include the end point
        if numel(indx)>1  % enough points for interpolation
            ypix = interp1(X(indx),Y(indx),xpix,cmethod);
            %ypix = spline(X,Y,xpix);  % cubic spline interpolation
            
            set(hline,'XDATA',xpix,'YDATA',ypix);
            
            hpinpoint(iset).points = hppNew;
        else  %  not enough points for interpolation
            delete(hline);
            
            for k = 1:numel(hppNew)
                htmp = hppNew(k);
                set(htmp,'DeleteFcn','');
                delete(htmp);
            end  % for k = 1:numel(hppNew)
            
            hpinpoint(iset).points = [];
            hpinpoint(iset).line = [];
        end  % if numel(indx)>1 ...
        
        setappdata(h1stripe.axI,'hpinpoint',hpinpoint);
        
        try, set(h1stripe.f,'WindowButtonDownFcn',''), end
        try, set(h1stripe.f,'WindowButtonMotionFcn',''), end
        try, set(h1stripe.f,'WindowButtonUpFcn',''), end
        
        resetZoom;
    end

    function updatePinPoint(hobj, pos)
        %hobj = gco;  % object that was moved
        % hobj = get(gco,'uicontextmenu');
        %         pos = round(pos);
        %         setPosition(hobj,pos); % position of impoint objects cannot be
        %         changed
        
        hpinpoint = getappdata(h1stripe.axI,'hpinpoint');
        
        iset = str2double(get(hobj,'tag'));
                    
        % check to see if the x position is independent
        x = round(pos(1));
        y = pos(2);
        rtmp = round(hpinpoint(iset).X)-x;  % get x values
        itmp = find(rtmp==0);
        ladd = true;
        if ~isempty(itmp)  % there are overlapping x positions
            if isempty(find(rtmp==1))  % if there are no existing point one pixel to the right
                x = x + 1;
            elseif isempty(find(rtmp==-1)) % availability to the left
                x = x - 1;
            else  
            end  % if isempty(rind(rtmp==1))
        end  % if ~isempty(itmp)
        
        hppNew = hpinpoint(iset).points;
        hline = hpinpoint(iset).line;
        
        X = []; X0 = [];
        Y = []; Y0 = [];
        for k = 1:numel(hppNew)  % build data
            if isvalid(hppNew(k)) & hppNew(k)~=hobj;
                rpos = round(getPosition(hppNew(k)));
                X0(end+1) = rpos(1);
                Y0(end+1) = rpos(2);
            elseif hppNew(k)==hobj;
                X0(end+1) = x;
                Y0(end+1) = y;
            end  % if isvalid(hppNew(k))
        end  % for k = 1:numel(hppNew)
        [X indx] = sort(X0);  % sort to make sure X is in ascending order
        Y = Y0(indx);
        hpinpoint(iset).X = X0;
        hpinpoint(iset).Y = Y0;
        
        xpix = round(X(1)):round(X(end));
        
        % determine the interpolation method
        cmethod = 'spline';  % default value
        if isappdata(H.f1,'cto')
            cto = getappdata(H.f1,'cto');
            if isfield(cto,'Interp')
                if cto.Interp==2  % linear
                    cmethod = 'linear';
                end  % if cto.Interp==2
            end  % if isfield(cto,'Interp')
        end  % if isappdata(H.f1,'cto')
        
        % fit spline
        rtmp = diff(X);
        indx = [(rtmp~=0) true];  % last true to include end point
        ypix = interp1(X(indx),Y(indx),xpix,cmethod);
        %ypix = spline(X(indx),Y(indx),xpix);  % cubic spline interpolation
        
        set(hline,'XDATA',xpix,'YDATA',ypix);
        
        setappdata(h1stripe.axI,'hpinpoint',hpinpoint);
        resetZoom;
    end

    function mask_OnOff(hObject,eventdata)
        % this function allows the user to turn the mask on and off so that
        % the original image can be checked
        
        resetZoom;  % make sure all zoom/pan buttons are off
        
%        if strcmpi(get(htool.mask,'off'))  % mask turned off
            set(hrm_line,'state','off');
            set(hrm_region,'state','off');
            set(htool.retrieveRegion,'state','off');
%        end  % if strcmpi(get(htool.mask,'off'))
        update_plotS;
    end

    function retrieve_regionS(hObject,eventdata)
        % this function allows users to remove mask
        
        resetZoom;  % make sure all zoom/pan buttons are off

        if strcmpi(get(htool.mask,'state'),'off')
            set(htool.mask,'on');
            update_plotS;  % update plot so that the mask exists
        end
        set(hrm_line,'state','off');
        set(hrm_region,'state','off');
        
        try
            while strcmpi(get(htool.retrieveRegion,'state'),'on')
                
                hr=imfreehand(h1stripe.axI);
                try  % this part runs when h is not empty
                    hr.Deletable = false;
                    % stripe0=stripe; % save an undo copy
                    % stripe(h.createMask)=0;
                     indxM0 = indxM;
%                     indxM = union(indxM,find(h.createMask));
                    indxM = setdiff(indxM,find(hr.createMask));
                    delete(hr);
                    update_plotS;
                    set(htundo,'Enable','on');
                    %setappdata(htundo,'stripe0',stripe0);
                    % fixes the bug(?) with imfreehand... PETROS
                    try, set(h1stripe.f,'WindowButtonDownFcn',''), end
                    try, set(h1stripe.f,'WindowButtonMotionFcn',''), end
                    try, set(h1stripe.f,'WindowButtonUpFcn',''), end
                end
           end
        end
    end

    function zoom2all(hObject, eventdata)
        set(h1stripe.axI,'Xlim',[xpatch(1) xpatch(end)],'Ylim',[ypatch(1) ypatch(end)],'Xaxislocation','Top')
        drawnow;
    end

    function constrained_position = symetric_fixedCenterY_fcn(new_position)
        constrained_position=new_position;
        constrained_position(2)=YC-new_position(4)/2;
    end

    function update_plotS(hObject, ~)
        xylimits=axis;
        %cla;
        
        % remove function associated with deleting of manually placed
        % points
        if isappdata(h1stripe.axI,'hpinpoint')  % traced by hand
            hpinpoint = getappdata(h1stripe.axI,'hpinpoint');
            for m = 1:numel(hpinpoint)
                hpp = hpinpoint(m).points;  % get all points
                
                for k = 1:numel(hpp)
                    if isvalid(hpp(k))  % if the point still exists
                        % the deleted points are still residing within
                        % the hpinpoint variable
                        
                        set(hpp(k),'DeleteFcn',{});
                        % set function associated with deletion of the
                        % object to nothing - otherwise there will be error
                        % messages as the window closes
                    end  % if any(ishandle(hpp(k)))
                end  % for k = 1:numel(hpp)
            end   % for m = 1:numel(hpinpoint)
        end  % if isappdata(h1stripe.axI,'hpinpoint')

        delete(himS);
        stripeM = stripe;
        if strcmpi(get(htool.mask,'state'),'on')  % mask is on
            stripeM(indxM) = 0;
        end
        %set(himS,'CDATA',stripeM);
        himS = imagesc(xpatch,ypatch,stripeM,'Parent',h1stripe.axI);colormap(h1stripe.axI,gray(256));
        
        % recover traced points and lines if they exist
        if isappdata(h1stripe.axI,'hpinpoint')
            % determine the interpolation method
            cmethod = 'spline';  % default value
            if isappdata(H.f1,'cto')
                cto = getappdata(H.f1,'cto');
                if isfield(cto,'Interp')
                    if cto.Interp==2  % linear
                        cmethod = 'linear';
                    end  % if cto.Interp==2
                end  % if isfield(cto,'Interp')
            end  % if isappdata(H.f1,'cto')
            
            hpinpoint = getappdata(h1stripe.axI,'hpinpoint');  % existing points
            
            for k = 1:numel(hpinpoint)  % loop over groups
                if ~isempty(hpinpoint(k).points)
                    X = hpinpoint(k).X;
                    Y = hpinpoint(k).Y;
                    
                    % do the line first
                    xpix = round(min(X)):round(max(X));
                    rtmp = diff(X);
                    indx = [(rtmp~=0) true];  % last true to include the end point
                    if numel(indx)>1  % need at least two points
                        ypix = interp1(X(indx),Y(indx),xpix,cmethod);
                        %                 ypix = spline(X(indx),Y(indx),xpix);  % cubic spline interpolation
                        hline = line(xpix,ypix,'color','r','parent',h1stripe.axI);
                        
                        % add movable points
                        hpp = [];  % collection of handles
                        for m = 1:numel(X)  % loop over each point
                            htmp = impoint(h1stripe.axI,X(m),Y(m));  % movable point
                            anFnc = @(pos) updatePinPoint(htmp,pos);
                            addNewPositionCallback(htmp,anFnc);
                            setColor(htmp,defcol);  % default colour
                            set(htmp,'DeleteFcn',@point_deleted);
                            set(htmp,'tag',num2str(k));
                            
                            hpp = [hpp; htmp];
                        end  % for m = 1:numel(X)
                        hpinpoint(k).points = hpp;
                        hpinpoint(k).line = hline;
                    else  % not enough points
                        for m = 1:numel(hpp)
                            htmp = hpp(m);
                            set(htmp,'DeleteFcn','');
                            delete(htmp);
                        end  % for m = 1:numel(hpp)
                        hline = hpinpoint(k).line;
                        delete(hline);
                        hpinpoint(k).points = [];
                        hpinpoint(k).line = [];
                    end  % if numel(indx)>1
                end  % if ~isempty(hpinpoint(k).points)
            end  % for k = 1:numel(hpinpoint)
            
            setappdata(h1stripe.axI,'hpinpoint', hpinpoint);
        end  % if isappdata(h1stripe.axI,'hpinpoin')
        
        axis(xylimits);
    end

    function remove_regionS(hObject, eventdata)
        %        global stripe0
        
        resetZoom;  % make sure all zoom/pan buttons are off

        set(hrm_line,'state','off');
        set(htool.mask,'state','on');
        set(htool.retrieveRegion,'state','off');
        
        try
            while strcmpi(get(hrm_region,'state'),'on')
                
                    try, set(h1stripe.f,'WindowButtonDownFcn',''), end
                    try, set(h1stripe.f,'WindowButtonMotionFcn',''), end
                    try, set(h1stripe.f,'WindowButtonUpFcn',''), end
                h=imfreehand(h1stripe.axI);
                try  % this part runs when h is not empty
                    h.Deletable = false;
                    % stripe0=stripe; % save an undo copy
                    % stripe(h.createMask)=0;
                    indxM0 = indxM;
                    indxM = union(indxM,find(h.createMask));
                    delete(h);
                    update_plotS;
                    set(htundo,'Enable','on');
                    %setappdata(htundo,'stripe0',stripe0);
                    % fixes the bug(?) with imfreehand... PETROS
                    try, set(h1stripe.f,'WindowButtonDownFcn',''), end
                    try, set(h1stripe.f,'WindowButtonMotionFcn',''), end
                    try, set(h1stripe.f,'WindowButtonUpFcn',''), end
                end
            end
        end
    end


    function remove_along_pathS(hObject, eventdata)
        %        global stripe0
        
        resetZoom;  % make sure all zoom/pan buttons are off

        set(hrm_region,'state','off');
        set(htool.mask,'state','on');
        set(htool.retrieveRegion,'state','off');
        
        try
            while strcmpi(get(hrm_line,'state'),'on')
                
                hfh=imfreehand(h1stripe.axI,'closed',false);
                try  % this part runs when hfh is not empty
                    hfh.Deletable = false;
                    set(hfh,'visible','off');  % so that it doesn't get deleted by update_plotS
                    pos=hfh.getPosition;
                    [cx,cy,~] = improfile(xpatch,ypatch,stripe,pos(:,1),pos(:,2));
                    linearInd = sub2ind([r,c], min(r,max(1,round(cy-ypatch(1)+1))),min(c,max(1,round(cx-xpatch(1)+1))));
                    % stripe0=stripe;  % starting copy for undo purpose
                    % stripe(linearInd)=0;
                    indxM0 = indxM;
                    indxM = union(indxM,linearInd);
                    delete(hfh);
                    update_plotS
                    set(htundo,'Enable','on');
                    % setappdata(htundo,'stripe0',stripe0);
                    % fixes the bug(?) with imfreehand... PETROS
                    try, set(h1stripe.f,'WindowButtonDownFcn',''), end
                    try, set(h1stripe.f,'WindowButtonMotionFcn',''), end
                    try, set(h1stripe.f,'WindowButtonUpFcn',''), end
                    
                end  % try
                
            end
        end
    end



    function UNDO(hObject, eventdata)
        %       global stripe0
        if isappdata(htundo,'stripe0')  % if stripe0 variable exists
            stripe0 = getappdata(htundo,'stripe0');
            stripe=stripe0;
            stripe = stripe0;
            rmappdata(htundo,'stripe0');
        else  % update only mask
            indxM = indxM0;
        end  % if isappdata(htundo,'stripe0')...
        update_plotS;
        drawnow;
        set(htundo,'Enable','off');
        set(hrm_region,'state','off');
    end


    function adjust_contrastS(hObject, eventdata)
%        global stripe0

        resetZoom;  % make sure all zoom/pan buttons are off

        set(hrm_region,'state','off');

        stripe0=stripe;  % save a version of original image for undo
        
        hFig = figure('Color','w','NumberTitle','off','Toolbar','none',...
            'Menubar','none','Visible','off','closerequestfcn','','Name',['Trace ' num2str(iTRACE)]);
        stripeM = stripe;
        stripeM(indxM) = 0;
        hIm = imshow(stripeM);
        hSP = imscrollpanel(hFig,hIm);
        set(hSP,'Units','normalized',...
            'Position',[0.0 0.05 1 .95])
        hMagBox = immagbox(hFig,hIm);
        pos = get(hMagBox,'Position');
        set(hMagBox,'Position',[0 0 pos(3) pos(4)])
        %imoverview(hIm0);
        hImCon=imcontrast(hIm);
        set(hFig,'Visible','on')
        waitfor(hImCon);
        
        % update stripe
        Itmp=get(hIm,'Cdata');
        indx = logical(zeros(numel(stripe),1));
        indx(indxM) = true;
        rtmp = Itmp(~indx);  % all valid pixels
        imin = min(rtmp);
        imax = max(rtmp);
        rtmp0 = stripe(~indx);  % all valid pixels in the original image
        idx = find(rtmp==imin);  % minimum value
        imin0 = max(rtmp0(idx));
        idx = find(rtmp==imax);
        imax0 = min(rtmp0(idx));
        stripe = imadjust(stripe,double([imin0 imax0])/255, double([imin imax])/255);

        % update figure
        update_plotS;
        drawnow
        delete(hFig);
        % enable undo
        set(htundo,'Enable','on');
        setappdata(htundo,'stripe0',stripe0);
    end


    function SeperateS(hObject, eventdata)
        % Go to classification mode, if 1st time create classification
  
        resetZoom;  % make sure all zoom/pan buttons are off

        set(hrm_region,'state','off');
        set(hrm_line,'state','off');
        set(htool.retrieveRegion,'state','off');
        
        % zoom out to entire view
        set(h1stripe.axI,'Xlim',[xpatch(1) xpatch(end)],'Ylim',[ypatch(1) ypatch(end)])


        if strcmpi(get(hObject,'State'),'ON')
            
            % set([h1stripe.Bremove_region,h1stripe.Bselect_region,h1stripe.Bselect_region_mirror,h1stripe.BSeperate,h1stripe.BUndo],'Enable','off')
%            set(h1stripe.tbar([2 10:13]),'Enable','off');
%            set(h1stripe.tbar([4:8]),'Enable','on');
            set([htool.removeRegion; htool.line],'Enable','off');
            
            if ~isappdata(h1stripe.axI,'hpinpoint')  % if lines haven't been traced
                set([htool.reject; htool.tm; htool.trace;...
                    htool.recalculate; hbdigitize_single],'Enable','on');
                
                
                if isempty(get(hObject,'UserData'))
                    Classify_from_scratchS;
                    set(hObject,'UserData',1)
                else
                    update_classplotS;
                end
                
            else  % lines have been traced
                set([htool.reject; htool.tm; htool.trace;...
                    hbdigitize_single],'Enable','on');
                
                classify_trace;

                update_classplotS;
            end  % if ~isappdata(h1stripe.axI,'hpinpoint')...
        else
            
            update_plotS;
            set([htool.removeRegion; htool.line],'Enable','on');
            set([htool.reject; htool.tm; htool.trace;...
                htool.recalculate],'Enable','off');
            % once the classification button is clicked, make digitization
            % option always available
        end
       % set(hbut_draw,'Enable','off'); % not ready yet. 
    end

    function CaLLClassify_from_scratchS(hObject, eventdata)
        
        Classify_from_scratchS;
%         resetZoom;  % make sure all zoom/pan buttons are off
% 
%         set(hrm_region,'state','off');
% 
%         button = questdlg({'Are you sure you want to recalculate classification?';'Current classification will be lost!'},'Confirmation','Yes','No','No');
%         if strcmpi(button,'NO')
%             return
%         else
%             Classify_from_scratchS;
%         end

    end



    function classify_trace
        % create binary and calculate classification. Previous values are lost.
        global tick_length_lim
        
        tick_length=str2double(get(H.edit_tick_xlength,'String'));
        ltmT = true;
        if isnan(tick_length)
            errordlg('Invalid length of time marks!','Error!')
            return
        elseif tick_length<0  % no time marks
            ltmT = false;
        end
        
        
        % recalculate objects based upon trace information
        clear SCLA;  % first clean up
        nct = 1;  % counter for objects
        
        hpinpoint = getappdata(h1stripe.axI,'hpinpoint');
        
        ptraceY = get(ptrace(iTRACE),'YDATA');
        indx = [];  
        if ishandle(ptraceSTD(iTRACE))
            ptraceSTDY = get(ptraceSTD(iTRACE),'YDATA');
            indx = find(~isnan(ptraceY) & ~isnan(ptraceSTDY));  %when both trace and STD have values
        end  % if ishandle(ptraceSTD(iTRACE))

        if isempty(indx)  % ptraceY and ptraceSTDY not available
        rwid = [];  % vector collecting width for each x value
        ltmS = logical(zeros(1,numel(hpinpoint)));  % all non-time marks
        for k = 1:numel(hpinpoint)  % loop over each set
            hpp = hpinpoint(k).points;  % points for this set
            if ~isempty(hpp)
            rstdB = [];
            rstdT = [];
            X = [];
            ltm = false;  % the object is not a time mark
            for m = 1:numel(hpp)  % loop over each point
                if isvalid(hpp(m));
                    rpos = round(getPosition(hpp(m)));  % get reference position
                    rpos(1) = rpos(1)-xpatch(1)+1;  % convert to row/col numbers for stripe
                    rpos(2) = rpos(2)-ypatch(1)+1;
                    col = getColor(hpp(m));
                    if col~=defcol  % colour has been changed
                        ltm = true;
                    end  % if (col~=defcol)
                    rcol = double(stripe(:,rpos(1)));  % vertical strip for this x position
                    ns = max(rpos(2)-dpixels,1); % starting row position
                    ne = min(rpos(2)+dpixels,r); % ending row poition
                    rmax = max(rcol(ns:ne));
                    indx = find(~((rcol>rmax*thresLum) & (rcol<=rmax)));
                    rtmp= min(indx(indx>ns)-ne);  % bottom side
                    if isempty(rtmp)  % boundary at the edge
                        rtmp = r;
                    end  % if isempty(rtmp)
                    rstdB(end+1) = rtmp(1)+ne;
                    rtmp = min(-(indx(indx<ns)-ns));
                    if isempty(rtmp) % boundary at the edge
                        rtmp = 1;
                    end  % if isempty(rtmp)
                    rstdT(end+1) = -rtmp(1)+ns;
                    X(end+1) = rpos(1);
                end  % if isvalid(hpp(m))
            end  % for m = 1:numel(hpp)
            if (ltm)
                for m = 1:numel(hpp)
                    setColor(hpp(m),'y');
                end  % for m = 1:numel(hpp)
                htmS(k) = true;
            end
            %             line(X,rstdB,'color','g');
            %             line(X,rstdT,'color','c');
            rwid = [rwid rstdB-rstdT];
            end  % if ~isempty(hpp)
        end  % for k = 1:numel(hpinpoint)
        
        % find most frequent width range
        [ncount yedges] = histcounts(rwid);
        [rtmp indx] = max(ncount);
        rwidth = mean(yedges(indx) + yedges(indx+1))/2.0; % width for defining objects
        
        else  % if ptraceY and ptraceSTDY are available
            rwidth = 2*round(double(mean(abs(ptraceY(indx) - ptraceSTDY(indx)))));
        end  % if ~isempty(indx)
        
        %        bwAll = logical(zeros(size(stripe)));
        for k = 1:numel(hpinpoint)  % loop over each set
                
            hline = hpinpoint(k).line;
            
            if ~isempty(hline)  % line exists
                
                X = get(hline,'XData');
                Y = get(hline,'YData');
                
                Ytop = Y - rwidth/2;
                Ybottom = Y + rwidth/2;
                
                %             line(X,Ytop,'color','g');
                %             line(X,Ybottom,'color','c');
                
                x = [X X(end:-1:1) X(1)]-xpatch(1)+1;
                y = [Ytop Ybottom(end:-1:1) Ytop(1)]-ypatch(1)+1;
                
                BWS = poly2mask(x,y,r,c);
                Sall=regionprops(BWS,{'BoundingBox','PixelIdxList'});
                %            bwAll(find(BWS)) = true;
                % taken into account if there are multiple objects in Sall
                Stmp.PixelIdxList = cell2mat({Sall.PixelIdxList}');
                rtmp = cell2mat({Sall.BoundingBox}');  % get all bounding box information
                rtmp(:,3) = rtmp(:,1) + rtmp(:,3);
                rtmp(:,4) = rtmp(:,2) + rtmp(:,4);
                Stmp.BoundingBox = [min(rtmp(:,1)) min(rtmp(:,2)) ...
                    max(rtmp(:,3)-min(rtmp(:,1))) max(rtmp(:,4)-min(rtmp(:,2)))];
                
                idTick = [];
                if ltmT  % if time marks exist
                    % check if this object is a time mark
                    time_length=cell2mat({Stmp.BoundingBox}');
                    time_length=time_length(:,3)';% width;
                    
                    idTick=abs(time_length-tick_length)<=tick_length_lim*tick_length;
                    
                end  % if ltm
                
                SCLA(nct).BoundingBox = Stmp.BoundingBox;
                SCLA(nct).PixelIdxList = Stmp.PixelIdxList;
                SCLA(nct).ID = 2;  % trace by default
                if idTick  % time mark
                    SCLA(nct).ID = 3;  % time mark
                end
                nct = nct + 1;  % next object
            end  % if ~isempty(hline)
        end  % for k = 1:numel(hpinpoint)
        %[SCLA(ltmS).ID] = deal(3);  % time marks
        [SCLA(:).TraceNum] = deal(iTRACE);
        
        setappdata(h1stripe.axI,'rwidth',rwidth);
        
        %                 itrace = ([SCLA.ID]==2);
        %                 itick = ([SCLA.ID]==3);
        
        irej=cell2mat({SCLA([SCLA.ID]==-1).PixelIdxList}');
        itick=cell2mat({SCLA([SCLA.ID]==1|[SCLA.ID]==3).PixelIdxList}');
        itrace=cell2mat({SCLA([SCLA.ID]==0|[SCLA.ID]==2).PixelIdxList}');
                
        %% create outputs
        BWstripe=false(r,c); BWstripe(itrace)=true;
        BWstripeNULL=false(r,c); BWstripeNULL(itick)=true;

        set(hbdigitize_single,'Enable','on')
    end



    function Classify_from_scratchS(BW)
        % create binary and calculate classification. Previous values are lost.
        global tick_length_lim
        
        % level=str2double(get(H.edit_luminance,'string'));
        level = get(jSpinner,'value');
        tick_length=str2double(get(H.edit_tick_xlength,'String'));
        if isnan(tick_length)
            errordlg('Invalid length of time marks!','Error!')
            return
        end
        
        stripeM = stripe;
        if strcmpi(get(htool.mask,'state'),'on') % mask on
            stripeM(indxM) = 0;
        end
        sh=smooth(sum(stripeM,2));
        [~,imax]=max(sh);
        
        if nargin==1
            BWS=BW;
        else
            BWS=im2bw(stripeM,level/100);
        end
        
        % calculate initial classification
        clear SCLA;  % first clean up
        SCLA=regionprops(BWS,{'Centroid','BoundingBox','PixelIdxList'});
        
        
        
        time_length=cell2mat({SCLA.BoundingBox}');
        time_length=time_length(:,3)';% width;
        
        temp1=cell2mat({SCLA.Centroid}'); temp1=temp1(:,2)';
        %idRej=time_length-tick_length>5 | (abs(tick_length-time_length)<=5  & temp1>imax);
        
        if tick_length>0
            if  strcmpi(get(H.timemarkpos,'State'),'off') % time marks above the trace
                idTick=abs(time_length-tick_length)<=tick_length_lim*tick_length & temp1<=imax;
            elseif strcmpi(get(H.timemarkpos,'State'),'on')  % time marks bellow the trace
                idTick=abs(time_length-tick_length)<=tick_length_lim*tick_length & temp1>=imax;
            else % It should never go to else
                error('Problem with time marks button status')
            end
        else
            idTick=[];
        end
        
        
        idTrace=time_length-tick_length>tick_length_lim*tick_length;
        
        % add classification
        [SCLA(idTrace).ID]=deal(0);
        [SCLA(idTick).ID]=deal(1);
        idRej=cellfun(@(xvar) isempty(xvar),{SCLA.ID});
        [SCLA(idRej).ID]=deal(-1);
        %             rejected_indx=cell2mat({SCLA(idRej).PixelIdxList}');
        %             tick_indx=cell2mat({SCLA(idTick).PixelIdxList}');
        %             main_indx=cell2mat({SCLA(idTrace).PixelIdxList}');
        %
        irej=cell2mat({SCLA([SCLA.ID]==-1).PixelIdxList}');
        itick=cell2mat({SCLA([SCLA.ID]==1).PixelIdxList}');
        itrace=cell2mat({SCLA([SCLA.ID]==0).PixelIdxList}');
        
        update_classplotS(itrace,itick,irej);
        
        %% create outputs
        BWstripe=false(r,c); BWstripe(itrace)=true;
        BWstripeNULL=false(r,c); BWstripeNULL(itick)=true;

        set(hbdigitize_single,'Enable','on')
    end


    function modify_classification(hObject, eventdata)
 
        resetZoom;  % make sure all zoom/pan buttons are off

        set(hrm_region,'state','off');
        set(hrm_line,'state','off');
        set(htool.retrieveRegion,'state','off');
        
        % save enable status of all buttons so that we can recover them
        % later
        cen = [];
        for k = 1:numel(hbuttons)
            cen{k} = get(hbuttons(k),'enable');
        end  % for k = 1:numel(hbuttons)
        set(hbuttons,'enable','off');
        
        lpinpoint = isappdata(h1stripe.axI,'hpinpoint');
        % this is true if hpinpoint variable exists, i.e., trace has been
        % created manually

        tag=get(hObject,'tag');
        if strcmp(tag,'s2Trace')
            if ~lpinpoint  % if not traced by hand
                id=0;
            else  % if traced by hand
                id = 2;
            end  % if lpinpoint
        elseif strcmp(tag,'s2Reject')
            id=-1;
        elseif strcmp(tag,'s2Timemark')
            if ~lpinpoint  % if not traced by hand
                id=1;
            else  % traced by hand
                id = 3;
            end  % if ~lpinpoint
        else
            error('Problem in modify classification, single trace')
        end
        
        [colSub,rowSub,butt]=ginput(1);
        while butt==1
            if (colSub>=xpatch(1)&colSub<=xpatch(end)) & (rowSub>=ypatch(1)&rowSub<=ypatch(end))
                linearInd = sub2ind([r,c], min(r,max(1,round(rowSub-ypatch(1)+1))),min(c,max(1,round(colSub-xpatch(1)+1))));
                for iter=1:length(SCLA)
                    if any(linearInd==(SCLA(iter).PixelIdxList))
                        SCLA(iter).ID=id;
                        update_classplotS;
                        break;
                    end
                end
            end  % if (colSub>0&colSub<=c) & (rowSub>0&rowSub<=r)
            [colSub,rowSub,butt]=ginput(1);
        end
        
        irej=cell2mat({SCLA([SCLA.ID]==-1).PixelIdxList}');
        itick=cell2mat({SCLA([SCLA.ID]==1|[SCLA.ID]==3).PixelIdxList}');
        itrace=cell2mat({SCLA([SCLA.ID]==0|[SCLA.ID]==2).PixelIdxList}');
        
        %% modify plot and outputs
        update_classplotS(itrace,itick,irej);
        %The following can be accelerated in a future version by updating only what is changed
        BWstripe=false(r,c); BWstripe(itrace)=true;
        BWstripeNULL=false(r,c); BWstripeNULL(itick)=true;

        % return enable functions
        for k = 1:numel(hbuttons)
            set(hbuttons(k),'enable',cen{k});
        end  % for k = 1:numel(hbuttons)
    end


    function update_classplotS(itrace,itick,irej)
        
        if nargin<3
            irej=cell2mat({SCLA([SCLA.ID]==-1).PixelIdxList}');
%             itick=cell2mat({SCLA([SCLA.ID]==1).PixelIdxList}');
%             itrace=cell2mat({SCLA([SCLA.ID]==0).PixelIdxList}');
            itick=cell2mat({SCLA([SCLA.ID]==1|[SCLA.ID]==3).PixelIdxList}');
            itrace=cell2mat({SCLA([SCLA.ID]==0|[SCLA.ID]==2).PixelIdxList}');
        end
        
        % if impoint objects exist, remove their DeleteFcn callback
        % function so that error messages are not generated
        try  % there are no points in the window if this function is called from updating object types
            if isappdata(h1stripe.axI,'hpinpoint')
                hpinpoint = getappdata(h1stripe.axI,'hpinpoint');
                for k = 1:numel(hpinpoint)
                    hpp = hpinpoint(k).points;
                    for m = 1:numel(hpp)
                        set(hpp(m),'DeleteFcn',{});
                    end  % for m = 1:numel(hpp)
                end  % for k = 1:numel(hpinpoint)
            end  % if isappdata(h1stripe.axI,'hpinpoint')
        end
        
        G=zeros(r,c,'uint8'); R=G; B=G;
        
        if strcmpi(get(bColorSchemeS,'state'),'off')
            R([irej;itrace])=255;
            G([itick; itrace])=255;
            B(itrace)=255;
        else
            R([irej;itrace])=255;
            B([itick;itrace])=255;
            G(itick)=220;
            G(itrace)=255;
        end
        
        
        stripeRGB=zeros(r,c,3,'uint8');
        stripeRGB(:,:,1)=R; stripeRGB(:,:,2)=G; stripeRGB(:,:,3)=B;
        
        xl=get(h1stripe.axI,'xlim');
        yl=get(h1stripe.axI,'ylim');
        
        hold(h1stripe.axI,'off')
        imagesc(xpatch,ypatch,stripeRGB,'parent',h1stripe.axI)
        set(h1stripe.axI,'xlim',xl, 'ylim',yl);
        %        hold(h1stripe.axI,'on')
        %        %plot(1:c,repmat(imax,1,c),'y')
        %         plot(h1stripe.axI,200,20,'rsq','MarkerFaceColor','r','MarkerSize',10),text(600,20,'Rejected objects','Color','w')
        %         plot(h1stripe.axI,200,50,'gsq','MarkerFaceColor','g','MarkerSize',10),text(600,50,'Time-marks','Color','w')
        %         plot(h1stripe.axI,200,80,'wsq','MarkerFaceColor','w','MarkerSize',10),text(600,80,'Main trace','Color','w')
        %         set(h1stripe.axI,'Xlim',xl,'Ylim',yl);
        %        hold(h1stripe.axI,'off')
        drawnow
    end

%     function res=check_if_inside(xpatch,ypatch,qbox)
%         
%         res=false;
%         if ((qbox(1) >= xpatch(1) &&  qbox(1) <= xpatch(end)) && (qbox(2) >= ypatch(1) &&  qbox(2) <= ypatch(end)) ) ||...
%            ((qbox(1)+ qbox(3) >= xpatch(1) && qbox(1)+ qbox(3) <= xpatch(end)) && (qbox(2)+ qbox(4) >= ypatch(1) && qbox(2)+ qbox(4) <= ypatch(end)))  
%             
%   
%             res=true;
%         end
%         
%     end
        
    function res=check_if_insideX(xpatch,qbox)
        
        res=false;
        
        if ( ( qbox(1)<=xpatch(1) && qbox(1)+qbox(3) >= xpatch(end) ) || ...
            ( qbox(1) >= xpatch(1) && qbox(1) < xpatch(end) ) || ...
            ( qbox(1)+qbox(3) > xpatch(1) && qbox(1)+qbox(3) <= xpatch(end) ) )
%         if ((qbox(1) >= xpatch(1) &&  qbox(1) <= xpatch(end)) ||...
%            ((qbox(1)+ qbox(3) >= xpatch(1) && qbox(1)+ qbox(3) <= xpatch(end))))  
%             
            res=true;
        end
        
    end

    function res=check_if_inside(xpatch,ypatch,qbox)
        
        res=false;
        
        lx = false;
        if ( ( qbox(1)<=xpatch(1) && qbox(1)+qbox(3) >= xpatch(end) ) || ...
                ( qbox(1) >= xpatch(1) && qbox(1) < xpatch(end) ) || ...
                ( qbox(1)+qbox(3) > xpatch(1) && qbox(1)+qbox(3) <= xpatch(end) ) )
            %         if ((qbox(1) >= xpatch(1) &&  qbox(1) <= xpatch(end)) ||...
            %            ((qbox(1)+ qbox(3) >= xpatch(1) && qbox(1)+ qbox(3) <= xpatch(end))))
            %
            lx=true;
        end
        
        if lx  % x values within range
            
            if ( ( qbox(2)<=ypatch(1) && qbox(2)+qbox(4) >= ypatch(end) ) || ...
                    ( qbox(2) >= ypatch(1) && qbox(2) < ypatch(end) ) || ...
                    ( qbox(2)+qbox(4) > ypatch(1) && qbox(2)+qbox(4) <= ypatch(end) ) )
                res = true;
            end
        end  % if lx
    end

    function put_SClass_to_BigClass(xpatch_tmp)
        %global BW S
        %global BW
        
        % This function puts the single trace classification into the big
        % whole-seismogram classification structure
        
       % BW(patch_indexes)=false;
        
        % clear S from empty boxes.
        iok= cellfun(@(x) isempty(x),{S.BoundingBox});
        S(iok)=[];
        iok= cellfun(@(x) isempty(x),{S.PixelIdxList});
        S(iok)=[];
        
        % Check if S includes TraceNum field.
        if isfield(S,'TraceNum')
            ineg= cellfun(@isempty,{S.TraceNum});
            [S(ineg).TraceNum]=deal(-1);
        else
            [S.TraceNum]=deal(-1);
        end
        
        % remove SCLA objects that are noise
        ipos = find([SCLA.ID]==-1);
        SCLA(ipos) = [];
                
        % first, work with S objects that have the same Trace Number
        indx = find([S.TraceNum]==iTRACE);
        iokS = cellfun(@(x) check_if_insideX(xpatch,x),{S(indx).BoundingBox});
        % since we are going to force the objects to have the same trace
        % number, only the x position needs to be checked.  Checking for y
        % position can potentially miss very far off objects
        iokSA = indx(iokS);
        
        indx_delete = [];
        for it = iokSA  % loop over objects that need to be checked
            [rows cols] = ind2sub(size(I0),S(it).PixelIdxList);
            itmp = ismember(cols,xpatch_tmp); % find ones that overlap in x
            
            % turn pixels to be removed to false in the black and white image 
%             rtmp1 = rows(itmp);
%             rtmp2 = cols(itmp);
            %BW(sub2ind(size(I0),rtmp1,rtmp2)) = false;
            
            % remove overlapping x position pixels
            rows(itmp) = [];
            cols(itmp) = [];
            if ~isempty(rows) && numel(rows)>npix_min  % pixels left
                S(it).PixelIdxList = sub2ind(size(I0),rows,cols);
                S(it).BoundingBox = [min(cols)-0.5 min(rows)-0.5 ...
                    max(cols)-min(cols)+1 max(rows)-min(rows)+1];
            else  % no pixels left, i.e., object needs to be removed
                indx_delete = [indx_delete it];
            end  % if ~isempty(rows)
        end  % for it = iokSA
         
        % remove unnecessary S objects
        S(indx_delete) = [];
        
        
        % now, work with other, non-trace-id-ed objects
        indx = find([S.TraceNum]==-1);
        
        % Find all S noise objects with parts inside the patch
        %iokS=cellfun(@(x) check_if_inside(xpatch,ypatch,x),{S.BoundingBox});
        iokS=cellfun(@(x) check_if_inside(xpatch,ypatch,x),{S(indx).BoundingBox});
        iokSA = indx(iokS);
        
        % xpatch_tmp contains x positions where data will be replaced by
        % those of the SCLA results
        % any objects for this trace within the xpatch_tmp need to be
        % removed
        indx_delete = [];
        for it = iokSA  % loop over objects that need to be checked
            
            % erode these objects with pixels of the SCLA objects
            for k = 1:length(SCLA)
                sbox = SCLA(k).BoundingBox;  % box for current object
                sbox(1) = sbox(1)+min(xpatch)-1;
                sbox(2) = sbox(2)+min(ypatch)-1;  % put back to I0 coordinates
                if check_if_inside([sbox(1) sbox(1)+sbox(3)], [sbox(2) sbox(2)+sbox(4)], [S(it).BoundingBox])
                    pixelsR = patch_indexes(SCLA(k).PixelIdxList);
                    pixS = S(it).PixelIdxList;
                    [itmp iS] = intersect(pixS,pixelsR);
                    pixS(iS) = [];  % erode ones overlapping with objects
                    S(it).PixelIdxList = pixS;  % update new set of pixels
%                     % update bounding box
%                     [ro co] = ind2sub(size(I0),pixS);
%                     S(it).BoundingBox = [min(co)-0.5 min(ro)-0.5 max(co)-min(co)+1 max(ro)-min(ro)+1];
                end  % if overlap in outline boxes
            end  % for k = 1:length(SCLA)
            
            if (numel(S(it).PixelIdxList))<npix_min
                indx_delete = [indx_delete it];
            else  % object to be kept -> update bounding box
                [ro co] = ind2sub(size(I0),S(it).PixelIdxList);
                S(it).BoundingBox = [min(co)-0.5 min(ro)-0.5 max(co)-min(co)+1 max(ro)-min(ro)+1];
            end  % if (numel(S(it).PixelIdxList))
            
            %
%             [rows cols] = ind2sub(size(I0),S(it).PixelIdxList);
%             itmp = ismember(cols,xpatch_tmp); % find ones that overlap in x
%             
%             % turn pixels to be removed to false in the black and white image 
%             rtmp1 = rows(itmp);
%             rtmp2 = cols(itmp);
%             BW(sub2ind(size(I0),rtmp1,rtmp2)) = false;
%             
%             % remove overlapping x position pixels
%             rows(itmp) = [];
%             cols(itmp) = [];
%             if ~isempty(rows)  % pixels left
%                 S(it).PixelIdxList = sub2ind(size(I0),rows,cols);
%                 S(it).BoundingBox = [min(cols)-0.5 min(rows)-0.5 ...
%                     max(cols)-min(cols)+1 max(rows)-min(rows)+1];
%             else  % no pixels left, i.e., object needs to be removed
%                 indx_delete = [indx_delete it];
%             end  % if ~isempty(rows)
        end  % for it = iokSA
         
        % remove unnecessary S objects
        S(indx_delete) = [];
        
        % add SCLA objects
        ipos = length(S);
        for it = 1:length(SCLA)
            ipos = ipos + 1;
            S(ipos).PixelIdxList = patch_indexes(SCLA(it).PixelIdxList);
            [ro co] = ind2sub(size(I0),S(ipos).PixelIdxList);
            S(ipos).BoundingBox = [min(co)-0.5 min(ro)-0.5 max(co)-min(co)+1 max(ro)-min(ro)+1];
            %rectangle('Position',S(ipos).BoundingBox,'EdgeColor','y','parent',h1stripe.axI)
            S(ipos).ID = SCLA(it).ID;
            S(ipos).TraceNum = iTRACE;
            %BW(S(ipos).PixelIdxList) = true;
        end  % for it = 1:length(SCLA)
        
        
%         for it = 1:length(SCLA)
%                         
%             iok=cellfun(@(x) ~isempty(intersect(patch_indexes(SCLA(it).PixelIdxList),x)),{S(iokS).PixelIdxList});
%             
%             if any(iok) % Find Crossings - common points
%                 for it2=iokS(iok)
%                     
%                     [C,ia,ib] = intersect(patch_indexes(SCLA(it).PixelIdxList),S(it2).PixelIdxList);
%                     % If it is already in S just update ID & TraceNum etc.
%                     if numel(S(it2).PixelIdxList)==numel(C)
%                         BW(S(it2).PixelIdxList)=true;
%                         S(it2).ID=SCLA(it).ID;  % possibly changing the type
%                         %S(it2).TraceNum=iTRACE;
%                     else
%                         % remove intersections of SCLA with S
%                         S(it2).PixelIdxList(ib)=[];
%                         % Update Box, ID does not change, same for TraceNum
%                         [ro,co] = ind2sub(size(I0),S(it2).PixelIdxList);
%                         S(it2).BoundingBox=[min(co)-0.5 min(ro)-0.5 max(co)-min(co)+1 max(ro)-min(ro)+1];
%                         BW(S(it2).PixelIdxList)=true;
%                         
%                         % replace in remaining S TraceNums not to be the current trace
% %                         if median(ro(:))> median(ypa)
% %                             S(it2).TraceNum=max(iTRACE+1,length(pt_traces));
% %                         else
% %                             S(it2).TraceNum=min(1,iTRACE-1);
% %                         end
%                         
%                         % Put intersections as seperate objects belonging to both
%                         lastobj=lastobj+1;
%                         S(lastobj).PixelIdxList=C(:);
%                         [ro,co] = ind2sub(size(I0),C(:));
%                         S(lastobj).BoundingBox=[min(co)-0.5 min(ro)-0.5 max(co)-min(co)+1 max(ro)-min(ro)+1];
%                         S(lastobj).ID=SCLA(it).ID;
%                         %S(lastobj).TraceNum=unique([S(lastobj).TraceNum iTRACE]); %belonging to both
%                         S(lastobj).TraceNum=iTRACE;
%                         BW(S(lastobj).PixelIdxList)=true;
%                     end
%                     % put remaining pxls of SCLA(it) (if any) in S and BW
%                     SCLA(it).PixelIdxList(ia)=[];
%                     if ~isempty(SCLA(it).PixelIdxList)
%                         % Put the rest of the pixels of SCLA(it).PixelIdxList as seperate objects
%                         lastobj=lastobj+1;
%                         S(lastobj).PixelIdxList=patch_indexes(SCLA(it).PixelIdxList);
%                         [ro,co] = ind2sub(size(I0),S(lastobj).PixelIdxList(:));
%                         S(lastobj).BoundingBox=[min(co)-0.5 min(ro)-0.5 max(co)-min(co)+1 max(ro)-min(ro)+1];
%                         S(lastobj).ID=SCLA(it).ID;
%                         S(lastobj).TraceNum=iTRACE;
%                         BW(S(lastobj).PixelIdxList)=true;
%                     end
%                 end
%             else % No common points just in SCLA
%                 % SCLA only
%                 lastobj=lastobj+1;
%                 S(lastobj).PixelIdxList=patch_indexes(SCLA(it).PixelIdxList);
%                 [ro,co] = ind2sub(size(I0),S(lastobj).PixelIdxList);
%                 S(lastobj).BoundingBox=[min(co)-0.5 min(ro)-0.5 max(co)-min(co)+1 max(ro)-min(ro)+1];
%                 S(lastobj).ID=SCLA(it).ID;
%                 S(lastobj).TraceNum=iTRACE;
%                 BW(S(lastobj).PixelIdxList)=true;
%             end
%         end
        %         fprintf('S after messing. length(S)=%d\n', length(S))
        %         whos S
        
        % remove any objects that end up having no pixels
        indx = find(cellfun('isempty',{S.PixelIdxList}));
        S(indx) = [];  % remove empty pixels
        
    end





    function extract_curvature_correctionS(hObject,eventdata)
        
        resetZoom;  % make sure all zoom/pan buttons are off

        stripeC=stripe;
        
        set(hrm_region,'state','off');

        % Launch GUI to correct curvature
        h1stripeC=figure('Color','w','WindowStyle','normal','Name',['Edit trace ' num2str(iTRACE)] ,'NumberTitle','Off',...
            'Menubar','none','ToolBar','figure','Units','Normalized','Position',[0.1 0.5 0.8 0.3]);
        
        
        % modify tool bar
        temp= findall(findall(h1stripeC,'Type','uitoolbar'));
        delete(temp([2:7 9 13 16 17]));
        % % Reorder tools
        % set(temp(1),'Children',[temp(4:end-1);temp(2:3);temp(end)])
        
        
        uipushtool(temp(1),'Tag','Select region mirrored','Cdata',imread('icons/CurvedTraces.png'),...
            'Separator','on','TooltipString','Define curvature using impoly object',...
            'HandleVisibility','on','ClickedCallback',@DefineCurvature);
        
        
        bpreview=uicontrol('Style', 'pushbutton','units','normalized',...
            'String', 'Preview','Position', [0.92 0.26 0.07 0.08],...
            'Callback',@preview_curvature_corection,'enable','off');
        
        
        bpapply=uicontrol('Style', 'pushbutton','units','normalized',...
            'String', 'Save & Close','Position', [0.92 0.17 0.07 0.08],...
            'Callback',@save_curvature_model);
        
        uicontrol('Style', 'pushbutton','units','normalized',...
            'String', 'Cancel','Position', [.92 0.08 0.07 0.08],...
            'Callback',@close_without_saving_curvature_model);
        
        
        
        ax1=axes('Position',[0.06 0.55 .8 .44]);
        % plot trace
        imagesc(stripeC,'Parent',ax1); colormap(ax1,gray(256));
        ax2=axes('Position',[0.06 0.05 .8 .44]);
        hpoly=[];
        fcurv=[];
        
        
        
        
        
        function constrained_position = symetric_fixedCenterY_fcn(new_position)
            constrained_position=new_position;
            constrained_position(2)=YC-new_position(4)/2;
        end
        
        function DefineCurvature(ho,ev)
            try
                delete(hpoly)
            end
            delete(findobj([ax1 ax2],'Color','r'))
            delete(findobj([ax1 ax2],'Color','c'))
            delete(findobj([ax1 ax2],'Color','y'))
            hpoly=impoly(ax1,'Closed',false);
            hpoly.Deletable = true;
            set(bpreview,'Enable','on');
            set(bpapply,'Enable','on');
            
        end
        
        function preview_curvature_corection(ho, ev)
            % delete previous diagnostic lines
            delete(findobj([ax1 ax2],'Color','r'))
            delete(findobj([ax1 ax2],'Color','c'))
            delete(findobj([ax1 ax2],'Color','y'))
            
            % get current position of impoly obj
            pos=getPosition(hpoly);
            pos2=pos(:,2)-r/2;
            
           % [r,c]=size(stripeC);
            % fit a 2nd degree eq
            fcurv = fit(pos2,pos(:,1),'poly2');
            
            yi=linspace(min(get(gca,'Ylim')),max(get(gca,'Ylim')),r);
            xi=fcurv(linspace(-r/2,r/2,r));
            
            [~,imin2]=min(xi);
            
            
            hold(ax1,'on')
            plot(ax1,xi,yi,'r');
            [~,imin1]=min(abs(gradient(xi,yi)));
            plot(ax1,get(ax1,'Xlim'),yi(imin1)*[1 1] ,'y--');
            plot(ax1,xi(imin2)*[1 1],get(gca,'Ylim'),'c--');
            plot(ax1,get(ax1,'Xlim'),yi(imin2)*[1 1],'c:');
            hold(ax1,'off')
            
            % create displacement field
            D=zeros(r,c,2);
            D(:,:,1)=repmat(xi-min(xi),1,c);
            B=imwarp(stripeC,D,'cubic');
            
            axes(ax2)
            cla(ax2)
            imagesc(B,'Parent',ax2)
            colormap(ax2,gray(256))
            hold(ax2,'on')
            plot(ax2,xi,yi,'r--');
            plot(ax2,xi(imin2)*[1 1],get(gca,'Ylim'),'c--');
            
            set(ax2,'Xgrid','on')
            linkaxes([ax1 ax2])
        end
        
        function save_curvature_model(ho,ev)
            set(curvButton,'UserData',fcurv,'State','on')
            close(h1stripeC)
        end
        
        
        function close_without_saving_curvature_model(ho,ev)
            set(curvButton,'UserData',[],'State','off')
            close(h1stripeC)
        end
              
    end
        
        



        function Digitize_Single(hObject, eventdata)
            global output_message %BWstripe BWstripeNULL
            global pt_single pt_singleSTD
            
        resetZoom;  % make sure all zoom/pan buttons are off

        try, delete(pt_single), end
            try, delete(pt_singleSTD),end
            
            % update user
            output_message{end+1}=['Digitizing single trace: '  num2str(iTRACE) ', please wait...'];
            set(H.output_mess,'String',output_message,'Value',length(output_message)); drawnow
            
            % Correct offset curvature
            if lcurve  % if curvature is used
                if isfield(H, 'curvButton')
%                     correctcurv=false;
                    % use the model for current trace
                    if ~isempty( get(curvButton,'UserData') )
                        fcurv=get(curvButton,'UserData');
%                         correctcurv=true;
                    elseif ~isempty( get(H.curvButton,'UserData') )
                        % use global model
                        fcurv=get(H.curvButton,'UserData');
%                         correctcurv=true;
                    end
                    
%                     if correctcurv % apply correction if it was selected.
                        xi=fcurv(linspace(-r/2,r/2,r));
                        % create displacement field
                        D=zeros(r,c,2);
                        D(:,:,1)=repmat(xi-min(xi),1,c);
                        % apply displacement field
                        stripe1=imwarp(stripe,D,'cubic');
                        BWstripe1=imwarp(BWstripe,D,'cubic');
                        BWstripeNULL1=imwarp(BWstripeNULL,D,'cubic');
%                     else
%                         stripe1=stripe;
%                         BWstripe1=BWstripe;
%                         BWstripeNULL1=BWstripeNULL;
%                     end
                end
            else  % if non-curved trace
                if ~isappdata(h1stripe.axI,'hpinpoint')  % not traced by hand
                    stripe1=stripe;
                else  % traced by hand
                    rtmp = double(BWstripe) + double(BWstripeNULL);
                    stripe1 = uint8(255*double(rtmp>0));
                end  % if ~isappdata(h1stripe.axI,'hpinpoint')...
                BWstripe1=BWstripe;
                BWstripeNULL1=BWstripeNULL;
            end
             
            % digitize
            [yy,yySTD]=digitize_trace_and_tickmarks(stripe1,stripe1,BWstripe1,BWstripeNULL1);
            
            % for patch
            yy=yy+min(ypatch);
            yySTD=yySTD+yy;
            
            % plot at current gui
            hold(h1stripe.axI,'on')
            pt_single=plot(h1stripe.axI,xpatch,yy,'-','color',[1 .7 0],'linewidth',2);
            pt_singleSTD=plot(h1stripe.axI,xpatch,yySTD,'-','color',[0.3 .1 1],'linewidth',2);
            
            
            hold(h1stripe.axI,'off')
            
            % update user
            output_message{end+1}='Ready';
            set(H.output_mess,'String',output_message,'Value',length(output_message)); drawnow

            try, set(h1stripe.f,'WindowButtonDownFcn',''), end
            try, set(h1stripe.f,'WindowButtonMotionFcn',''), end
            try, set(h1stripe.f,'WindowButtonUpFcn',''), end
            
        end
        
    
        function Apply_to_main(hObject, eventdata)
            global pt_single pt_singleSTD

            resetZoom;  % make sure all zoom/pan buttons are off
            
            if isempty(pt_single) | ~isvalid(pt_single) | ~ishandle(pt_single)   % trace has not been digitized
                h = errordlg('The image has not been digitized - no digitized trace exists','Missing Trace');
%                try; delete(h); end
                return
            end  % if isempty(pt_single)
            
            yy=get(pt_single,'YDATA');
            yySTD=get(pt_singleSTD,'YDATA');
            
            
            set(hrm_region,'state','off');
            set(hrm_line,'state','off');
            set(htool.retrieveRegion,'state','off');

%             button = questdlg({['Click at "Skip NaNs" if you want to skip NaN values ',...
%                 'returned from the digitization. If you click at "Include NaNs" then NaNs will be included.']},...
%                 'NaN numbers','Skip NaNs','Include NaNs','Skip NaNs');
            if strcmpi(get(hObject,'tag'),'skipNaN')
                %replace nans in yy and yySTD with the current value of the trace
                notanan=~isnan(yy);
                yy=yy(notanan);
                yySTD=yySTD(notanan);
                xpatch_temp=xpatch(notanan);
            else
                xpatch_temp=xpatch;
            end
            
            ptraceSTD_EXISTS=true;
            try
                yitraceSTD=get(ptraceSTD(iTRACE),'YData');
            catch
                ptraceSTD_EXISTS=false;
                htemp=errordlg('Could not find handles for digitized traces STD. If this file is from an older version of DigitSeis without this feature, ignore this message',...
                    'Possible error?','modal');
                uiwait(htemp);
                
            end
            
            yitrace=get(ptrace(iTRACE),'YData');
            xitrace = get(ptrace(iTRACE),'XData');
            [~,iok,iokxpatch] = intersect(xitrace,xpatch_temp);
            
            yitrace(iok)=yy(iokxpatch);%+ftrend(xpatch(iokxpatch))';
            % update traces
            set(ptrace(iTRACE),'XData',xitrace,'YData',yitrace,'color',[1 .7 0])
            
            if ptraceSTD_EXISTS
                yitraceSTD(iok)=yySTD(iokxpatch);
                set(ptraceSTD(iTRACE),'XData',xitrace,'YData',yitraceSTD,'color',[.3 .1 1])
            end
            
            % check to make sure pt_traces(iTRACE) covers the entire range
            % of x used in ptrace
            xtmp = xitrace(~isnan(yitrace));  % valid values of x in ptrace
            if min(xtmp)<min(get(pt_traces(iTRACE),'Xdata'))  % extend reference line to the left
                x0 = get(pt_traces(iTRACE),'XData');
                y0 = get(pt_traces(iTRACE),'YData');
                extrapoints = min(xtmp):min(x0)-1;
                % if ~exist('pt_tracesY0') || isempty([pt_tracesY0{iTRACE}])  % reference trace Y values do not exist
                if ~iscell('pt_tracesY0') || isempty(pt_tracesY0) % reference trace Y values do not exist
                    ynew = interp1(x0,y0,extrapoints,'linear','extrap');
                else  % reference values exist
                    if ~isempty([pt_tracesY0{iTRACE}])
                        yRef = pt_tracesY0{iTRACE};
                        ynew = yRef(extrapoints);
                    else  % if pt_tracesY0{iTRACE} is empty
                        ynew = interp1(x0,y0,extrapoints,'linear','extrap');
                    end  % if ~isempty([pt_tracesY0{iTRACE}])...
                end  % if ~exist(pt_tracesY0) || isempty([pt_tracesY0{iTRACE}])...
                x0 = [extrapoints x0];
                y0 = [ynew y0];
                set(pt_traces(iTRACE),'XData',x0,'YData',y0);
                
                % update start time information for this trace
                if any(ishandle(pt_start_time))
                    x0 = get(pt_start_time,'Xdata');
                    y0 = get(pt_start_time,'Ydata');
                    x0(iTRACE) = min(xtmp);
                    y0(iTRACE) = ynew(1);
                    set(pt_start_time,'Xdata',x0,'Ydata',y0);
                end  % if ~isempty(pt_start_time)
                
            end  % if (min(xtmp)<min(pt_traces(iTRACE))
            if max(xtmp)>max(get(pt_traces(iTRACE),'Xdata'))  % extend reference line to the left
                x0 = get(pt_traces(iTRACE),'XData');
                y0 = get(pt_traces(iTRACE),'YData');
                extrapoints = max(x0)+1:max(xtmp);
                %if ~exist('pt_tracesY0') || isempty([pt_tracesY{iTRACE}])  % reference trace Y values do not exist
                if ~iscell('pt_tracesY0') || isempty(pt_tracesY0) % reference trace Y values do not exist
                    ynew = interp1(x0,y0,extrapoints,'linear','extrap');
                else  % reference values exist
                    if ~isempty([pt_tracesY0{iTRACE}])  % values available
                        yRef = pt_tracesY0{iTRACE};
                        ynew = yRef(extrapoints);
                    else  % pt_tracesY0 values not available
                        ynew = interp1(x0,y0,extrapoints,'linear','extrap');
                    end  % if ~isempty([pt_tracesY0{iTRACE}])...
                end  % if ~exist(pt_tracesY0) || isempty([pt_tracesY0{iTRACE}])...
                x0 = [x0 extrapoints];
                y0 = [y0 ynew];
                set(pt_traces(iTRACE),'XData',x0,'YData',y0);
                
                % update ending time information for this trace
                if any(ishandle(pt_end_time))
                    x0 = get(pt_end_time,'Xdata');
                    y0 = get(pt_end_time,'Ydata');
                    x0(iTRACE) = max(xtmp);
                    y0(iTRACE) = ynew(end);
                    set(pt_end_time,'Xdata',x0,'Ydata',y0);
                end  % if ~isempty(pt_end_time)
                
            end  % if (min(xtmp)<min(pt_traces(iTRACE))
            
            
            % Apply classification/Binary changes
%              hmsg = dialog('Units','Normalized','Position',[0.4 0.4 0.25 0.1],'Name','Updating Classification');
%              uicontrol('Parent',hmsg,...
%                  'Style','text','HorizontalAlignment','center',...
%                  'Units','Normalized','Position',[0.1 .25 0.8 0.5],...
%                  'String','Updating the classification structure and the binary image, please wait...','FontSize',12);
             drawnow
             put_SClass_to_BigClass(xpatch_temp);
            try
            close(hmsg);
            end
            
            if get(H.find_gaps,'value')  % checkbox for finding gaps on
                hgap = getappdata(H.ax1,'hgap');  % get gap rectangles
                delete(hgap);  % delete
                try; rmappdata(H.ax1,'hgap'); end 
                find_gaps(H.find_gaps);  % update the boxes
            end  % if get(H.find_gaps,'value')==1
            
            if get(H.find_vert,'value')  % checkbox for finding vertical lines on
                hvert = getappdata(H.ax1,'hvert');
                delete(hvert);  % delete
                try; rmappdata(H.ax1,'hvert'); end
                find_gaps(H.find_vert);  % update the boxes
            end  % if get(H.find_vert,'value')
        end
        
    
    
    function resetZoom
        % this function checks if zoom or pan toggle tool is on and if any
        % one is, turns it off and removes window functions
        
        htmp = [htool.zoomIn; htool.zoomOut; htool.pan];  % handles
        
        ipos = find(strcmpi(get(htmp,'state'),'on'));
        
        if ~isempty(ipos)  % one of them is on
            if ipos==1 || ipos==2  % zoom icon
                zoom off
            else  % pan
                pan off
            end
        end  % if ~isempty(ipos)
        
        % make sure pointer is back to default style
        set(h1stripe.f,defPointer);
    end  % function resetZoom



        function Closethefigure(hObject, eventdata)         
%             if isappdata(h1stripe.axI,'hpinpoint')  % traced by hand
%                 hpinpoint = getappdata(h1stripe.axI,'hpinpoint');
%                 for m = 1:numel(hpinpoint)
%                     hpp = hpinpoint(m).points;  % get all points
%                     for k = 1:numel(hpp)
%                         set(hpp(k),'DeleteFcn',{});
%                         % set function associated with deletion of the
%                         % object to nothing - otherwise there will be error
%                         % messages as the window closes
%                     end  % for k = 1:numel(hpp)
%                 end   % for m = 1:numel(hpinpoint)
%             end  % if isappdata(h1stripe.axI,'hpinpoint')
%             
%             rmappdata(H.f1,'h1stripe');  % remove data saved
        resetZoom;  % make sure all zoom/pan buttons are off
            deleteFigure;
            close(h1stripe.f);
        end

        function deleteFigure(hObject, eventdata)         
            if isappdata(h1stripe.axI,'hpinpoint')  % traced by hand
                hpinpoint = getappdata(h1stripe.axI,'hpinpoint');
                for m = 1:numel(hpinpoint)
                    hpp = hpinpoint(m).points;  % get all points
                    for k = 1:numel(hpp)
                        if isvalid(hpp(k))  % if the point still exists
                            % the deleted points are still residing within
                            % the hpinpoint variable
                            
                            set(hpp(k),'DeleteFcn',{});
                            % set function associated with deletion of the
                            % object to nothing - otherwise there will be error
                            % messages as the window closes
                        end  % if any(ishandle(hpp(k)))
                    end  % for k = 1:numel(hpp)
                end   % for m = 1:numel(hpinpoint)
                rmappdata(h1stripe.axI,'hpinpoint');  % remove data saved
            end  % if isappdata(h1stripe.axI,'hpinpoint')   
            
            if isappdata(H.f1,'h1stripe');  % if the figure still exists
                rmappdata(H.f1,'h1stripe');
            end  % if isappdata(H.f1,'h1stripe')
            
            set(H.digitize_1,'enable','on');  % make the pushbutton in main window active
            % if the correct trace option window is open, make the Recover pushbutton
            % invisible
            if isappdata(H.f1,'cto')  % if correct trace option variable exists
                cto = getappdata(H.f1,'cto');
                if ishandle(cto.hf)  % if window exists
                    set(cto.hRecover,'visible','off');
                end  % if ishandle(cto.hf)
            end  % if isappdata(H.f1,'cto')
        end

end








%%

%function load_analysis(hObject, eventdata)
function load_analysis
global H HIM I0 Iorig HIMRGB BW ptrace ptraceSTD pt_traces pt_traceLABEL 
global Time ftrend ftrend_tick S output_message  sumI ww 
global pt_start_time pt_end_time p_tick_1st p_tick_last hresult_time_marks
global posD

[filename, pathname]=uigetfile({'*.mat','mat files';...
    '*.*','All Files' },'Select analysis file');
if isequal(filename,0)
    return
end
close(gcf); % close old one,

hmsg = dialog('Units','Normalized','Position',[0.5 0.5 0.25 0.1],'Name','Loading file');
uicontrol('Parent',hmsg,...
    'Style','text','HorizontalAlignment','center',...
    'Units','Normalized','Position',[0.01 .25 0.9 0.5],...
    'String','Loading data, Please wait...','FontSize',16);
drawnow
% % pause(2)
% close(hmsg)
disp('Loading data. This can take some time, please wait...')
%load everything
load(fullfile(pathname,filename));

closeAdditionalWindows;  % close all additional windows (to be left with the main window only)

if ~isfield(H,'version')
    H.version = [0.0];
end  % if ~isfield(H,'version')

set(H.f1,'Visible','off')
set(H.f1,'OuterPosition',get(0,'ScreenSize'));
set(H.f1,'WindowButtonMotionFcn','');  % reset button motion function
set(H.f1,'menu','none','toolbar','figure');
set(H.f1,'Name',filename);  % show filename
set(H.f1,'DeleteFcn',@closeDigitSeis);  % function to be executed when the figure closes

% modify tool bar
temp= findall(findall(H.f1,'Type','uitoolbar'));
%    ' 1'     'FigureToolBar'
%     ' 2'    'Plottools.PlottoolsOn'
%     ' 3'    'Plottools.PlottoolsOff'
%     ' 4'    'Annotation.InsertLegend'
%     ' 5'    'Annotation.InsertColorbar'
%     ' 6'    'DataManager.Linking'
%     ' 7'    'Exploration.Brushing'
%     ' 8'    'Exploration.DataCursor'
%     ' 9'    'Exploration.Rotate'
%     '10'    'Exploration.Pan'
%     '11'    'Exploration.ZoomOut'
%     '12'    'Exploration.ZoomIn'
%     '13'    'Standard.EditPlot'
%     '14'    'Standard.PrintFigure'
%     '15'    'Standard.SaveFigure'
%     '16'    'Standard.FileOpen'
%     '17'    'Standard.NewFigure'
%     '18'    ''

htool.save = findobj(temp,'tag','Standard.SaveFigure');
htool.datacursor = findobj(temp,'tag','Exploration.DataCursor');
htool.pan = findobj(temp,'tag','Exploration.Pan');
htool.zoomOut = findobj(temp,'tag','Exploration.ZoomOut');
htool.zoomIn = findobj(temp,'tag','Exploration.ZoomIn');
%htool.fileOpen = temp(16);
set(htool.save,'ClickedCallback',@Saveresults,'TooltipString','Save results');
set(htool.datacursor,'ClickedCallback',@dataCursorFnc)

% remove icons that need to be reset
itmp = ismember(temp,[temp(1) htool.save htool.datacursor htool.pan htool.zoomOut htool.zoomIn]);
delete(temp(~itmp));

% uipushtool(temp(1),'Tag','Reload image from file','Cdata',imread('reload.png','BackgroundColor',[1 1 1]),...
%     'Separator','on','TooltipString','Reload image from file',...
%     'HandleVisibility','on','ClickedCallback',@Load_Image);

% uipushtool(temp(1),'Tag','View and edit image file','Cdata',imread('imtool.png','BackgroundColor',[1 1 1]),...
%     'Separator','on','TooltipString','View info and edit image with imtool',...
%     'HandleVisibility','on','ClickedCallback',@viewimage);
htool.editImage = uipushtool(temp(1),'Tag','View and edit portion of the image','Cdata',imread('icons/imtool.png','BackgroundColor',[1 1 1]),...
    'Separator','on','TooltipString','View and edit portion of the image',...
    'HandleVisibility','on','ClickedCallback',@viewimage);

%temp= findall(findall(H.f1,'Type','uitoolbar'));
%set(temp(end),'ClickedCallback',@Browse_and_Load)

% Reorder tools
%set(temp(1),'Children',[temp(4:end-2);temp(2:3);temp(end)])

% htool.loadAnalysis = uipushtool(temp(1),'Tag','Load existing analysis','Cdata',imread('load_analysis.png','BackgroundColor',[1 1 1]),...
%     'Separator','on','TooltipString','Load existing analysis from file',...
%     'HandleVisibility','on','ClickedCallback',@load_analysis);

% add some tools
htool.whole = uipushtool(temp(1),'Tag','Show whole seismogram','Cdata',imread('icons/whole_seismogram.png','BackgroundColor',[1 1 1]),...
    'Separator','on','TooltipString','Show whole seismogram',...
    'HandleVisibility','on','ClickedCallback',@zoom2seismogram);

htool.crop = uipushtool(temp(1),'Tag','Crop Image','Cdata',imread('icons/crop.png','BackgroundColor',[1 1 1]),...
    'Separator','on','TooltipString','Crop image',...
    'HandleVisibility','on','ClickedCallback',@crop_image);
htool.contrast = uipushtool(temp(1),'Tag','Adjust contrast','Cdata',imread('icons/contrast-icon.png','BackgroundColor',[1 1 1]),...
    'Separator','on','TooltipString','Adjust contrast',...
    'HandleVisibility','on','ClickedCallback',@adjust_contrast);

% htool.removeBG = uipushtool(temp(1),'Tag','Remove background','Cdata',imread('icons/Removebackground.png','BackgroundColor',[1 1 1]),...
%     'Separator','on','TooltipString','Click remove large background stains-colorization',...
%     'HandleVisibility','on','ClickedCallback',@remove_background_withGAUSSFILT);

% Create button image & Button
tempim=imnoise(zeros([16,16,3],'uint8'),'salt & pepper',0.2);
for ii=2:16, tempim(ii-1:ii,ii-1:ii,1)=255; tempim(ii-1:ii,ii-1:ii,2:3)=0 ;tempim([ii-1:ii],17-[ii-1:ii],1)=255; tempim([ii-1:ii],17-[ii-1:ii],2:3)=0; end
htool.saltPepper = uipushtool(temp(1),'Cdata',tempim,'Tag','Salt & Pepper Removal',...
    'Separator','off','TooltipString','Remove salt & pepper noise',...
    'HandleVisibility','on','Enable','on','ClickedCallback',@remove_SaltandPepper);

% htool.rotate = uipushtool(temp(1),'Tag','Correct rotation','Cdata',imread('icons/correct_rotation.png','BackgroundColor',[1 1 1]),...
%     'Separator','on','TooltipString','Cprrect for rotation',...
%     'HandleVisibility','on','ClickedCallback',@estimate_rotation);
htool.recoverRegion = uipushtool(temp(1),'Tag','Recover Region','Cdata',imread('icons/retrieveR.png','BackgroundColor',[1 1 1]),...
    'Separator','on','TooltipString','Draw a region to recover from the original image',...
    'HandleVisibility','on','ClickedCallback',@recoverOrig,'enable','off');
if ~isempty(Iorig) & numel(I0)==numel(Iorig)  % if original image exists and of same size as I0
    set(htool.recoverRegion,'enable','on');
end  % if isempty(Iorig)
htool.removeRegion = uipushtool(temp(1),'Tag','Remove Region','Cdata',imread('icons/removeS.png','BackgroundColor',[1 1 1]),...
    'Separator','on','TooltipString','Draw a region to remove',...
    'HandleVisibility','on','ClickedCallback',@remove_region);
htool.undo = uipushtool(temp(1),'Tag','Undo last','Cdata',imread('icons/undo.jpeg'),...
    'Separator','on','TooltipString','Undo last action',...
    'HandleVisibility','on','Enable','off','ClickedCallback',@Undo_remove);
htool.measureLength = uipushtool(temp(1),'Tag','Measure tick length','Cdata',imread('icons/tick_length.png','BackgroundColor',[1 1 1]),...
    'Separator','on','TooltipString','Measure tick length',...
    'HandleVisibility','on','ClickedCallback',@meassure_pix_dist0);
% Image complement
% polarity_ic=zeros([16,16,3]); for i=1:16, polarity_ic(1:i,i:16,:)=1;end
% htool.IComplement = uipushtool(temp(1),'Tag','Measure tick length','Cdata',polarity_ic,...
%     'Separator','on','TooltipString','Click to change image polarity',...
%     'HandleVisibility','on','ClickedCallback',@change_I0_polarity);
% clear polarity_ic;

% Time marks up or down
icon_m=zeros([16,16,3]); icon_m(8,:,:)=1; icon_m(4,6:10,:)=1; icon_m(13,6:10,1)=1;
if isfield(H,'timemarkpos')  % if timemarkpos field exists
    cstr = get(H.timemarkpos,'state');  % saved state
else
    cstr = 'off';
end
if strcmpi(cstr,'off')  % time mark above
    icon_m=zeros([16,16,3]); icon_m(11:12,[1:6, 11:16],:)=1; icon_m(6:7,7:10,2)=1;
else  % time mark below
    icon_m=zeros([16,16,3]); icon_m(6:7,[1:6, 11:16],:)=1; icon_m(11:12,7:10,2)=1;
end  % if strcmpi(cstr,'off')
H.timemarkpos=uitoggletool(temp(1),'Tag','Time marks relative position','Cdata',icon_m,...
    'Separator','on','TooltipString','Click to indicate that time mark offset occurs downward',...
    'HandleVisibility','on', 'State',cstr,'ClickedCallback',@check_timemarkpos);
clear icon_m

% CurvedTraces
if isfield(H,'curvButton')  % if curvButton field exists
    cstr = get(H.curvButton,'state');  % saved state
else
    cstr = 'off';
end  % if (isfield(H,'curvButton'))
H.curvButton=uitoggletool(temp(1),'Tag','Measure tick length','Cdata',imread('icons/CurvedTraces.png'),...
    'Separator','on','TooltipString','Click to model and correct curved offsets',...
    'HandleVisibility','on','ClickedCallback',@extract_curvature_correction,'state',cstr);

if isfield(H,'origImage')
    if strcmpi(get(H.origImage,'state'),'on')  % need to switch to processed image
        set(HIM,'Cdata',I0);
    end  % if strcmpi(get(H.origImage,'state'),'on')
end
H.origImage=uitoggletool(temp(1),'Tag','Original Image','Cdata',imread('icons/origImage.png'),...
    'Separator','on','TooltipString','Switch between original and processed images',...
    'HandleVisibility','on', 'State','off','ClickedCallback',@viewOrigImage,...
    'enable',get(htool.recoverRegion,'enable'));

htool.tmPos = H.timemarkpos;
htool.curvButton = H.curvButton;
htool.origImage = H.origImage;

temp= findall(findall(H.f1,'Type','uitoolbar'));
% htmp = [htool.whole; htool.pan; htool.zoomOut; htool.zoomIn; 
%     htool.datacursor; htool.undo; 
%     htool.tmPos; htool.curvButton; htool.measureLength;
%     htool.removeRegion; htool.editImage; 
%     htool.removeBG; htool.saltPepper; 
%     htool.IComplement; htool.rotate; htool.contrast; htool.crop;
%     htool.save
%     ];
htmp = [htool.whole; htool.pan; htool.zoomOut; htool.zoomIn; 
    htool.datacursor; htool.undo; 
    htool.tmPos; htool.curvButton; htool.measureLength;
    htool.removeRegion; htool.recoverRegion; htool.origImage; htool.editImage; 
    htool.saltPepper; 
    htool.contrast; htool.crop;
    htool.save
    ];
set(temp(1),'Children',htmp);



H.tbar=findall(findall(H.f1,'Type','uitoolbar'));
H.tbar_UNDO=findobj(temp(1),'Tag','Undo last');

% try to remove features that existed in previous versions
try, delete(H.CursorPosText), end
try, delete(H.strMousePos), end

% axes objects (repositioning)
set(H.output_mess,'position',posD.output_mess);
set(H.ax1,'position',posD.ax1);
set(H.ax2,'position',posD.ax2);
if isfield(H,'ax3')  % if objects for vertical analysis exists
    if ishandle(H.ax3)  % if the object still exists
        delete(H.ax3);
        rmfield(H,'ax3');
    end  % if ishandle(H.ax3)
end  % if isfield(H,'ax3')
set(H.axwait,'position',posD.axwait);

% uicontrols
% retrieve data, delete and reconstruct

% update the filename to loaded filename
set(H.textfilename,'string',filename);

temptimestr='yyyymmdd HH:MM';
try
temptimestr=H.edit_t0.String;
end
% tempnumticks='';
% try, tempnumticks=H.edit_num_of_ticks.String; end

if (isfield(H,'edit_num_of_ticks')) % if edit_num_of_ticks field exists
    cnumTicks = get(H.edit_num_of_ticks,'string');
else  % if field does not exist
    cnumTicks = '';
end  % if (isfield(H,'edit_num_of_ticks'))
if (isfield(H,'cb_timeEnd'))  % if cb_timeEnd field exists
    rvalTimeEnd = get(H.cb_timeEnd,'value');
else
    rvalTimeEnd = 0;
end  % if (isfield(H,'cb_timeEnd')

% get enable properties for time mark pushbuttons
clear ctmp
cfields = {'mark_1st','edit_mark_1st','mark_last','edit_mark_last',...
    'edit_num_of_ticks'};
[ctmp{1:numel(cfields)}] = deal('on');
for k = 1:numel(cfields)
    try
        ctmp{k} = get(H.(cfields{k}),'enable');
    end
end  % for k = 1:numel(cfields)

ctmpTimeEnd = 'on';
try; ctmpTimeEnd = get(H.cb_timeEnd,'enable'); end
    
% create again
try; delete(H.hp1); end  % if panel existed, all the objects within the panel are also deleted

% H.hp1 = uipanel('Title','Timing Information',...
%     'BackgroundColor','white',...
%     'Position',posD.hp1,'Parent',H.f1);

try; delete(H.edit_t0); end
cen = get(H.digitize_1,'enable');
try; cen = get(H.pbedit_start,'enable'); end
H.edit_t0=uicontrol('Style', 'pushbutton','Parent',H.f1,'units','normalized',...
    'TooltipString','Date & Time of the 1st reference time mark (seismogram with time marks), or of the begining of the 1st trace (seismigram without time marks)',...
    'string', temptimestr,'Position',posD.edit_t0,...
    'BackgroundColor',[1 1 1],'Callback',@Get_DATE_TIME,'enable',cen);
% uicontrol('Style', 'Text','Parent',H.hp1,'units','normalized',...
%     'String','# of time marks',...
%     'Position',posD.timeText,'BackgroundColor',[1 1 1]);
try; delete(H.edit_num_of_ticks); end
H.edit_num_of_ticks=uicontrol('Style', 'edit','Parent',H.f1,'units','normalized',...
    'TooltipString','Number of time marks between 1st and last (including 1st and last)',...
    'string',cnumTicks,'Position',posD.edit_num_of_ticks,'BackgroundColor',[1 1 1],...
    'enable',ctmp{5});
try; delete(H.cb_timeEnd); end
H.cb_timeEnd = uicontrol('style','checkbox','parent',H.f1,'units','normalized',...
    'TooltipString','Select if minute starts at the beginning of a time mark',...
    'position',posD.cb_timeEnd,'backgroundcolor',[1 1 1],'value',rvalTimeEnd,...
    'string','time left','horizontalAlignment','left','enable',ctmpTimeEnd);
try; delete(H.mark_1st); end
H.mark_1st=uicontrol('Style', 'pushbutton','Parent',H.f1,'units','normalized',...
    'Tooltip','Mark the ending pixels of the 1st time marks',...
    'String', '1st mark','Position',posD.mark_1st,...
    'Callback',@(h,e) mark_1st_and_last(h,e,'FIRST',false),...
    'enable',ctmp{1});
try; delete(H.edit_mark_1st); end
H.edit_mark_1st=uicontrol('Style', 'pushbutton','Parent',H.f1,'units','normalized',...
    'Tooltip','Edit the ending pixels of the 1st time marks',...
    'String', 'Edit','Position',posD.edit_mark_1st,...
    'Callback',@(h,e) mark_1st_and_last(h,e,'FIRST',true),'Enable',ctmp{2});
try; delete(H.mark_last); end
H.mark_last=uicontrol('Style', 'pushbutton','Parent',H.f1,'units','normalized',...
    'Tooltip','Mark the last pixel of the last time marks',...
    'String', 'Last mark','Position', posD.mark_last,...
    'Callback',@(h,e) mark_1st_and_last(h,e,'LAST',false),'enable',ctmp{3});
try; delete(H.edit_mark); end
H.edit_mark_last=uicontrol('Style', 'pushbutton','Parent',H.f1,'units','normalized',...
    'Tooltip','Edit ending pixels of the last time marks',...
    'String', 'Edit','Position', posD.edit_mark_last,...
    'Callback',@(h,e) mark_1st_and_last(h,e,'LAST',true),'Enable',ctmp{4});

clear ctmp
ctmp = get(H.digitize_1,'enable');
try; ctmp = get(H.mark_start,'enable'); end
delete(H.mark_start)
H.mark_start=uicontrol('Style', 'pushbutton','Parent',H.f1,'units','normalized',...
    'String', 'Ends of traces','Position', posD.mark_start,...
    'Callback',@(h,e) mark_start(h,e,0),'enable',ctmp,...
    'Tooltip',['Identify the start and end of each trace ',...
    '(start first, double click to finalize and move on to end, ',...
    'double click to finish)']);

ctmp = get(H.digitize_1,'enable');
try; ctmp = get(H.pbedit_start,'enable'); end
delete(H.pbedit_start)
H.pbedit_start=uicontrol('Style', 'pushbutton','Parent',H.f1,'units','normalized',...
    'String', 'Edit','Position', posD.pbedit_start,...
    'Callback',@(h,e) mark_start(h,e,1),'Enable',ctmp);

if isfield(H,'mark_end')  % mark_end field exists
    delete(H.mark_end);
    rmfield(H,'mark_end');
    delete(H.pbedit_end);
    rmfield(H,'pbedit_end');
end
% delete(H.mark_end)
% H.mark_end=uicontrol('Style', 'pushbutton','Parent',H.f1,'units','normalized',...
%     'String', 'End of traces','Position', [0.002 0.69 0.05 0.04],...
%     'Callback',@(h,e) mark_end(h,e,0));
% 
% delete(H.pbedit_end)
% H.pbedit_end=uicontrol('Style', 'pushbutton','Parent',H.f1,'units','normalized',...
%     'String', 'Edit','Position', [0.052 0.69 0.028 0.04],...
%     'Callback',@(h,e) mark_end(h,e,1),'Enable','on');
% 
cstr = get(H.togle_timebounds_visibility,'enable');
rtmp = get(H.togle_timebounds_visibility,'value');
delete(H.togle_timebounds_visibility),
H.togle_timebounds_visibility=uicontrol('Style', 'checkbox','Parent',H.f1,'units','normalized',...
    'String', 'Time boundaries visible','Enable',cstr,'BackgroundColor','w',...
    'Position', posD.togle_timebounds_visibility,'Callback',@togle_timebounds_visibility,...
    'value',rtmp);


try, delete(H.togle_time_marks), end
% H.togle_time_marks=uicontrol('Style', 'checkbox','Parent',H.f1,'units','normalized',...
%     'String', 'Timing symbols visible','Enable','on','BackgroundColor','w',...
%     'Position', [0.002 0.62 0.08 0.03],'Callback',@togle_timesymbols_visibility);

%DT

try; delete(H.imageMod); end
try; delete(H.imageMod_options); end

H.imageMod_options = uicontrol('style','pushbutton','parent',H.f1,...
    'units','normalized','position',posD.imageOption,'string','+',...
    'Tooltip','Additional parameters/algorithms for image processing',...
    'enable','on','callback',@imageMod_options);
H.imageMod=uicontrol('Style', 'pushbutton','Parent',H.f1,'units','normalized',...
    'String', 'Piecewise Contrast','Position', posD.image,...
    'Callback',@imageMod,'Enable','on',...
    'Tooltip','Modify image by adjusting contrast in small regions');

temp={'60','1','720','25'};
try
    temp{1}=H.edit_DT.String; temp{2}=H.edit_DTtrace.String;
    temp{3}=H.edit_DP.String; temp{4}=H.edit_tick_xlength.String;
end

delete(H.edit_DT)
delete(H.edit_DTtrace)
delete(H.edit_DP)
delete(H.edit_tick_xlength)
H.edit_DT=uicontrol('Style', 'edit','Parent',H.f1,'units','normalized',...
    'TooltipString','Time difference between timemarks (s)','String',temp{1},...
    'Position',posD.edit_DT,'BackgroundColor',[1 1 1],...
    'Enable','on','callback',@Check_length_input);
H.edit_DTtrace=uicontrol('Style', 'edit','Parent',H.f1,'units','normalized',...
    'TooltipString','time difference between succeding traces (in hours)','String',temp{2},...
    'Position',posD.edit_DTtrace,'BackgroundColor',[1 1 1],...
    'Enable','on','callback',@Check_length_input);
H.edit_DP=uicontrol('Style', 'edit','Parent',H.f1,'units','normalized',...
    'TooltipString','pixel dist between timemarks (px)','String', temp{3},...
    'Position',posD.edit_DP,'BackgroundColor',[1 1 1],...
    'Enable','on','callback',@Check_length_input);
H.edit_tick_xlength=uicontrol('Style', 'edit','Parent',H.f1,'units','normalized',...
    'TooltipString','Approximate time mark length (px)','String', temp{4},...
    'Position',posD.edit_tick_xlength,'BackgroundColor',[1 1 1],...
    'Callback',@Check_length_input);

ctmp = get(H.auto_num_of_traces,'enable');
delete(H.auto_num_of_traces)
H.auto_num_of_traces=uicontrol('Style', 'pushbutton','Parent',H.f1,'units','normalized',...
    'String', 'Identify traces','Position', posD.auto_num_of_traces,...
    'Callback',@find_traces,'Enable',ctmp);
H.idTrace_options = uicontrol('style','pushbutton','parent',H.f1,...
    'units','normalized','position',posD.idTrace_options,'string','+',...
    'Tooltip','Additional parameters that can be set for identify trace routine',...
    'enable',ctmp,'callback',@idTrace_options);

ctmp = get(H.adjust_traces,'enable');
delete(H.adjust_traces)
H.adjust_traces=uicontrol('Style', 'pushbutton','Parent',H.f1,'units','normalized',...
    'String', 'Adjust traces','Position', posD.adjust_traces,'Callback',@adjust_traces,...
    'enable',ctmp);

zero_visible = get(H.togle_traces_visibility,'value');
cstr = get(H.togle_traces_visibility,'enable');
delete(H.togle_traces_visibility)
H.togle_traces_visibility=uicontrol('Style', 'checkbox','Parent',H.f1,'units','normalized',...
    'String', 'Traces 0-line visible','Enable',cstr,'Position', posD.togle_traces_visibility,...
    'BackgroundColor','w','value',zero_visible,'Callback',@traces_visibility);
cstr = get(H.togle_digital_traces_visibility,'enable');
trace_visible = get(H.togle_digital_traces_visibility,'value');
delete(H.togle_digital_traces_visibility)
H.togle_digital_traces_visibility=uicontrol('Style', 'checkbox','Parent',H.f1,'units','normalized',...
    'String', 'Traces visible','Enable',cstr,'Position', posD.togle_digital_traces_visibility,...
    'BackgroundColor','w','value',trace_visible,'Callback',@digital_traces_visibility);
cstr = 'off';
std_visible = 0;  % not visible by default
try
cstr = get(H.togle_digital_traces_STD_visibility,'enable'); 
std_visible = get(H.togle_digital_traces_STD_visibility,'value');
delete(H.togle_digital_traces_STD_visibility);
end;
H.togle_digital_traces_STD_visibility=uicontrol('Style', 'checkbox','Parent',H.f1,'units','normalized',...
    'String', 'STD traces visible','Enable',cstr,'Position', posD.togle_digital_traces_STD_visibility,...
    'BackgroundColor','w','value',std_visible,'Callback',@digital_traces_visibility);

ctmp = get(H.correct_classification,'enable');
delete(H.correct_classification)
H.correct_classification=uicontrol('Style', 'pushbutton','Parent',H.f1,'units','normalized',...
    'String', 'Edit classification','Position', posD.correct_classification,...
    'Callback',@correct_classification,'Enable',ctmp);

ctmp = get(H.Create_Time,'enable');
delete(H.Create_Time)
H.Create_Time=uicontrol('Style', 'pushbutton','Parent',H.f1,'units','normalized',...
    'String', 'Calculate Timing','Position', posD.Create_Time,'Callback',@Create_Time,'Enable',ctmp);

try
    delete(H.text_luminance)
end
% H.text_luminance=uicontrol('Style', 'text','Parent',H.f1,'units','normalized',...
%     'String',{'Intensity';'threshold'},'Position', posD.text_luminance,'BackgroundColor',[1 1 1]);

temp='10';
try, temp=H.edit_luminance.String; end
delete(H.edit_luminance)
H.edit_luminance=uicontrol('Style', 'edit','Parent',H.f1,'units','normalized',...
    'TooltipString','Luminance thershold, should be between 1 and 99',...
    'String',temp,'Position', posD.edit_luminance,'BackgroundColor',[1 1 1],...
    'callback',@check_edit_luminance);

try
delete(H.text_offset)
end
% H.text_offset=uicontrol('Style', 'text','Parent',H.f1,'units','normalized',...
%     'String',{'Offset from trace';'to digitize'},'Position', posD.text_offset,...
%     'BackgroundColor',[1 1 1]);

% temp='[-0.3 0.3]';
% try, temp=H.edit_offset.String; end
if any(ishandle(H.edit_offset))  % if H.edit_offset is a handle
    cstr = get(H.edit_offset,'string');
    delete(H.edit_offset);
    H.edit_offset = cstr;  % reset to a string
end  % if any(ishandle(H.edit_offset))
% H.edit_offset=uicontrol('Style', 'edit','Parent',H.f1,'units','normalized',...
%     'TooltipString',...
%     'Offset bellow and over the trace as a portion of the average trace distance, typical value: [0.3 0.3]',...
%     'String',temp,'Position', posD.edit_offset,'BackgroundColor',[1 1 1]);
try
    delete(H.digitize_options)
end
ctmp = get(H.digitize,'enable');
delete(H.digitize)
H.digitize=uicontrol('Style', 'pushbutton','Parent',H.f1,'units','normalized',...
    'String', 'Digitize','Position', posD.digitize,'Callback',@digitize_traces,'Enable',ctmp);
H.digitize_options = uicontrol('style','pushbutton','parent',H.f1,...
    'units','normalized','position',posD.digitize_options,'string','+',...
    'Tooltip','Additional parameters that can be set for digitization routine',...
    'enable',ctmp,'callback',@digitize_options);

ctmp = get(H.digitize_1,'enable');
delete(H.digitize_1)
H.digitize_1=uicontrol('Style', 'pushbutton','Parent',H.f1,'units','normalized',...
    'String', 'Correct trace','Position', posD.digitize_1,...
    'Enable',ctmp,'Callback',@correct_trace,'backgroundcolor',[0.8 1 0.8]);
try; delete(H.correct_trace_options); end  % try deleting if the button already exists
H.correct_trace_options = uicontrol('style','pushbutton','parent',H.f1,...
    'units','normalized','position',posD.correct_trace_options,'string','+',...
    'Tooltip','Additional parameters that can be set for correct trace routine',...
    'enable',ctmp,'callback',@correct_trace_options);

try  % accommodate earlier versions without saveSAC function
    ctmp = get(H.saveSAC,'enable');
    delete(H.saveSAC);  % delete the object handle if saveSAC exists
end  % try
H.saveSAC=uicontrol('Style', 'pushbutton','Parent',H.f1,'units','normalized',...
    'String', 'Extract SAC','Position', posD.saveSAC,'Callback',@saveSAC,'Enable',ctmp);

try  % accommodate earlier versions without find_gaps checkbox
    ctmp = get(H.find_gaps,'enable');
    delete(H.find_gaps);
end
if isappdata(H.ax1,'hgap')
    hgap = getappdata(H.ax1,'hgap');
    delete(hgap);  % delete existing boxes showing gaps
    rmappdata(H.ax1,'hgap');
end  % if isappdata(H.ax1,'hgap')
H.find_gaps=uicontrol('Style', 'checkbox','Parent',H.f1,'units','normalized',...
    'String', 'Check for gaps','Enable',get(H.digitize,'enable'),'Position', posD.find_gaps,...
    'BackgroundColor','w','value',0,'Callback',@find_gaps,...
    'Tooltip','Check for any gaps in the digitized trace');
try  % accommodate earlier versions without find_gaps checkbox
    delete(H.find_vert);
end
if isappdata(H.ax1,'hvert')
    hgap = getappdata(H.ax1,'hvert');
    delete(hgap);  % delete existing boxes showing gaps
    rmappdata(H.ax1,'hvert');
end  % if isappdata(H.ax1,'hgap')
H.find_vert=uicontrol('Style', 'checkbox','Parent',H.f1,'units','normalized',...
    'String', 'Check for vertical','Enable',get(H.digitize,'enable'),'Position', posD.find_vert,...
    'BackgroundColor','w','value',0,'Callback',@find_gaps,...
    'Tooltip','Check for any nearly vertical lines in the digitized trace');

%delete(H.strMousePos)
%H.strMousePos=uicontrol('Style', 'pushbutton','Parent',H.f1,'units','normalized',...
%'String', '','Tooltip','Push to reset','Position', [0.002 0.002 0.08 0.04],'Callback',set (H.f1, 'WindowButtonMotionFcn', @mouseMove));

%set (H.f1, 'WindowButtonMotionFcn', @mouseMove); % Display continiously mouse location

% make sure all the lines in the image windows are visible as a default
try  % see if the trace-zero lines can be found
    if zero_visible  % trace-zero lines are to be shown
        set([pt_traces; pt_traceLABEL],'visible','on');  % turn trace-zero lines on
    else zero_visible  % trace-zero lines are to be shown
        set([pt_traces; pt_traceLABEL],'visible','off');  % turn trace-zero lines on
    end  % if zero_visible
catch  % trace-zero line couldn't be found
    disp('trace_zero lines couldn''t be identified: calculating');
    pt_traces=flipud(findobj(H.ax1,'Color','c','Linestyle','--'));
    pt_traceLABEL=flipud(findobj(H.ax1,'type','text','FontSize',10,'BackgroundColor',[1 1 0.800]));
    if zero_visible  % trace-zero lines are to be displayed
        set([pt_traces; pt_traceLABEL],'visible','on');  % turn trace-zero lines on
    else  % trace-zero lines not visible
        set([pt_traces; pt_traceLABEL],'visible','on');  % turn trace-zero lines on
    end  % if zero_visible
end

try  % see if digitized traces can be displayed
    %disp('trying mode');
    if trace_visible  % digitized traces visible
        set(ptrace,'visible','on');  % turn it on
    else  % digitized traces not visible
        set(ptrace,'visible','off');  % turn it on
    end  % if trace_visible
catch
    %disp('something wrong happened');
    disp('digitized traces couldn''t be identified: calculating');
    
    % recover traces
    h1=findobj(H.ax1,'Color',[0 0.8 1],'Visible','on'); % light blue
    h2=findobj(H.ax1,'Color',[0 0.6 1],'Visible','on'); % darker blue
    h3=findobj(H.ax1,'Color',[1 0.7 0],'Visible','on'); % orange
    hhh=[h1;h2;h3]; % gather them in a vector
    
    % check to find lines that are closest to the trace-zero lines
    if ~isempty(hhh)  % if there are traces to be checked
        
        ptrace = [];  % initial matrix
        for k = 1:numel(pt_traces)  % loop over trace-zero line
            xZ = get(pt_traces(k),'XData');  % x values
            yZ0 = get(pt_traces(k),'YData');  % y values
            rdist = [];  % distance
            for m = 1:numel(hhh)  % loop over left-over trace handles
                xT = get(hhh(m),'XData');  % x values
                yT = get(hhh(m),'YData');  % y values
                
                [rtmp iZ iT] = intersect(xZ,xT);  % find indices of same values
                yZ = yZ0(iZ);  yT = yT(iT);  % update y values
                indx = find(~isnan(yT));  % find not NaN values
                yZ = yZ(indx);  yT = yT(indx);  % update y values, now the arrays contain no NaN
                
                rtmp = sum((yZ - yT).^2)/numel(indx);  % average distance
                rdist = [rdist rtmp];
            end  % for m = 1:numel(hhh)
            
            [rtmp indx] = min(rdist);  % find closest trace
            ptrace = [ptrace; hhh(indx)];
            
            hhh(indx) = [];  % remove current index
        end  % for k = 1:numel(pt_traces)
        
    else  % if ~isempty(hhh)
        ptrace = [];  % no traces
    end  % if ~isempty(hhh)
    
    
    
    % when digitized traces are not available, STD are also not available
    % do the recover analysis now
    hhh=findobj(H.ax1,'Color',[0.3 0.1 1]);
    
    if ~isempty(hhh)
        
        ptraceSTD = [];  % initialize
        
       for k = 1:numel(ptrace)  % loop over trace-zero line
            xZ = get(ptrace(k),'XData');  % x values
            yZ0 = get(ptrace(k),'YData');  % y values
            rdist = [];  % distance
            for m = 1:numel(hhh)  % loop over left-over trace handles
                xT = get(hhh(m),'XData');  % x values
                yT = get(hhh(m),'YData');  % y values
                
                [rtmp iZ iT] = intersect(xZ,xT);  % find indices of same values
                yZ = yZ0(iZ);  yT = yT(iT);  % update y values
                indx = find(~isnan(yT));  % find not NaN values
                yZ = yZ(indx);  yT = yT(indx);  % update y values, now the arrays contain no NaN
                
                rtmp = sum((yZ - yT).^2)/numel(indx);  % average distance
                rdist = [rdist rtmp];
            end  % for m = 1:numel(hhh)
            
            [rtmp indx] = min(rdist);  % find closest trace
            ptraceSTD = [ptraceSTD; hhh(indx)];
            
            hhh(indx) = [];  % remove current index
        end  % for k = 1:numel(pt_traces)
        
    end
    
    if (trace_visible)  % display digitized traces
        set(ptrace,'visible','on');  % turn it on
    else  % do not display digitized traces
        set(ptrace,'visible','off');  % turn it on
    end  % if (trace_visible)...

end

% % if digitized traces exist, turn Correct Trace button on
% if exist('ptrace')
% rtmp = get(ptrace,'XData');
% if ~isempty(rtmp)
%     set(H.digitize_1,'enable','on');
% end
% end

try  % see if digitized STDtraces can be displayed
    %disp('trying mode (STD)');
    if std_visible  % if std lines visible
        set(ptraceSTD,'visible','on');  % turn it on
    else  % if std lines not visible
        set(ptraceSTD,'visible','off');  % turn it on
    end  % if std_visible...
catch
    %disp('something wrong happened (STD)');
%    errordlg('Could not find handles for digitized traces STD. If this file is from an older version of DigitSeis without this feature, ignore this message');
    %set(ptraceSTD,'visible','off');
    set(H.togle_digital_traces_STD_visibility,'value',0);
end

% make sure all objects have trace numbers
if (numel(S)>1)  % there are S objects
    if ~isfield(S,'TraceNum'); % TraceNum field does not exist
        [S.TraceNum] = deal(-1);
    else  % find if there are any empty TraceNum fields
        indx = find(cellfun('isempty',{S.TraceNum}));
        if ~isempty(indx)
            [S(indx).TraceNum] = deal(-1);
        end  % if ~isempty(indx)
    end  % if ~isfield(S,'TraceNum')...
end

disp('Completed.')
close(hmsg)

set(H.f1,'Visible','on')

drawnow
end



function Saveresults(hObject, eventdata)
global H HIM I0 Iorig HIMRGB BW ptrace ptraceSTD pt_traces pt_traceLABEL Time ftrend ftrend_tick S output_message  sumI ww pt_start_time pt_end_time p_tick_1st p_tick_last hresult_time_marks
global SEIS DigitSeisVersion
global pt_tracesY0

%savefig(H.f1,['FIG_' get(H.textfilename,'String') '_.fig'])

%Gui_data = guidata(H.f1);

% Make everything visible
try
    set(H.togle_timebounds_visibility,'value',1); togle_timebounds_visibility;
end
try
    set(H.togle_time_marks,'value',1); togle_timesymbols_visibility;
end
try
    set(H.togle_traces_visibility,'value',1); traces_visibility;
end
try
    set(H.togle_digital_traces_visibility,'value',1); digital_traces_visibility;
end
if ~isfield(H,'version')
    H.version = DigitSeisVersion;
end  % if ~isfield(H,'version')


cstr = get(H.textfilename,'String');
DefaultName = [];
if length(cstr)>10 & strcmpi(cstr(1:10),'DigitSeis_')  % string longer than 10 characters and starts with 'DigitSeis_'
    cstr(1:10) = [];  % remove 'DigitSeis_' part
end
DefaultName = ['DigitSeis_',cstr];
if ~strcmpi(DefaultName(end-3:end),'.mat')  % filename not ending with .mat
    DefaultName = [DefaultName,'.mat'];
end  % if ~strcmpi(cstr(end-3:end),'.mat')


[FileName,PathName] = uiputfile('.mat','Save Analysis and Results',DefaultName)

if isequal(FileName,0) || isequal(PathName,0)
    return
else
    
    
    % this is for making sure pt_traces are saved (DEBUGING)
    
%     pt_tracesY=get(pt_traces,'Ydata');
%     pt_tracesX=get(pt_traces,'Xdata');
    
    if any(ishandle(ptrace))
        itr=find(ishandle(ptrace));
        for iii=itr(:)' % to force itr be a row vector
            ptraceY{iii}=get(ptrace(iii),'Ydata');
            ptraceX{iii}=get(ptrace(iii),'Xdata');
        end
    else
        ptraceX={};
        ptraceY={};
    end
    
    hmess = dialog('Name','Saving Data');
    cstr = sprintf('%s\n\n%s','Saving Data',...
        'Please be patient as this may take some time.  This window will close when save completes.');
    ctxt = uicontrol('parent',hmess,'style','text','units','normalized',...
        'position',[0.1 0.3 0.8 0.4],'string', cstr,...
        'horizontalAlignment','center','fontSize',18);
    drawnow
    %     pause(5);
    %     delete(hmess);
    
%     % clean up structure before saving
%     try  % run if S exists
%     indx = find([S.ID]==-1);  % noise objects
%     S(indx) = [];  % remove noise objects
%     end
    
    % update version information
    itmp = H.version;
    itmp(end+1) = DigitSeisVersion;
    H.version = unique(itmp);
    
    save(fullfile(PathName,FileName),'H','HIM','HIMRGB','BW',...
        'I0', 'Iorig','ptrace','ptraceSTD', 'pt_traces',... %'pt_tracesX',...
        'pt_tracesY0','ptraceX','ptraceY','pt_traceLABEL', 'Time',...
        'ftrend','ftrend_tick','S','output_message','sumI', 'ww', ...
        'pt_start_time', 'pt_end_time', 'p_tick_1st', 'p_tick_last',...
        'hresult_time_marks','-v7.3');
    
    % if SAC format structure has been created
    if exist('SEIS')>0  % SEIS variable exists
        if numel(SEIS)>0  % something exists
            save(fullfile(PathName,FileName),'SEIS','-append')
        end  % if numel(SEIS)>0
    end
    
    delete(hmess);
end

end

function saveSAC(hObj,eventdata)

global H HIM I0 HIMRGB BW ptrace ptraceSTD pt_traces pt_traceLABEL Time 
global ftrend ftrend_tick S output_message  sumI ww 
global pt_start_time pt_end_time p_tick_1st p_tick_last hresult_time_marks
global SEIS

clear SEIS

% check if STD traces exist (in earlier version of DigitSeis, STD was not
% calculated)
lstd = true;  % by default assume STD traces do exist
if min([H.version]<1)  % possibility of STD not existing
    if isempty(ptraceSTD)  % ptraceSTD empty
        lstd = false;
    else  % ptraceSTD is not empty
        if (numel(ptraceSTD)~=numel(ptrace))  % number of traces do not match
            lstd = false;
        else  % same number of ptraceSTD and ptrace exist
            for k = 1:numel(ptrace)  % loop over traces
                rtmp = find(~isnan(get(ptrace(k),'YDATA')));
                rtmp2 = get(ptraceSTD(k),'YDATA');
                if ~isempty(rtmp2)
                    if numel(rtmp)~=numel(find(~isnan(rtmp2)))
                        lstd = false;
                        break
                    end  % if numel(rtmp)~=numel(find(~isnan(rtmp2)))
                else
                    lstd = false;
                    break
                end  % if ~isempty(rtmp2)...
            end  % for k = 1:numel(ptrace)
        end  % if (numel(ptraceSTD)~=numel(ptrace))...
    end  % if isempty(ptraceSTD)
end  % if min([H.version]==0)

SEIS(length(ptrace))=struct('B',[],'DELTA',[],'x',[],'t',[],'E',[],'DATA1',[]);
for i=1:numel(ptrace)
    SEIS(i).DELTA=Time.DELTA;
%     SEIS(i).x=get(ptrace(i),'XDATA');
%     SEIS(i).x = get(pt_traces(i),'XDATA');  % limit range to start/end marks selected
    % this takes into account when digitization is performed before
    % start/end time selection 
    [rtmp indxZ indxT] = intersect(get(pt_traces(i),'XDATA'),get(ptrace(i),'XDATA'));
    SEIS(i).x = rtmp;  % valid X position information
    
    % This need update for efficiency.
    if str2double(get(H.edit_tick_xlength,'String'))>0
        SEIS(i).t=Time.trace(i).fx_t(SEIS(i).x);
    else
        SEIS(i).t=Time.trace(i).fx_t;
    end
    
    SEIS(i).FILENAME=[datestr(datestr(SEIS(i).t(1)),'yyyymmdd_HHMM') '_' datestr(datestr(SEIS(i).t(end)),'yyyymmdd_HHMM') '.SAC'];
    SEIS(i).B=etime(datevec(SEIS(i).t(1)),datevec(Time.RefDate));  % relative time from Reference Time
    SEIS(i).E=etime(datevec(SEIS(i).t(end)),datevec(Time.RefDate));  % relative time from Reference Time
    
    rtmp = get(pt_traces(i),'YDATA');
    SEIS(i).trend=rtmp(indxZ);
    
    SEIS(i).reference_date=Time.RefDate;
    
    reltime=SEIS(i).B : SEIS(i).DELTA : SEIS(i).E;
    abstime=repmat(datevec(SEIS(i).reference_date),length(reltime),1);
    abstime(:,6)=abstime(:,6)+reltime(:);
    abstime=datenum(abstime);
    
    % remove the quadratic trend from each trace
    rtmp = get(ptrace(i),'YDATA');
    y = rtmp(indxT) - SEIS(i).trend;  % removing the trend associated with the one calculated for zero-trace line
    indx = find(~isnan(y));  % find indices that are not NaN's
    x = SEIS(i).x;  % x values
    p = polyfit(x(indx), y(indx),2);  % n = 2 for quadratic fit p(1)*x^2 + p(2)*x + p(3)
    ytrend = polyval(p,x);  % trend values
    y(indx) = y(indx) - ytrend(indx);  % only update numerical entries
    % update std values with the same trend correction
    if lstd  % STD trace exists
        rtmp = get(ptraceSTD(i),'YDATA');
        ystd = rtmp(indxT) - SEIS(i).trend;
        ystd(indx) = ystd(indx) - ytrend(indx);
    else  % if std line doesn't exist, set to zero line
        ystd = SEIS(i).trend;
    end  % if lstd
 
    SEIS(i).y=y;
    SEIS(i).ySTD=ystd;
    
    SEIS(i).DATA1 = y;

%     SEIS(i).DATA1=SEIS(i).y-SEIS(i).trend;  
%     meanDATA1=mean(SEIS(i).DATA1,'omitnan');
%     SEIS(i).DATA1=SEIS(i).DATA1-meanDATA1;
    SEIS(i).DATA1=interp1(SEIS(i).t,SEIS(i).DATA1,abstime);
    
    if ~isempty(ptraceSTD)
        SEIS(i).DATA2=SEIS(i).ySTD-SEIS(i).y;
        SEIS(i).DATA2=interp1(SEIS(i).t,SEIS(i).DATA2,abstime);
    else
        SEIS(i).DATA2=[];
    end
    
    SEIS(i).NPTS=length(SEIS(i).DATA1);
    
    
    [y, m, d, HH, M, SEC]=datevec(Time.RefDate);
    SEIS(i).NZYEAR=y;
    SEIS(i).NZJDAY=datenum(y,m,d) - datenum(y, 1, 1) + 1;
    SEIS(i).NZHOUR=HH;
    SEIS(i).NZMIN=M;
    SEIS(i).NZSEC=floor(SEC);
    SEIS(i).NZMSEC=round(1000*(SEC - SEIS(i).NZSEC));
    
end

saveassac(SEIS);

end



function check_edit_luminance(hobj,eventdata)
% this function checks the value entered for the bw conversion threshold
% value and makes sure that it is between 1 and 99

htmp = hobj;
ndef = 10;  % default value - this is the value used if a value not between 1 and 99 is entered

nnew = str2num(get(htmp,'string'));

if ~isempty(nnew)  % check the entry value
    nnew = round(nnew);  % make sure it is an integer
    if nnew>0 & nnew<100  % valid number
        ndef = nnew;
    else  % inform user
        errordlg('Please enter a number between 1 and 99','Threshold Value Check');
    end  % if nnew>0 & nnew<100
else  % let the user know what is expected
    errordlg('Please enter a number between 1 and 99','Threshold Value Check');
end  % if ~isempty(nnew)

set(htmp,'string',num2str(ndef));  % update value

end  % function check_edit_luminance




function Create_Time(hobject,eventdata)
global H Time pt_traces

tol_time_check = 0.2;  % fraction of tolerance for time checks

% check to make sure time has been selected
try  % see if we can get a meaningful date
    date0=datenum(get(H.edit_t0,'String'),'yyyymmdd HH:MM');
catch
    errordlg('Please provide starting trace/time mark date/time before calculating timing!');
    return
end  % try

% check to make sure that time mark number is consistent with different
% settings
dtm = str2double(get(H.edit_DT,'string'));  % duration between time marks (in seconds)
dtrace = str2double(get(H.edit_DTtrace,'string'));  % length of a single trace in hours
pixMain = str2double(get(H.edit_DP,'string'));  % # of pixels between time marks
pixTM = str2double(get(H.edit_tick_xlength,'string'));  % # of pixels for a given time mark
ntm=str2double(get(H.edit_num_of_ticks,'String'));

if pixTM>0  % time marks exist
    xend=cell2mat(cellfun(@(v) v(end),get(pt_traces,'Xdata'),'UniformOutput',false));
    dtrace_pre = max(xend)/pixMain*dtm/3600;  % predicted length of trace
    
    ntm_pre = round(dtrace*3600/dtm);  % predicted number of time marks/trace
    
    if ( (abs(dtrace_pre - dtrace)>tol_time_check*dtrace) | ...
            (abs(ntm_pre-ntm)>tol_time_check*ntm) )
        cstr = sprintf('%s\n%s %3.2f%s%3.2f%s\n%s%d%s%d%s\n%s',...
            'There seems to be discrepancy in the time setting:',...
            '   Duration/trace: ',dtrace,' (input) & ',dtrace_pre,' (based upon pixels)',...
            '   Number of time marks: ', ntm, ' (input) & ', ntm_pre, ' (based upon duration)',...
            'Are you sure to continue?');
        cans = questdlg(cstr,'Time Setting Confirmation','Yes','No','No');
        if ~strcmpi(cans,'Yes')  % stop execution
            return
        end  % if ~strcmpi(cans,'Yes')
    end  % if either of the two parameters is off by more than tol_time_check fraction
end

try  % make sure time calculation doesn't fail
    
    % check to make sure that parameters make sense
    
    if str2double(get(H.edit_tick_xlength,'String'))>0
        Time=pixels2time;
    else
        Time=pixels2time_no_timemarks;
    end
    
    % if everything goes well, activate SAC extraction button
    set(H.saveSAC,'enable','on');
end

end




function Time=pixels2time_no_timemarks()
global I0 H ftrend pt_traces hresult_time_marks
% if no time marks, then tick_length should be < 0 e.g., -999
% then Reference time SHOULD CORRESPOND to the beginging of the 1st trace.
% the p_tick_1st and last will not taken into consideration (the user can just ignore them)

Time=[];

% update user
%% needs improvement
output_message=get(H.output_mess,'String');
output_message{end+1}=['Timing seismogram... please wait'];
set(H.output_mess,'String',output_message,'Value',length(output_message)); drawnow
%%


try, delete(hresult_time_marks); end;
delete(findobj(H.ax1,'color','y'))
delete(findobj(H.ax1,'color','m'))
delete(findobj(H.ax1,'color',[1 1 0]))
delete(findobj('Type','rectangle'))

[rows,cols]=size(I0);

date0=datenum(get(H.edit_t0,'String'),'yyyymmdd HH:MM');
%t0=datevec(date0);
pt_traces_Xdata=get(pt_traces,'Xdata');
%pt_traces_Ydata=get(pt_traces,'Ydata');

Dtrace=str2double(get(H.edit_DTtrace,'String')); % time between different traces in hours

% approximate num of pixels per trace.
%         <----l2---->|<--l1-->
%                 date0~~~~~~~~current_date
%          ~~~~~~~~~~~|~~~~~~~~
%          ~~~~~~~~~~~~~~~~~~~~


x1=pt_traces_Xdata{1}(1);
l1=abs(diff(pt_traces_Xdata{1}([1:end])));
[~,ix2]=min(pt_traces_Xdata{2}-x1);
l2=abs(diff(pt_traces_Xdata{2}([1 ix2]))) + ( pt_traces_Xdata{1}(end) - pt_traces_Xdata{2}(end) );

l1Time=l1*Dtrace/(l1+l2);
%l2Time=l2*Dtrace/(l1+l2);


tleft=datevec(addtodate(date0,round(1000*l1Time),'millisecond')); %the time at the left of the current trace

Time.RefDate=date0;
Time.DELTA=-inf;
for i=1:length(pt_traces)
    pxs=1:numel(pt_traces_Xdata{i});
    DELTAx=Dtrace/numel(pt_traces_Xdata{i}) * sqrt(1+diff(1+ftrend(pt_traces_Xdata{i}(1:end))).^2);
    Time.DELTA=max([DELTAx(:);Time.DELTA]);
    DELTAx=[DELTAx(1);DELTAx];
    
    tv=datevec(datenum(repmat([tleft(1:3) tleft(4)+(i-1)*Dtrace tleft(5:6)],length(pxs),1)));
    tv(:,6)=cumsum(DELTAx(pxs))-DELTAx(1);
    
    % Create time vector for each trace: Fx_t in this case is directly the
    % time vector.
    Time.trace(i).fx_t=datenum(tv);
    
    %% have to do
    %plot time marks in this case
    
    
end



% update user
output_message{end+1}=['Ready'];
set(H.output_mess,'String',output_message,'Value',length(output_message)); drawnow




end


function Time=pixels2time()
global I0 H S ftrend pt_traces pt_traceLABEL p_tick_1st p_tick_last hresult_time_marks

% check to make sure # of time marks is available
num_of_ticks=str2double(get(H.edit_num_of_ticks,'String'));
if isnan(num_of_ticks)  % if number isn't available
    errordlg('Please provide number of time marks before calculating timing!');
    return
end  % if isnan(num_of_ticks)

% update user
%% needs improvement
output_message=get(H.output_mess,'String');
output_message{end+1}=['Timing seismogram... please wait'];
set(H.output_mess,'String',output_message,'Value',length(output_message)); drawnow
%%

ltend = true;  % true if the minute starts at the end of a time mark (default)f
% turn this to false if the minute starts at the beginning
if get(H.cb_timeEnd,'value')  %  minute starts at the beginning of a time mark
    ltend = false;  % true if the minute starts at the end of a time mark
end

Time=[];

try, delete(hresult_time_marks); end;
set(H.find_gaps,'value',0);
if isappdata(H.ax1,'hgaps')
    rmappdata(H.ax1,'hgaps'); % remove information since the objects will be deleted below
end  % if isappdata(H.ax1)

delete(findobj(H.ax1,'color','y'))
delete(findobj(H.ax1,'color','m'))
delete(findobj(H.ax1,'color',[1 1 0]))
delete(findobj('Type','rectangle'))

[rows,cols]=size(I0);

date0=datenum(get(H.edit_t0,'String'),'yyyymmdd HH:MM');
t0=datevec(date0);
pt_traces_Xdata=get(pt_traces,'Xdata');
pt_traces_Ydata=get(pt_traces,'Ydata');
x0=get(p_tick_1st,'Xdata');
x1=get(p_tick_last,'Xdata');
num_of_ticks=str2double(get(H.edit_num_of_ticks,'String'));

% Find the time kmark that corresponds to the reference time (1st time mark)
i_reference= find(~(isnan(x0)),1);


Dtrace=str2double(get(H.edit_DTtrace,'String')); % time between different traces
dt=str2double(get(H.edit_DT,'String'));

% theoretical delta correction as a function of x pixel coordinate
dp1=str2double(get(H.edit_DP,'String'));
tick_length=str2double(get(H.edit_tick_xlength,'String'));
itics=([S.ID]==1|[S.ID]==3);

CALL=cell2mat({S(itics).BoundingBox}');
tmTraceNum = [S(itics).TraceNum];  % trace number of each box

hold(H.ax1,'on')


a=nan(length(pt_traces),1);
for i=1:length(pt_traces) % optimize ftrend
    if ~isnan(x0(i)) &&  ~isnan(x1(i))
        a(i) = fminbnd(@(x)CostDelta(x,dt,dp1,ftrend,cols,x0(i):x1(i),num_of_ticks) ,0.9,1.1);
    end
end
a=median(a(~isnan(a)));


% set length unit for the vertical bars based upon trace spacing
rtmp=cell2mat(cellfun(@(v) v(1),get(pt_traces,'Ydata'),'UniformOutput',false));
%dy=round(mode(abs(diff(rtmp)))/2);
% [ncount yedges] = histcounts(diff(rtmp));
% [rtmp indx] = max(ncount);
% dy = mean(yedges(indx) + yedges(indx+1))/2.0; % width for defining objects
dy = mean(abs(diff(rtmp)))/2;
% rtmp = [];
% for k = 2:numel(pt_traces_Ydata)  % loop over y positions of the traces
%     rtmp = [rtmp pt_traces_Ydata{k}(1)-pt_traces_Ydata{k-1}(1)];
% end  % for k = 1:numel(pt_traces_Ydata)
% dy = round(mean(abs(rtmp))/2);
if strcmpi(get(H.timemarkpos,'state'),'off')
    npt_traces_togo=length(pt_traces);
    ylimits=[-dy; round(dy/10)];
else
    npt_traces_togo=length(pt_traces)-1;
    ylimits=[dy; -round(dy/10)];
end

hwait = waitbar(0,'Processing time marks and calculating time for each trace');
nwait = 0;

DELTA=-inf;
hminline = [];  % vector containing handles of imline objects
indxLine = [];  % matrix containing the indices to identify the trace # of the line object and which time point
% tmwidthAll = [];  % valid time mark distance for average calculation
ntmNT = 1;  % default value for the traces with trace being shorter than selected
lcheck = true;
if (min([H.version])>0)  % all time marks have been identified with a trace
    lcheck = false;
end  % if (min([H.version])>0)
for i=1:npt_traces_togo
    
    if (~lcheck)  % if all relevant objects have trace assignments
        ic = find(tmTraceNum==i);
    else  % use object locations to infer time marks that belong to each trace
        if strcmpi(get(H.timemarkpos,'state'),'off')
            ic= CALL(:,2)+CALL(:,4)/2<=pt_traces_Ydata{i}(1)+ftrend(CALL(:,1)) &...
                CALL(:,1)>=pt_traces_Xdata{i}(1) & CALL(:,1)+CALL(:,3)<=pt_traces_Xdata{i}(end);
        else
            ic= CALL(:,2)+CALL(:,4)/2<=pt_traces_Ydata{i+1}(1)+ftrend(CALL(:,1)) &...
                CALL(:,1)>=pt_traces_Xdata{i}(1) & CALL(:,1)+CALL(:,3)<=pt_traces_Xdata{i}(end);
        end
    end  % if (min([H.version])>0)...
    
    if ~isnan(x0(i)) &&  ~isnan(x1(i))  
        
        ipxs=find(pt_traces_Xdata{i}==x0(i)):find(pt_traces_Xdata{i}==x1(i));
        pxs=x0(i):x1(i);
        
        DELTAx=dt/dp1 * a*sqrt(1+diff(1+ftrend(1:cols)).^2); DELTA=max([DELTA; DELTAx(:)]);
        tempY=pt_traces_Ydata{i}(ipxs);
        
        tv=datevec(datenum(repmat([t0(1:3) t0(4)+(i-i_reference)*Dtrace t0(5:6)],length(pxs),1)));
        
        tv(:,6)=cumsum(DELTAx(pxs))-DELTAx(ipxs(1));
        tnum=datenum(tv);
        tv=datevec(tnum);
        iMM=[true; abs(diff(tv(:,5)))>eps]'; % find round minutes
        if (tv(end,6)<eps)
            iMM(end)=true;
        end
        
        theor_ticks=pxs(iMM);
        
        tempX=repmat(theor_ticks,2,1);
        tempY=repmat(tempY(iMM),2,1);
        tempY=tempY+repmat(ylimits,1,size(tempY,2));
        plot(H.ax1,tempX,tempY,'m--','LineWidth',1);
        
        % find tick marks of current trace
        
        %ic= CALL(:,2)<=[pt_traces_Ydata{i}(1)+ftrend(CALL(:,1))];
        C=CALL(ic,:);
        if lcheck  % time marks are being sought
        CALL(ic,:)=[];
        end  % if lcheck
        
        
        %plot(H.ax1,1:cols,[pt_traces_Ydata{i}(1)+ftrend(1:cols)],'y:')
        
        % combine very close tickmarks
        [~,isortx]=sort(C(:,1));
        C=C(isortx,:);
        iclose=find([C(2:end,1) - C(1:end-1,1)] < tick_length);
        if ~isempty(iclose)
            C(iclose,3)=max(C(iclose,1)+C(iclose,3),C(iclose+1,1)+C(iclose+1,3))-C(iclose,1);
            C(iclose,4)=max(C(iclose,2)+C(iclose,4),C(iclose+1,2)+C(iclose+1,4))-min(C(iclose,2),C(iclose+1,2));
            C(iclose+1,:)=[];
        end
        if (ltend)  % minute starts at the end of the time mark
            actual_ticks=C(:,1)+C(:,3)-0.5; 
            % offset of 0.5 to take into account that the boxes are 0.5 larger than the pixels of time marks
        else  % if minute starts at the beginning of the time mark
            actual_ticks = C(:,1)+0.5;
        end  % if (ltend)...
        tmwidth = C(:,3);
        
        for k=1:size(C,1)
            rectangle('Position',C(k,:),'EdgeColor','y','Parent',H.ax1);
        end
        
        ty=tnum(iMM);
        [Dist,iDist] = pdist2(theor_ticks',actual_ticks,'euclidean','Smallest',1);
        ty=ty(iDist);
        
        % delete distant ticks
        ito_delete=abs(Dist)>dp1/3;
        iDist(ito_delete)=[];
        ty(ito_delete)=[];
        actual_ticks(ito_delete)=[];
        tmwidth(ito_delete) = [];
        
        % delete doubles
        ito_delete=false(size(iDist));
        for j=1:length(iDist)-1
            if iDist(j)==iDist(j+1);
                ito_delete(j:j+1)=true;
            end
        end
        ty(ito_delete)=[];
        actual_ticks(ito_delete)=[];
        iDist(ito_delete)=[];
        tmwidth(ito_delete) = [];
        
        %      if numel(unique(iDist))~=numel(iDist);
        %         error('iDist doubles ')
        %     end
        
%         tmwidthAll = [tmwidthAll; tmwidth];  % collect time marks
%         disp(['in pixels2time i, # of actual_ticks: ', num2str(i),',', num2str(numel(actual_ticks))]);
        if (numel(actual_ticks)>1)  % require at least three time marks
            
            % Create function(x)-->time for each trace
            Time.trace(i).fx_t=fit(actual_ticks,ty,'linearinterp');
            
            % plot
            tv=datevec(Time.trace(i).fx_t(pt_traces_Xdata{i}));
            iMM=[abs(diff(tv(:,5)))>eps; false]';
            if (tv(end,6)<0.001)
                iMM(end)=true;
            end
            
            % make sure the time is snapped to the minute
            tv(iMM,5) = tv(iMM,5) + round(tv(iMM,6)/60);
            tv(iMM,6) = 0;

            xpos = pt_traces_Xdata{i}(iMM)';
            indx_tmp = (xpos>-1000);
            if (numel(actual_ticks)>0)  % there are time marks
                for k = 1:numel(actual_ticks)  % check for actual time mark locations against available time points
                    rtmp = abs(xpos - actual_ticks(k));  % distance from actual tick end location
                    %itmp = find(rtmp<tmwidth(k));  % find time points very close to the actual time mark
                    itmp = find(rtmp<tick_length);  % find time points very close to the actual time mark
                    if ~isempty(itmp)  % corresponding minute position found
                        if (numel(itmp)==1)  % there is only one minute mark
                            xpos(itmp) = actual_ticks(k);  % replace with the actual time mark position
                            indx_tmp(itmp) = false;
                        else
                            disp(['!!Error(pixels2time): trace ',num2str(i),...
                                ', tick number ', num2str(k), ', # of minima = ', num2str(numel(itmp))]);
                        end  % if (numel(itmp)==1)
                    end  % if ~isempty(itmp)
                end  % for k = 1:numel(actual_ticks)
            end  % if (numel(actual_ticks)>0)...
                        
            tempX=repmat(xpos',2,1);
            tempY=repmat(pt_traces_Ydata{i}(iMM),2,1);
            tempY=tempY+repmat(ylimits,1,size(tempY,2));
            htmp = plot(H.ax1,tempX(:,indx_tmp),tempY(:,indx_tmp),'-o','Color',[1 1 0],...
                'markerfacecolor',[1 0 0]);
            htmp2 = plot(H.ax1,tempX(:,~indx_tmp),tempY(:,~indx_tmp),'-o','Color',[1 1 0]);
            htmp = [htmp; htmp2];
            hminline = [hminline; htmp];
            rtmp2 = [find(indx_tmp); find(~indx_tmp)];
            rtmp = [i*ones(size(htmp)) rtmp2];
            indxLine = [indxLine; rtmp];
            %         disp(['trace number: ', num2str(i)]);
            %         tempX = pt_traces_Xdata{i}(iMM);
            %         tempY = pt_traces_Ydata{i}(iMM);
            %         % plot individual vertical time bars as draggable objects
            %         for k = 1:numel(find(iMM))
            %             htmp = imline(H.ax1,[tempX(k); tempX(k)],[tempY(k)+ylimits]);
            %             htmp.Deletable = false;
            %             set(htmp,'tag',sprintf('%d.%d',i,k));
            %             setColor(htmp,[1 1 0]);
            %             setPositionConstraintFcn(htmp,@linePosConstraint);
            %             himline = [himline htmp];
            %         end  % for k = 1:numel(iMM)
            
            Time.trace(i).tmXloc = xpos;  % locations of the "time marks" (even when they are not classified as such)
            Time.trace(i).tmTime = datenum(tv(iMM,:));  % times of the "time marks"
        else   % there are enough time marks
            tmNT(ntmNT).traceNo = i;
            if (ltend)  % minute starts at the end of the time mark
                tmNT(ntmNT).actual_ticks=C(:,1)+C(:,3)-0.5;
            else  % if minute starts at the beginning of the time mark
                tmNT(ntmNT).actual_ticks = C(:,1)+0.5;
            end  % if (ltend)...
%             tmNT(ntmNT).actual_ticks = C(:,1)+C(:,3)-0.5;
            tmNT(ntmNT).width = C(:,3);
            ntmNT = ntmNT + 1;
            %         actual_ticks=C(:,1)+C(:,3);
            
            Time.trace(i).fx_t=[];
            Time.trace(i).tmXloc = [];
            Time.trace(i).tmTime = [];
        end  % if there are more than 3 time marks
        
    else
        % plot boxes around time marks
        C = CALL(ic,:);
        % combine very close tickmarks
        [~,isortx]=sort(C(:,1));
        C=C(isortx,:);
        iclose=find([C(2:end,1) - C(1:end-1,1)] < tick_length);
        if ~isempty(iclose)
            C(iclose,3)=max(C(iclose,1)+C(iclose,3),C(iclose+1,1)+C(iclose+1,3))-C(iclose,1);
            C(iclose,4)=max(C(iclose,2)+C(iclose,4),C(iclose+1,2)+C(iclose+1,4))-min(C(iclose,2),C(iclose+1,2));
            C(iclose+1,:)=[];
        end
        tmNT(ntmNT).traceNo = i;
        if (ltend) % if minute starts at the end of the time mark
            tmNT(ntmNT).actual_ticks = C(:,1)+C(:,3)-0.5;
        else  % if minute starts at the beginning of the time mark
            tmNT(ntmNT).actual_ticks = C(:,1)+0.5;
        end  % if (ltend)...
        tmNT(ntmNT).width = C(:,3);
        ntmNT = ntmNT + 1;
%         actual_ticks=C(:,1)+C(:,3);
        
        for k=1:size(C,1)  % plot rectangles
            rectangle('Position',C(k,:),'EdgeColor','y','Parent',H.ax1);
        end
        
        % find and remove tick marks of current trace
        if (lcheck)  % if time marks are being identified
        CALL(ic,:)=[];
        end  % if (lcheck)
        
        Time.trace(i).fx_t=[];
        Time.trace(i).tmXloc = [];
        Time.trace(i).tmTime = [];
    end
    
    nwait = nwait + 1;
    waitbar((nwait/length(pt_traces)),hwait);

end
% put an empty vector for the last trace if time-marks are downwards
if npt_traces_togo<length(pt_traces)
    i = length(pt_traces);
    
    if (~lcheck)  % trace assignments exist
        ic = (tmTraceNum==i);
    else  % trace assignements not very reliable
        ic= CALL(:,2)+CALL(:,4)/2>=pt_traces_Ydata{i}(1)+ftrend(CALL(:,1)) &...
            CALL(:,1)>=pt_traces_Xdata{i}(1) & CALL(:,1)+CALL(:,3)<=pt_traces_Xdata{i}(end);
    end  % if (min([H.version])>0)...
    % draw rectanbles
    % plot boxes around time marks
    C = CALL(ic,:);
    % combine very close tickmarks
    [~,isortx]=sort(C(:,1));
    C=C(isortx,:);
    iclose=find([C(2:end,1) - C(1:end-1,1)] < tick_length);
    if ~isempty(iclose)
        C(iclose,3)=max(C(iclose,1)+C(iclose,3),C(iclose+1,1)+C(iclose+1,3))-C(iclose,1);
        C(iclose,4)=max(C(iclose,2)+C(iclose,4),C(iclose+1,2)+C(iclose+1,4))-min(C(iclose,2),C(iclose+1,2));
        C(iclose+1,:)=[];
    end
    tmNT(ntmNT).traceNo = i;
    if (ltend)  % if the minute starts at the end of the time mark
        tmNT(ntmNT).actual_ticks = C(:,1)+C(:,3)-0.5;
    else  % if the minute starts at the beginning of the time mark
        tmNT(ntmNT).actual_ticks = C(:,1)+0.5;
    end  % if (ltend)...
    tmNT(ntmNT).width = C(:,3);
    ntmNT = ntmNT + 1;
    %         actual_ticks=C(:,1)+C(:,3);
    
    for k=1:size(C,1)  % plot rectangles
                    rectangle('Position',C(k,:),'EdgeColor','y','Parent',H.ax1);
    end
    
    Time.trace(length(pt_traces)).fx_t=[];
    Time.trace(length(pt_traces)).tmXloc=[];
    Time.trace(length(pt_traces)).tmTime= [];
    waitbar(1,hwait);
end
delete(hwait);    
hold(H.ax1,'off')

Time.DELTA=DELTA;
Time.RefDate=date0;

% if ~isempty(tmwidthAll)
%     tmwidthAve = mean(tmwidthAll);
% else  % no time marks available
    tmwidthAve = tick_length;
% end  % if ~isempty(tmwidthAll)

% If trace is too short to be included, calculate time from closest ok trace
itodo=find(cell2mat(cellfun(@(x) isempty(x),{Time.trace(:).fx_t},'UniformOutput',false)));
hold(H.ax1,'on')
for i=itodo
    
    % create vector (i-1,i+1,i-2,i+2, etc)
    totest1=[i-1:-1:1]; n1=numel(totest1);
    totest2=[i+1:length(pt_traces)]; n2=numel(totest2);
    totest= zeros(1,n1+n2); %sort(abs (i - [1:i-1, i+1:length(pt_traces)]));
    if n1<n2
        totest(1:2:2*n1)=totest1;
        totest(2:2:2*n1)=totest2(1:n1);
        totest(2*n1+1:end)=totest2(n1+1:end);
    else
        totest(1:2:2*n2)=totest2;
        totest(2:2:2*n2)=totest1(1:n2);
        totest(2*n2+1:end)=totest1(n2+1:end);
    end
    
    
    for j=totest;  % looping over closest traces to see which one has time marks available
        if ~isempty(Time.trace(j).fx_t)
            ty=datevec(Time.trace(j).fx_t(pt_traces_Xdata{i}(:)));
            ty(:,5)=ty(:,5)+(i-j)*60*Dtrace;
%             ty = datevec(datenum(ty));  % update time
            iMM=[abs(diff(ty(:,5)))>eps; false]';  % find position of minutes
            if (ty(end,6)<0.001)
                iMM(end) = true;
            end  % if (ty(end,6)<0.001)
            
            % make sure time snaps to the minute
            ty(iMM,5) = ty(iMM,5) + round(ty(iMM,6)/60);
            ty(iMM,6) = 0;
            
            % incorporate any time marks that are available
            ipos = find([tmNT.traceNo]==i);  % get trace number position
            actual_ticks = tmNT(ipos).actual_ticks;  % time mark positions
            tmwidth = tmNT(ipos).width;  % width of each classified timemark
            tmwidth(find(tmwidth==0)) = tmwidthAve;  % set width to some finite value
            
            xpos = pt_traces_Xdata{i}(iMM)';  % minute locations
            indx_tmp = (xpos>-1000);
            if (numel(actual_ticks)>0)  % there are time marks
                for k = 1:numel(actual_ticks)  % check for actual time mark locations against available time points
                    rtmp = abs(xpos - actual_ticks(k));  % distance from actual tick end location
                    itmp = find(rtmp<tmwidth(k));  % find time points very close to the actual time mark
                    if ~isempty(itmp)  % corresponding minute position found
                        if (numel(itmp)==1)  % there is only one minute mark
                            xpos(itmp) = actual_ticks(k);  % replace with the actual time mark position
                            indx_tmp(itmp) = false;
                        end  % if (numel(itmp)==1)
                    end  % if ~isempty(itmp)
                end  % for k = 1:numel(actual_ticks)
                Time.trace(i).fx_t = fit(xpos,datenum(ty(iMM,:)),'linearinterp');  % set up trend for the time
            else  % if there are no time marks
                Time.trace(i).fx_t=fit(pt_traces_Xdata{i}(:),datenum(ty),'cubicinterp');  % time of the trace
            end  % if (numel(actual_ticks)>0)...
            
            % Time.trace(i).fx_t=fit(pt_traces_Xdata{i}(:),datenum(ty),'cubicinterp');  % time of the trace
%            Time.trace(i).fx_t = fit(xpos,datenum(ty(iMM,:)),'linearinterp');  % set up trend for the time
            Time.trace(i).tmXloc = xpos;  % locations of the "time marks"
%             Time.trace(i).tmXloc = pt_traces_Xdata{i}(iMM)';  % locations of the "time marks"
            Time.trace(i).tmTime = datenum(ty(iMM,:));  % times of the time marks
            
            % plot the new "time mark" locations
            tempX = repmat(xpos',2,1);
            tempY = repmat(pt_traces_Ydata{i}(iMM),2,1);
            tempY = tempY+repmat(ylimits,1,size(tempY,2));
            htmp = plot(H.ax1,tempX(:,indx_tmp),tempY(:,indx_tmp),'-o','Color',[1 1 0],...
                'markerfacecolor',[1 0 0]);
            htmp2 = plot(H.ax1,tempX(:,~indx_tmp),tempY(:,~indx_tmp),'-o','Color',[1 1 0]);
            htmp = [htmp; htmp2];
            rtmp2 = [find(indx_tmp); find(~indx_tmp)];
            rtmp = [i*ones(size(htmp)) rtmp2];
            indxLine = [indxLine; rtmp];
            
            hminline = [hminline; htmp];

%             tempX = pt_traces_Xdata{i}(iMM);
%             tempY = pt_traces_Ydata{i}(iMM);
%             % plot individual vertical time bars as draggable objects
%             for k = 1:numel(find(iMM))
%                 htmp = imline(H.ax1,[tempX(k); tempX(k)],[tempY(k)+ylimits]);
%                 htmp.Deletable = false;
%                 set(htmp,'tag',sprintf('%d.%d',i,k));
%                 setColor(htmp,[1 1 0]);
%                 setPositionConstraintFcn(htmp,@linePosConstraint);
%                 himline = [himline htmp];
%             end  % for k = 1:numel(iMM)
            
            break
        end
    end
    
end
hold(H.ax1,'off')

% set vertical minute marks to be draggable
set(hminline,'ButtonDownFcn',@lineButtonDownFcn);

% get handles for time symbols
hresult_time_marks=findobj(H.ax1,'color','y');
set(hresult_time_marks,'LineWidth',2)
hresult_time_marks=[hresult_time_marks;...
    findobj(H.ax1,'color','m');...
    findobj(H.ax1,'Edgecolor','y')];

% try
% set(H.togle_time_marks,'Enable','on');
% catch me
%     msgText = getReport(me,'extended');
%     warning(msgText);
% end
% update user
output_message{end+1}=['Ready'];
set(H.output_mess,'String',output_message,'Value',length(output_message)); drawnow

    function lineButtonDownFcn(hObj,eventdata)
        set(gcf,'WindowButtonMotionFcn','');  % clear out function
        set(gcf,'WindowButtonMotionFcn',{@figButtonMotion,hObj});
        set(gcf,'WindowButtonUpFcn',@ButtonUp);
    end  % funciton dragLine

    function figButtonMotion(hObj, eventdata, htmp, dx)
       curr_pt = get(gca,'CurrentPoint');
        set(htmp,'Xdata',[curr_pt(1,1) curr_pt(1,1)]);
    end  % function figButtonMotion

    function ButtonUp(hObj,eventdata)
        htmp = gco;
        indx = find(hminline==htmp);
        if numel(indx)~=0
            x = round(get(htmp,'Xdata'));
            set(htmp,'Xdata',x);
%             y = get(htmp,'Ydata');
%             disp([x y])
%             disp(['trace # and line position: ', num2str(indxLine(indx,1)),', ',num2str(indxLine(indx,2))])
            i = indxLine(indx,1);  % trace number
            k = indxLine(indx,2);  % minute mark position number
            Time.trace(i).tmXloc(k) = x(1);  % update x position for corresponding time mark
            Time.trace(i).fx_t=fit(Time.trace(i).tmXloc,Time.trace(i).tmTime,'linearinterp');
            set(gcf,'WindowButtonMotionFcn','');  % clean up functions
            set(gcf,'WindowButtonUpFcn','');
        end
    end  % function ButtonUp


%disp('xoxoxo')
end


function cost=CostDelta(x,dt,dp1,ftrend,cols,pxs,num_of_ticks)

DELTAx=dt/dp1 *x*sqrt(1+diff(1+ftrend(1:cols)).^2);
tv=repmat([0 0 0 0 0 0],numel(pxs),1);
tv(:,6)=cumsum(DELTAx(pxs))-DELTAx(pxs(1));
tv=datevec(datenum(tv));
iMM=[abs(diff(tv(:,5)))>eps]; % find round minutes
theor_ticks=pxs(iMM);
cost=abs( num_of_ticks-length(theor_ticks) -1)*2*dp1 +  abs(theor_ticks(end)-pxs(end));

end






function saveassac(SEIS)
global H
% save digitized traces in sac format
%    FILENAME   - File name of SAC data file
%    DELTA      - Data sampling interval
%    DEPMIN     - Minimum value of dependent variable
%    DEPMAX     - Maximum value of dependent variable
%    SCALE      - Multiplying factor for dependent variable
%    ODELTA     - Observed increment if different from nominal
%    B          - Beginning value of independent variable
%    E          - Ending value of independent variable
%    O          - Event origin time
%    A          - First arrival time
%    INTERNAL1  - First internal variable
%    T0         - First user-defined time picks or markers
%    T1         - Second user-defined time picks or markers
%    T2         - Third user-defined time picks or markers
%    T3         - Fourth user-defined time picks or markers
%    T4         - Fifth user-defined time picks or markers
%    T5         - Sixth user-defined time picks or markers
%    T6         - Seventh user-defined time picks or markers
%    T7         - Eighth user-defined time picks or markers
%    T8         - Ninth user-defined time picks or markers
%    T9         - Tenth user-defined time picks or markers
%    F          - Final or end of event time
%    RESP0      - First instrument response parameter
%    RESP1      - Second instrument response parameter
%    RESP2      - Third instrument response parameter
%    RESP3      - Fourth instrument response parameter
%    RESP4      - Fifth instrument response parameter
%    RESP5      - Sixth instrument response parameter
%    RESP6      - Seventh instrument response parameter
%    RESP7      - Eighth instrument response parameter
%    RESP8      - Ninth instrument response parameter
%    RESP9      - Tenth instrument response parameter
%    STLA       - Station latitude
%    STLO       - Station longitude
%    STEL       - Station elevation
%    STDP       - Station depth
%    EVLA       - Event latitude
%    EVLO       - Event longitude
%    EVEL       - Event elevation
%    EVDP       - Event depth
%    MAG        - Event magnitude
%    USER0      - First user-defined variable
%    USER1      - Second user-defined variable
%    USER2      - Third user-defined variable
%    USER3      - Fourth user-defined variable
%    USER4      - Fifth user-defined variable
%    USER5      - Sixth user-defined variable
%    USER6      - Seventh user-defined variable
%    USER7      - Eighth user-defined variable
%    USER8      - Ninth user-defined variable
%    USER9      - Tenth user-defined variable
%    DIST       - Station-to-event distance (km)
%    AZ         - Event-to-station azimuth (degree)
%    BAZ        - Station-to-event azimuth (degree)
%    GCARC      - Station-to-event great-circle arc length (degree)
%    INTERNAL2  - Second internal variable
%    INTERNAL3  - Third internal variable
%    DEPMEN     - Mean value of dependent variable
%    CMPAZ      - Component azimuth
%    CMPINC     - Component incident angle
%    XMINIMUM   - Minimum value of X (spectral file only)
%    XMAXIMUM   - Maximum value of X (spectral file only)
%    YMINIMUM   - Minimum value of Y (spectral file only)
%    YMAXIMUM   - Maximum value of Y (spectral file only)
%    UNUSED1    - First unused variable
%    UNUSED2    - Second unused variable
%    UNUSED3    - Third unused variable
%    UNUSED4    - Fourth unused variable
%    UNUSED5    - Fifth unused variable
%    UNUSED6    - Sixth unused variable
%    UNUSED7    - Seventh unused variable
%    NZYEAR     - GMT year corresponding to reference time
%    NZJDAY     - GMT julian day corresponding to reference time
%    NZHOUR     - GMT hour corresponding to reference time
%    NZMIN      - GMT minute corresponding to reference time
%    NZSEC      - GMT second corresponding to reference time
%    NZMSEC     - GMT milisecond corresponding to reference time
%    NVHDR      - Header version
%    NORID      - Origin ID
%    NEVID      - Event ID
%    NPTS       - Number of data points
%    INTERNAL4  - Fourth internal variable
%    NWFID      - Waveform ID
%    NXSIZE     - Spectral length (spectral file only)
%    NYSIZE     - Spectral width (spectral file only)
%    UNUSED8    - Eighth unused variable
%    IFTYPE     - Type of file
%    IDEP       - Type of dependent variable
%    IZTYPE     - Reference-time equivalence
%    UNUSED9    - Ninth unused variable
%    IINST      - Type of recording instrument
%    ISTREG     - Station geographic region
%    IEVREG     - Event geographic region
%    IEVTYP     - Type of event
%    IQUAL      - Quality of data
%    ISYNTH     - Synthetic data flag
%    IMAGTYP    - Magnitude type
%    IMAGSRC    - Magnitude source
%    UNUSED10   - Tenth unused variable
%    UNUSED11   - Eleventh unused variable
%    UNUSED12   - Twelveth unused variable
%    UNUSED13   - Thirteenth unused variable
%    UNUSED14   - Fourteenth unused variable
%    UNUSED15   - Fifteenth unused variable
%    UNUSED16   - Sixteenth unused variable
%    UNUSED17   - Seventeenth unused variable
%    LEVEN      - True if data is evenly spaced
%    LPSPOL     - True if station polarity follows left-hand rule
%    LOVROK     - True if it is ok to overwrite this file on disk
%    LCALDA     - True if DIST, AZ, BAZ and GCARC are to be calculated from
%                 station and event coordinates
%    UNUSED18   - Eighteenth unused variable
%    KSTNM      - Station name
%    KEVNM      - Event name
%    KHOLE      - Hole ID for nuclear test
%    KO         - Event origin time ID
%    KA         - First arrival time ID
%    KT0        - First user-defined time pick ID
%    KT1        - Second user-defined time pick ID
%    KT2        - Third user-defined time pick ID
%    KT3        - Fourth user-defined time pick ID
%    KT4        - Fifth user-defined time pick ID
%    KT5        - Sixth user-defined time pick ID
%    KT6        - Seventh user-defined time pick ID
%    KT7        - Eighth user-defined time pick ID
%    KT8        - Ninth user-defined time pick ID
%    KT9        - Tenth user-defined time pick ID
%    KF         - Final or end event time ID
%    KUSER0     - First user-defined text string
%    KUSER1     - Second user-defined text string
%    KUSER2     - Third user-defined text string
%    KCMPNM     - Component name
%    KNETWK     - Network name
%    KDATRD     - Date data were read onto computer
%    KINST      - Generic name of recording instrument
%    DATA1      - First data block

field_names={'FILENAME','DELTA','DEPMIN','DEPMAX','SCALE','ODELTA','B','E','O','A','INTERNAL1',...
    'T0','T1','T2','T3','T4','T5','T6','T7','T8','T9','F','RESP0','RESP1','RESP2','RESP3',...
    'RESP4','RESP5','RESP6','RESP7','RESP8','RESP9','STLA','STLO','STEL','STDP','EVLA',...
    'EVLO','EVEL','EVDP','MAG','USER0','USER1','USER2','USER3','USER4','USER5','USER6',...
    'USER7','USER8','USER9','DIST','AZ','BAZ','GCARC','INTERNAL2','INTERNAL3','DEPMEN',...
    'CMPAZ','CMPINC','XMINIMUM','XMAXIMUM','YMINIMUM','YMAXIMUM','UNUSED1','UNUSED2','UNUSED3',...
    'UNUSED4','UNUSED5','UNUSED6','UNUSED7','NZYEAR','NZJDAY','NZHOUR','NZMIN','NZSEC','NZMSEC',...
    'NVHDR','NORID','NEVID','NPTS','INTERNAL4','NWFID','NXSIZE','NYSIZE','UNUSED8','IFTYPE','IDEP',...
    'IZTYPE','UNUSED9','IINST','ISTREG','IEVREG','IEVTYP','IQUAL','ISYNTH','IMAGTYP','IMAGSRC',...
    'UNUSED10','UNUSED11','UNUSED12','UNUSED13','UNUSED14','UNUSED15','UNUSED16','UNUSED17',...
    'LEVEN','LPSPOL','LOVROK','LCALDA','UNUSED18','KSTNM','KEVNM','KHOLE','KO','KA','KT0','KT1',...
    'KT2','KT3','KT4','KT5','KT6','KT7','KT8','KT9','KF','KUSER0','KUSER1','KUSER2','KCMPNM',...
    'KNETWK','KDATRD','KINST'};
% DATA1

% Initialize
N = length(SEIS);
clear tS;
tS(N)=struct();
nfields=length(field_names);
for it=1:86
    [tS.(field_names{it})]=deal(nan);
end
% 87 up to 89
[tS.IFTYPE]=deal('ITIME');
[tS.IDEP]=deal('IUNKN'); % This should be changed in future version to reflect the units.
[tS.IZTYPE]=deal(''); % This should be changed in future version to reflect the units.
for it=90:93
    [tS.(field_names{it})]=deal(nan);
end
for it=94:98    % IEVTYP up to IMAGSRC = ''
    [tS.(field_names{it})]=deal('');
end
for it=99:111
    [tS.(field_names{it})]=deal(nan);
end
for it=112:nfields
    [tS.(field_names{it})]=deal('');
end


[tS.LEVEN]=deal(1); % Even sampling distance

[tS.FILENAME]=deal(SEIS.FILENAME);
[tS.DELTA]=deal(SEIS.DELTA);
[tS.B]=deal(SEIS.B);
[tS.E]=deal(SEIS.E);
[tS.NPTS]=deal(SEIS.NPTS);
[tS.NZYEAR]=deal(SEIS.NZYEAR);
[tS.NZJDAY]=deal( SEIS.NZJDAY);
[tS.NZHOUR]= deal(SEIS.NZHOUR);
[tS.NZMIN]= deal(SEIS.NZMIN);
[tS.NZSEC]= deal(SEIS.NZSEC);
[tS.NZMSEC]= deal(SEIS.NZMSEC);

% HRV defaults
[tS.KSTNM]=deal('HRV');
[tS.STLA]=deal(42.506);
[tS.STLO]=deal(-71.558);
[tS.STEL]=deal(200);


Fig_table = figure('Units','Normalized','Position',[0.2 0.1 0.4 0.7],...
    'color','w','WindowStyle','Normal','NumberTitle','Off','Name','Save in SAC format');
columnname =   num2str([1:length(SEIS)]');
columneditable =  true(size(field_names));
columneditable(end)=false;
htabl = uitable('Units','normalized','Position',...
    [0.01 0.15 0.98 0.85], 'Data',struct2cell( tS(:) ),...
    'ColumnName', columnname,...
    'ColumnEditable', columneditable,...
    'RowName',field_names,'Parent',Fig_table);



uicontrol('Style', 'Text','Parent',Fig_table,'units','normalized',...
    'String', 'Network','Tooltip','Enter network name (KNETWK field)','BackgroundColor',[1 1 1],...
    'Position', [0.01 0.1 0.095 0.03]);
uicontrol('Style', 'Text','Parent',Fig_table,'units','normalized',...
    'String', 'Station','Tooltip','Enter station name (KSTNM field)','BackgroundColor',[1 1 1],...
    'Position', [0.11 0.1 0.095 0.03]);
uicontrol('Style', 'Text','Parent',Fig_table,'units','normalized',...
    'String', 'Component','Tooltip','Enter component name (KCMPNM field)','BackgroundColor',[1 1 1],...
    'Position', [0.21 0.1 0.095 0.03]);
uicontrol('Style', 'Text','Parent',Fig_table,'units','normalized',...
    'String', 'Hole ID','Tooltip','Enter Hole ID (KHOLE field)','BackgroundColor',[1 1 1],...
    'Position', [0.31 0.1 0.095 0.03]);
uicontrol('Style', 'Text','Parent',Fig_table,'units','normalized',...
    'String', 'Instrument','Tooltip','Enter generic name of recording instrument (KINST field)','BackgroundColor',[1 1 1],...
    'Position', [0.41 0.1 0.095 0.03]);
uicontrol('Style', 'Text','Parent',Fig_table,'units','normalized',...
    'String', 'latitude','Tooltip','Station latitude (STLA field)','BackgroundColor',[1 1 1],...
    'Position', [0.51 0.1 0.095 0.03]);
uicontrol('Style', 'Text','Parent',Fig_table,'units','normalized',...
    'String', 'longitude','Tooltip','Station longitude  (STLO field)','BackgroundColor',[1 1 1],...
    'Position', [0.61 0.1 0.095 0.03]);
uicontrol('Style', 'Text','Parent',Fig_table,'units','normalized',...
    'String', 'Elevation','Tooltip','Station elevation (STEL field)','BackgroundColor',[1 1 1],...
    'Position', [0.71 0.1 0.095 0.03]);
uicontrol('Style', 'Text','Parent',Fig_table,'units','normalized',...
    'String', 'Depth','Tooltip','Station depth','BackgroundColor',[1 1 1],...
    'Position', [0.81 0.1 0.095 0.03]);

ht_eKNETWK=uicontrol('Style', 'Edit','Parent',Fig_table,'units','normalized','tag','KNETWK',...
    'String', '','Tooltip','Enter network name (KNETWK field)','BackgroundColor',[1 1 1],...
    'Position', [0.01 0.07 0.095 0.03],'Callback',@update_table);
ht_eKSTNM=uicontrol('Style', 'Edit','Parent',Fig_table,'units','normalized','tag','KSTNM',...
    'String', 'HRV','Tooltip','Enter station name (KSTNM field)','BackgroundColor',[1 1 1],...
    'Position', [0.11 0.07 0.095 0.03],'Callback',@update_table);
ht_eKCMPNM=uicontrol('Style', 'Edit','Parent',Fig_table,'units','normalized','tag','KCMPNM',...
    'String', '','Tooltip','Enter component name (KCMPNM field)','BackgroundColor',[1 1 1],...
    'Position', [0.21 0.07 0.095 0.03],'Callback',@update_table);
ht_eKHOLE=uicontrol('Style', 'Edit','Parent',Fig_table,'units','normalized','tag','KHOLE',...
    'String', '','Tooltip','Enter Hole ID (KHOLE field)','BackgroundColor',[1 1 1],...
    'Position', [0.31 0.07 0.095 0.03],'Callback',@update_table);
ht_eKINST=uicontrol('Style', 'Edit','Parent',Fig_table,'units','normalized','tag','KINST',...
    'String', '','Tooltip','Enter generic name of recording instrument (KINST field)',...
    'BackgroundColor',[1 1 1],'Position', [0.41 0.07 0.095 0.03],'Callback',@update_table);
ht_eSTLA=uicontrol('Style', 'Edit','Parent',Fig_table,'units','normalized','tag','STLA',...
    'String', '42.506','Tooltip','Station latitude (STLA field)',...
    'BackgroundColor',[1 1 1],'Position', [0.51 0.07 0.095 0.03],'Callback',@update_table);
ht_eSTLO=uicontrol('Style', 'Edit','Parent',Fig_table,'units','normalized','tag','STLO',...
    'String', '-71.558','Tooltip','Station longitude  (STLO field)',...
    'BackgroundColor',[1 1 1],'Position', [0.61 0.07 0.095 0.03],'Callback',@update_table);
ht_eSTEL=uicontrol('Style', 'Edit','Parent',Fig_table,'units','normalized','tag','STEL',...
    'String', '200','Tooltip','Station elevation (STEL field)',...
    'BackgroundColor',[1 1 1],'Position', [0.71 0.07 0.095 0.03],'Callback',@update_table);
ht_eSTDP=uicontrol('Style', 'Edit','Parent',Fig_table,'units','normalized','tag','STDP',...
    'String', '','Tooltip','Station depth',...
    'BackgroundColor',[1 1 1],'Position', [0.81 0.07 0.095 0.03],'Callback',@update_table);


ht_eCMPAZ=uicontrol('Style', 'Edit','Parent',Fig_table,'units','normalized','tag','CMPAZ',...
    'String', '','Tooltip','Enter component azimuth (CMPAZ field)','BackgroundColor',[1 1 1],...
    'Position', [0.21 0.036 0.095 0.025],'Callback',@update_table);
ht_eCMPINC=uicontrol('Style', 'Edit','Parent',Fig_table,'units','normalized','tag','CMPINC',...
    'String', '','Tooltip','Enter component incidence angle (CMPINC field)','BackgroundColor',[1 1 1],...
    'Position', [0.21 0.01 0.095 0.025],'Callback',@update_table);



hp_saveassac=uicontrol('Style', 'pushbutton','Parent',Fig_table,'units','normalized',...
    'String', 'Generate Sac files','Position', [0.82 0.01 0.17 0.04],'Callback',@saveassac);




    function update_table(hObject, eventdata)
        data = get(htabl,'Data');
        cstr = get(hObject,'tag');
        if (strcmpi(cstr,'STLA') || strcmpi(cstr,'STLO') || ...
                strcmpi(cstr,'STEL') || strcmpi(cstr,'STDP') ) % numerical entries
            if ~isempty(get(hObject,'String'));  % new values input
                data(~cellfun('isempty',strfind(field_names, get(hObject,'tag'))),:) = {str2double(get(hObject,'String'))};
            else  % values removed
                data(~cellfun('isempty',strfind(field_names, get(hObject,'tag'))),:) = {NaN};
            end  % if ~isempty(get(hObject,'String'));
        else
            if ~isempty(get(hObject,'String'));  % new values input
                data(~cellfun('isempty',strfind(field_names, get(hObject,'tag'))),:) = {get(hObject,'String')};
            else  % values removed
                data(~cellfun('isempty',strfind(field_names, get(hObject,'tag'))),:) = {''};
            end  % if ~isempty(get(hObject,'String'));
        end  % if cstr = STLA, STLO, STEL, or STDP
        set(htabl,'Data',data);
    end


    function saveassac(hObject, eventdata)
        
        folder_name = uigetdir(get(H.textfilename,'Userdata'),'Select folder to generate SAC files');
        if (folder_name==0), return, end
        
        hbar=waitbar(0,'please wait...','Name','Generating SAC files');
        
        clear SAC;
        data = get(htabl,'Data');
        [nrow ncol] = size(data);
        for k = 1:ncol  % loop over SAC files to update the filename
            data{1,k} = [folder_name,'/',data{1,k}];  % give it the full path
        end  % for k = 1:nrow
        SAC = cell2struct(data, field_names, 1);
        nS=length(SAC);
        total_wait=nS+30;
        for i=1:nS
            waitbar(i/total_wait,hbar,'Creating SAC structure, please wait...')
            SAC(i).DATA1=SEIS(i).DATA1;
            if isfield(SEIS,'DATA2')
                SAC(i).DATA2=SEIS(i).DATA2;
            end
            
        end
        
        
%         oldFolder = cd(folder_name);
        waitbar(0.8,hbar,'Writing SAC files, please wait...')
        writesac(SAC);
        waitbar(0.99,hbar,'Completed.')
%         cd(oldFolder)
        delete(hbar);
        close(Fig_table);
    end


end

function dataCursorFnc(hobj,eventdata)
% modify the operation of the data cursor (mainly for display purpose)

% global H

hf = gcbf;
% htext = H.CursorPosText;

% remove any hggroup objects that exist in the figure (there shouldn't be
% any at this point remaining
htmp = findall(hf,'type','hggroup');
delete(htmp);

if strcmpi(get(hobj,'state'),'on')  % object turned on
    dcm_obj = datacursormode(hf);
    set(dcm_obj,'DisplayStyle','datatip','Enable','on');
    set(dcm_obj,'UpdateFcn',{@UpdateDataCursor,hf});
else  % object turned off
    datacursormode off;
%     set(htext,'visible','off');
end

end

function output_txt = UpdateDataCursor(~,event_obj,hf)
% global H
pos = get(event_obj,'Position');  % data cursor position
% set(H.CursorPosText,'String',['(X,Y) = (', num2str(pos(1),'%6.0f'), ', ',num2str(pos(2),'%6.0f'), ')'],...
%     'visible','on');
htmp = findall(hf,'type','hggroup');
set(htmp,'FontSize',20);
output_txt = ['(X,Y) = (', num2str(pos(1),'%6.0f'), ', ',num2str(pos(2),'%6.0f'), ')'];
end


function y = median_nonan( x )
%  calculates the median in a vector. Leaving out the nans
% Petros Bogiatzis 2014
y=median(x(~isnan(x)));

end





function find_gaps(hObj,eventdata)
% this function examines the traced data and puts boxes where there are
% gaps in traces for checking purposes.

global H ptrace pt_traces pt_tracesY0

nsum = 3;  % number of pixels to be summed for "vertical" line identification

if get(hObj,'value')==1  % checkbox turned on
    
    %hObj = H.find_gaps;
    if get(H.togle_timebounds_visibility,'value')  % if timebounds are on
        % turn the timebounds objects off
        set(H.togle_timebounds_visibility,'value',0);
        togle_timebounds_visibility;
    end  % if get(H.togle_timebounds_visibility,'value')
    
    % check to see if this function was called by find_gaps button or find_vert
    % button
    itype = -1;  % default value
    if strcmpi(get(hObj,'string'),'Check for gaps')  % find_gaps call
        % if gap boxes exist, delete them first
        if isappdata(H.ax1,'hgap')
            hgap = getappdata(H.ax1,'hgap');
            delete(hgap);
            try; rmappdata(H.ax1,'hgap'); end
        end  % if isappdata(H.ax1,'hgap')
        
        % turn the other set off
        set(H.find_vert,'value',0);        
        % remove all rectangles
        if isappdata(H.ax1,'hvert')  % rectangle objects for vertical lines exist
            hvert = getappdata(H.ax1,'hvert');
            delete(hvert);
            
            try; rmappdata(H.ax1,'hvert'); end
        end  % if isappdata(H.ax1,'hvert')
        drawnow
        
        itype = 0;  % find gaps
        cappdata = 'hgap';
        
    elseif strcmpi(get(hObj,'string'),'Check for vertical') % find_vert call
        % if vertical boxes exist, delete them first
        if isappdata(H.ax1,'hvert')
            hvert = getappdata(H.ax1,'hvert');
            delete(hvert);
            try; rmappdata(H.ax1,'hvert'); end
        end  % if isappdata(H.ax1,'hvert')
        
        % turn the other set off
        set(H.find_gaps,'value',0);
        if isappdata(H.ax1,'hgap')  % rectangle objects exist
            hgap = getappdata(H.ax1,'hgap');
            delete(hgap);
            
            try; rmappdata(H.ax1,'hgap'); end
        end  % if isappdata(H.ax1,'hgap')
        drawnow
        
        itype = 1;  % find vertical lines
        cappdata = 'hvert';
        
        clear yshift
        if isappdata(H.ax1,'tmshift')  % if time mark shift information is available
            tmshift = getappdata(H.ax1,'tmshift');
            if numel(tmshift(~isnan(tmshift)))>0
                yshift = mean(tmshift(~isnan(tmshift)));
                ry = 0.5;
            end
        elseif isappdata(H.find_vert,'yshift')  % if pre-computed value of yshift available
            yshift = getappdata(H.find_vert,'yshift');
            ry = getappdata(H.find_vert,'ry');
        else  % calculate value
            [yshift ry] = find_yshift;
        end  % if isappdata(H.ax1,'tmshift')
        
        ydif_min = yshift*(1-ry);
        ydif_max = yshift*(1+ry);
        % disp(['ydif_min/max = ', num2str(ydif_min),', ', num2str(ydif_max)]);
    else  % shouldn't get here
        disp('in find_gaps routine');
        disp('shouldn''t get here...');
        get(hObj)  % display object properties
        return
    end  % if strcmpi(get(hObj,'string'),'Check for gaps')...
    
%     if itype==0  % checking for gaps
        % determine the height of box for all the gaps based upon trace spacing
        rtmp=cell2mat(cellfun(@(v) v(1),get(pt_traces,'Ydata'),'UniformOutput',false));
        [ncount yedges] = histcounts(diff(rtmp));
        [rtmp indx] = max(ncount);
        dy = mean(yedges(indx) + yedges(indx+1))*2/5.0; % height of the boxes
        
        % for each trace, find gap and place rectangles
        hgap = [];  % collection of handles of rectangles
        for k = 1:numel(ptrace)  % loop over traces
            x = get(ptrace(k),'XDATA');
            y = get(ptrace(k),'YDATA');
            
            if itype==0  % finding gaps
                indx = find(~isnan(y));  % find numerical y values
            elseif itype==1  % finding verticals
                rtmp = diff(y);  % differentiating y values
                ydif = rtmp;
                for m = 2:nsum  % sum over pixels
                    ydif = ydif + [rtmp(m:end) nan*ones(1,(m-1))];
                end  % for m = 1:nsum
                ydif = abs(ydif);
                indx = find( isnan(ydif) | (ydif<ydif_min) | (ydif>ydif_max));  % all non-vertical parts of the trace
            end  % if itype==0
            
            x = x(indx);  % x values without corresponding y value that is NaN
            indxGap = find(diff(x)>1);  % locations where gaps exist
            
            if ~isempty(indxGap)
                % find position of y for each box
                x0 = get(pt_traces(k),'Xdata');
                [rtmp iref itmp] = intersect(x0,x(indxGap));
                y0 = get(pt_traces(k),'YData');
                y0 = y0(iref);
%                 if ~iscell(pt_tracesY0)|| isempty(pt_tracesY0) % reference trace Y values do not exist
%                     x0 = get(pt_traces(k),'XData');
%                     [rtmp iref itmp] = intersect(x0,x(indxGap));
%                     y0 = get(pt_traces(k),'YDATA');
%                     y0 = y0(iref); % make sure it's a column vector
%                 else  % pt_tracesY0{k} exists and is not empty
%                     if ~isempty([pt_tracesY0{k}])
%                         y0 = pt_tracesY0{k};
%                         y0 = y0(indx(indxGap));
%                     else  % pt_tracesY0 values not available
%                         x0 = get(pt_traces(k),'XData');
%                         [rtmp iref itmp] = intersect(x0,x(indxGap));
%                         y0 = get(pt_traces(k),'YDATA');
%                         y0 = y0(iref); % make sure it's a column vector
%                     end  % if ~isempty([pt_tracesY0{k}])...
%                 end  % if ~exist(pt_tracesY0)|| isempty([pt_tracesY{k}])...
                
                gbox = [x(indxGap)-0.5; y0-dy/2; ...
                    x(indxGap+1)-x(indxGap)+1; ones(1,numel(indxGap))*dy];
                
                for m = 1:numel(indxGap)  % loop over boxes for plotting
                    htmp = rectangle('Position',gbox(:,m),'EdgeColor','y',...
                        'linewidth',2.0,'Parent',H.ax1);
                    hgap = [hgap; htmp];
                end  % for m = 1:numel(indxGap)
                
            end  % if ~isempty(indxGap)
        end  % for k = 1:numel(ptrace)
        
        % store all the handles of the rectangles in the axes object handle
        if ~isempty(hgap)
            setappdata(H.ax1,cappdata,hgap);
        end  % if ~isempty(hgap)
        
%     elseif itype==1  % checking for vertical lines
%         
%         % for each trace, find steep slopes and place rectangles
%         hvert = [];  % collection of handles of rectangles
%         for k = 1:numel(ptrace)  % loop over traces
%             x = get(ptrace(k),'XDATA');
%             y = get(ptrace(k),'YDATA');
%             
%             ydif = abs(diff(y));  % differentiating y values
%             indx = find( isnan(ydif) | (ydif<ydif_min) );  % all non-vertical parts of the trace
%             
%             x = x(indx);  % x values without corresponding steep y slope
%             indxGap = find(diff(x)>1);  % locations where "vertical lines" exist
%             
%             if ~isempty(indxGap)
%                 % find position of y for each box
%                 y = get(pt_traces(k),'YDATA');
%                 y0 = y(indx(indxGap)); % make sure it's a column vector
%                 
%                 gbox = [x(indxGap)-0.5; y0-dy/2; ...
%                     x(indxGap+1)-x(indxGap)+1; ones(1,numel(indxGap))*dy];
%                 
%                 for m = 1:numel(indxGap)  % loop over boxes for plotting
%                     htmp = rectangle('Position',gbox(:,m),'EdgeColor','y',...
%                         'linewidth',2.0,'Parent',H.ax1);
%                     hgap = [hgap; htmp];
%                 end  % for m = 1:numel(indxGap)
%                 
%             end  % if ~isempty(indxGap)
%         end  % for k = 1:numel(ptrace)
%         
%         % store all the handles of the rectangles in the axes object handle
%         if ~isempty(hvert)
%             setappdata(H.ax1,'hvert',hvert);
%         end  % if ~isempty(hvert)
%         
%     end  % if itype==0
    
else  % checkbox turned off
    % remove all rectangles
    if isappdata(H.ax1,'hgap')  % rectangle objects exist
        hgap = getappdata(H.ax1,'hgap');
        delete(hgap);
        
        try; rmappdata(H.ax1,'hgap'); end
    end  % if isappdata(H.ax1,'hgap')
    
    if isappdata(H.ax1,'hvert')  % rectangle objects for vertical lines exist
        hvert = getappdata(H.ax1,'hvert');
        delete(hvert);
        
        try; rmappdata(H.ax1,'hvert'); end
    end  % if isappdata(H.ax1,'hvert')
    
    % check to make sure no rectangles exist in the axes object
    htmp = get(H.ax1,'children');
    h = findobj(htmp,'type','rectangle');
    if ~isempty(h)  % rectangles exist -> remove
        delete(h);
    end  % if ~isempty(h)
end  % if get(hObj,'value')==1

    function [yshift ry] = find_yshift
        %  this function finds average shift value between the main trace
        %  and time marks
        
        global S
        
        % preset value
        nwidth = 10;
        yshift = 100;
        ry = 0.5;
        
        dy = [];
        for k = 1:numel(ptrace)  % loop over traces
            xtrace = get(ptrace(k),'XData');
            ytrace = get(ptrace(k),'YDATA');
            
            indx = (([S.ID]==0 | [S.ID]==2) & [S.TraceNum]==k);   % traces
            indxTM = (([S.ID]==1 | [S.ID]==3) & [S.TraceNum]==k);  % time marks
            
            if ~isempty(find(indx)) & ~isempty(find(indxTM))
                box0 = cell2mat({S(indx).BoundingBox}');
                xlim0 = [box0(:,1) box0(:,1)+box0(:,3)];  % starting and ending values of x
                boxTM = cell2mat({S(indxTM).BoundingBox}');
                xlimTM = [boxTM(:,1) boxTM(:,1)+boxTM(:,3)];  % limits of x
                
                for m = 1:numel(find(indxTM))  % loop over timemark boxes
                    ytm = boxTM(m,2)+boxTM(m,4)/2;  % central y position of the time mark box
                    
                    % compare beginning of the time mark
                    rtmp = abs(xlim0(:,2)-xlimTM(m,1));
                    indx = find(rtmp<=nwidth);
                    if ~isempty(indx)  % xval exists
                        xval = round(xlim0(indx,2)-0.5);
                        indx = find(xtrace==xval);  % find x position
                        if ~isempty(indx)
                            dy(end+1) = abs(ytm-ytrace(indx));
                        end  % if ~isempty(indx)
                    end  % if ~isempty(xval)
                    
                    % compare end of the time mark
                    rtmp = abs(xlim0(:,1)-xlimTM(m,2));
                    indx = find(rtmp<=nwidth);
                    if ~isempty(indx)  % xval exists
                        xval = round(xlim0(indx,1)+0.5);
                        indx = find(xtrace==xval);  % find x position
                        if ~isempty(indx)
                            dy(end+1) = abs(ytm-ytrace(indx));
                        end  % if ~isempty(indx)
                    end  % if ~isempty(xval)
                    
                end  % for m = 1:numel(find(indxTM))
                
            end  % if ~isempty(indx) & ~isempty(indxTM)
        end  % for k = 1:numel(ptrace)
        
        if ~isempty(dy)  % time mark and corresponding traces found
            yshift = mean(dy);
            if numel(dy)>3  % calculate standard deviation
                rtmp = std(dy)/yshift;
                ry = max(rtmp,ry);
            end  % if numel(dy)>3
        end  % if ~isempty(dy)...
        
        setappdata(H.find_vert,'yshift',yshift);
        setappdata(H.find_vert,'ry',ry);
    end  % function find_yshift

end   % function find_gaps

function check_timemarkpos(hObj,eventdata)
% if this button is pushed after digitization has happened, it resets the
% trace number assignment of all time marks

global H ptrace S 

if strcmpi(get(hObj,'state'),'on')  % time mark below
    icon_m=zeros([16,16,3]); icon_m(6:7,[1:6, 11:16],:)=1; icon_m(11:12,7:10,2)=1;
else  % time mark above
    icon_m=zeros([16,16,3]); icon_m(11:12,[1:6, 11:16],:)=1; icon_m(6:7,7:10,2)=1;
end  % if strcmpi(get(hObj,'state'),'on')...
set(hObj,'Cdata',icon_m);
clear icon_m

if any(ishandle(ptrace))
    indx = find([S.ID]==1);  % ignore ones where time marks are generated manually, i.e., ID=3
    [S(indx).TraceNum] = deal(-1);
end  % if any(ishandle(ptrace))

if isappdata(H.ax1,'tmshift');  % if time mark positions from previous round of calculations exist
    rmappdata(H.ax1,'tmshift');  % remove since the side of the time mark has changed
end  % if isappdata(H.ax1,'tmshift')

end  % function check_timemarkpos





function imageMod_options(hObject, eventdata)
global H

% default division values
himMod.nx = 25;
himMod.ny = 10; 

% positions of the properties window
r_wh = 396/564;  % width to height ratio
p = get(0,'ScreenSize');  % get reference screen size
% set up max values for box setup (in pixels)
rval_max = [floor(p(3)/2) floor(p(4)/2) p(3) p(4)];
% height = p(4)/2;  % height of the box
% width = r_wh*height;
height = 564;
width = 396;
x = p(3)-width;
y = p(4)-height;
pos.hf = [x y width height];

sizeImage = 25;  % pixel number of icons to be shown
wid = 0.55;  % width of pushbuttons
hgt = 0.2;  % height of pushbuttons
% calculate number of pixels corresponding to x1
xiconW = sizeImage/pos.hf(3);  % icon width in normailzed values
yiconH = sizeImage/pos.hf(4);  % icon height
x1 = (1.0-xiconW-0.01-wid)/2+xiconW+0.01;  % pushbutton starting position
% positions
pos.axPol = [x1-xiconW-0.01 0.85+hgt/2-yiconH/2 xiconW yiconH];
pos.pbPol = [x1 0.85+hgt/2-yiconH/2  0.55 yiconH];
pos.axRot = [x1-xiconW-0.01 0.85+hgt/2-2*yiconH xiconW yiconH];
pos.pbRot = [x1 0.85+hgt/2-2*yiconH  0.55 yiconH];
pos.axBG = [x1-xiconW-0.01 0.85+hgt/2-3.5*yiconH xiconW yiconH];
pos.pbBG = [x1 0.85+hgt/2-3.5*yiconH  0.55 yiconH];
pos.axRec = [x1-xiconW-0.01 0.85+hgt/2-5.0*yiconH xiconW yiconH];
pos.pbRec = [x1 0.85+hgt/2-5.0*yiconH  0.55 yiconH];
% panel for piecewise contrast algorithm
pos.panel = [0.05 pos.pbRec(2)-0.2 0.9 0.15];
pos.panelx = [0.05 0.55 0.15 0.35];  % # of x division
pos.panelxT = [0.23 0.52 0.62 0.35];  % text
pos.panely = [0.05 0.15 0.15 0.35];  % # of y division
pos.panelyT = [0.23 0.12 0.62 0.35];  % text
pos.panelApply = [0.4 0.05 0.2 0.25];  % Apply button



% positions of the execution buttons
pos.Apply = [0.32 0.1 0.15 0.05];
pos.Close = [0.425 0.05 0.15 0.05];

lnewFig = true;  % will become false if new figure window is not going to be created

% check to see if the option window already exists
if isappdata(H.f1,'himMod');
    himMod = getappdata(H.f1,'himMod');  % get old value of cto
    
    if ishandle(himMod.hf)  % if figure still exists
        hchil = get(0,'children');
        set(himMod.hf,'visible','on');  % make sure this window is visibled
        set(0,'children',[himMod.hf; hchil(hchil~=himMod.hf)]);  % bring relevant figure to front
        lnewFig = false;
    end  % if any(ishandle(cto.figure))    
end  % if isappdata(H.f1,'hcorrect_trace_options');

if lnewFig  % if new figure is going to be created
    himMod.hf = figure('Name','Additional Image Modification Options','NumberTitle','off',...
        'Units','pixels','menu','none','toolbar','none',...
        'OuterPosition',pos.hf);
    
    hpolAx = axes('Units','Normalized','Position',pos.axPol,...
        'Xtick',[],'Ytick',[],'box','off','Parent',himMod.hf,'CLimMode','manual','Clim',[0 255]);
    polarity_ic=zeros([16,16,3]); for i=1:16, polarity_ic(1:i,i:16,:)=1;end
    % Ianalysis = imread('icons/file.png');
    image(polarity_ic,'parent',hpolAx);
    set(hpolAx,'visible','off');
    clear polarity_ic;
    hpolarity = uicontrol('style','pushbutton','parent',himMod.hf,'units','normalized',...
        'position',pos.pbPol,'string','Change Polarity','tooltipstring','Change image polarity',...
        'fontsize',14,'callback',@change_I0_polarity);

    hrotAx = axes('Units','Normalized','Position',pos.axRot,...
        'Xtick',[],'Ytick',[],'box','off','Parent',himMod.hf,'CLimMode','manual','Clim',[0 255]);
    image(imread('icons/correct_rotation.png','BackgroundColor',[1 1 1]),'parent',hrotAx);
    set(hrotAx,'visible','off');
    hrot = uicontrol('style','pushbutton','parent',himMod.hf,'units','normalized',...
        'position',pos.pbRot,'string','Rotation','tooltipstring','Correct image rotation',...
        'fontsize',14,'callback',@estimate_rotation);

    hBGAx = axes('Units','Normalized','Position',pos.axBG,...
        'Xtick',[],'Ytick',[],'box','off','Parent',himMod.hf,'CLimMode','manual','Clim',[0 255]);
    image(imread('icons/Removebackground.png','BackgroundColor',[1 1 1]),'parent',hBGAx);
    set(hBGAx,'visible','off');
    hBG = uicontrol('style','pushbutton','parent',himMod.hf,'units','normalized',...
        'position',pos.pbBG,'string','Remove Background','tooltipstring','Remove large-scale stain-like features',...
        'fontsize',14,'callback',@remove_background_withGAUSSFILT);
    
    hpanel = uipanel('Title','Piecewise Contrast Modification',...
        'units','normalized','Position',pos.panel,'Parent',himMod.hf,'fontsize',14);
    hpanelx=uicontrol('Style', 'edit','Parent',hpanel,'units','normalized',...
        'TooltipString','number of image division in the x direction',...
        'String',num2str(himMod.nx),'Position', pos.panelx,'BackgroundColor',[1 1 1],...
        'fontsize',14,'callback',@checkValueInt,'tag','x');
    hpanelxT=uicontrol('Style', 'text','Parent',hpanel,'units','normalized',...
        'String','Number of image divisions in x','Position', pos.panelxT,...
        'fontsize',14,'horizontalAlignment','left');
    hpanely=uicontrol('Style', 'edit','Parent',hpanel,'units','normalized',...
        'TooltipString','number of image division in the y direction',...
        'String',num2str(himMod.ny),'Position', pos.panely,'BackgroundColor',[1 1 1],...
        'fontsize',14,'callback',@checkValueInt,'tag','y');
    hpanelyT=uicontrol('Style', 'text','Parent',hpanel,'units','normalized',...
        'String','Number of image divisions in y','Position', pos.panelyT,...
        'fontsize',14,'horizontalAlignment','left');
%     hpanelApply = uicontrol('style','pushbutton','parent',hpanel,'units','normalized',...
%         'position',pos.panelApply,'string','Apply',...
%         'tooltipstring','Run piecewise contrast modification calculation',...
%         'fontsize',14,'callback',@PWcontrastApply);
%     
%     hRecAx = axes('Units','Normalized','Position',pos.axRec,...
%         'Xtick',[],'Ytick',[],'box','off','Parent',himMod.hf,'CLimMode','manual','Clim',[0 255]);
%     image(imread('icons/retrieveR.png','BackgroundColor',[1 1 1]),'parent',hRecAx);
%     set(hRecAx,'visible','off');
%     hRec = uicontrol('style','pushbutton','parent',himMod.hf,'units','normalized',...
%         'position',pos.pbRec,'string','Recover Original Image',...
%         'tooltipstring','Recover original image before modification by selecting rectangular region',...
%         'fontsize',14,'enable','off','callback',@recoverOrig);
% 
%     if ~isempty(Iorig)
%         set(hRec,'enable','on');
%     end  % if ~isempty(Iorig)
    
%     hApply = uicontrol('style','pushbutton','parent',himMod.hf,'units','normalized',...
%         'position',pos.Apply,'string','Apply','tooltipstring','Apply changes',...
%         'fontsize',14,'callback',@imoApply);
    hClose = uicontrol('style','pushbutton','parent',himMod.hf,'units','normalized',...
        'position',pos.Close,'string','Close','tooltipstring','Close this window',...
        'fontsize',14,'callback',@imoClose);
    
end  % if lnewFig

setappdata(H.f1,'himMod',himMod);  % update variable

%     function PWcontrastApply(hobj,eventdata)
%         %  This function runs piecewise contrast calculation
%         
%         % define expression needed for Gaussian fit
%         gaussEqn = 'a*exp(-((x-b)/c)^2)+d';
%         
%         Iundo = I0;  % set up undo possibility
%         set(H.tbar_UNDO,'Enable','on');
%         
%         % get number of divisions
%         nx = str2num(get(hpanelx,'string'));
%         ny = str2num(get(hpanely,'string'));
%         
%         [nrow ncol] = size(I0);
%         
%         nxpos = linspace(1,ncol,nx+1);
%         nypos = linspace(1,nrow,ny+1);
%         % make sure we have integers
%         nxpos(2:end-1) = round(nxpos(2:end-1));
%         nypos(2:end-1) = round(nypos(2:end-1));
%         % let the last point be end+1 so that we can loop over assuming that each
%         % position is the starting value
%         nxpos(end) = nxpos(end)+1;
%         nypos(end) = nypos(end)+1;
%         
%         for k = 1:nx  % loop over horizontal segments
%             for m = 1:ny  % loop over vertical segments
%                 rI = double(I0(nypos(m):nypos(m+1)-1,nxpos(k):nxpos(k+1)-1));
%                 rtmp = reshape(rI,1,numel(rI));  % make it into a vector
%                 
%                 % get value distribution
%                 ncounts = hist(rtmp,0:255);
%                 
%                 % work with upper limit (white)
%                 [rmaxW ipos] = max(ncounts(125:end-1));  % avoid last value
%                 iposW = round(ipos+124);
%                 
%                 % work with lower range
%                 [rmax ipos] = max(ncounts(2:125));  % avoid first value
%                 ipos = ipos + 1;
%                 indx = 1:2*ipos;
%                 startPoints = [rmax ipos ipos/2 0];
%                 Gfit = fit(indx', ncounts(indx)', gaussEqn, 'Start',startPoints);  % fit Gaussian
%                 iposB = round(ipos+sqrt(log(rmax/(2*rmaxW)))*Gfit.c);  % get position
%                 
%                 I0(nypos(m):nypos(m+1)-1,nxpos(k):nxpos(k+1)-1) = ...
%                     imadjust(I0(nypos(m):nypos(m+1)-1,nxpos(k):nxpos(k+1)-1),...
%                     [iposB,iposW]/255, [0,255]/255);
%                 
%                 set(HIM,'Cdata',I0);
%             end  % for m = 1:ny
%         end  % for k = 1:nx
%     
%     end  % function PWcontrastApply

    function checkValueInt(hobj,eventdata)
        % this function is called from an edit box, and makes sure that the
        % value entered is a positive integer
        
        % initial value
        lx = true;
        if strcmpi(get(hobj,'tag'),'x')  % if x division value
            nnew = himMod.nx;
        else  % assume y division value
            nnew = himMod.ny;
            lx = false;
        end  % if strcmpi(get(hobj,'tag'),'x')...
        
        cstr = get(hobj,'string');  % get input value
        
        nval = str2num(cstr);
        if ~isempty(nval) & nval>0  % numerical value
            nnew = round(nval);
        end  % if ~isempty(nval) & nval>0
        
        set(hobj,'string',num2str(nnew));
        if lx  % x value updated
            himMod.nx = nnew;
        else  % y value updated
            himMod.ny = nnew;
        end  % if lx
        
        setappdata(H.f1,'himMod',himMod);  % update variable
    end  % function checkValue(hobj,eventdata)

%     function imoApply(hobj,eventdata)
%         % this function will copy the values in the edit fields into cto
%         % structure so that they can be used by the main program
%         setappdata(H.f1,'himMod',himMod);  % update information
%     end  % function ctoApply



    function imoClose(hobj,eventdata)
        % this function closes the property setup window
        delete(himMod.hf);
    end  % function ctoClose

end  % function imageMod_options




function imageMod(hobj,eventdata)
%  This function runs piecewise contrast calculation

global H I0 Iundo HIM

% define expression needed for Gaussian fit
gaussEqn = 'a*exp(-((x-b)/c)^2)+d';

% default values (used if himMod appdata does not exist in H.f1)
nx = 25;
ny = 10;

% threshold level: floor(rthresLevel*number of pixels) will be used to
% expand the contrast
rthresLevel = 0.0001;
ngthres = 3;  % threshold for determining if the low level is continuous

Iundo = I0;  % set up undo possibility
set(H.tbar_UNDO,'Enable','on');

% get number of divisions
if isappdata(H.f1,'himMod');  % if values exist
    himMod = getappdata(H.f1,'himMod');
    nx = himMod.nx;
    ny = himMod.ny;
end  % if isappdata(H.f1,'himMod');

[nrow ncol] = size(I0);

nxpos = linspace(1,ncol,nx+1);
nypos = linspace(1,nrow,ny+1);
% make sure we have integers
nxpos(2:end-1) = round(nxpos(2:end-1));
nypos(2:end-1) = round(nypos(2:end-1));
% let the last point be end+1 so that we can loop over assuming that each
% position is the starting value
nxpos(end) = nxpos(end)+1;
nypos(end) = nypos(end)+1;

hwait = waitbar(0,'Calculating Piecewise Contrast Modifications');
ntot = nx*ny;
ncount = 0;
Itmp = I0;
for k = 1:nx  % loop over horizontal segments
    for m = 1:ny  % loop over vertical segments
        rI = double(Itmp(nypos(m):nypos(m+1)-1,nxpos(k):nxpos(k+1)-1));
        rtmp = reshape(rI,1,numel(rI));  % make it into a vector
        
        % get value distribution
        ncounts = hist(rtmp,0:255);
        
        % first, treat the case when the histogram is tight (e.g., there
        % are values where there are no entries)
        nthres = floor(numel(rI)*rthresLevel);
        indx = ncounts<nthres;
        indxF = find(indx);
        rtmp = diff(indxF);  % find if they are grouped
        indxG = find(rtmp>ngthres);  % find groups
        ns = indxF(min(indxG)); ne = indxF(max(indxG)+1);  % starting and ending intensity values
        % expand intensity
        Itmp(nypos(m):nypos(m+1)-1,nxpos(k):nxpos(k+1)-1) = ...
            imadjust(Itmp(nypos(m):nypos(m+1)-1,nxpos(k):nxpos(k+1)-1),...
            [ns,ne]/255, [0,255]/255);
        
        % get value distribution
        rI = double(Itmp(nypos(m):nypos(m+1)-1,nxpos(k):nxpos(k+1)-1));
        rtmp = reshape(rI,1,numel(rI));  % make it into a vector
        ncounts = hist(rtmp,0:255);

        % find maximum peak - assume this is associated with the background
        [rmax ipos] = max(ncounts(2:254));  % find maximum within the range
        ipos = ipos + 1;

        % work with upper limit (white)
%         [rmaxW ipos] = max(ncounts(125:end-1));  % avoid last value
%         iposW = round(ipos+124);
        % determine the upper limit as the point at which the slope changes
        indx = find(ncounts~=0);
        rtmp = diff(ncounts(indx));
        rtmp2 = find(rtmp(1:end-1)>0);
        iposW = indx(rtmp2(end)); 
        if iposW<ipos
            iposW = 255;
        end  % if iposW<ipos
        rmaxW = ncounts(iposW);
        
        % work with lower range
%         [rmax ipos] = max(ncounts(2:125));  % avoid first value
%         ipos = ipos + 1;
%         %indx = 1:2*ipos;
%         indx = find(ncounts(1:125)~=0);  % remove bins where count is zero 
        % (this happens when the intensity range has been changed by imadjust)
        ns = max(1,ipos-50); ne = min(ns+100,255);  % defining range for Gaussian fitting
        indx = find(ncounts(ns:ne)~=0)+ns-1;  % remove bins where count is zero
        % which happens when the intensity range has been modified by
        % imadjust
        startPoints = [rmax ipos ipos/2 0];
        Gfit = fit(indx', ncounts(indx)', gaussEqn, 'Start',startPoints);  % fit Gaussian
        clear rtmp;
        if (rmax < rmaxW)  % no values found for upper range
            rtmp(1) = Gfit.b + sqrt(Gfit.c^2*log(5));
            rtmp(2) = Gfit.b - sqrt(Gfit.c^2*log(5));
            %iposB = round(ipos+sqrt(log(5))*Gfit.c);  % get position
        else
            if (log(Gfit.a)-log(2*rmaxW-Gfit.d))<0 | 2*rmaxW-Gfit.d<0  % 2*upper peak is below the baseline
                rtmp(1) = Gfit.b + sqrt(Gfit.c^2*log(5));
                rtmp(2) = Gfit.b - sqrt(Gfit.c^2*log(5));
            else
                rtmp(1) = Gfit.b + sqrt(Gfit.c^2*(log(Gfit.a)-log(2*rmaxW-Gfit.d)));
                rtmp(2) = Gfit.b - sqrt(Gfit.c^2*(log(Gfit.a)-log(2*rmaxW-Gfit.d)));
            end  % if 2*rmaxW-Gfit.d<0
            %iposB = round(ipos+sqrt(log(rmax/(2*rmaxW)))*Gfit.c);  % get position
        end  % if (rmaxW==0)
        iposB = max(round(rtmp)); clear rtmp;
        
        if iposB>iposW
            iposW = 255;
        end  % if iposB>iposW
        
        %disp([nypos(m),nypos(m+1)-1,nxpos(k),nxpos(k+1)-1]);
        Itmp(nypos(m):nypos(m+1)-1,nxpos(k):nxpos(k+1)-1) = ...
            imadjust(Itmp(nypos(m):nypos(m+1)-1,nxpos(k):nxpos(k+1)-1),...
            [iposB,iposW]/255, [0,255]/255);
        
       % set(HIM,'Cdata',I0);
        ncount = ncount + 1;
        waitbar(ncount/ntot,hwait);
    end  % for m = 1:ny
end  % for k = 1:nx
delete(hwait);
I0 = Itmp;  % transfer information to I0 only after all boxes are successfully converted
set(HIM,'CDATA',I0);

end  % function imageMod


function recoverOrig(hobj,eventdata)
% this function allows user to select a rectangular region and recovers the
% original image in that region

global H I0 Iorig HIM Iundo

% turn off zoon and pan
zoom off; pan off;

% check to make sure Iorig exists
if isempty(Iorig)
    return
end  % exit function

if any(size(I0)~=size(Iorig))  % if sizes are different
    disp('ERROR: size of I0 and Iorig different');
    return  % exit function
end  % if any(size(I0)~=size(Iorig))

Iundo = I0;  % set up value for allowing undo

% select rectangular region
%zoom(H.f1,'enable','off'); pan(H.f1,'enable','off');
hchil = get(0,'children');
set(0,'children',[H.f1; hchil(hchil~=H.f1)]);
hrec = imrect(H.ax1);  % create rectangle
pos = wait(hrec);  % get positions of the rectangle
delete(hrec);

% get nx and ny ranges
[nrow ncol] = size(I0);
ncs = max(round(pos(1)),1);  % starting column number
nce = min(ncs+round(pos(3)),ncol);  % ending column number
nrs = max(round(pos(2)),1);  % starting row number
nre = min(round(pos(2)+pos(4)),nrow);  % ending row number

I0(nrs:nre,ncs:nce) = Iorig(nrs:nre,ncs:nce);
set(HIM,'CDATA',I0); drawnow

set(H.tbar_UNDO,'Enable','on');  % turn undo on

end  % function recoverOrig




function viewOrigImage(hobj,eventdata)
% this function switches between original and processed image for H.ax1
% space

global H I0 Iorig HIM

cstate = get(hobj,'state');

if strcmpi(cstate,'on')  % turn on the original image
    set(HIM,'Cdata',Iorig);
else  % turn on the processed image
    set(HIM,'Cdata',I0);
end  % if strcmpi(cstate,'on')...

end  % function viewOrigImage




function closeDigitSeis(hObj,eventdata)
% this function is called when the DigitSeis window is closed

global H
global nSizeDefaultUicontrolFontSize

% set the default font size to original
set(0,'defaultUicontrolFontSize',nSizeDefaultUicontrolFontSize);

closeAdditionalWindows;  % close all additional windows

% if isappdata(H.f1,'ito')  % check if digitization property window is still open
%     ito = getappdata(H.f1,'ito');
%     if isvalid(ito.hf)  % if figure still exists
%         delete(ito.hf);
%     end  % if any (ishandle(digitize_opt.hf))
%     try; rmappdata(H.f1,'ito'); end;
% end  % if isappdata(H.f1,'digitize_opt')
% 
% if isappdata(H.f1,'Hfdistinguish')  % check if Edit Classification window is still open
%     Hfdistinguish = getappdata(H.f1,'Hfdistinguish');
%     if isvalid(Hfdistinguish)  % figure still exists
%         delete(Hfdistinguish);
%     end  % if any(ishandle(Hfdistinguish))
%     % rmappdata(H.f1,'Hfdistinguish');
% end  % if isappdata(H.f1,'Hfdistinguish')
% 
% if isappdata(H.f1,'digitize_opt')  % check if digitization property window is still open
%     digitize_opt = getappdata(H.f1,'digitize_opt');
%     if isvalid(digitize_opt.hf)  % if figure still exists
%         delete(digitize_opt.hf);
%     end  % if any (ishandle(digitize_opt.hf))
%     try; rmappdata(H.f1,'digitize_opt'); end;
% end  % if isappdata(H.f1,'digitize_opt')
% 
% if isappdata(H.f1,'h1stripe')  % check if correct_trace window is still open
%     h1stripe = getappdata(H.f1,'h1stripe');
%     if isvalid(h1stripe.f)  % if window still exists
%         delete(h1stripe.f);
%     end  % if any(ishandle(h1stripe.f))
%     try; rmappdata(H.f1,'h1stripe'); end
% end  % if isappdata(H.f1,'h1stripe')
% 
% if isappdata(H.f1,'cto')  % check if correct_trace property window is still open
%     cto = getappdata(H.f1,'cto');
%     if isvalid(cto.hf)  % if figure window still exists
%         delete(cto.hf);  % delete figure
%     end  % if any(ishandle(cto.hf))
% end  % if isappdata(H.f1,'cto')
% 
% if isappdata(H.f1,'himMod')  % image modification window
%     himMod = getappdata(H.f1,'himMod');
%     if isvalid(himMod.hf)  % if figure window still exists
%         delete(himMod.hf);
%     end  % if any(ishandle(himMod.hf))
% end  % if isappdata(H.f1,'himMod')

end  % function closeDigitSeis

function closeAdditionalWindows
% this function is called when window other than the main window are to be closed

global H

if isappdata(H.f1,'ito')  % check if digitization property window is still open
    ito = getappdata(H.f1,'ito');
    if isvalid(ito.hf)  % if figure still exists
        delete(ito.hf);
    end  % if any (ishandle(digitize_opt.hf))
    try; rmappdata(H.f1,'ito'); end;
end  % if isappdata(H.f1,'digitize_opt')

if isappdata(H.f1,'Hfdistinguish')  % check if Edit Classification window is still open
    Hfdistinguish = getappdata(H.f1,'Hfdistinguish');
    if isvalid(Hfdistinguish)  % figure still exists
        delete(Hfdistinguish);
    end  % if any(ishandle(Hfdistinguish))
    % rmappdata(H.f1,'Hfdistinguish');
end  % if isappdata(H.f1,'Hfdistinguish')

if isappdata(H.f1,'digitize_opt')  % check if digitization property window is still open
    digitize_opt = getappdata(H.f1,'digitize_opt');
    if isvalid(digitize_opt.hf)  % if figure still exists
        delete(digitize_opt.hf);
    end  % if any (ishandle(digitize_opt.hf))
    try; rmappdata(H.f1,'digitize_opt'); end;
end  % if isappdata(H.f1,'digitize_opt')

if isappdata(H.f1,'h1stripe')  % check if correct_trace window is still open
    h1stripe = getappdata(H.f1,'h1stripe');
    if isvalid(h1stripe.f)  % if window still exists
        delete(h1stripe.f);
    end  % if any(ishandle(h1stripe.f))
    try; rmappdata(H.f1,'h1stripe'); end
end  % if isappdata(H.f1,'h1stripe')

if isappdata(H.f1,'cto')  % check if correct_trace property window is still open
    cto = getappdata(H.f1,'cto');
    if isvalid(cto.hf)  % if figure window still exists
        delete(cto.hf);  % delete figure
    end  % if any(ishandle(cto.hf))
end  % if isappdata(H.f1,'cto')

if isappdata(H.f1,'himMod')  % image modification window
    himMod = getappdata(H.f1,'himMod');
    if isvalid(himMod.hf)  % if figure window still exists
        delete(himMod.hf);
    end  % if any(ishandle(himMod.hf))
end  % if isappdata(H.f1,'himMod')

end  % function closeDigitSeis









